(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    841515,      18818]
NotebookOptionsPosition[    836318,      18674]
NotebookOutlinePosition[    836711,      18690]
CellTagsIndexPosition[    836668,      18687]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[TextData[StyleBox["Model definitions", "Title"]], "Subsection",
 CellChangeTimes->{{3.627888094472248*^9, 3.627888097056507*^9}}],

Cell[CellGroupData[{

Cell["\<\

Definition of the two models, StochasticDeath and IndividualTolerance, for \
different dose metrices, with 3, 4 or 5 model parameters\
\>", "Subsubtitle",
 CellChangeTimes->{{3.587829073851083*^9, 3.5878290965194807`*^9}, {
  3.6278881956763678`*^9, 3.6278882169254923`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"$accuracy", "=", "14"}], ";"}]], "Input",
 CellChangeTimes->{{3.647920526779357*^9, 3.647920535977277*^9}, {
  3.648539061785339*^9, 3.6485390619413404`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "SDmodel", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SDmodel", "[", 
    RowBox[{"Ci0_", ",", "Cw_", ",", "params_", ",", "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"KD", ",", "Haz$BG", ",", "kk", ",", "z"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Haz$BG", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"KD", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kk", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"z", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"locSol", "=", 
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Ci", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"KD", "*", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cw", "[", "t", "]"}], "-", 
               RowBox[{"Ci", "[", "t", "]"}]}], ")"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Haz", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"kk", "*", 
             RowBox[{"Max", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Ci", "[", "t", "]"}], "-", "z"}], ",", "0"}], 
              "]"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"SOT", "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{
               RowBox[{"-", "Haz$BG"}], " ", "t"}]], " ", "*", 
             RowBox[{"Exp", "[", 
              RowBox[{"-", 
               RowBox[{"Haz", "[", "t", "]"}]}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Ci", "[", "0", "]"}], "\[Equal]", "Ci0"}], ",", 
           RowBox[{
            RowBox[{"Haz", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
           RowBox[{
            RowBox[{"SOT", "[", "0", "]"}], "\[Equal]", "1"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Ci", ",", "SOT", ",", "Haz"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
         RowBox[{"AccuracyGoal", "\[Rule]", "$accuracy"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<After NDSolve:\>\"", ",", 
          RowBox[{"ToString", "[", "tEnd", "]"}]}], "]"}], ";"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Important", " ", 
        RowBox[{"convention", ":", " ", 
         RowBox[{"Survival", " ", "over", " ", "time", " ", 
          RowBox[{"(", "SOT", ")"}], " ", "is", " ", "the", " ", "third", " ",
           "element", " ", "of", " ", "the", " ", "resturned", " ", 
          RowBox[{"list", "."}]}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Ci", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{
          RowBox[{"Haz", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{
          RowBox[{"SOT", "[", "t", "]"}], "/.", "locSol"}]}], "}"}], 
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.58772974271323*^9, 3.587729775395649*^9}, {
   3.5877298235690665`*^9, 3.5877299231295433`*^9}, {3.5877299720673704`*^9, 
   3.5877300035953746`*^9}, {3.5877301681774845`*^9, 
   3.5877301821240635`*^9}, {3.587730828477537*^9, 3.5877308494910064`*^9}, {
   3.5877308895377197`*^9, 3.5877309217209325`*^9}, {3.5877309737632*^9, 
   3.5877310456645217`*^9}, {3.5877310794545546`*^9, 3.58773115784556*^9}, {
   3.587731295454924*^9, 3.5877313332542086`*^9}, {3.5877323197652617`*^9, 
   3.5877323209196763`*^9}, {3.587737885887328*^9, 3.587737891284998*^9}, {
   3.5877379285694757`*^9, 3.5877379423756523`*^9}, 3.587740189160697*^9, {
   3.5877888502741594`*^9, 3.587788852442587*^9}, {3.5877976669607058`*^9, 
   3.5877977032426805`*^9}, {3.5877981701638656`*^9, 3.587798171942209*^9}, 
   3.5877982038431864`*^9, {3.5880539743950443`*^9, 3.588054079478552*^9}, {
   3.58805444930453*^9, 3.588054456584258*^9}, {3.5883126381170254`*^9, 
   3.5883126662478695`*^9}, 3.5956583178866377`*^9, {3.595844452492592*^9, 
   3.5958445398733287`*^9}, {3.595845057088045*^9, 3.5958450616395006`*^9}, {
   3.5958451188662224`*^9, 3.5958451256589017`*^9}, {3.617002431960808*^9, 
   3.6170024615411034`*^9}, {3.6170070883726892`*^9, 
   3.6170070906191034`*^9}, {3.6170213975459337`*^9, 3.617021401617586*^9}, {
   3.6170216694143295`*^9, 3.61702166944553*^9}, {3.6170873253028173`*^9, 
   3.6170873259268217`*^9}, {3.6170873869388127`*^9, 3.617087408654152*^9}, 
   3.617087570130787*^9, {3.617087885197814*^9, 3.617087885353815*^9}, 
   3.6179838560165596`*^9, {3.6179846952964144`*^9, 3.6179846972476096`*^9}, {
   3.6186551372165155`*^9, 3.618655202339862*^9}, {3.618718798019719*^9, 
   3.618718798107728*^9}, 3.618824178129185*^9, {3.6188242509240627`*^9, 
   3.618824279799848*^9}, {3.618825148726448*^9, 3.6188251487420483`*^9}, {
   3.618905974761194*^9, 3.618906003961114*^9}, {3.6194385050924373`*^9, 
   3.6194385072386518`*^9}, {3.619439626271544*^9, 3.6194396295518723`*^9}, {
   3.647695209496339*^9, 3.6476952106804066`*^9}, 3.647920541169796*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "SDmodelMOSF", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SDmodelMOSF", "[", 
    RowBox[{"Ci0_", ",", "Cw_", ",", "params_", ",", "tEnd_", ",", "MOSF_"}], 
    "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"KD", ",", "Haz$BG", ",", "kk", ",", "z"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Haz$BG", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"KD", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kk", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"z", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"locSol", "=", 
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Ci", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"KD", "*", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"MOSF", "*", 
                RowBox[{"Cw", "[", "t", "]"}]}], "-", 
               RowBox[{"Ci", "[", "t", "]"}]}], ")"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Haz", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"kk", "*", 
             RowBox[{"Max", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Ci", "[", "t", "]"}], "-", "z"}], ",", "0"}], 
              "]"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"SOT", "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{
               RowBox[{"-", "Haz$BG"}], " ", "t"}]], " ", "*", 
             RowBox[{"Exp", "[", 
              RowBox[{"-", 
               RowBox[{"Haz", "[", "t", "]"}]}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Ci", "[", "0", "]"}], "\[Equal]", "Ci0"}], ",", 
           RowBox[{
            RowBox[{"Haz", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
           RowBox[{
            RowBox[{"SOT", "[", "0", "]"}], "\[Equal]", "1"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Ci", ",", "SOT", ",", "Haz"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
         RowBox[{"AccuracyGoal", "\[Rule]", "$accuracy"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<After NDSolve:\>\"", ",", 
          RowBox[{"ToString", "[", "tEnd", "]"}]}], "]"}], ";"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Important", " ", 
        RowBox[{"convention", ":", " ", 
         RowBox[{"Survival", " ", "over", " ", "time", " ", 
          RowBox[{"(", "SOT", ")"}], " ", "is", " ", "the", " ", "third", " ",
           "element", " ", "of", " ", "the", " ", "resturned", " ", 
          RowBox[{"list", "."}]}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Ci", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{
          RowBox[{"Haz", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{
          RowBox[{"SOT", "[", "t", "]"}], "/.", "locSol"}]}], "}"}], 
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6405240646965733`*^9, 3.6405240801304564`*^9}, {
   3.6478271275144525`*^9, 3.647827127670453*^9}, 3.647920542536933*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "SDmodel4p", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SDmodel4p", "[", 
    RowBox[{"Ci0_", ",", "Cw_", ",", "params_", ",", "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"kin", ",", "kout", ",", "Haz$BG", ",", "kk", ",", "z"}], "}"}],
      ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Haz$BG", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kin", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kout", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kk", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"z", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "5", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"locSol", "=", 
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Ci", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             RowBox[{"kin", "*", 
              RowBox[{"Cw", "[", "t", "]"}]}], "-", 
             RowBox[{"kout", "*", 
              RowBox[{"Ci", "[", "t", "]"}]}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Haz", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"kk", "*", 
             RowBox[{"Max", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Ci", "[", "t", "]"}], "-", "z"}], ",", "0"}], 
              "]"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"SOT", "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{
               RowBox[{"-", "Haz$BG"}], " ", "t"}]], " ", "*", 
             RowBox[{"Exp", "[", 
              RowBox[{"-", 
               RowBox[{"Haz", "[", "t", "]"}]}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Ci", "[", "0", "]"}], "\[Equal]", "Ci0"}], ",", 
           RowBox[{
            RowBox[{"Haz", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
           RowBox[{
            RowBox[{"SOT", "[", "0", "]"}], "\[Equal]", "1"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Ci", ",", "SOT", ",", "Haz"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
         RowBox[{"AccuracyGoal", "\[Rule]", "$accuracy"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<After NDSolve:\>\"", ",", 
          RowBox[{"ToString", "[", "tEnd", "]"}]}], "]"}], ";"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Important", " ", 
        RowBox[{"convention", ":", " ", 
         RowBox[{"Survival", " ", "over", " ", "time", " ", 
          RowBox[{"(", "SOT", ")"}], " ", "is", " ", "the", " ", "third", " ",
           "element", " ", "of", " ", "the", " ", "resturned", " ", 
          RowBox[{"list", "."}]}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Ci", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{
          RowBox[{"Haz", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{
          RowBox[{"SOT", "[", "t", "]"}], "/.", "locSol"}]}], "}"}], 
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.58772974271323*^9, 3.587729775395649*^9}, {
   3.5877298235690665`*^9, 3.5877299231295433`*^9}, {3.5877299720673704`*^9, 
   3.5877300035953746`*^9}, {3.5877301681774845`*^9, 
   3.5877301821240635`*^9}, {3.587730828477537*^9, 3.5877308494910064`*^9}, {
   3.5877308895377197`*^9, 3.5877309217209325`*^9}, {3.5877309737632*^9, 
   3.5877310456645217`*^9}, {3.5877310794545546`*^9, 3.58773115784556*^9}, {
   3.587731295454924*^9, 3.5877313332542086`*^9}, {3.5877323197652617`*^9, 
   3.5877323209196763`*^9}, {3.587737885887328*^9, 3.587737891284998*^9}, {
   3.5877379285694757`*^9, 3.5877379423756523`*^9}, 3.587740189160697*^9, {
   3.5877888502741594`*^9, 3.587788852442587*^9}, {3.5877976669607058`*^9, 
   3.5877977032426805`*^9}, {3.5877981701638656`*^9, 3.587798171942209*^9}, 
   3.5877982038431864`*^9, {3.5880539743950443`*^9, 3.588054079478552*^9}, {
   3.58805444930453*^9, 3.588054456584258*^9}, {3.5883126381170254`*^9, 
   3.5883126662478695`*^9}, 3.5956583178866377`*^9, {3.595844452492592*^9, 
   3.5958445398733287`*^9}, {3.595845057088045*^9, 3.5958450616395006`*^9}, {
   3.5958451188662224`*^9, 3.5958451256589017`*^9}, {3.617002431960808*^9, 
   3.6170024615411034`*^9}, {3.6170070883726892`*^9, 
   3.6170070906191034`*^9}, {3.6170213975459337`*^9, 3.617021401617586*^9}, {
   3.6170216694143295`*^9, 3.61702166944553*^9}, {3.6170873253028173`*^9, 
   3.6170873259268217`*^9}, {3.6170873869388127`*^9, 3.617087408654152*^9}, 
   3.617087570130787*^9, {3.617087885197814*^9, 3.617087885353815*^9}, 
   3.6179838560165596`*^9, {3.6179846952964144`*^9, 3.6179846972476096`*^9}, {
   3.6186551372165155`*^9, 3.618655202339862*^9}, {3.618718798019719*^9, 
   3.618718798107728*^9}, 3.618824178129185*^9, {3.6188242509240627`*^9, 
   3.618824279799848*^9}, {3.618825148726448*^9, 3.6188251487420483`*^9}, {
   3.618905974761194*^9, 3.618906003961114*^9}, {3.6194385050924373`*^9, 
   3.6194385072386518`*^9}, {3.619439626271544*^9, 3.6194396295518723`*^9}, {
   3.6207192209362946`*^9, 3.620719273634564*^9}, {3.6209987491881638`*^9, 
   3.6209987496362085`*^9}, 3.620998784733718*^9, 3.6209990412963715`*^9, {
   3.6478271320228605`*^9, 3.6478271321788607`*^9}, 3.647920545449224*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "SDmodel5p", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SDmodel5p", "[", 
    RowBox[{"Ci0_", ",", "Cw_", ",", "params_", ",", "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "kin", ",", "kout", ",", "Haz$BG", ",", "kk", ",", "z", ",", "kr"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Haz$BG", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kin", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kout", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kk", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"z", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "5", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kr", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "6", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"locSol", "=", 
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Ci", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             RowBox[{"kin", "*", 
              RowBox[{"Cw", "[", "t", "]"}]}], "-", 
             RowBox[{"kout", "*", 
              RowBox[{"Ci", "[", "t", "]"}]}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"scD", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"kr", "*", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ci", "[", "t", "]"}], "-", 
               RowBox[{"scD", "[", "t", "]"}]}], ")"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Haz", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"kk", "*", 
             RowBox[{"Max", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"scD", "[", "t", "]"}], "-", "z"}], ",", "0"}], 
              "]"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"SOT", "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             SuperscriptBox["\[ExponentialE]", 
              RowBox[{
               RowBox[{"-", "Haz$BG"}], " ", "t"}]], " ", "*", 
             RowBox[{"Exp", "[", 
              RowBox[{"-", 
               RowBox[{"Haz", "[", "t", "]"}]}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Ci", "[", "0", "]"}], "\[Equal]", "Ci0"}], ",", 
           RowBox[{
            RowBox[{"Haz", "[", "0", "]"}], "\[Equal]", "0"}], ",", 
           RowBox[{
            RowBox[{"SOT", "[", "0", "]"}], "\[Equal]", "1"}], ",", 
           RowBox[{
            RowBox[{"scD", "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Ci", ",", "SOT", ",", "Haz", ",", "scD"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
         RowBox[{"AccuracyGoal", "\[Rule]", "$accuracy"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<After NDSolve:\>\"", ",", 
          RowBox[{"ToString", "[", "tEnd", "]"}]}], "]"}], ";"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Important", " ", 
        RowBox[{"convention", ":", " ", 
         RowBox[{"Survival", " ", "over", " ", "time", " ", 
          RowBox[{"(", "SOT", ")"}], " ", "is", " ", "the", " ", "third", " ",
           "element", " ", "of", " ", "the", " ", "resturned", " ", 
          RowBox[{"list", "."}]}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Ci", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{
          RowBox[{"Haz", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{
          RowBox[{"SOT", "[", "t", "]"}], "/.", "locSol"}], ",", 
         RowBox[{"scD", "/.", "locSol"}]}], "}"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.58772974271323*^9, 3.587729775395649*^9}, {
   3.5877298235690665`*^9, 3.5877299231295433`*^9}, {3.5877299720673704`*^9, 
   3.5877300035953746`*^9}, {3.5877301681774845`*^9, 
   3.5877301821240635`*^9}, {3.587730828477537*^9, 3.5877308494910064`*^9}, {
   3.5877308895377197`*^9, 3.5877309217209325`*^9}, {3.5877309737632*^9, 
   3.5877310456645217`*^9}, {3.5877310794545546`*^9, 3.58773115784556*^9}, {
   3.587731295454924*^9, 3.5877313332542086`*^9}, {3.5877323197652617`*^9, 
   3.5877323209196763`*^9}, {3.587737885887328*^9, 3.587737891284998*^9}, {
   3.5877379285694757`*^9, 3.5877379423756523`*^9}, 3.587740189160697*^9, {
   3.5877888502741594`*^9, 3.587788852442587*^9}, {3.5877976669607058`*^9, 
   3.5877977032426805`*^9}, {3.5877981701638656`*^9, 3.587798171942209*^9}, 
   3.5877982038431864`*^9, {3.5880539743950443`*^9, 3.588054079478552*^9}, {
   3.58805444930453*^9, 3.588054456584258*^9}, {3.5883126381170254`*^9, 
   3.5883126662478695`*^9}, 3.5956583178866377`*^9, {3.595844452492592*^9, 
   3.5958445398733287`*^9}, {3.595845057088045*^9, 3.5958450616395006`*^9}, {
   3.5958451188662224`*^9, 3.5958451256589017`*^9}, {3.617002431960808*^9, 
   3.6170024615411034`*^9}, {3.6170070883726892`*^9, 
   3.6170070906191034`*^9}, {3.6170213975459337`*^9, 3.617021401617586*^9}, {
   3.6170216694143295`*^9, 3.61702166944553*^9}, {3.6170873253028173`*^9, 
   3.6170873259268217`*^9}, {3.6170873869388127`*^9, 3.617087408654152*^9}, 
   3.617087570130787*^9, {3.617087885197814*^9, 3.617087885353815*^9}, 
   3.6179838560165596`*^9, {3.6179846952964144`*^9, 3.6179846972476096`*^9}, {
   3.6186551372165155`*^9, 3.618655202339862*^9}, {3.618718798019719*^9, 
   3.618718798107728*^9}, 3.618824178129185*^9, {3.6188242509240627`*^9, 
   3.618824279799848*^9}, {3.618825148726448*^9, 3.6188251487420483`*^9}, {
   3.618905974761194*^9, 3.618906003961114*^9}, {3.6194385050924373`*^9, 
   3.6194385072386518`*^9}, {3.619439626271544*^9, 3.6194396295518723`*^9}, {
   3.6207192209362946`*^9, 3.620719273634564*^9}, {3.6209987491881638`*^9, 
   3.6209987496362085`*^9}, 3.620998784733718*^9, 3.6209990412963715`*^9, {
   3.6261568014563885`*^9, 3.62615689087733*^9}, {3.626158725812805*^9, 
   3.626158726852909*^9}, {3.6478271365780687`*^9, 3.6478271367184687`*^9}, 
   3.6479205474444237`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ITmodel", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ITmodel", "[", 
    RowBox[{"Ci0_", ",", "Cw_", ",", "params_", ",", "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\[Delta]t", ",", "CSol", ",", "CTab", ",", "F1", ",", "F2", ",", "F3", 
       ",", "CMaxVal", ",", "CMaxTab", ",", "FfuncTab", ",", "SOTTab", ",", 
       "Ci", ",", "Haz", ",", "SOTloc", ",", "KD", ",", "\[Alpha]", ",", 
       "\[Beta]", ",", "Haz$BG", ",", "maxC"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Clear", "[", 
       RowBox[{
       "\[Delta]t", ",", "CSol", ",", "CTab", ",", "F1", ",", "F2", ",", "F3",
         ",", "CMaxVal", ",", "CMaxTab", ",", "FfuncTab", ",", "SOTTab", ",", 
        "Ci", ",", "Haz", ",", "SOTloc", ",", "KD", ",", "\[Alpha]", ",", 
        "\[Beta]", ",", "Haz$BG", ",", "maxC"}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"Haz$BG", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"KD", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Alpha]", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Beta]", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Delta]t", "=", "0.025"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"CSol", "=", 
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Ci", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"KD", "*", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Cw", "[", "t", "]"}], "-", 
               RowBox[{"Ci", "[", "t", "]"}]}], ")"}]}]}], ",", 
           RowBox[{
            RowBox[{"Ci", "[", "0", "]"}], "\[Equal]", "Ci0"}]}], "}"}], ",", 
         RowBox[{"{", "Ci", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
         RowBox[{"AccuracyGoal", "\[Rule]", "$accuracy"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"CTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Ci", "[", "t", "]"}], "/.", "CSol"}], ")"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd", ",", "\[Delta]t"}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"CMaxVal", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"CMaxTab", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "<=", 
         RowBox[{"Length", "[", "CTab", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"CTab", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "2"}], "]"}], "]"}], ">", "CMaxVal"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"CMaxTab", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"i", "-", "1"}], ")"}], "*", "\[Delta]t"}], ",", 
               RowBox[{"CTab", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "}"}]}], "]"}], ";", 
           RowBox[{"CMaxVal", "=", 
            RowBox[{"CTab", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "2"}], "]"}], "]"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"CMaxTab", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{"i", "-", "1"}], ")"}], "*", "\[Delta]t"}], ",", 
              "CMaxVal"}], "}"}]}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Clear", "[", "maxC", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"maxC", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"CMaxTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"FfuncTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"maxC", "[", "t", "]"}], ">", "0"}], " ", "&&", " ", 
              RowBox[{"\[Alpha]", ">", "0"}]}], ",", 
             FractionBox["1", 
              RowBox[{"1", "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 FractionBox[
                  RowBox[{"maxC", "[", "t", "]"}], "\[Alpha]"], ")"}], 
                RowBox[{"-", "\[Beta]"}]]}]], ",", "0"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "\[Delta]t", ",", "tEnd", ",", "\[Delta]t"}], 
          "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"PrependTo", "[", 
       RowBox[{"FfuncTab", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Haz", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"FfuncTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"SOTTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"Haz", "[", "t", "]"}]}], ")"}], "*", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "Haz$BG"}], " ", "t"}]]}]}], "}"}], " ", ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd", ",", "\[Delta]t"}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"SOTloc", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"SOTTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"maxC", "[", "t", "]"}], ",", 
         RowBox[{"Haz", "[", "t", "]"}], ",", 
         RowBox[{"SOTloc", "[", "t", "]"}]}], "}"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5877888622239122`*^9, 3.5877888633315268`*^9}, {
   3.5877889204750595`*^9, 3.5877890188811207`*^9}, {3.587789062561681*^9, 
   3.5877892342116814`*^9}, {3.5877905293890862`*^9, 3.587790550605358*^9}, 
   3.587790594051915*^9, {3.5877907330937138`*^9, 3.5877909426043997`*^9}, {
   3.5877910034295797`*^9, 3.5877910137569118`*^9}, {3.5877912065471897`*^9, 
   3.5877912069995956`*^9}, {3.587795689708955*^9, 3.5877957271007166`*^9}, {
   3.587795826047711*^9, 3.5877958573557067`*^9}, {3.5877959555238795`*^9, 
   3.5877960124167147`*^9}, {3.587796048717682*^9, 3.5877960848938503`*^9}, {
   3.5877961165616474`*^9, 3.587796118932832*^9}, {3.587796246756984*^9, 
   3.587796250859784*^9}, {3.5877962898753843`*^9, 3.587796294508584*^9}, {
   3.587796336269784*^9, 3.5877963944421844`*^9}, {3.587796428231784*^9, 
   3.587796457700184*^9}, {3.587796530832984*^9, 3.5877965644353843`*^9}, {
   3.587796683986136*^9, 3.5877966856410437`*^9}, {3.587797450447692*^9, 
   3.5877974574679728`*^9}, {3.587797488989234*^9, 3.587797493349408*^9}, {
   3.587797627460493*^9, 3.587797659909325*^9}, {3.587797854531562*^9, 
   3.5877978996421347`*^9}, 3.587798043433528*^9, {3.587798115284825*^9, 
   3.587798131820295*^9}, {3.5877981796795607`*^9, 3.5877981817074957`*^9}, 
   3.5878201186578417`*^9, {3.5878216802095823`*^9, 3.5878216809115686`*^9}, 
   3.5878236605750427`*^9, {3.588054066234227*^9, 3.5880540949160953`*^9}, {
   3.588054384616062*^9, 3.5880544422598257`*^9}, {3.588320663985593*^9, 
   3.5883206766978636`*^9}, {3.5910815790354104`*^9, 3.591081589908498*^9}, {
   3.5910820341429167`*^9, 3.5910821049950013`*^9}, 3.591089023963829*^9, {
   3.591346189249627*^9, 3.59134619027673*^9}, 3.5956014424451456`*^9, 
   3.5956017226789474`*^9, {3.5958451422295585`*^9, 3.5958451437937145`*^9}, {
   3.595846740594379*^9, 3.5958467410814276`*^9}, 3.5958468499593143`*^9, {
   3.6192540175298157`*^9, 3.619254031230186*^9}, {3.6207017192091703`*^9, 
   3.6207017319034395`*^9}, {3.6207192836465654`*^9, 
   3.6207192947676773`*^9}, {3.621160078872097*^9, 3.6211600792311325`*^9}, {
   3.6264183429152193`*^9, 3.6264183721006136`*^9}, {3.626423283991006*^9, 
   3.626423284790086*^9}, 3.6303322903583264`*^9, 3.6303323348654118`*^9, {
   3.6331555653556013`*^9, 3.6331555776485586`*^9}, 3.633164716857084*^9, {
   3.6345334725297413`*^9, 3.6345334727467628`*^9}, {3.647695233008684*^9, 
   3.647695246857476*^9}, 3.64792054931161*^9, {3.648222931864921*^9, 
   3.6482229322969646`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ITmodelMOSF", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ITmodelMOSF", "[", 
    RowBox[{"Ci0_", ",", "Cw_", ",", "params_", ",", "tEnd_", ",", "MOSF_"}], 
    "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\[Delta]t", ",", "CSol", ",", "CTab", ",", "F1", ",", "F2", ",", "F3", 
       ",", "CMaxVal", ",", "CMaxTab", ",", "FfuncTab", ",", "SOTTab", ",", 
       "Ci", ",", "Haz", ",", "SOTloc", ",", "KD", ",", "\[Alpha]", ",", 
       "\[Beta]", ",", "Haz$BG", ",", "maxC"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Clear", "[", 
       RowBox[{
       "\[Delta]t", ",", "CSol", ",", "CTab", ",", "F1", ",", "F2", ",", "F3",
         ",", "CMaxVal", ",", "CMaxTab", ",", "FfuncTab", ",", "SOTTab", ",", 
        "Ci", ",", "Haz", ",", "SOTloc", ",", "KD", ",", "\[Alpha]", ",", 
        "\[Beta]", ",", "Haz$BG", ",", "maxC"}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"Haz$BG", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"KD", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Alpha]", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Beta]", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Delta]t", "=", "0.01"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"CSol", "=", 
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Ci", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"KD", "*", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"MOSF", "*", 
                RowBox[{"Cw", "[", "t", "]"}]}], "-", 
               RowBox[{"Ci", "[", "t", "]"}]}], ")"}]}]}], ",", 
           RowBox[{
            RowBox[{"Ci", "[", "0", "]"}], "\[Equal]", "Ci0"}]}], "}"}], ",", 
         RowBox[{"{", "Ci", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
         RowBox[{"AccuracyGoal", "\[Rule]", "$accuracy"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"CTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Ci", "[", "t", "]"}], "/.", "CSol"}], ")"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd", ",", "\[Delta]t"}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"CMaxVal", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"CMaxTab", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "<=", 
         RowBox[{"Length", "[", "CTab", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"CTab", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "2"}], "]"}], "]"}], ">", "CMaxVal"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"CMaxTab", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"i", "-", "1"}], ")"}], "*", "\[Delta]t"}], ",", 
               RowBox[{"CTab", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "}"}]}], "]"}], ";", 
           RowBox[{"CMaxVal", "=", 
            RowBox[{"CTab", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "2"}], "]"}], "]"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"CMaxTab", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{"i", "-", "1"}], ")"}], "*", "\[Delta]t"}], ",", 
              "CMaxVal"}], "}"}]}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Clear", "[", "maxC", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"maxC", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"CMaxTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"FfuncTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"maxC", "[", "t", "]"}], ">", "0"}], " ", "&&", " ", 
              RowBox[{"\[Alpha]", ">", "0"}]}], ",", 
             FractionBox["1", 
              RowBox[{"1", "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 FractionBox[
                  RowBox[{"maxC", "[", "t", "]"}], "\[Alpha]"], ")"}], 
                RowBox[{"-", "\[Beta]"}]]}]], ",", "0"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "\[Delta]t", ",", "tEnd", ",", "\[Delta]t"}], 
          "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"PrependTo", "[", 
       RowBox[{"FfuncTab", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Haz", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"FfuncTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"SOTTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"Haz", "[", "t", "]"}]}], ")"}], "*", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "Haz$BG"}], " ", "t"}]]}]}], "}"}], " ", ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd", ",", "\[Delta]t"}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"SOTloc", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"SOTTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"maxC", "[", "t", "]"}], ",", 
         RowBox[{"Haz", "[", "t", "]"}], ",", 
         RowBox[{"SOTloc", "[", "t", "]"}]}], "}"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6405240997445784`*^9, 3.6405241129983363`*^9}, {
   3.647827141273677*^9, 3.647827144159682*^9}, 3.6479205524779267`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ITmodel4p", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ITmodel4p", "[", 
    RowBox[{"Ci0_", ",", "Cw_", ",", "params_", ",", "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\[Delta]t", ",", "CSol", ",", "CTab", ",", "F1", ",", "F2", ",", "F3", 
       ",", "CMaxVal", ",", "CMaxTab", ",", "FfuncTab", ",", "SOTTab", ",", 
       "Ci", ",", "Haz", ",", "SOTloc", ",", "KD", ",", "\[Alpha]", ",", 
       "\[Beta]", ",", "Haz$BG", ",", "maxC", ",", "kin", ",", "kout"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Clear", "[", 
       RowBox[{
       "\[Delta]t", ",", "CSol", ",", "CTab", ",", "F1", ",", "F2", ",", "F3",
         ",", "CMaxVal", ",", "CMaxTab", ",", "FfuncTab", ",", "SOTTab", ",", 
        "Ci", ",", "Haz", ",", "SOTloc", ",", "kin", ",", "kout", ",", 
        "\[Alpha]", ",", "\[Beta]", ",", "Haz$BG", ",", "maxC"}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"Haz$BG", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kin", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kout", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Alpha]", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Beta]", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "5", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Delta]t", "=", "0.025"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"CSol", "=", 
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Ci", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             RowBox[{"kin", "*", 
              RowBox[{"Cw", "[", "t", "]"}]}], "-", 
             RowBox[{"kout", "*", 
              RowBox[{"Ci", "[", "t", "]"}]}]}]}], ",", 
           RowBox[{
            RowBox[{"Ci", "[", "0", "]"}], "\[Equal]", "Ci0"}]}], "}"}], ",", 
         RowBox[{"{", "Ci", "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
         RowBox[{"AccuracyGoal", "\[Rule]", "$accuracy"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"CTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Ci", "[", "t", "]"}], "/.", "CSol"}], ")"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd", ",", "\[Delta]t"}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"CMaxVal", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"CMaxTab", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "<=", 
         RowBox[{"Length", "[", "CTab", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"CTab", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "2"}], "]"}], "]"}], ">", "CMaxVal"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"CMaxTab", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"i", "-", "1"}], ")"}], "*", "\[Delta]t"}], ",", 
               RowBox[{"CTab", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "}"}]}], "]"}], ";", 
           RowBox[{"CMaxVal", "=", 
            RowBox[{"CTab", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "2"}], "]"}], "]"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"CMaxTab", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{"i", "-", "1"}], ")"}], "*", "\[Delta]t"}], ",", 
              "CMaxVal"}], "}"}]}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Clear", "[", "maxC", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"maxC", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"CMaxTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"FfuncTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"maxC", "[", "t", "]"}], ">", "0"}], " ", "&&", " ", 
              RowBox[{"\[Alpha]", ">", "0"}]}], ",", 
             FractionBox["1", 
              RowBox[{"1", "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 FractionBox[
                  RowBox[{"maxC", "[", "t", "]"}], "\[Alpha]"], ")"}], 
                RowBox[{"-", "\[Beta]"}]]}]], ",", "0"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "\[Delta]t", ",", "tEnd", ",", "\[Delta]t"}], 
          "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"PrependTo", "[", 
       RowBox[{"FfuncTab", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Haz", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"FfuncTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"SOTTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"Haz", "[", "t", "]"}]}], ")"}], "*", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "Haz$BG"}], " ", "t"}]]}]}], "}"}], " ", ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd", ",", "\[Delta]t"}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"SOTloc", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"SOTTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"maxC", "[", "t", "]"}], ",", 
         RowBox[{"Haz", "[", "t", "]"}], ",", 
         RowBox[{"SOTloc", "[", "t", "]"}]}], "}"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5877888622239122`*^9, 3.5877888633315268`*^9}, {
   3.5877889204750595`*^9, 3.5877890188811207`*^9}, {3.587789062561681*^9, 
   3.5877892342116814`*^9}, {3.5877905293890862`*^9, 3.587790550605358*^9}, 
   3.587790594051915*^9, {3.5877907330937138`*^9, 3.5877909426043997`*^9}, {
   3.5877910034295797`*^9, 3.5877910137569118`*^9}, {3.5877912065471897`*^9, 
   3.5877912069995956`*^9}, {3.587795689708955*^9, 3.5877957271007166`*^9}, {
   3.587795826047711*^9, 3.5877958573557067`*^9}, {3.5877959555238795`*^9, 
   3.5877960124167147`*^9}, {3.587796048717682*^9, 3.5877960848938503`*^9}, {
   3.5877961165616474`*^9, 3.587796118932832*^9}, {3.587796246756984*^9, 
   3.587796250859784*^9}, {3.5877962898753843`*^9, 3.587796294508584*^9}, {
   3.587796336269784*^9, 3.5877963944421844`*^9}, {3.587796428231784*^9, 
   3.587796457700184*^9}, {3.587796530832984*^9, 3.5877965644353843`*^9}, {
   3.587796683986136*^9, 3.5877966856410437`*^9}, {3.587797450447692*^9, 
   3.5877974574679728`*^9}, {3.587797488989234*^9, 3.587797493349408*^9}, {
   3.587797627460493*^9, 3.587797659909325*^9}, {3.587797854531562*^9, 
   3.5877978996421347`*^9}, 3.587798043433528*^9, {3.587798115284825*^9, 
   3.587798131820295*^9}, {3.5877981796795607`*^9, 3.5877981817074957`*^9}, 
   3.5878201186578417`*^9, {3.5878216802095823`*^9, 3.5878216809115686`*^9}, 
   3.5878236605750427`*^9, {3.588054066234227*^9, 3.5880540949160953`*^9}, {
   3.588054384616062*^9, 3.5880544422598257`*^9}, {3.588320663985593*^9, 
   3.5883206766978636`*^9}, {3.5910815790354104`*^9, 3.591081589908498*^9}, {
   3.5910820341429167`*^9, 3.5910821049950013`*^9}, 3.591089023963829*^9, {
   3.591346189249627*^9, 3.59134619027673*^9}, 3.5956014424451456`*^9, 
   3.5956017226789474`*^9, {3.5958451422295585`*^9, 3.5958451437937145`*^9}, {
   3.595846740594379*^9, 3.5958467410814276`*^9}, 3.5958468499593143`*^9, {
   3.6192540175298157`*^9, 3.619254031230186*^9}, {3.6207017192091703`*^9, 
   3.6207017319034395`*^9}, {3.6207192836465654`*^9, 3.620719353609561*^9}, {
   3.620719969804174*^9, 3.6207199731085043`*^9}, {3.6261570067819195`*^9, 
   3.626157008647106*^9}, 3.6264182207180214`*^9, {3.626418384877095*^9, 
   3.626418391039135*^9}, {3.626423260254633*^9, 3.626423276791286*^9}, 
   3.64782714898009*^9, 3.6479205551121902`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ITmodel5p", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ITmodel5p", "[", 
    RowBox[{"Ci0_", ",", "Cw_", ",", "params_", ",", "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\[Delta]t", ",", "CSol", ",", "CTab", ",", "F1", ",", "F2", ",", "F3", 
       ",", "CMaxVal", ",", "CMaxTab", ",", "FfuncTab", ",", "SOTTab", ",", 
       "Ci", ",", "Haz", ",", "SOTloc", ",", "KD", ",", "\[Alpha]", ",", 
       "\[Beta]", ",", "Haz$BG", ",", "maxC", ",", "kin", ",", "kout", ",", 
       "kr", ",", "scD"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Clear", "[", 
       RowBox[{
       "\[Delta]t", ",", "CSol", ",", "CTab", ",", "F1", ",", "F2", ",", "F3",
         ",", "CMaxVal", ",", "CMaxTab", ",", "FfuncTab", ",", "SOTTab", ",", 
        "Ci", ",", "Haz", ",", "SOTloc", ",", "kin", ",", "kout", ",", 
        "\[Alpha]", ",", "\[Beta]", ",", "Haz$BG", ",", "maxC"}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"Haz$BG", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kin", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kout", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Alpha]", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Beta]", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "5", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"kr", "=", 
       RowBox[{"params", "[", 
        RowBox[{"[", "6", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"\[Delta]t", "=", "0.01"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"CSol", "=", 
       RowBox[{"NDSolve", "[", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Ci", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{
             RowBox[{"kin", "*", 
              RowBox[{"Cw", "[", "t", "]"}]}], "-", 
             RowBox[{"kout", "*", 
              RowBox[{"Ci", "[", "t", "]"}]}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"scD", "'"}], "[", "t", "]"}], "\[Equal]", 
            RowBox[{"kr", "*", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Ci", "[", "t", "]"}], "-", 
               RowBox[{"scD", "[", "t", "]"}]}], ")"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Ci", "[", "0", "]"}], "\[Equal]", "Ci0"}], ",", 
           RowBox[{
            RowBox[{"scD", "[", "0", "]"}], "\[Equal]", "0"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"Ci", ",", "scD"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
         RowBox[{"AccuracyGoal", "\[Rule]", "$accuracy"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"CTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"scD", "[", "t", "]"}], "/.", "CSol"}], ")"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd", ",", "\[Delta]t"}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"CMaxVal", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"CMaxTab", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "<=", 
         RowBox[{"Length", "[", "CTab", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"CTab", "[", 
            RowBox[{"[", 
             RowBox[{"i", ",", "2"}], "]"}], "]"}], ">", "CMaxVal"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"CMaxTab", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"i", "-", "1"}], ")"}], "*", "\[Delta]t"}], ",", 
               RowBox[{"CTab", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "}"}]}], "]"}], ";", 
           RowBox[{"CMaxVal", "=", 
            RowBox[{"CTab", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "2"}], "]"}], "]"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"CMaxTab", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{"i", "-", "1"}], ")"}], "*", "\[Delta]t"}], ",", 
              "CMaxVal"}], "}"}]}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Clear", "[", "maxC", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"maxC", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"CMaxTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"FfuncTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"maxC", "[", "t", "]"}], ">", "0"}], " ", "&&", " ", 
              RowBox[{"\[Alpha]", ">", "0"}]}], ",", 
             FractionBox["1", 
              RowBox[{"1", "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 FractionBox[
                  RowBox[{"maxC", "[", "t", "]"}], "\[Alpha]"], ")"}], 
                RowBox[{"-", "\[Beta]"}]]}]], ",", "0"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "\[Delta]t", ",", "tEnd", ",", "\[Delta]t"}], 
          "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"PrependTo", "[", 
       RowBox[{"FfuncTab", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Haz", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"FfuncTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"SOTTab", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"t", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"Haz", "[", "t", "]"}]}], ")"}], "*", 
            SuperscriptBox["\[ExponentialE]", 
             RowBox[{
              RowBox[{"-", "Haz$BG"}], " ", "t"}]]}]}], "}"}], " ", ",", 
         RowBox[{"{", 
          RowBox[{"t", ",", "0", ",", "tEnd", ",", "\[Delta]t"}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"SOTloc", "=", 
       RowBox[{"Interpolation", "[", 
        RowBox[{"SOTTab", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"maxC", "[", "t", "]"}], ",", 
         RowBox[{"Haz", "[", "t", "]"}], ",", 
         RowBox[{"SOTloc", "[", "t", "]"}]}], "}"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5877888622239122`*^9, 3.5877888633315268`*^9}, {
   3.5877889204750595`*^9, 3.5877890188811207`*^9}, {3.587789062561681*^9, 
   3.5877892342116814`*^9}, {3.5877905293890862`*^9, 3.587790550605358*^9}, 
   3.587790594051915*^9, {3.5877907330937138`*^9, 3.5877909426043997`*^9}, {
   3.5877910034295797`*^9, 3.5877910137569118`*^9}, {3.5877912065471897`*^9, 
   3.5877912069995956`*^9}, {3.587795689708955*^9, 3.5877957271007166`*^9}, {
   3.587795826047711*^9, 3.5877958573557067`*^9}, {3.5877959555238795`*^9, 
   3.5877960124167147`*^9}, {3.587796048717682*^9, 3.5877960848938503`*^9}, {
   3.5877961165616474`*^9, 3.587796118932832*^9}, {3.587796246756984*^9, 
   3.587796250859784*^9}, {3.5877962898753843`*^9, 3.587796294508584*^9}, {
   3.587796336269784*^9, 3.5877963944421844`*^9}, {3.587796428231784*^9, 
   3.587796457700184*^9}, {3.587796530832984*^9, 3.5877965644353843`*^9}, {
   3.587796683986136*^9, 3.5877966856410437`*^9}, {3.587797450447692*^9, 
   3.5877974574679728`*^9}, {3.587797488989234*^9, 3.587797493349408*^9}, {
   3.587797627460493*^9, 3.587797659909325*^9}, {3.587797854531562*^9, 
   3.5877978996421347`*^9}, 3.587798043433528*^9, {3.587798115284825*^9, 
   3.587798131820295*^9}, {3.5877981796795607`*^9, 3.5877981817074957`*^9}, 
   3.5878201186578417`*^9, {3.5878216802095823`*^9, 3.5878216809115686`*^9}, 
   3.5878236605750427`*^9, {3.588054066234227*^9, 3.5880540949160953`*^9}, {
   3.588054384616062*^9, 3.5880544422598257`*^9}, {3.588320663985593*^9, 
   3.5883206766978636`*^9}, {3.5910815790354104`*^9, 3.591081589908498*^9}, {
   3.5910820341429167`*^9, 3.5910821049950013`*^9}, 3.591089023963829*^9, {
   3.591346189249627*^9, 3.59134619027673*^9}, 3.5956014424451456`*^9, 
   3.5956017226789474`*^9, {3.5958451422295585`*^9, 3.5958451437937145`*^9}, {
   3.595846740594379*^9, 3.5958467410814276`*^9}, 3.5958468499593143`*^9, {
   3.6192540175298157`*^9, 3.619254031230186*^9}, {3.6207017192091703`*^9, 
   3.6207017319034395`*^9}, {3.6207192836465654`*^9, 3.620719353609561*^9}, {
   3.620719969804174*^9, 3.6207199731085043`*^9}, {3.626156911899432*^9, 
   3.626156999128154*^9}, 3.62615706149839*^9, {3.6261573297662144`*^9, 
   3.626157331438382*^9}, 3.6264182266772594`*^9, {3.626418269686735*^9, 
   3.6264182778463945`*^9}, 3.6264184324105997`*^9, {3.626423267790386*^9, 
   3.626423270877695*^9}, {3.627302503053048*^9, 3.6273025076095037`*^9}, {
   3.6478271525992966`*^9, 3.647827155750502*^9}, 3.6479205586445436`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Objective functions for optimisation", "Title",
 CellChangeTimes->{{3.6278881191767187`*^9, 3.6278881268334846`*^9}}],

Cell["\<\
\[LineSeparator]Definition of the likelihood function for determining the \
mortality rate constant in the controls\
\>", "Subsubtitle",
 CellChangeTimes->{{3.587829114460746*^9, 3.5878291451793118`*^9}, 
   3.5878292583808937`*^9, {3.5878311570205207`*^9, 3.5878311584868927`*^9}, 
   3.6170024636611247`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "goalFunc$Control", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"goalFunc$Control", "[", " ", 
   RowBox[{"data_", ",", "model_", ",", "tEnd_", ",", 
    RowBox[{"Haz$BG_", "?", "NumericQ"}], ",", 
    RowBox[{"treatments_:", "1"}]}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "data$days", ",", "data$initAbundance", ",", "modelrun", ",", "SOT", ",",
       "data$currentAbundances", ",", "ges$err", ",", "step$err"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"data$days", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$initAbundance", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"ges$err", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"tr$i", "=", "0"}], ",", 
       RowBox[{"tr$i", "<", "treatments"}], ",", 
       RowBox[{"tr$i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"data$currentAbundances", "=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{"4", "+", "tr$i"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"modelrun", "=", 
         RowBox[{"Flatten", "[", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"model", "[", 
              RowBox[{"0", ",", "0", ",", 
               RowBox[{"{", 
                RowBox[{"Haz$BG", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
               "tEnd"}], "]"}], "[", 
             RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"step$err", "=", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "*", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"data$initAbundance", "[", 
                  RowBox[{"[", 
                   RowBox[{"tr$i", "+", "1"}], "]"}], "]"}], "*", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "-", 
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", "+", "1"}], "]"}], "]"}]}], ")"}]}], ")"}], 
               "*", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "-", 
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", "+", "1"}], "]"}], "]"}]}], "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "1", ",", 
                RowBox[{
                 RowBox[{"Length", "[", "data$currentAbundances", "]"}], "-", 
                 "1"}]}], "}"}]}], "]"}], "+", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"data$initAbundance", "[", 
               RowBox[{"[", 
                RowBox[{"tr$i", "+", "1"}], "]"}], "]"}], "*", 
              RowBox[{"data$currentAbundances", "[", 
               RowBox[{"[", 
                RowBox[{"Length", "[", "data$currentAbundances", "]"}], "]"}],
                "]"}], "*", 
              RowBox[{"Log", "[", 
               RowBox[{"modelrun", "[", 
                RowBox[{"[", 
                 RowBox[{"Length", "[", "data$currentAbundances", "]"}], 
                 "]"}], "]"}], "]"}]}], ")"}]}], ")"}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"ges$err", "+=", "step$err"}], ";"}]}], "\[IndentingNewLine]",
       "]"}], ";", "\[IndentingNewLine]", "ges$err"}]}], 
   "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.5877333574759655`*^9, 3.58773336488706*^9}, {
   3.587733397304276*^9, 3.5877334427164583`*^9}, 3.5877334814517546`*^9, {
   3.5877336220563574`*^9, 3.5877336576716137`*^9}, {3.5877342033761168`*^9, 
   3.5877343344957976`*^9}, {3.5877367879900527`*^9, 
   3.5877367946201377`*^9}, {3.587788793879436*^9, 3.5877888232078123`*^9}, {
   3.587798244183493*^9, 3.587798258769026*^9}, {3.5880541184824514`*^9, 
   3.588054132690872*^9}, {3.5883435838333483`*^9, 3.5883436174227066`*^9}, {
   3.5883436870926733`*^9, 3.588343736931657*^9}, {3.5883437808410473`*^9, 
   3.5883438040093637`*^9}, {3.588344060056966*^9, 3.588344079109871*^9}, 
   3.58961715091317*^9, 3.5898567274764814`*^9, {3.5956578551410103`*^9, 
   3.595657861441073*^9}, {3.595662480267726*^9, 3.5956624981979055`*^9}}],

Cell[CellGroupData[{

Cell["\[LineSeparator]Definition of the likelihood function for the \
treatments", "Subsubtitle",
 CellChangeTimes->{{3.5878292334503355`*^9, 3.587829254745837*^9}, {
   3.5878315242403316`*^9, 3.5878315250827312`*^9}, 3.61700246518114*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "goalFunc$treatments", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"goalFunc$treatments", "[", " ", 
   RowBox[{"data_", ",", "model_", ",", "Ci0_", ",", 
    RowBox[{"p1_", "?", "NumericQ"}], ",", 
    RowBox[{"p2_", "?", "NumericQ"}], ",", 
    RowBox[{"p3_", "?", "NumericQ"}], ",", "Haz$BG_", ",", "tEnd_", ",", 
    RowBox[{"method_:", "\"\<LL\>\""}], ",", 
    RowBox[{"detailedErrors_:", "\"\<false\>\""}]}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "data$days", ",", "data$initAbundance", ",", "data$conc", ",", 
      "modelrun", ",", "N$of$treatments", ",", "ges$err", ",", "step$err", 
      ",", "data$currentAbundances", ",", "SOT", ",", "Cw$temp", ",", 
      "gesErr$SSR", ",", "stepErr$SSR", ",", "detailedErrs", ",", "trErrs", 
      ",", "tr$i", ",", "di", ",", "numAcc", ",", "ttt"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"numAcc", "=", 
      RowBox[{"10", "^", 
       RowBox[{"-", "5"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$days", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$initAbundance", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$conc", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"N$of$treatments", "=", 
      RowBox[{"Length", "[", "data$initAbundance", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ges$err", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"gesErr$SSR", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"detailedErrs", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"IFform", "=", 
      RowBox[{"TrueQ", "[", " ", 
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", 
            RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
        "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"tr$i", "=", "1"}], ",", 
       RowBox[{"tr$i", "<=", "N$of$treatments"}], ",", 
       RowBox[{"tr$i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "tr$i"}], "]"}], "]"}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"tr$i", "+", "1"}]}], "]"}], "]"}]}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"data$currentAbundances", "=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{"4", "+", "tr$i", "-", "1"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
          RowBox[{"modelrun", "=", 
           RowBox[{"Flatten", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"model", "[", 
                RowBox[{"Ci0", ",", 
                 RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"Haz$BG", ",", "p1", ",", "p2", ",", "p3"}], "}"}], 
                 ",", "tEnd"}], "]"}], "[", 
               RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"modelrun", "=", 
           RowBox[{"Flatten", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"model", "[", 
                RowBox[{"Ci0", ",", "Cw$temp", ",", 
                 RowBox[{"{", 
                  RowBox[{"Haz$BG", ",", "p1", ",", "p2", ",", "p3"}], "}"}], 
                 ",", "tEnd"}], "]"}], "[", 
               RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"di", "=", "1"}], ",", 
          RowBox[{"di", "\[LessEqual]", 
           RowBox[{"Length", "[", "data$days", "]"}]}], ",", 
          RowBox[{"di", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"modelrun", "[", 
              RowBox[{"[", "di", "]"}], "]"}], "<", "numAcc"}], ",", 
            RowBox[{
             RowBox[{"modelrun", "[", 
              RowBox[{"[", "di", "]"}], "]"}], "=", 
             RowBox[{"numAcc", "/", "di"}]}]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"step$err", "=", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "*", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"data$initAbundance", "[", 
                  RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", "-", "1"}], "]"}], "]"}], "-", 
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], ")"}]}], ")"}], "*", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", "-", "1"}], "]"}], "]"}], "-", 
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "2", ",", 
                RowBox[{"Length", "[", "data$currentAbundances", "]"}]}], 
               "}"}]}], "]"}], "+", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"data$initAbundance", "[", 
               RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
              RowBox[{"data$currentAbundances", "[", 
               RowBox[{"[", 
                RowBox[{"-", "1"}], "]"}], "]"}], "*", 
              RowBox[{"Log", "[", 
               RowBox[{"modelrun", "[", 
                RowBox[{"[", 
                 RowBox[{"-", "1"}], "]"}], "]"}], "]"}]}], ")"}]}], 
           ")"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"ges$err", "+=", "step$err"}], ";", "\[IndentingNewLine]", 
        RowBox[{"stepErr$SSR", "=", 
         RowBox[{"Sum", "[", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"NumberQ", "[", 
              RowBox[{"data$currentAbundances", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}], " ", ",", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"data$initAbundance", "[", 
                  RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "-", 
                   RowBox[{"modelrun", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], ")"}]}], ")"}], ")"}], 
              "^", "2"}], ",", "0"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "data$currentAbundances", "]"}]}], 
            "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"gesErr$SSR", "+=", "stepErr$SSR"}]}]}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Treatment:\>\"", "<>", 
          RowBox[{"ToString", "[", "tr$i", "]"}], "<>", "\"\<, error: \>\"", "<>", 
          RowBox[{"ToString", "[", "step$err", "]"}]}], "]"}], ";"}], "*)"}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"globC", "++"}], ";", "\[IndentingNewLine]", 
     RowBox[{"resMon", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\"\<Step \>\"", "<>", 
         RowBox[{"ToString", "[", "globC", "]"}]}], ",", 
        RowBox[{"\"\<current error value :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "ges$err"}], "]"}]}], ",", 
        RowBox[{"\"\< p1:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p1"}], "]"}]}], ",", 
        RowBox[{"\"\< p2:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p2"}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\< p3:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p3"}], "]"}]}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"resMonDat", "=", 
      RowBox[{"{", 
       RowBox[{
       "ges$err", ",", "p1", ",", "p2", ",", "p3", ",", "gesErr$SSR"}], 
       "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{
        RowBox[{"globResMonDat", "[", 
         RowBox[{"[", "globC", "]"}], "]"}], "=", "resMonDat"}], ";"}], 
      "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"detailedErrors", "==", "\"\<false\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"method", "\[Equal]", "\"\<LL\>\""}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"GlobErr", "=", "ges$err"}], ";", 
          RowBox[{"Return", "[", "ges$err", "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"GlobErr", "=", "gesErr$SSR"}], ";", 
          RowBox[{"Return", "[", "gesErr$SSR", "]"}]}]}], 
        "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "detailedErrs", "]"}]}], "\[IndentingNewLine]", 
      "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.587733831165067*^9, 3.5877338670157843`*^9}, {
   3.58773443218425*^9, 3.5877345160041246`*^9}, {3.587734595051338*^9, 
   3.5877348565418906`*^9}, {3.587735173896759*^9, 3.5877352870148096`*^9}, {
   3.587735326966922*^9, 3.5877353439555397`*^9}, {3.5877354286802254`*^9, 
   3.587735436355524*^9}, {3.5877355814061832`*^9, 3.587735596787981*^9}, {
   3.5877356739777703`*^9, 3.58773573481855*^9}, {3.58773576832778*^9, 
   3.5877359038477173`*^9}, {3.587735938948168*^9, 3.587736013719926*^9}, {
   3.5877361378507175`*^9, 3.58773618246729*^9}, {3.5877362172557354`*^9, 
   3.587736291543888*^9}, 3.58773639703244*^9, {3.5877364432558327`*^9, 
   3.5877365084968696`*^9}, {3.5877371850629435`*^9, 3.587737220756201*^9}, {
   3.587737258508685*^9, 3.5877372875874577`*^9}, {3.5877373252463408`*^9, 
   3.5877373435921755`*^9}, {3.5877373925300035`*^9, 3.58773747883031*^9}, {
   3.5877375148355713`*^9, 3.5877375171444006`*^9}, {3.587737713177514*^9, 
   3.587737733676177*^9}, {3.5877380128261557`*^9, 3.587738022825884*^9}, {
   3.5877381528911552`*^9, 3.587738157212411*^9}, {3.5877394884767137`*^9, 
   3.58773949289157*^9}, {3.587739614604331*^9, 3.587739693260539*^9}, {
   3.587739768827908*^9, 3.587739775863598*^9}, {3.5877398648003387`*^9, 
   3.5877399028180256`*^9}, {3.5877402264919753`*^9, 3.587740233262462*^9}, {
   3.5877403036349645`*^9, 3.587740340092632*^9}, {3.5877403706846237`*^9, 
   3.587740505080347*^9}, {3.5877405637370987`*^9, 3.587740568042754*^9}, {
   3.5877411849848633`*^9, 3.587741223844962*^9}, {3.587741739571082*^9, 
   3.587741740507094*^9}, {3.5877419236222415`*^9, 3.5877419328887606`*^9}, {
   3.587744371110965*^9, 3.5877443770810843`*^9}, {3.58779836703274*^9, 
   3.5877984106035404`*^9}, {3.587830131228649*^9, 3.587830168778331*^9}, {
   3.587830242894881*^9, 3.587830277308922*^9}, {3.5878303781174145`*^9, 
   3.587830434340535*^9}, {3.5878304897368455`*^9, 3.5878304922172775`*^9}, {
   3.5878391882588096`*^9, 3.587839194229884*^9}, {3.5880541694965525`*^9, 
   3.588054181631766*^9}, {3.5880542159121933`*^9, 3.5880542187914815`*^9}, {
   3.588054279509553*^9, 3.5880543225518565`*^9}, {3.5908032615117397`*^9, 
   3.590803290762325*^9}, {3.5908075925380983`*^9, 3.5908075927481027`*^9}, {
   3.590807659321436*^9, 3.5908076674715986`*^9}, {3.595657725529714*^9, 
   3.5956577465399237`*^9}, {3.595657865551114*^9, 3.595657868651145*^9}, {
   3.5956581377648363`*^9, 3.5956581882553415`*^9}, {3.5956585088285465`*^9, 
   3.5956585119885783`*^9}, {3.5956605502839637`*^9, 
   3.5956606155746164`*^9}, {3.5956606525249863`*^9, 3.595660658475045*^9}, {
   3.5956607154856157`*^9, 3.5956607785662465`*^9}, {3.5956608100875616`*^9, 
   3.5956608175776367`*^9}, {3.5956609059885206`*^9, 
   3.5956609286087465`*^9}, {3.5956610257897186`*^9, 3.595661051419975*^9}, {
   3.5956610979104395`*^9, 3.5956611001104617`*^9}, {3.5956611330507913`*^9, 
   3.5956611898913593`*^9}, {3.595661817048631*^9, 3.595661983180292*^9}, {
   3.5956620303507643`*^9, 3.5956620370808315`*^9}, {3.595662099621457*^9, 
   3.5956621007814684`*^9}, {3.5956625059779835`*^9, 
   3.5956625288482122`*^9}, {3.5956638279952054`*^9, 
   3.5956638295052204`*^9}, {3.5956639574865007`*^9, 3.5956639913068385`*^9}, 
   3.595664038047306*^9, 3.5956641152980785`*^9, {3.5956642331192565`*^9, 
   3.595664233329259*^9}, {3.5956642794897203`*^9, 3.595664293579861*^9}, {
   3.595664498142907*^9, 3.5956645588535137`*^9}, {3.5956646102040277`*^9, 
   3.59566461245405*^9}, {3.5956648194061193`*^9, 3.595664831336239*^9}, {
   3.595664862806554*^9, 3.595664922907154*^9}, {3.595664983167757*^9, 
   3.595665013178057*^9}, {3.595665075059676*^9, 3.595665104659972*^9}, 
   3.595665389542821*^9, {3.595666521718145*^9, 3.5956665247081747`*^9}, 
   3.5956693244078937`*^9, {3.5956706860349708`*^9, 3.5956706878949895`*^9}, {
   3.59567072493536*^9, 3.5956707251253614`*^9}, {3.5956707555166655`*^9, 
   3.59567075596667*^9}, {3.5956708735878468`*^9, 3.5956708738878493`*^9}, {
   3.5958418986511707`*^9, 3.5958419754527063`*^9}, {3.5958420492941833`*^9, 
   3.595842246708132*^9}, {3.595842342290043*^9, 3.5958423941420803`*^9}, {
   3.595844257172062*^9, 3.595844273267671*^9}, {3.595844327184062*^9, 
   3.595844336133957*^9}, {3.595844402249568*^9, 3.5958444106254053`*^9}, {
   3.5958451518745227`*^9, 3.5958451590432396`*^9}, {3.5962729081117134`*^9, 
   3.596272974744046*^9}, {3.596273111486781*^9, 3.5962731437774267`*^9}, {
   3.5962732577397056`*^9, 3.59627330296061*^9}, {3.596273352361598*^9, 
   3.596273359221735*^9}, {3.5962733985325212`*^9, 3.5962735623857985`*^9}, {
   3.5962736571286936`*^9, 3.596273665578862*^9}, {3.5962737025496016`*^9, 
   3.5962737029896107`*^9}, {3.596273748590523*^9, 3.59627374995055*^9}, {
   3.59627380096157*^9, 3.596273804311637*^9}, {3.596273967064892*^9, 
   3.596273976155074*^9}, {3.596274733671224*^9, 3.5962748594237385`*^9}, {
   3.596274904184634*^9, 3.5962749046446433`*^9}, {3.6041261415200434`*^9, 
   3.6041261952854195`*^9}, {3.604126459566845*^9, 3.604126464534342*^9}, {
   3.61684119320485*^9, 3.616841222598643*^9}, 3.6170019032245207`*^9, {
   3.617001940444893*^9, 3.617001940844897*^9}, {3.617002169648185*^9, 
   3.6170021831383195`*^9}, {3.6170022775392637`*^9, 3.617002280699295*^9}, {
   3.6170023357998466`*^9, 3.617002355880047*^9}, {3.617002480881297*^9, 
   3.617002489561384*^9}, {3.6170026029925184`*^9, 3.617002615612644*^9}, {
   3.617005232593464*^9, 3.617005287240614*^9}, {3.6170053426733866`*^9, 
   3.6170053611907053`*^9}, {3.6170183645846863`*^9, 
   3.6170183780631723`*^9}, {3.6170184208230467`*^9, 
   3.6170184401047697`*^9}, {3.6170191737505045`*^9, 
   3.6170191788985376`*^9}, {3.61701925420022*^9, 3.617019271391531*^9}, {
   3.617019409610424*^9, 3.617019435148595*^9}, {3.6170194917477655`*^9, 
   3.617019507113864*^9}, {3.6170195386416655`*^9, 3.6170195437896986`*^9}, 
   3.617019575691903*^9, {3.6170196076575084`*^9, 3.617019611167531*^9}, {
   3.617019874659412*^9, 3.6170199550188937`*^9}, {3.6170211578840046`*^9, 
   3.617021169565031*^9}, {3.6170219647529945`*^9, 3.617021966125803*^9}, {
   3.6170264416817684`*^9, 3.6170264607606907`*^9}, {3.617026570444994*^9, 
   3.6170265721298046`*^9}, {3.617453309352045*^9, 3.617453318896*^9}, {
   3.6174546480959063`*^9, 3.617454650271124*^9}, {3.6179830635322204`*^9, 
   3.617983091097597*^9}, {3.6179831260730214`*^9, 3.6179831264006233`*^9}, {
   3.617983162124852*^9, 3.6179831625460553`*^9}, {3.6186550349389734`*^9, 
   3.6186550352290373`*^9}, {3.6188236943440065`*^9, 3.618823694749609*^9}, {
   3.619253963071371*^9, 3.6192540027333364`*^9}, {3.6192552259076414`*^9, 
   3.6192552384909*^9}, {3.6192553214881988`*^9, 3.619255376385688*^9}, {
   3.619255593786426*^9, 3.619255618441891*^9}, {3.619256011126156*^9, 
   3.6192560165086937`*^9}, {3.6194393923861575`*^9, 3.6194394968045983`*^9}, 
   3.619439564920409*^9, 3.619442084222946*^9, {3.6194432700875216`*^9, 
   3.6194432726457767`*^9}, 3.6194436281163206`*^9, {3.6194436834808564`*^9, 
   3.619443702661774*^9}, {3.6207019676790147`*^9, 3.620701977391986*^9}, {
   3.6207021147437196`*^9, 3.620702117869032*^9}, 3.6207027826675053`*^9, 
   3.624705225524925*^9, {3.627708943703267*^9, 3.6277089440593033`*^9}, {
   3.6278062561489196`*^9, 3.6278062893472395`*^9}, 3.627806516365939*^9, {
   3.627806603936695*^9, 3.627806609441245*^9}, {3.6278161271649227`*^9, 
   3.627816127305937*^9}, {3.6278163114843526`*^9, 3.6278163360788116`*^9}, {
   3.6278303346135817`*^9, 3.627830343421462*^9}, {3.627894653893125*^9, 
   3.6278946551572514`*^9}, {3.647692685312964*^9, 3.6476926959165707`*^9}, {
   3.6476953047727885`*^9, 3.647695341724902*^9}, {3.6476961320611067`*^9, 
   3.647696136349352*^9}, 3.647828469928426*^9, {3.6479422307905984`*^9, 
   3.6479422370122204`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "goalFunc$treatments2p", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"goalFunc$treatments2p", "[", " ", 
   RowBox[{"data_", ",", "model_", ",", "Ci0_", ",", 
    RowBox[{"p1_", "?", "NumericQ"}], ",", 
    RowBox[{"p2_", "?", "NumericQ"}], ",", 
    RowBox[{"p3_", "?", "NumericQ"}], ",", 
    RowBox[{"p4_", "?", "NumericQ"}], ",", "Haz$BG_", ",", "tEnd_", ",", 
    RowBox[{"method_:", "\"\<LL\>\""}], ",", 
    RowBox[{"detailedErrors_:", "\"\<false\>\""}]}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "data$days", ",", "data$initAbundance", ",", "data$conc", ",", 
      "modelrun", ",", "N$of$treatments", ",", "ges$err", ",", "step$err", 
      ",", "data$currentAbundances", ",", "SOT", ",", "Cw$temp", ",", 
      "gesErr$SSR", ",", "stepErr$SSR", ",", "detailedErrs", ",", "trErrs", 
      ",", "tr$i", ",", "di", ",", "numAcc", ",", "ttt"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"numAcc", "=", 
      RowBox[{"10", "^", 
       RowBox[{"-", "5"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$days", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$initAbundance", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$conc", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"N$of$treatments", "=", 
      RowBox[{"Length", "[", "data$initAbundance", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ges$err", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"gesErr$SSR", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"detailedErrs", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"IFform", "=", 
      RowBox[{"TrueQ", "[", " ", 
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", 
            RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
        "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"tr$i", "=", "1"}], ",", 
       RowBox[{"tr$i", "<=", "N$of$treatments"}], ",", 
       RowBox[{"tr$i", "++"}], ",", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "tr$i"}], "]"}], "]"}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"tr$i", "+", "1"}]}], "]"}], "]"}]}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"data$currentAbundances", "=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{"4", "+", "tr$i", "-", "1"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
          RowBox[{"modelrun", "=", 
           RowBox[{"Flatten", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"model", "[", 
                RowBox[{"Ci0", ",", 
                 RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
                 RowBox[{"{", 
                  RowBox[{
                  "Haz$BG", ",", "p1", ",", "p2", ",", "p3", ",", "p4"}], 
                  "}"}], ",", "tEnd"}], "]"}], "[", 
               RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"modelrun", "=", 
           RowBox[{"Flatten", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"model", "[", 
                RowBox[{"Ci0", ",", "Cw$temp", ",", 
                 RowBox[{"{", 
                  RowBox[{
                  "Haz$BG", ",", "p1", ",", "p2", ",", "p3", ",", "p4"}], 
                  "}"}], ",", "tEnd"}], "]"}], "[", 
               RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"di", "=", "1"}], ",", 
          RowBox[{"di", "\[LessEqual]", 
           RowBox[{"Length", "[", "data$days", "]"}]}], ",", 
          RowBox[{"di", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"modelrun", "[", 
              RowBox[{"[", "di", "]"}], "]"}], "<", "numAcc"}], ",", 
            RowBox[{
             RowBox[{"modelrun", "[", 
              RowBox[{"[", "di", "]"}], "]"}], "=", 
             RowBox[{"numAcc", "/", "di"}]}]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"step$err", "=", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "*", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"data$initAbundance", "[", 
                  RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", "-", "1"}], "]"}], "]"}], "-", 
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], ")"}]}], ")"}], "*", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", "-", "1"}], "]"}], "]"}], "-", 
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "2", ",", 
                RowBox[{"Length", "[", "data$currentAbundances", "]"}]}], 
               "}"}]}], "]"}], "+", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"data$initAbundance", "[", 
               RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
              RowBox[{"data$currentAbundances", "[", 
               RowBox[{"[", 
                RowBox[{"-", "1"}], "]"}], "]"}], "*", 
              RowBox[{"Log", "[", 
               RowBox[{"modelrun", "[", 
                RowBox[{"[", 
                 RowBox[{"-", "1"}], "]"}], "]"}], "]"}]}], ")"}]}], 
           ")"}]}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"ges$err", "+=", "step$err"}], ";", "\[IndentingNewLine]", 
        RowBox[{"stepErr$SSR", "=", 
         RowBox[{"Sum", "[", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"NumberQ", "[", 
              RowBox[{"data$currentAbundances", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}], " ", ",", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"data$currentAbundances", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "-", 
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", "i", "]"}], "]"}]}], ")"}], ")"}], "^", "2"}], 
             ",", "0"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "data$currentAbundances", "]"}]}], 
            "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"gesErr$SSR", "+=", "stepErr$SSR"}]}]}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Treatment:\>\"", "<>", 
          RowBox[{"ToString", "[", "tr$i", "]"}], "<>", "\"\<, error: \>\"", "<>", 
          RowBox[{"ToString", "[", "step$err", "]"}]}], "]"}], ";"}], "*)"}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"globC", "++"}], ";", "\[IndentingNewLine]", 
     RowBox[{"resMon", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\"\<Step \>\"", "<>", 
         RowBox[{"ToString", "[", "globC", "]"}]}], ",", 
        RowBox[{"\"\<current error value :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "ges$err"}], "]"}]}], ",", 
        RowBox[{"\"\< p1:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p1"}], "]"}]}], ",", 
        RowBox[{"\"\< p2:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p2"}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\< p3:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p3"}], "]"}]}], ",", 
        RowBox[{"\"\< p4:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p4"}], "]"}]}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"resMonDat", "=", 
      RowBox[{"{", 
       RowBox[{"ges$err", ",", "p1", ",", "p2", ",", "p3", ",", "p4"}], 
       "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{
        RowBox[{"globResMonDat", "[", 
         RowBox[{"[", "globC", "]"}], "]"}], "=", "resMonDat"}], ";"}], 
      "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"detailedErrors", "==", "\"\<false\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"method", "\[Equal]", "\"\<LL\>\""}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"GlobErr", "=", "ges$err"}], ";", 
          RowBox[{"Return", "[", "ges$err", "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"GlobErr", "=", "gesErr$SSR"}], ";", 
          RowBox[{"Return", "[", "gesErr$SSR", "]"}]}]}], 
        "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "detailedErrs", "]"}]}], "\[IndentingNewLine]", 
      "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.587733831165067*^9, 3.5877338670157843`*^9}, {
   3.58773443218425*^9, 3.5877345160041246`*^9}, {3.587734595051338*^9, 
   3.5877348565418906`*^9}, {3.587735173896759*^9, 3.5877352870148096`*^9}, {
   3.587735326966922*^9, 3.5877353439555397`*^9}, {3.5877354286802254`*^9, 
   3.587735436355524*^9}, {3.5877355814061832`*^9, 3.587735596787981*^9}, {
   3.5877356739777703`*^9, 3.58773573481855*^9}, {3.58773576832778*^9, 
   3.5877359038477173`*^9}, {3.587735938948168*^9, 3.587736013719926*^9}, {
   3.5877361378507175`*^9, 3.58773618246729*^9}, {3.5877362172557354`*^9, 
   3.587736291543888*^9}, 3.58773639703244*^9, {3.5877364432558327`*^9, 
   3.5877365084968696`*^9}, {3.5877371850629435`*^9, 3.587737220756201*^9}, {
   3.587737258508685*^9, 3.5877372875874577`*^9}, {3.5877373252463408`*^9, 
   3.5877373435921755`*^9}, {3.5877373925300035`*^9, 3.58773747883031*^9}, {
   3.5877375148355713`*^9, 3.5877375171444006`*^9}, {3.587737713177514*^9, 
   3.587737733676177*^9}, {3.5877380128261557`*^9, 3.587738022825884*^9}, {
   3.5877381528911552`*^9, 3.587738157212411*^9}, {3.5877394884767137`*^9, 
   3.58773949289157*^9}, {3.587739614604331*^9, 3.587739693260539*^9}, {
   3.587739768827908*^9, 3.587739775863598*^9}, {3.5877398648003387`*^9, 
   3.5877399028180256`*^9}, {3.5877402264919753`*^9, 3.587740233262462*^9}, {
   3.5877403036349645`*^9, 3.587740340092632*^9}, {3.5877403706846237`*^9, 
   3.587740505080347*^9}, {3.5877405637370987`*^9, 3.587740568042754*^9}, {
   3.5877411849848633`*^9, 3.587741223844962*^9}, {3.587741739571082*^9, 
   3.587741740507094*^9}, {3.5877419236222415`*^9, 3.5877419328887606`*^9}, {
   3.587744371110965*^9, 3.5877443770810843`*^9}, {3.58779836703274*^9, 
   3.5877984106035404`*^9}, {3.587830131228649*^9, 3.587830168778331*^9}, {
   3.587830242894881*^9, 3.587830277308922*^9}, {3.5878303781174145`*^9, 
   3.587830434340535*^9}, {3.5878304897368455`*^9, 3.5878304922172775`*^9}, {
   3.5878391882588096`*^9, 3.587839194229884*^9}, {3.5880541694965525`*^9, 
   3.588054181631766*^9}, {3.5880542159121933`*^9, 3.5880542187914815`*^9}, {
   3.588054279509553*^9, 3.5880543225518565`*^9}, {3.5908032615117397`*^9, 
   3.590803290762325*^9}, {3.5908075925380983`*^9, 3.5908075927481027`*^9}, {
   3.590807659321436*^9, 3.5908076674715986`*^9}, {3.595657725529714*^9, 
   3.5956577465399237`*^9}, {3.595657865551114*^9, 3.595657868651145*^9}, {
   3.5956581377648363`*^9, 3.5956581882553415`*^9}, {3.5956585088285465`*^9, 
   3.5956585119885783`*^9}, {3.5956605502839637`*^9, 
   3.5956606155746164`*^9}, {3.5956606525249863`*^9, 3.595660658475045*^9}, {
   3.5956607154856157`*^9, 3.5956607785662465`*^9}, {3.5956608100875616`*^9, 
   3.5956608175776367`*^9}, {3.5956609059885206`*^9, 
   3.5956609286087465`*^9}, {3.5956610257897186`*^9, 3.595661051419975*^9}, {
   3.5956610979104395`*^9, 3.5956611001104617`*^9}, {3.5956611330507913`*^9, 
   3.5956611898913593`*^9}, {3.595661817048631*^9, 3.595661983180292*^9}, {
   3.5956620303507643`*^9, 3.5956620370808315`*^9}, {3.595662099621457*^9, 
   3.5956621007814684`*^9}, {3.5956625059779835`*^9, 
   3.5956625288482122`*^9}, {3.5956638279952054`*^9, 
   3.5956638295052204`*^9}, {3.5956639574865007`*^9, 3.5956639913068385`*^9}, 
   3.595664038047306*^9, 3.5956641152980785`*^9, {3.5956642331192565`*^9, 
   3.595664233329259*^9}, {3.5956642794897203`*^9, 3.595664293579861*^9}, {
   3.595664498142907*^9, 3.5956645588535137`*^9}, {3.5956646102040277`*^9, 
   3.59566461245405*^9}, {3.5956648194061193`*^9, 3.595664831336239*^9}, {
   3.595664862806554*^9, 3.595664922907154*^9}, {3.595664983167757*^9, 
   3.595665013178057*^9}, {3.595665075059676*^9, 3.595665104659972*^9}, 
   3.595665389542821*^9, {3.595666521718145*^9, 3.5956665247081747`*^9}, 
   3.5956693244078937`*^9, {3.5956706860349708`*^9, 3.5956706878949895`*^9}, {
   3.59567072493536*^9, 3.5956707251253614`*^9}, {3.5956707555166655`*^9, 
   3.59567075596667*^9}, {3.5956708735878468`*^9, 3.5956708738878493`*^9}, {
   3.5958418986511707`*^9, 3.5958419754527063`*^9}, {3.5958420492941833`*^9, 
   3.595842246708132*^9}, {3.595842342290043*^9, 3.5958423941420803`*^9}, {
   3.595844257172062*^9, 3.595844273267671*^9}, {3.595844327184062*^9, 
   3.595844336133957*^9}, {3.595844402249568*^9, 3.5958444106254053`*^9}, {
   3.5958451518745227`*^9, 3.5958451590432396`*^9}, {3.5962729081117134`*^9, 
   3.596272974744046*^9}, {3.596273111486781*^9, 3.5962731437774267`*^9}, {
   3.5962732577397056`*^9, 3.59627330296061*^9}, {3.596273352361598*^9, 
   3.596273359221735*^9}, {3.5962733985325212`*^9, 3.5962735623857985`*^9}, {
   3.5962736571286936`*^9, 3.596273665578862*^9}, {3.5962737025496016`*^9, 
   3.5962737029896107`*^9}, {3.596273748590523*^9, 3.59627374995055*^9}, {
   3.59627380096157*^9, 3.596273804311637*^9}, {3.596273967064892*^9, 
   3.596273976155074*^9}, {3.596274733671224*^9, 3.5962748594237385`*^9}, {
   3.596274904184634*^9, 3.5962749046446433`*^9}, {3.6041261415200434`*^9, 
   3.6041261952854195`*^9}, {3.604126459566845*^9, 3.604126464534342*^9}, {
   3.61684119320485*^9, 3.616841222598643*^9}, 3.6170019032245207`*^9, {
   3.617001940444893*^9, 3.617001940844897*^9}, {3.617002169648185*^9, 
   3.6170021831383195`*^9}, {3.6170022775392637`*^9, 3.617002280699295*^9}, {
   3.6170023357998466`*^9, 3.617002355880047*^9}, {3.617002480881297*^9, 
   3.617002489561384*^9}, {3.6170026029925184`*^9, 3.617002615612644*^9}, {
   3.617005232593464*^9, 3.617005287240614*^9}, {3.6170053426733866`*^9, 
   3.6170053611907053`*^9}, {3.6170183645846863`*^9, 
   3.6170183780631723`*^9}, {3.6170184208230467`*^9, 
   3.6170184401047697`*^9}, {3.6170191737505045`*^9, 
   3.6170191788985376`*^9}, {3.61701925420022*^9, 3.617019271391531*^9}, {
   3.617019409610424*^9, 3.617019435148595*^9}, {3.6170194917477655`*^9, 
   3.617019507113864*^9}, {3.6170195386416655`*^9, 3.6170195437896986`*^9}, 
   3.617019575691903*^9, {3.6170196076575084`*^9, 3.617019611167531*^9}, {
   3.617019874659412*^9, 3.6170199550188937`*^9}, {3.6170211578840046`*^9, 
   3.617021169565031*^9}, {3.6170219647529945`*^9, 3.617021966125803*^9}, {
   3.6170264416817684`*^9, 3.6170264607606907`*^9}, {3.617026570444994*^9, 
   3.6170265721298046`*^9}, {3.617453309352045*^9, 3.617453318896*^9}, {
   3.6174546480959063`*^9, 3.617454650271124*^9}, {3.6179830635322204`*^9, 
   3.617983091097597*^9}, {3.6179831260730214`*^9, 3.6179831264006233`*^9}, {
   3.617983162124852*^9, 3.6179831625460553`*^9}, {3.6186550349389734`*^9, 
   3.6186550352290373`*^9}, {3.6188236943440065`*^9, 3.618823694749609*^9}, {
   3.619253963071371*^9, 3.6192540027333364`*^9}, {3.6192552259076414`*^9, 
   3.6192552384909*^9}, {3.6192553214881988`*^9, 3.619255376385688*^9}, {
   3.619255593786426*^9, 3.619255618441891*^9}, {3.619256011126156*^9, 
   3.6192560165086937`*^9}, {3.6194393923861575`*^9, 3.6194394968045983`*^9}, 
   3.619439564920409*^9, 3.619442084222946*^9, {3.6194432700875216`*^9, 
   3.6194432726457767`*^9}, 3.6194436281163206`*^9, {3.6194436834808564`*^9, 
   3.619443702661774*^9}, {3.6207019676790147`*^9, 3.620701977391986*^9}, {
   3.6207021147437196`*^9, 3.620702117869032*^9}, 3.6207027826675053`*^9, {
   3.6207200023664303`*^9, 3.620720073983591*^9}, {3.6207204948136697`*^9, 
   3.6207204962598143`*^9}, 3.624705877807147*^9, 3.624705989477313*^9, {
   3.6261585435955853`*^9, 3.626158543971623*^9}, {3.6278062286771727`*^9, 
   3.6278062497572803`*^9}, 3.6278065050258045`*^9, {3.6278066192252235`*^9, 
   3.627806627629064*^9}, {3.627816103555562*^9, 3.627816103674574*^9}, 
   3.6278163459648004`*^9, 3.6278163764698505`*^9, {3.627830367658886*^9, 
   3.6278303689530153`*^9}, {3.627894696135349*^9, 3.627894697217457*^9}, {
   3.6478271672019224`*^9, 3.6478271695575266`*^9}, {3.647827219995415*^9, 
   3.6478272631032915`*^9}, {3.647827294490546*^9, 3.6478273011527576`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "goalFunc$treatments5p", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"goalFunc$treatments5p", "[", " ", 
   RowBox[{"data_", ",", "model_", ",", "Ci0_", ",", 
    RowBox[{"p1_", "?", "NumericQ"}], ",", 
    RowBox[{"p2_", "?", "NumericQ"}], ",", 
    RowBox[{"p3_", "?", "NumericQ"}], ",", 
    RowBox[{"p4_", "?", "NumericQ"}], ",", 
    RowBox[{"p5_", "?", "NumericQ"}], ",", "Haz$BG_", ",", "tEnd_", ",", 
    RowBox[{"method_:", "\"\<LL\>\""}], ",", 
    RowBox[{"detailedErrors_:", "\"\<false\>\""}]}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "data$days", ",", "data$initAbundance", ",", "data$conc", ",", 
      "modelrun", ",", "N$of$treatments", ",", "ges$err", ",", "step$err", 
      ",", "data$currentAbundances", ",", "SOT", ",", "Cw$temp", ",", 
      "gesErr$SSR", ",", "stepErr$SSR", ",", "detailedErrs", ",", "trErrs", 
      ",", "tr$i", ",", "di", ",", "numAcc", ",", "ttt"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"numAcc", "=", 
      RowBox[{"10", "^", 
       RowBox[{"-", "5"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$days", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$initAbundance", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"data$conc", "=", 
      RowBox[{"data", "[", 
       RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"N$of$treatments", "=", 
      RowBox[{"Length", "[", "data$initAbundance", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ges$err", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"gesErr$SSR", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"detailedErrs", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"IFform", "=", 
      RowBox[{"TrueQ", "[", " ", 
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", 
            RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
        "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"tr$i", "=", "1"}], ",", 
       RowBox[{"tr$i", "<=", "N$of$treatments"}], ",", 
       RowBox[{"tr$i", "++"}], ",", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "tr$i"}], "]"}], "]"}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"tr$i", "+", "1"}]}], "]"}], "]"}]}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"data$currentAbundances", "=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{"4", "+", "tr$i", "-", "1"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
          RowBox[{"modelrun", "=", 
           RowBox[{"Flatten", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"model", "[", 
                RowBox[{"Ci0", ",", 
                 RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
                 RowBox[{"{", 
                  RowBox[{
                  "Haz$BG", ",", "p1", ",", "p2", ",", "p3", ",", "p4", ",", 
                   "p5"}], "}"}], ",", "tEnd"}], "]"}], "[", 
               RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"modelrun", "=", 
           RowBox[{"Flatten", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"model", "[", 
                RowBox[{"Ci0", ",", "Cw$temp", ",", 
                 RowBox[{"{", 
                  RowBox[{
                  "Haz$BG", ",", "p1", ",", "p2", ",", "p3", ",", "p4", ",", 
                   "p5"}], "}"}], ",", "tEnd"}], "]"}], "[", 
               RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"di", "=", "1"}], ",", 
          RowBox[{"di", "\[LessEqual]", 
           RowBox[{"Length", "[", "data$days", "]"}]}], ",", 
          RowBox[{"di", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"modelrun", "[", 
              RowBox[{"[", "di", "]"}], "]"}], "<", "numAcc"}], ",", 
            RowBox[{
             RowBox[{"modelrun", "[", 
              RowBox[{"[", "di", "]"}], "]"}], "=", 
             RowBox[{"numAcc", "/", "di"}]}]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"step$err", "=", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "*", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"data$initAbundance", "[", 
                  RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", "-", "1"}], "]"}], "]"}], "-", 
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], ")"}]}], ")"}], "*", 
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", "-", "1"}], "]"}], "]"}], "-", 
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "2", ",", 
                RowBox[{"Length", "[", "data$currentAbundances", "]"}]}], 
               "}"}]}], "]"}], "+", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"data$initAbundance", "[", 
               RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
              RowBox[{"data$currentAbundances", "[", 
               RowBox[{"[", 
                RowBox[{"-", "1"}], "]"}], "]"}], "*", 
              RowBox[{"Log", "[", 
               RowBox[{"modelrun", "[", 
                RowBox[{"[", 
                 RowBox[{"-", "1"}], "]"}], "]"}], "]"}]}], ")"}]}], 
           ")"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"ges$err", "+=", "step$err"}], ";", "\[IndentingNewLine]", 
        RowBox[{"stepErr$SSR", "=", 
         RowBox[{"Sum", "[", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"NumberQ", "[", 
               RowBox[{"data$currentAbundances", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "]"}], " ", "&&", " ", 
              RowBox[{
               RowBox[{"data$currentAbundances", "[", 
                RowBox[{"[", "i", "]"}], "]"}], ">", "0"}]}], ",", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"data$currentAbundances", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "-", 
                  RowBox[{"modelrun", "[", 
                   RowBox[{"[", "i", "]"}], "]"}]}], ")"}], ")"}], "^", "2"}],
               "/", 
              RowBox[{"data$currentAbundances", "[", 
               RowBox[{"[", "i", "]"}], "]"}]}], ",", "0"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "data$currentAbundances", "]"}]}], 
            "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"gesErr$SSR", "+=", "stepErr$SSR"}]}]}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Treatment:\>\"", "<>", 
          RowBox[{"ToString", "[", "tr$i", "]"}], "<>", "\"\<, error: \>\"", "<>", 
          RowBox[{"ToString", "[", "step$err", "]"}]}], "]"}], ";"}], "*)"}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"globC", "++"}], ";", "\[IndentingNewLine]", 
     RowBox[{"resMon", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\"\<Step \>\"", "<>", 
         RowBox[{"ToString", "[", "globC", "]"}]}], ",", 
        RowBox[{"\"\<current error value :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "ges$err"}], "]"}]}], ",", 
        RowBox[{"\"\< p1:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p1"}], "]"}]}], ",", 
        RowBox[{"\"\< p2:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p2"}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\< p3:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p3"}], "]"}]}], ",", 
        RowBox[{"\"\< p4:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p4"}], "]"}]}], ",", 
        RowBox[{"\"\< p5:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "p5"}], "]"}]}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"resMonDat", "=", 
      RowBox[{"{", 
       RowBox[{
       "ges$err", ",", "p1", ",", "p2", ",", "p3", ",", "p4", ",", "p5"}], 
       "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{
        RowBox[{"globResMonDat", "[", 
         RowBox[{"[", "globC", "]"}], "]"}], "=", "resMonDat"}], ";"}], 
      "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"detailedErrors", "==", "\"\<false\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"method", "\[Equal]", "\"\<LL\>\""}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"GlobErr", "=", "ges$err"}], ";", 
          RowBox[{"Return", "[", "ges$err", "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"GlobErr", "=", "gesErr$SSR"}], ";", 
          RowBox[{"Return", "[", "gesErr$SSR", "]"}]}]}], 
        "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "detailedErrs", "]"}]}], "\[IndentingNewLine]", 
      "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.587733831165067*^9, 3.5877338670157843`*^9}, {
   3.58773443218425*^9, 3.5877345160041246`*^9}, {3.587734595051338*^9, 
   3.5877348565418906`*^9}, {3.587735173896759*^9, 3.5877352870148096`*^9}, {
   3.587735326966922*^9, 3.5877353439555397`*^9}, {3.5877354286802254`*^9, 
   3.587735436355524*^9}, {3.5877355814061832`*^9, 3.587735596787981*^9}, {
   3.5877356739777703`*^9, 3.58773573481855*^9}, {3.58773576832778*^9, 
   3.5877359038477173`*^9}, {3.587735938948168*^9, 3.587736013719926*^9}, {
   3.5877361378507175`*^9, 3.58773618246729*^9}, {3.5877362172557354`*^9, 
   3.587736291543888*^9}, 3.58773639703244*^9, {3.5877364432558327`*^9, 
   3.5877365084968696`*^9}, {3.5877371850629435`*^9, 3.587737220756201*^9}, {
   3.587737258508685*^9, 3.5877372875874577`*^9}, {3.5877373252463408`*^9, 
   3.5877373435921755`*^9}, {3.5877373925300035`*^9, 3.58773747883031*^9}, {
   3.5877375148355713`*^9, 3.5877375171444006`*^9}, {3.587737713177514*^9, 
   3.587737733676177*^9}, {3.5877380128261557`*^9, 3.587738022825884*^9}, {
   3.5877381528911552`*^9, 3.587738157212411*^9}, {3.5877394884767137`*^9, 
   3.58773949289157*^9}, {3.587739614604331*^9, 3.587739693260539*^9}, {
   3.587739768827908*^9, 3.587739775863598*^9}, {3.5877398648003387`*^9, 
   3.5877399028180256`*^9}, {3.5877402264919753`*^9, 3.587740233262462*^9}, {
   3.5877403036349645`*^9, 3.587740340092632*^9}, {3.5877403706846237`*^9, 
   3.587740505080347*^9}, {3.5877405637370987`*^9, 3.587740568042754*^9}, {
   3.5877411849848633`*^9, 3.587741223844962*^9}, {3.587741739571082*^9, 
   3.587741740507094*^9}, {3.5877419236222415`*^9, 3.5877419328887606`*^9}, {
   3.587744371110965*^9, 3.5877443770810843`*^9}, {3.58779836703274*^9, 
   3.5877984106035404`*^9}, {3.587830131228649*^9, 3.587830168778331*^9}, {
   3.587830242894881*^9, 3.587830277308922*^9}, {3.5878303781174145`*^9, 
   3.587830434340535*^9}, {3.5878304897368455`*^9, 3.5878304922172775`*^9}, {
   3.5878391882588096`*^9, 3.587839194229884*^9}, {3.5880541694965525`*^9, 
   3.588054181631766*^9}, {3.5880542159121933`*^9, 3.5880542187914815`*^9}, {
   3.588054279509553*^9, 3.5880543225518565`*^9}, {3.5908032615117397`*^9, 
   3.590803290762325*^9}, {3.5908075925380983`*^9, 3.5908075927481027`*^9}, {
   3.590807659321436*^9, 3.5908076674715986`*^9}, {3.595657725529714*^9, 
   3.5956577465399237`*^9}, {3.595657865551114*^9, 3.595657868651145*^9}, {
   3.5956581377648363`*^9, 3.5956581882553415`*^9}, {3.5956585088285465`*^9, 
   3.5956585119885783`*^9}, {3.5956605502839637`*^9, 
   3.5956606155746164`*^9}, {3.5956606525249863`*^9, 3.595660658475045*^9}, {
   3.5956607154856157`*^9, 3.5956607785662465`*^9}, {3.5956608100875616`*^9, 
   3.5956608175776367`*^9}, {3.5956609059885206`*^9, 
   3.5956609286087465`*^9}, {3.5956610257897186`*^9, 3.595661051419975*^9}, {
   3.5956610979104395`*^9, 3.5956611001104617`*^9}, {3.5956611330507913`*^9, 
   3.5956611898913593`*^9}, {3.595661817048631*^9, 3.595661983180292*^9}, {
   3.5956620303507643`*^9, 3.5956620370808315`*^9}, {3.595662099621457*^9, 
   3.5956621007814684`*^9}, {3.5956625059779835`*^9, 
   3.5956625288482122`*^9}, {3.5956638279952054`*^9, 
   3.5956638295052204`*^9}, {3.5956639574865007`*^9, 3.5956639913068385`*^9}, 
   3.595664038047306*^9, 3.5956641152980785`*^9, {3.5956642331192565`*^9, 
   3.595664233329259*^9}, {3.5956642794897203`*^9, 3.595664293579861*^9}, {
   3.595664498142907*^9, 3.5956645588535137`*^9}, {3.5956646102040277`*^9, 
   3.59566461245405*^9}, {3.5956648194061193`*^9, 3.595664831336239*^9}, {
   3.595664862806554*^9, 3.595664922907154*^9}, {3.595664983167757*^9, 
   3.595665013178057*^9}, {3.595665075059676*^9, 3.595665104659972*^9}, 
   3.595665389542821*^9, {3.595666521718145*^9, 3.5956665247081747`*^9}, 
   3.5956693244078937`*^9, {3.5956706860349708`*^9, 3.5956706878949895`*^9}, {
   3.59567072493536*^9, 3.5956707251253614`*^9}, {3.5956707555166655`*^9, 
   3.59567075596667*^9}, {3.5956708735878468`*^9, 3.5956708738878493`*^9}, {
   3.5958418986511707`*^9, 3.5958419754527063`*^9}, {3.5958420492941833`*^9, 
   3.595842246708132*^9}, {3.595842342290043*^9, 3.5958423941420803`*^9}, {
   3.595844257172062*^9, 3.595844273267671*^9}, {3.595844327184062*^9, 
   3.595844336133957*^9}, {3.595844402249568*^9, 3.5958444106254053`*^9}, {
   3.5958451518745227`*^9, 3.5958451590432396`*^9}, {3.5962729081117134`*^9, 
   3.596272974744046*^9}, {3.596273111486781*^9, 3.5962731437774267`*^9}, {
   3.5962732577397056`*^9, 3.59627330296061*^9}, {3.596273352361598*^9, 
   3.596273359221735*^9}, {3.5962733985325212`*^9, 3.5962735623857985`*^9}, {
   3.5962736571286936`*^9, 3.596273665578862*^9}, {3.5962737025496016`*^9, 
   3.5962737029896107`*^9}, {3.596273748590523*^9, 3.59627374995055*^9}, {
   3.59627380096157*^9, 3.596273804311637*^9}, {3.596273967064892*^9, 
   3.596273976155074*^9}, {3.596274733671224*^9, 3.5962748594237385`*^9}, {
   3.596274904184634*^9, 3.5962749046446433`*^9}, {3.6041261415200434`*^9, 
   3.6041261952854195`*^9}, {3.604126459566845*^9, 3.604126464534342*^9}, {
   3.61684119320485*^9, 3.616841222598643*^9}, 3.6170019032245207`*^9, {
   3.617001940444893*^9, 3.617001940844897*^9}, {3.617002169648185*^9, 
   3.6170021831383195`*^9}, {3.6170022775392637`*^9, 3.617002280699295*^9}, {
   3.6170023357998466`*^9, 3.617002355880047*^9}, {3.617002480881297*^9, 
   3.617002489561384*^9}, {3.6170026029925184`*^9, 3.617002615612644*^9}, {
   3.617005232593464*^9, 3.617005287240614*^9}, {3.6170053426733866`*^9, 
   3.6170053611907053`*^9}, {3.6170183645846863`*^9, 
   3.6170183780631723`*^9}, {3.6170184208230467`*^9, 
   3.6170184401047697`*^9}, {3.6170191737505045`*^9, 
   3.6170191788985376`*^9}, {3.61701925420022*^9, 3.617019271391531*^9}, {
   3.617019409610424*^9, 3.617019435148595*^9}, {3.6170194917477655`*^9, 
   3.617019507113864*^9}, {3.6170195386416655`*^9, 3.6170195437896986`*^9}, 
   3.617019575691903*^9, {3.6170196076575084`*^9, 3.617019611167531*^9}, {
   3.617019874659412*^9, 3.6170199550188937`*^9}, {3.6170211578840046`*^9, 
   3.617021169565031*^9}, {3.6170219647529945`*^9, 3.617021966125803*^9}, {
   3.6170264416817684`*^9, 3.6170264607606907`*^9}, {3.617026570444994*^9, 
   3.6170265721298046`*^9}, {3.617453309352045*^9, 3.617453318896*^9}, {
   3.6174546480959063`*^9, 3.617454650271124*^9}, {3.6179830635322204`*^9, 
   3.617983091097597*^9}, {3.6179831260730214`*^9, 3.6179831264006233`*^9}, {
   3.617983162124852*^9, 3.6179831625460553`*^9}, {3.6186550349389734`*^9, 
   3.6186550352290373`*^9}, {3.6188236943440065`*^9, 3.618823694749609*^9}, {
   3.619253963071371*^9, 3.6192540027333364`*^9}, {3.6192552259076414`*^9, 
   3.6192552384909*^9}, {3.6192553214881988`*^9, 3.619255376385688*^9}, {
   3.619255593786426*^9, 3.619255618441891*^9}, {3.619256011126156*^9, 
   3.6192560165086937`*^9}, {3.6194393923861575`*^9, 3.6194394968045983`*^9}, 
   3.619439564920409*^9, 3.619442084222946*^9, {3.6194432700875216`*^9, 
   3.6194432726457767`*^9}, 3.6194436281163206`*^9, {3.6194436834808564`*^9, 
   3.619443702661774*^9}, {3.6207019676790147`*^9, 3.620701977391986*^9}, {
   3.6207021147437196`*^9, 3.620702117869032*^9}, 3.6207027826675053`*^9, {
   3.6207200023664303`*^9, 3.620720073983591*^9}, {3.6207204948136697`*^9, 
   3.6207204962598143`*^9}, 3.624705877807147*^9, 3.624705989477313*^9, {
   3.626157401440381*^9, 3.6261574398402205`*^9}, {3.626158565215747*^9, 
   3.6261585674469705`*^9}, {3.626160313266585*^9, 3.626160314096593*^9}, {
   3.6278061137476807`*^9, 3.6278061440247087`*^9}, {3.627806479435246*^9, 
   3.6278064839777*^9}, {3.6278066468459854`*^9, 3.627806649680269*^9}, {
   3.627816042342441*^9, 3.6278160889821043`*^9}, {3.6278163542596292`*^9, 
   3.6278163605922627`*^9}, {3.627830325506671*^9, 3.627830354704591*^9}, {
   3.6278947019649315`*^9, 3.6278947021899543`*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Methods for parameter confidence calculation", "Title",
 CellChangeTimes->{{3.6278881383696375`*^9, 3.6278881490987105`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"InvChiSquare", "[", 
    RowBox[{"p_", ",", 
     RowBox[{"\[Alpha]_:", "0.9"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Quiet", "[", "\[IndentingNewLine]", 
       RowBox[{"sol", "=", 
        RowBox[{"NSolve", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"CDF", "[", 
            RowBox[{
             RowBox[{"ChiSquareDistribution", "[", "p", "]"}], ",", "x"}], 
            "]"}], "\[Equal]", "\[Alpha]"}], ",", "x"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"sol", "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6478297965483665`*^9, 3.647829829671225*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Fdist", "[", 
    RowBox[{"p_", ",", "n_", ",", 
     RowBox[{"\[Alpha]_:", "0.9"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Quiet", "[", "\[IndentingNewLine]", 
       RowBox[{"sol", "=", 
        RowBox[{"NSolve", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"CDF", "[", 
            RowBox[{
             RowBox[{"FRatioDistribution", "[", 
              RowBox[{"p", ",", 
               RowBox[{"n", "-", "p"}]}], "]"}], ",", "x"}], "]"}], 
           "\[Equal]", "\[Alpha]"}], ",", "x"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"p", "/", 
        RowBox[{"(", 
         RowBox[{"n", "-", "p"}], ")"}]}], "*", 
       RowBox[{"sol", "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}]}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"errorProfiling", "[", 
   RowBox[{
   "data_", ",", "model_", ",", "tEnd_", ",", "ranges_", ",", "opt$Haz$BG_", 
    ",", "num_"}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "p1Arr", ",", "p2Arr", ",", "p3Arr", ",", "errProfilingParams", ",", 
      "errProfile$Ges", ",", "errProfileSorted", ",", "errProfile", ",", 
      "ix"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"p1Arr", "=", 
      RowBox[{"RandomReal", "[", 
       RowBox[{
        RowBox[{"ranges", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", "num"}], 
       StyleBox["]", "DesignDiscussion"]}]}], 
     StyleBox[";", "DesignDiscussion"], "\[IndentingNewLine]", 
     RowBox[{"p2Arr", "=", 
      RowBox[{"RandomReal", "[", 
       RowBox[{
        RowBox[{"ranges", "[", 
         RowBox[{"[", "2", "]"}], "]"}], ",", "num"}], 
       StyleBox["]", "DesignDiscussion"]}]}], 
     StyleBox[";", "DesignDiscussion"], "\[IndentingNewLine]", 
     RowBox[{"p3Arr", "=", 
      RowBox[{"RandomReal", "[", 
       RowBox[{
        RowBox[{"ranges", "[", 
         RowBox[{"[", "3", "]"}], "]"}], ",", "num"}], 
       StyleBox["]", "DesignDiscussion"]}]}], 
     StyleBox[";", "DesignDiscussion"], 
     StyleBox["\[IndentingNewLine]", "DesignDiscussion"], 
     RowBox[{"errProfilingParams", "=", 
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{"p1Arr", ",", "p2Arr", ",", "p3Arr"}], "}"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"errProfile", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"ix", "=", "1"}], ",", 
       RowBox[{"ix", "\[LessEqual]", "num"}], ",", 
       RowBox[{"ix", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"AppendTo", "[", 
         RowBox[{"errProfile", ",", 
          RowBox[{"goalFunc$treatments", "[", " ", 
           RowBox[{"data", ",", "model", ",", "0", ",", 
            RowBox[{"errProfilingParams", "[", 
             RowBox[{"[", 
              RowBox[{"ix", ",", "1"}], "]"}], "]"}], ",", 
            RowBox[{"errProfilingParams", "[", 
             RowBox[{"[", 
              RowBox[{"ix", ",", "2"}], "]"}], "]"}], ",", 
            RowBox[{"errProfilingParams", "[", 
             RowBox[{"[", 
              RowBox[{"ix", ",", "3"}], "]"}], "]"}], ",", "opt$Haz$BG", ",", 
            "tEnd"}], "]"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"errProfile$Ges", "=", 
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{"errProfile", ",", "p1Arr", ",", "p2Arr", ",", "p3Arr"}], 
        "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"errProfileSorted", "=", 
      RowBox[{"Sort", "[", 
       RowBox[{"errProfile$Ges", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#1", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "<", 
          RowBox[{"#2", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"g1", "=", 
      RowBox[{"ListPlot", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Transpose", "[", "errProfileSorted", "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", 
        RowBox[{"PlotRange", "->", "All"}], ",", 
        RowBox[{"AxesOrigin", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"g2", "=", 
      RowBox[{"ListLogPlot", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Transpose", "[", "errProfileSorted", "]"}], "[", 
         RowBox[{"[", "2", "]"}], "]"}], ",", 
        RowBox[{"PlotRange", "->", "All"}], ",", 
        RowBox[{"AxesOrigin", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"g3", "=", 
      RowBox[{"ListLogPlot", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Transpose", "[", "errProfileSorted", "]"}], "[", 
         RowBox[{"[", "3", "]"}], "]"}], ",", 
        RowBox[{"PlotRange", "->", "All"}], ",", 
        RowBox[{"AxesOrigin", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"g4", "=", 
      RowBox[{"ListLogPlot", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Transpose", "[", "errProfileSorted", "]"}], "[", 
         RowBox[{"[", "4", "]"}], "]"}], ",", 
        RowBox[{"PlotRange", "->", "All"}], ",", 
        RowBox[{"AxesOrigin", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0", ",", "0"}], "}"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"Return", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"g1", ",", "g2", ",", "g3", ",", "g4"}], "}"}], ",", 
        RowBox[{"errProfileSorted", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", "errProfileSorted"}], "}"}], 
      "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGCQBmIQfWNm9kPh3a8cr6VNANNmc8+8BtFpt+PegGi5708+
guj3E/g+gehitYPfQHRCgN9/EP3M6reoCJBOsY4WA9GGK2xNQDTfs34w7bCj
1gFEvz90BUzXtnzIA9H7NBzzwepy0tfqAekTbofAtHi+1DcQLXE+HUz3Xv5w
2RZIRzA4XAfRJ9JtboPoP4X3X4Dob+fb3oPoDvWkTyC64YT1DxD9v6zmN4he
5P9rvh2QvnqGYQGIzm+KWw2i5erTwLTUgZpjIPqNVSeYPmRWvM4JSHcnHtgA
ooVWBDX3AenQH41tINp8tu96ED3lWPc2EH3a4S+X1p5XjgsVf/KD6HnhO7RB
9MWeND0Q3SD/IBdE151cngeitxzifagDpKcEPnoEoj28j/Gf2P/K8QZbqyCI
Phy1yezgl1eOn2+qWYLoNI/eiSDaZUbpFBANABe97Cc=
  "]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "SOT$LRParamConfidence", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SOT$LRParamConfidence", "[", 
   RowBox[{
   "res_", ",", "model_", ",", "BGM_", ",", "data_", ",", "tEnd_", ",", 
    "deltaRel_", ",", "confLim_"}], "]"}], ":=", 
  RowBox[{"Block", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "$modErr", ",", "$deltaL", ",", "$modErrList", ",", "$searchPoints", ",",
       "$initStep", ",", "$allModErrList", ",", "$maxIter", ",", "$optParLR", 
      ",", "$pVar", ",", "$pVarTab", ",", "$pVarTabCorr", ",", "$tab", ",", 
      "\[IndentingNewLine]", "p1$lowLim", ",", "p1$lowConf", ",", "p1$upLim", 
      ",", "p1$upConf", ",", "p2$lowLim", ",", "p2$lowConf", ",", "p2$upLim", 
      ",", "p2$upConf", ",", "p3$lowLim", ",", "p3$lowConf", ",", "p3$upLim", 
      ",", "p3$upConf", ",", "\[IndentingNewLine]", "$bestErr", ",", 
      "$pVarNew", ",", "$startVector", ",", "$cons", ",", "$bestPVar", ",", 
      "$lowFold", ",", "$highFold", ",", "$minRelStep", ",", "$WPrec", ",", 
      "$p3Out", ",", "$initDeltaL"}], "}"}], ",", "\[IndentingNewLine]", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"$bestErr", "=", 
      RowBox[{"goalFunc$treatments", "[", 
       RowBox[{"data", ",", "model", ",", "0", ",", 
        RowBox[{"p1", "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", 
        RowBox[{"p2", "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", 
        RowBox[{"p3", "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd"}], 
       "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"$allModErrList", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"$maxIter", "=", "10"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$initStep", "=", "2"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$searchPoints", "=", "5"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"$lowFold", "=", "0.01"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$highFold", "=", "100"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$WPrec", "=", "6"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$minRelStep", "=", "0.05"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$initDeltaL", "=", "1.00001"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"minMethod", "=", 
      RowBox[{"{", "\"\<DifferentialEvolution\>\"", "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"minMethod", "=", 
      RowBox[{"{", 
       RowBox[{"\"\<NelderMead\>\"", ",", " ", 
        RowBox[{"\"\<ShrinkRatio\>\"", "\[Rule]", "0.95"}], ",", 
        RowBox[{"\"\<ContractRatio\>\"", "\[Rule]", "0.95"}], ",", 
        RowBox[{"\"\<ReflectRatio\>\"", "\[Rule]", "2"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"minMethod", "=", 
      RowBox[{"{", 
       RowBox[{"\"\<RandomSearch\>\"", ",", 
        RowBox[{"\"\<SearchPoints\>\"", "\[Rule]", "$searchPoints"}]}], 
       "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"minMethod", "=", 
      RowBox[{"{", 
       RowBox[{"\"\<SimulatedAnnealing\>\"", ",", 
        RowBox[{"\"\<PerturbationScale\>\"", "\[Rule]", "3"}], ",", 
        RowBox[{"\"\<SearchPoints\>\"", "\[Rule]", "$searchPoints"}]}], 
       "}"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"$p3Out", "=", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"p3", "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ")"}], "\[LessEqual]", 
       RowBox[{"10", "^", 
        RowBox[{"-", "10"}]}]}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"totalProfile", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"--", 
        RowBox[{"--", 
         RowBox[{"--", 
          RowBox[{"--", 
           RowBox[{"--", 
            RowBox[{"-", " ", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"p1", " ", "--"}], "--"}], "--"}], "--"}], "--"}],
                 "--"}], "--"}], "--"}]}]}]}]}]}]}], "-"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"SOT$gmonix", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"SOT$globResMonDat", "=", 
      RowBox[{"Table", "[", 
       RowBox[{"\"\<\>\"", ",", 
        RowBox[{"{", "100000", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "\"\<Profiling parameter p1\>\"", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$pVar", "=", 
      RowBox[{"(", 
       RowBox[{"p1", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$bestPVar", "=", 
      RowBox[{"(", 
       RowBox[{"p1", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Optimal p1 val: \>\"", "<>", 
       RowBox[{"ToString", "[", "$pVar", "]"}], "<>", 
       "\"\<, best error: \>\"", "<>", 
       RowBox[{"ToString", "[", "$bestErr", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Confidence level to be reached:\>\"", "<>", 
       RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$modErrList", "=", 
      RowBox[{"{", "}"}]}], ";", " ", 
     RowBox[{"globC", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$monRes", "=", 
      RowBox[{"{", 
       RowBox[{"$pVar", ",", "$bestErr", ",", "0"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
     RowBox[{"$modErr", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$modErrList", ",", 
       RowBox[{"{", 
        RowBox[{"$bestErr", ",", 
         RowBox[{"(", 
          RowBox[{"p1", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}], ",", 
         RowBox[{"(", 
          RowBox[{"p2", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}], ",", 
         RowBox[{"(", 
          RowBox[{"p3", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}], ",", "confLim"}], 
        "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", " ", "[", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
          RowBox[{"(", " ", 
           RowBox[{"$pVar", " ", ">", " ", 
            RowBox[{"$lowFold", " ", 
             RowBox[{"(", 
              RowBox[{"p1", "/.", 
               RowBox[{"res", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"$pVarNew", "=", 
           RowBox[{"$pVar", "*", " ", 
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"deltaRel", "*", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                 RowBox[{"(", 
                  RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}]}]}], 
             ")"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
             RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", 
               RowBox[{"(", 
                RowBox[{"1", "-", "$minRelStep"}], ")"}]}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"p2", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{
                  RowBox[{"$initStep", "*", "p2"}], "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{"p2", ">", "0"}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p2", ",", 
                  RowBox[{"p2", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p2"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p3", ",", 
                  RowBox[{"p3", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p3"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p2", ">", "0"}], "&&", 
                RowBox[{"p3", ">", "0"}]}], "}"}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"err", "=", 
                    RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "$pVar", ",", "p2", 
                    ",", "0", ",", "BGM", ",", "tEnd"}], "]"}]}], ",", 
                   "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"err", "=", 
                    RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "$pVar", ",", "p2", 
                    ",", "p3", ",", "BGM", ",", "tEnd"}], "]"}]}], ",", 
                   "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ",", "resMon"}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", "$pVar", ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", "BGM", ",",
                 "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", "$pVar", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", 
                 "$deltaL"}], "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", "$pVar", ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                RowBox[{"p3", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd"}],
                "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", "$pVar", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$deltaL"}], 
                "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"monRes", "=", 
           RowBox[{"{", 
            RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
          ";"}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}], ",", 
       "monRes"}], "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"p1$lowLim", "=", "$pVar"}], ";", 
     RowBox[{"p1$lowConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"$deltaL", ">=", "confLim"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
     RowBox[{"$modErr", "=", "0"}], ";", 
     RowBox[{"$pVar", "=", 
      RowBox[{"(", 
       RowBox[{"p1", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", " ", "[", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
          RowBox[{"(", " ", 
           RowBox[{"$pVar", " ", "<", " ", 
            RowBox[{"$highFold", " ", 
             RowBox[{"(", 
              RowBox[{"p1", "/.", 
               RowBox[{"res", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"$pVarNew", "=", 
           RowBox[{"$pVar", "*", " ", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              RowBox[{"deltaRel", "*", 
               RowBox[{"(", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
             ")"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
             RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", 
               RowBox[{"(", 
                RowBox[{"1", "+", "$minRelStep"}], ")"}]}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"p2", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{
                  RowBox[{"$initStep", "*", "p2"}], "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{"p2", ">", "0"}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p2", ",", 
                  RowBox[{"p2", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p2"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p3", ",", 
                  RowBox[{"p3", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p3"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p2", ">", "0"}], "&&", 
                RowBox[{"p3", ">", "0"}]}], "}"}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "$pVar", ",", "p2", 
                    ",", "0", ",", "BGM", ",", "tEnd"}], "]"}], ",", 
                   "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "$pVar", ",", "p2", 
                    ",", "p3", ",", "BGM", ",", "tEnd"}], "]"}], ",", 
                   "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ",", "resMon"}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", "$pVar", ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", "BGM", ",",
                 "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", "$pVar", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", 
                 "$deltaL"}], "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", "$pVar", ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                RowBox[{"p3", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd"}],
                "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", "$pVar", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$deltaL"}], 
                "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"monRes", "=", 
           RowBox[{"{", 
            RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
          ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"p1$upLim", "=", "$pVar"}], ";", 
     RowBox[{"p1$upConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"$deltaL", ">=", "confLim"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$allModErrList", ",", "$modErrList"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"traceDat", "=", 
      RowBox[{"SOT$globResMonDat", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"SOT$gmonix", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"totalProfile", ",", "traceDat"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"--", 
        RowBox[{"--", 
         RowBox[{"--", 
          RowBox[{"--", 
           RowBox[{"--", 
            RowBox[{"-", " ", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"p2", " ", "--"}], "--"}], "--"}], "--"}], "--"}],
                 "--"}], "--"}], "--"}]}]}]}]}]}]}], "-"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"SOT$gmonix", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"SOT$globResMonDat", "=", 
      RowBox[{"Table", "[", 
       RowBox[{"\"\<\>\"", ",", 
        RowBox[{"{", "100000", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "\"\<Profiling parameter p2\>\"", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$pVar", "=", 
      RowBox[{"(", 
       RowBox[{"p2", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$bestPVar", "=", 
      RowBox[{"(", 
       RowBox[{"p2", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Optimal p2 val: \>\"", "<>", 
       RowBox[{"ToString", "[", "$pVar", "]"}], "<>", 
       "\"\<, best error: \>\"", "<>", 
       RowBox[{"ToString", "[", "$bestErr", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Confidence level to be reached:\>\"", "<>", 
       RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$modErrList", "=", 
      RowBox[{"{", "}"}]}], ";", " ", 
     RowBox[{"globC", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"monRes", "=", 
      RowBox[{"{", 
       RowBox[{"$pVar", ",", "$bestErr", ",", "0"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
     RowBox[{"$modErr", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$modErrList", ",", 
       RowBox[{"{", 
        RowBox[{"$bestErr", ",", 
         RowBox[{"p1", "/.", 
          RowBox[{"res", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
         RowBox[{"p3", "/.", 
          RowBox[{"res", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], ",", "confLim"}], "}"}]}], "]"}],
      ";", "\[IndentingNewLine]", 
     RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", " ", "[", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
          RowBox[{"(", " ", 
           RowBox[{"$pVar", " ", ">", " ", 
            RowBox[{"$lowFold", " ", 
             RowBox[{"(", 
              RowBox[{"p2", "/.", 
               RowBox[{"res", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"$pVarNew", "=", 
           RowBox[{"$pVar", "*", " ", 
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"deltaRel", "*", 
               RowBox[{"(", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
             ")"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
             RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", 
               RowBox[{"(", 
                RowBox[{"1", "-", "$minRelStep"}], ")"}]}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"p1", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{
                  RowBox[{"$initStep", "*", "p1"}], "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{"p1", ">", "0"}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p1", ",", 
                  RowBox[{"p1", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p1"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p3", ",", 
                  RowBox[{"p3", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p3"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p1", ">", "0"}], "&&", 
                RowBox[{"p1", ">", "0"}]}], "}"}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          "\[IndentingNewLine]", 
          RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "$pVar", 
                    ",", "0", ",", "BGM", ",", "tEnd"}], "]"}], ",", 
                   "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "$pVar", 
                    ",", "p3", ",", "BGM", ",", "tEnd"}], "]"}], ",", 
                   "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ",", " ", "resMon"}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "0", 
                ",", "BGM", ",", "tEnd"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "0", 
                 ",", "$deltaL"}], "}"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                RowBox[{"p3", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd"}],
                "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$deltaL"}], 
                "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"monRes", "=", 
           RowBox[{"{", 
            RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
          ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"p2$lowLim", "=", "$pVar"}], ";", 
     RowBox[{"p2$lowConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"$deltaL", ">=", "confLim"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
     RowBox[{"$modErr", "=", "0"}], ";", 
     RowBox[{"$pVar", "=", 
      RowBox[{"(", 
       RowBox[{"p2", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", " ", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
          RowBox[{"(", " ", 
           RowBox[{"$pVar", " ", "<", 
            RowBox[{"$highFold", " ", 
             RowBox[{"(", 
              RowBox[{"p2", "/.", 
               RowBox[{"res", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"$pVarNew", "=", 
           RowBox[{"$pVar", "*", " ", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              RowBox[{"deltaRel", "*", 
               RowBox[{"(", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
             ")"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
             RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", 
               RowBox[{"(", 
                RowBox[{"1", "+", "$minRelStep"}], ")"}]}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"p1", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{
                  RowBox[{"$initStep", "*", "p1"}], "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{"p1", ">", "0"}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p1", ",", 
                  RowBox[{"p1", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p1"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p3", ",", 
                  RowBox[{"p3", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p3"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p1", ">", "0"}], "&&", 
                RowBox[{"p1", ">", "0"}]}], "}"}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "$pVar", 
                    ",", "0", ",", "BGM", ",", "tEnd"}], "]"}], ",", 
                   "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "$pVar", 
                    ",", "p3", ",", "BGM", ",", "tEnd"}], "]"}], ",", 
                   "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ",", "resMon"}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "0", 
                ",", "BGM", ",", "tEnd"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "0", 
                 ",", "$deltaL"}], "}"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                RowBox[{"p3", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd"}],
                "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$deltaL"}], 
                "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"monRes", "=", 
           RowBox[{"{", 
            RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
          ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"p2$upLim", "=", "$pVar"}], ";", 
     RowBox[{"p2$upConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"$deltaL", ">=", "confLim"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$allModErrList", ",", "$modErrList"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"traceDat", "=", 
      RowBox[{"SOT$globResMonDat", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"SOT$gmonix", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"totalProfile", ",", "traceDat"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"--", 
        RowBox[{"--", 
         RowBox[{"--", 
          RowBox[{"--", 
           RowBox[{"--", 
            RowBox[{"-", " ", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"p3", " ", "--"}], "--"}], "--"}], "--"}], "--"}],
                 "--"}], "--"}], "--"}]}]}]}]}]}]}], "-"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"SOT$gmonix", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"SOT$globResMonDat", "=", 
      RowBox[{"Table", "[", 
       RowBox[{"\"\<\>\"", ",", 
        RowBox[{"{", "100000", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
        "Print", "[", 
         "\"\<Profiling parameter p3 not applicable, best p3 value = 0\>\"", 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"$modErrList", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"$modErrList", ",", 
          RowBox[{"{", 
           RowBox[{"$bestErr", ",", 
            RowBox[{"p1", "/.", 
             RowBox[{"res", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", 
            RowBox[{"p2", "/.", 
             RowBox[{"res", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", "confLim", 
            ","}], "}"}]}], "]"}]}], "\[IndentingNewLine]", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Print", "[", "\"\<Profiling parameter p3\>\"", "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"$pVar", "=", 
         RowBox[{"(", 
          RowBox[{"p3", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"$bestPVar", "=", 
         RowBox[{"(", 
          RowBox[{"p3", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Optimal p3 val: \>\"", "<>", 
          RowBox[{"ToString", "[", "$pVar", "]"}], "<>", 
          "\"\<, best error: \>\"", "<>", 
          RowBox[{"ToString", "[", "$bestErr", "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Confidence level to be reached:\>\"", "<>", 
          RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"$modErrList", "=", 
         RowBox[{"{", "}"}]}], ";", 
        RowBox[{"globC", "=", "0"}], ";", "\[IndentingNewLine]", 
        RowBox[{"monRes", "=", 
         RowBox[{"{", 
          RowBox[{"$pVar", ",", "$bestErr", ",", "0"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
        RowBox[{"$modErr", "=", "0"}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"$modErrList", ",", 
          RowBox[{"{", 
           RowBox[{"$bestErr", ",", 
            RowBox[{"p1", "/.", 
             RowBox[{"res", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", 
            RowBox[{"p2", "/.", 
             RowBox[{"res", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
            "confLim"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Monitor", "[", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"While", " ", "[", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
             RowBox[{"(", " ", 
              RowBox[{"$pVar", " ", ">", " ", 
               RowBox[{"$lowFold", " ", 
                RowBox[{"(", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", " ", 
               RowBox[{"(", 
                RowBox[{"1", "-", 
                 RowBox[{"deltaRel", "*", 
                  RowBox[{"(", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                    RowBox[{"(", 
                    RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
                ")"}]}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
                RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"$pVarNew", "=", 
                 RowBox[{"$pVar", "*", 
                  RowBox[{"(", 
                   RowBox[{"1", "-", "$minRelStep"}], ")"}]}]}], ";"}]}], 
              "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p1", ">", "0"}], "&&", 
                RowBox[{"p2", ">", "0"}]}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p1", ",", 
                  RowBox[{"p1", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p1"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p2", ",", 
                  RowBox[{"p2", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p2"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"$optParLR", "=", 
                RowBox[{"NMinimize", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "p2", ",",
                     "$pVar", ",", "BGM", ",", "tEnd"}], "]"}], ",", 
                    "$cons"}], "}"}], ",", "$startVector", ",", 
                  RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                  RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                  RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                  RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", "resMon"}], 
              "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "BGM", 
                ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                 "$deltaL"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"monRes", "=", 
              RowBox[{"{", 
               RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
             ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"p3$lowLim", "=", "$pVar"}], ";", 
        RowBox[{"p3$lowConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", " ", "[", 
         RowBox[{
          RowBox[{"$deltaL", ">=", "confLim"}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<Confidence :\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], 
         "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
        RowBox[{"$modErr", "=", "0"}], ";", 
        RowBox[{"$pVar", "=", 
         RowBox[{"(", 
          RowBox[{"p3", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"While", " ", "[", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
             RowBox[{"(", " ", 
              RowBox[{"$pVar", " ", "<", " ", 
               RowBox[{"$highFold", " ", 
                RowBox[{"(", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
            "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", " ", 
               RowBox[{"(", 
                RowBox[{"1", "+", 
                 RowBox[{"deltaRel", "*", 
                  RowBox[{"(", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                    RowBox[{"(", 
                    RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
                ")"}]}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
                RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"$pVarNew", "=", 
                 RowBox[{"$pVar", "*", 
                  RowBox[{"(", 
                   RowBox[{"1", "+", "$minRelStep"}], ")"}]}]}], ";"}]}], 
              "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p1", ">", "0"}], "&&", 
                RowBox[{"p2", ">", "0"}]}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p1", ",", 
                  RowBox[{"p1", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p1"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p2", ",", 
                  RowBox[{"p2", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p2"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"$optParLR", "=", 
                RowBox[{"NMinimize", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "p2", ",",
                     "$pVar", ",", "BGM", ",", "tEnd"}], "]"}], ",", 
                    "$cons"}], "}"}], ",", "$startVector", ",", 
                  RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                  RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                  RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                  RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", "resMon"}], 
              "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
             "\[IndentingNewLine]", 
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "BGM", 
                ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "-", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                 "$deltaL"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"monRes", "=", 
              RowBox[{"{", 
               RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
             ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"p3$upLim", "=", "$pVar"}], ";", 
        RowBox[{"p3$upConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", " ", "[", 
         RowBox[{
          RowBox[{"$deltaL", ">=", "confLim"}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<Confidence :\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], 
         "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$allModErrList", ",", "$modErrList"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"traceDat", "=", 
      RowBox[{"SOT$globResMonDat", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"SOT$gmonix", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"totalProfile", ",", "traceDat"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Best likelihood value (Chi-squared test): \>\"", ",", " ", 
       RowBox[{"ToString", "[", "$bestErr", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Confidence limit: \>\"", "<>", 
       RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
     "Print", "[", 
      "\"\<Parameter | lower limit| best value | upper limit\>\"", "]"}], ";",
      "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<p1       | \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p1$lowLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p1$lowConf"}], "]"}], "<>", 
       "\"\<)\\t| \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{
         RowBox[{"AccountingForm", "@", "p1"}], "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "<>", "\"\<\\t| \>\"", 
       "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p1$upLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p1$upConf"}], "]"}], "<>", 
       "\"\<)\>\""}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<p2       | \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p2$lowLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p2$lowConf"}], "]"}], "<>", 
       "\"\<)\\t| \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{
         RowBox[{"AccountingForm", "@", "p2"}], "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "<>", "\"\<\\t| \>\"", 
       "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p2$upLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p2$upConf"}], "]"}], "<>", 
       "\"\<)\>\""}], "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<p3       | \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p3$lowLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p3$lowConf"}], "]"}], "<>", 
       "\"\<)\\t| \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{
         RowBox[{"AccountingForm", "@", "p3"}], "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "<>", "\"\<\\t| \>\"", 
       "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p3$upLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p3$upConf"}], "]"}], "<>", 
       "\"\<)\>\""}], "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Return", "[", "$allModErrList", "]"}], ";"}]}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.601636606878379*^9, 3.6016366768390784`*^9}, {
   3.601636708189392*^9, 3.6016368304606147`*^9}, {3.601636862180932*^9, 
   3.60163712197453*^9}, {3.6016372982362947`*^9, 3.601637329156604*^9}, {
   3.601637366116973*^9, 3.601637792391236*^9}, {3.6016378345416574`*^9, 
   3.601637927132583*^9}, {3.6016380383646975`*^9, 3.601638099805312*^9}, {
   3.6016388306958647`*^9, 3.6016389330278883`*^9}, {3.6016390489190474`*^9, 
   3.6016390814693727`*^9}, {3.6016391187097454`*^9, 
   3.6016391269598274`*^9}, {3.601639170060259*^9, 3.6016392148507066`*^9}, {
   3.6016395512240705`*^9, 3.6016395933444915`*^9}, {3.601639706885627*^9, 
   3.601639738485943*^9}, {3.6016398284268427`*^9, 3.6016398454370127`*^9}, {
   3.6016399424479823`*^9, 3.601639945558014*^9}, {3.6016400097096553`*^9, 
   3.6016400191397495`*^9}, {3.6016401125806837`*^9, 
   3.6016402082116404`*^9}, {3.601640294892507*^9, 3.6016403125426836`*^9}, {
   3.601640345483013*^9, 3.601640384083399*^9}, {3.6016405269448276`*^9, 
   3.6016405320148783`*^9}, {3.6016406386459446`*^9, 3.601640676896327*^9}, {
   3.601656179769906*^9, 3.6016562035182805`*^9}, {3.6016563848794146`*^9, 
   3.6016564436712933`*^9}, {3.6016564889088163`*^9, 
   3.6016565669926243`*^9}, {3.6016566336402884`*^9, 
   3.6016567152104445`*^9}, {3.6016568319711194`*^9, 3.601656867599682*^9}, {
   3.601657261227041*^9, 3.6016573928001966`*^9}, {3.6016574457914953`*^9, 
   3.601657550508966*^9}, {3.601657596274542*^9, 3.6016577336432776`*^9}, {
   3.6016577754064536`*^9, 3.60165782107802*^9}, {3.6016578532022324`*^9, 
   3.6016579315990715`*^9}, {3.6016583234232492`*^9, 3.601658331440051*^9}, {
   3.6016675829761715`*^9, 3.601667601028977*^9}, {3.601667676949568*^9, 
   3.601667894429742*^9}, {3.6016680728515263`*^9, 3.6016682333631315`*^9}, {
   3.6016683503843017`*^9, 3.6016683541343393`*^9}, {3.6016930099406333`*^9, 
   3.6016930947414813`*^9}, {3.601693157702111*^9, 3.601693394694481*^9}, {
   3.601693437278985*^9, 3.6016936139190087`*^9}, {3.6016936717388363`*^9, 
   3.6016937097552977`*^9}, {3.6016948978086033`*^9, 
   3.6016949211689777`*^9}, {3.6016949567293334`*^9, 3.601695103690803*^9}, {
   3.601695203011796*^9, 3.601695221571982*^9}, {3.6016952591223574`*^9, 
   3.601695276152528*^9}, {3.601696449865265*^9, 3.601696486785634*^9}, {
   3.6016965580863466`*^9, 3.601696664778414*^9}, 3.601696715098917*^9, {
   3.601697967048566*^9, 3.6016979672885685`*^9}, {3.601699329059886*^9, 
   3.601699516081756*^9}, {3.601700644174037*^9, 3.6017006854844503`*^9}, {
   3.6017023005084276`*^9, 3.601702344159737*^9}, {3.6017441783373013`*^9, 
   3.601744178717305*^9}, {3.6017470075943704`*^9, 3.6017470298005905`*^9}, {
   3.6041213826922083`*^9, 3.6041213850984488`*^9}, {3.6041245658424916`*^9, 
   3.6041246951634226`*^9}, {3.604124744193325*^9, 3.6041247871546206`*^9}, {
   3.6041248460495095`*^9, 3.604124992848188*^9}, {3.6041250657224746`*^9, 
   3.6041251342373257`*^9}, {3.604125349691869*^9, 3.604125584407338*^9}, {
   3.6041256274266396`*^9, 3.604125946343528*^9}, {3.6041262992488146`*^9, 
   3.604126299430833*^9}, {3.604126360944984*^9, 3.6041263842533145`*^9}, {
   3.6041368769924836`*^9, 3.604136880995884*^9}, {3.6049765310293493`*^9, 
   3.6049765417394567`*^9}, {3.6053234323455086`*^9, 3.605323434385529*^9}, {
   3.618906340543769*^9, 3.6189063740981236`*^9}, {3.6189064177814913`*^9, 
   3.618906441556869*^9}, {3.618918215512509*^9, 3.61891827992795*^9}, {
   3.6189307625418453`*^9, 3.618930882647015*^9}, {3.6189380781378145`*^9, 
   3.6189382848304815`*^9}, {3.6189384318761845`*^9, 
   3.6189384499129887`*^9}, {3.618938547663762*^9, 3.6189386973146095`*^9}, {
   3.6189767296943808`*^9, 3.618976751508103*^9}, {3.6189767912850075`*^9, 
   3.618976892804599*^9}, {3.6189790417275248`*^9, 3.6189790770463777`*^9}, {
   3.6192406428687553`*^9, 3.6192407692919655`*^9}, {3.6192418467956705`*^9, 
   3.6192418523181067`*^9}, {3.619244007609088*^9, 3.619244069323084*^9}, {
   3.619244207884172*^9, 3.6192443040743885`*^9}, {3.619244342232233*^9, 
   3.61924434795747*^9}, {3.6192444545373535`*^9, 3.6192444935688033`*^9}, {
   3.619244548459358*^9, 3.6192445497541666`*^9}, {3.6192504737274714`*^9, 
   3.6192505432984276`*^9}, 3.619253900598124*^9, {3.6193216648390584`*^9, 
   3.6193220582303934`*^9}, {3.61932209619619*^9, 3.619322636482213*^9}, {
   3.6193227335059147`*^9, 3.619322805224086*^9}, {3.6193228416517277`*^9, 
   3.6193229054951115`*^9}, {3.619440016160529*^9, 3.619440019012814*^9}, {
   3.619440101010013*^9, 3.619440111060018*^9}, {3.619440201867098*^9, 
   3.619440276241534*^9}, {3.619440308818792*^9, 3.6194403466305723`*^9}, 
   3.61944373562407*^9, 3.6194438451280193`*^9, {3.6194450892944236`*^9, 
   3.6194452716116533`*^9}, {3.6194453613596277`*^9, 
   3.6194453632228136`*^9}, {3.619795614071819*^9, 3.6197956734968185`*^9}, {
   3.6197957117655935`*^9, 3.619795711955612*^9}, {3.619795750953512*^9, 
   3.619795762017618*^9}, {3.6197986875646267`*^9, 3.6197986983910265`*^9}, {
   3.619834882563918*^9, 3.6198348834999247`*^9}, {3.6198349431079063`*^9, 
   3.619834950720755*^9}, {3.6198350089871287`*^9, 3.61983500914313*^9}, 
   3.6198359586740665`*^9, {3.619844500638008*^9, 3.619844513196089*^9}, {
   3.619844635097681*^9, 3.6198446529441953`*^9}, {3.619844727107071*^9, 
   3.6198447478552036`*^9}, {3.6198447794922066`*^9, 3.619844810399012*^9}, {
   3.619844847652051*^9, 3.6198450220915823`*^9}, {3.619849718088818*^9, 
   3.6198498194890895`*^9}, 3.6198519850998545`*^9, {3.6198562081284094`*^9, 
   3.6198562212324934`*^9}, {3.6198567933775063`*^9, 
   3.6198567960451236`*^9}, {3.6198568407770176`*^9, 
   3.6198568442090397`*^9}, {3.619857003357667*^9, 3.6198570766845446`*^9}, {
   3.6198830661416907`*^9, 3.6198830701200886`*^9}, {3.6198850876098175`*^9, 
   3.619885101627219*^9}, {3.6200301857397013`*^9, 3.620030189666094*^9}, {
   3.62003032363949*^9, 3.6200304938765116`*^9}, {3.620030528711995*^9, 
   3.6200307034364653`*^9}, {3.62003073598372*^9, 3.620030780620183*^9}, {
   3.620030812878409*^9, 3.62003084819594*^9}, {3.6200308904381638`*^9, 
   3.620030891677288*^9}, {3.6200309736894884`*^9, 3.620030999299049*^9}, {
   3.6237285638085127`*^9, 3.623728568798562*^9}, {3.623728604448919*^9, 
   3.623728604778922*^9}, {3.6237291000838747`*^9, 3.62372912557413*^9}, {
   3.623729159105465*^9, 3.6237291945558195`*^9}, {3.6237292828967032`*^9, 
   3.623729283806712*^9}, {3.6237293873677483`*^9, 3.6237295764596386`*^9}, {
   3.623730124169609*^9, 3.6237301483398514`*^9}, {3.6237335879016104`*^9, 
   3.623733640072132*^9}, {3.6237342299010324`*^9, 3.6237343073018064`*^9}, {
   3.6237346970667048`*^9, 3.623734715786892*^9}, {3.623734784837582*^9, 
   3.6237347862275963`*^9}, 3.623734864328377*^9, {3.6237349184489183`*^9, 
   3.6237349384491177`*^9}, {3.6237350195099287`*^9, 
   3.6237350281800156`*^9}, {3.6237355649096165`*^9, 
   3.6237356235044756`*^9}, {3.6237356595180764`*^9, 3.623735689495074*^9}, {
   3.623741623338399*^9, 3.623741710120076*^9}, {3.623743922228265*^9, 
   3.6237439329103327`*^9}, {3.623744854203453*^9, 3.623744854735506*^9}, {
   3.623744887567789*^9, 3.623744889672*^9}, {3.623744971337165*^9, 
   3.6237450360226336`*^9}, {3.6246759688218446`*^9, 3.624675975933551*^9}, {
   3.624676568965971*^9, 3.6246765693059845`*^9}, {3.6333550563398156`*^9, 
   3.633355057224904*^9}, 3.6333550886410456`*^9, {3.6346235139651833`*^9, 
   3.6346235156403513`*^9}, {3.6346235956718235`*^9, 3.634623683438549*^9}, {
   3.634631739651836*^9, 3.6346317933502054`*^9}, {3.6346318304979196`*^9, 
   3.6346318581006794`*^9}, {3.634631892329102*^9, 3.6346320767710133`*^9}, 
   3.6349985037975616`*^9, {3.647697515085042*^9, 3.6476975481727*^9}, {
   3.6477048513926897`*^9, 3.6477048550659*^9}, {3.647704918086997*^9, 
   3.6477049202710004`*^9}, {3.647828177531309*^9, 3.647828204675357*^9}, {
   3.6479123761005898`*^9, 3.6479123762656064`*^9}, {3.64792878436423*^9, 
   3.647928784544248*^9}, {3.6479407082200937`*^9, 3.6479407222133837`*^9}, {
   3.647940798887875*^9, 3.647940801555492*^9}, {3.6482880409334216`*^9, 
   3.6482880695182796`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "SOT$FratParamConfidence", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SOT$FratParamConfidence", "[", 
   RowBox[{
   "res_", ",", "model_", ",", "BGM_", ",", "data_", ",", "tEnd_", ",", 
    "deltaRel_", ",", "confLim_"}], "]"}], ":=", 
  RowBox[{"Block", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "$modErr", ",", "$deltaL", ",", "$modErrList", ",", "$searchPoints", ",",
       "$initStep", ",", "$allModErrList", ",", "$maxIter", ",", "$optParLR", 
      ",", "$pVar", ",", "$pVarTab", ",", "$pVarTabCorr", ",", "$tab", ",", 
      "\[IndentingNewLine]", "p1$lowLim", ",", "p1$lowConf", ",", "p1$upLim", 
      ",", "p1$upConf", ",", "p2$lowLim", ",", "p2$lowConf", ",", "p2$upLim", 
      ",", "p2$upConf", ",", "p3$lowLim", ",", "p3$lowConf", ",", "p3$upLim", 
      ",", "p3$upConf", ",", "\[IndentingNewLine]", "$bestErr", ",", 
      "$pVarNew", ",", "$startVector", ",", "$cons", ",", "$bestPVar", ",", 
      "$lowFold", ",", "$highFold", ",", "$minRelStep", ",", "$WPrec", ",", 
      "$p3Out", ",", "$initDeltaL"}], "}"}], ",", "\[IndentingNewLine]", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"$bestErr", "=", 
      RowBox[{"goalFunc$treatments", "[", 
       RowBox[{"data", ",", "model", ",", "0", ",", 
        RowBox[{"p1", "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", 
        RowBox[{"p2", "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", 
        RowBox[{"p3", "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd", ",", 
        "\"\<SSR\>\""}], "]"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"$allModErrList", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"$maxIter", "=", "10"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$initStep", "=", "2"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$searchPoints", "=", "5"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"$lowFold", "=", "0.01"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$highFold", "=", "100"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$WPrec", "=", "6"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$minRelStep", "=", "0.05"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$initDeltaL", "=", "1.00001"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"minMethod", "=", 
      RowBox[{"{", "\"\<DifferentialEvolution\>\"", "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"minMethod", "=", 
      RowBox[{"{", 
       RowBox[{"\"\<NelderMead\>\"", ",", " ", 
        RowBox[{"\"\<ShrinkRatio\>\"", "\[Rule]", "0.95"}], ",", 
        RowBox[{"\"\<ContractRatio\>\"", "\[Rule]", "0.95"}], ",", 
        RowBox[{"\"\<ReflectRatio\>\"", "\[Rule]", "2"}]}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"minMethod", "=", 
      RowBox[{"{", 
       RowBox[{"\"\<RandomSearch\>\"", ",", 
        RowBox[{"\"\<SearchPoints\>\"", "\[Rule]", "$searchPoints"}]}], 
       "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"minMethod", "=", 
      RowBox[{"{", 
       RowBox[{"\"\<SimulatedAnnealing\>\"", ",", 
        RowBox[{"\"\<PerturbationScale\>\"", "\[Rule]", "3"}], ",", 
        RowBox[{"\"\<SearchPoints\>\"", "\[Rule]", "$searchPoints"}]}], 
       "}"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"$p3Out", "=", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"p3", "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], ")"}], "\[Equal]", "0"}]}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"totalProfile", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"--", 
        RowBox[{"--", 
         RowBox[{"--", 
          RowBox[{"--", 
           RowBox[{"--", 
            RowBox[{"-", " ", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"p1", " ", "--"}], "--"}], "--"}], "--"}], "--"}],
                 "--"}], "--"}], "--"}]}]}]}]}]}]}], "-"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"SOT$gmonix", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"SOT$globResMonDat", "=", 
      RowBox[{"Table", "[", 
       RowBox[{"\"\<\>\"", ",", 
        RowBox[{"{", "100000", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "\"\<Profiling parameter p1\>\"", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$pVar", "=", 
      RowBox[{"(", 
       RowBox[{"p1", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$bestPVar", "=", 
      RowBox[{"(", 
       RowBox[{"p1", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Optimal p1 val: \>\"", "<>", 
       RowBox[{"ToString", "[", "$pVar", "]"}], "<>", 
       "\"\<, best error: \>\"", "<>", 
       RowBox[{"ToString", "[", "$bestErr", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Confidence level to be reached:\>\"", "<>", 
       RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$modErrList", "=", 
      RowBox[{"{", "}"}]}], ";", " ", 
     RowBox[{"globC", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"$monRes", "=", 
      RowBox[{"{", 
       RowBox[{"$pVar", ",", "$bestErr", ",", "0"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
     RowBox[{"$modErr", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$modErrList", ",", 
       RowBox[{"{", 
        RowBox[{"$bestErr", ",", 
         RowBox[{"(", 
          RowBox[{"p1", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}], ",", 
         RowBox[{"(", 
          RowBox[{"p2", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}], ",", 
         RowBox[{"(", 
          RowBox[{"p3", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}], ",", "confLim"}], 
        "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", " ", "[", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
          RowBox[{"(", " ", 
           RowBox[{"$pVar", " ", ">", " ", 
            RowBox[{"$lowFold", " ", 
             RowBox[{"(", 
              RowBox[{"p1", "/.", 
               RowBox[{"res", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"$pVarNew", "=", 
           RowBox[{"$pVar", "*", " ", 
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"deltaRel", "*", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                 RowBox[{"(", 
                  RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}]}]}], 
             ")"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
             RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", 
               RowBox[{"(", 
                RowBox[{"1", "-", "$minRelStep"}], ")"}]}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"p2", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{
                  RowBox[{"$initStep", "*", "p2"}], "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{"p2", ">", "0"}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p2", ",", 
                  RowBox[{"p2", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p2"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p3", ",", 
                  RowBox[{"p3", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p3"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p2", ">", "0"}], "&&", 
                RowBox[{"p3", ">", "0"}]}], "}"}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"err", "=", 
                    RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "$pVar", ",", "p2", 
                    ",", "0", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}]}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"err", "=", 
                    RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "$pVar", ",", "p2", 
                    ",", "p3", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}]}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ",", "resMon"}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", "$pVar", ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", "BGM", ",",
                 "tEnd", ",", "\"\<SSR\>\""}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", "$pVar", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", 
                 "$deltaL"}], "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", "$pVar", ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                RowBox[{"p3", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd", 
                ",", "\"\<SSR\>\""}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", "$pVar", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$deltaL"}], 
                "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"monRes", "=", 
           RowBox[{"{", 
            RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
          ";"}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}], ",", 
       "monRes"}], "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"p1$lowLim", "=", "$pVar"}], ";", 
     RowBox[{"p1$lowConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"$deltaL", ">=", "confLim"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
     RowBox[{"$modErr", "=", "0"}], ";", 
     RowBox[{"$pVar", "=", 
      RowBox[{"(", 
       RowBox[{"p1", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", " ", "[", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
          RowBox[{"(", " ", 
           RowBox[{"$pVar", " ", "<", " ", 
            RowBox[{"$highFold", " ", 
             RowBox[{"(", 
              RowBox[{"p1", "/.", 
               RowBox[{"res", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"$pVarNew", "=", 
           RowBox[{"$pVar", "*", " ", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              RowBox[{"deltaRel", "*", 
               RowBox[{"(", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
             ")"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
             RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", 
               RowBox[{"(", 
                RowBox[{"1", "+", "$minRelStep"}], ")"}]}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"p2", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{
                  RowBox[{"$initStep", "*", "p2"}], "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{"p2", ">", "0"}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p2", ",", 
                  RowBox[{"p2", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p2"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p3", ",", 
                  RowBox[{"p3", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p3"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p2", ">", "0"}], "&&", 
                RowBox[{"p3", ">", "0"}]}], "}"}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "$pVar", ",", "p2", 
                    ",", "0", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "$pVar", ",", "p2", 
                    ",", "p3", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ",", "resMon"}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", "$pVar", ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", "BGM", ",",
                 "tEnd", ",", "\"\<SSR\>\""}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", "$pVar", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", 
                 "$deltaL"}], "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", "$pVar", ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                RowBox[{"p3", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd", 
                ",", "\"\<SSR\>\""}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", "$pVar", ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$deltaL"}], 
                "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"monRes", "=", 
           RowBox[{"{", 
            RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
          ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"p1$upLim", "=", "$pVar"}], ";", 
     RowBox[{"p1$upConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"$deltaL", ">=", "confLim"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$allModErrList", ",", "$modErrList"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"traceDat", "=", 
      RowBox[{"SOT$globResMonDat", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"SOT$gmonix", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"totalProfile", ",", "traceDat"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"--", 
        RowBox[{"--", 
         RowBox[{"--", 
          RowBox[{"--", 
           RowBox[{"--", 
            RowBox[{"-", " ", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"p2", " ", "--"}], "--"}], "--"}], "--"}], "--"}],
                 "--"}], "--"}], "--"}]}]}]}]}]}]}], "-"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"SOT$gmonix", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"SOT$globResMonDat", "=", 
      RowBox[{"Table", "[", 
       RowBox[{"\"\<\>\"", ",", 
        RowBox[{"{", "100000", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "\"\<Profiling parameter p2\>\"", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$pVar", "=", 
      RowBox[{"(", 
       RowBox[{"p2", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$bestPVar", "=", 
      RowBox[{"(", 
       RowBox[{"p2", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Optimal p2 val: \>\"", "<>", 
       RowBox[{"ToString", "[", "$pVar", "]"}], "<>", 
       "\"\<, best error: \>\"", "<>", 
       RowBox[{"ToString", "[", "$bestErr", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Confidence level to be reached:\>\"", "<>", 
       RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$modErrList", "=", 
      RowBox[{"{", "}"}]}], ";", " ", 
     RowBox[{"globC", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"monRes", "=", 
      RowBox[{"{", 
       RowBox[{"$pVar", ",", "$bestErr", ",", "0"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
     RowBox[{"$modErr", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$modErrList", ",", 
       RowBox[{"{", 
        RowBox[{"$bestErr", ",", 
         RowBox[{"p1", "/.", 
          RowBox[{"res", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
         RowBox[{"p3", "/.", 
          RowBox[{"res", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], ",", "confLim"}], "}"}]}], "]"}],
      ";", "\[IndentingNewLine]", 
     RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", " ", "[", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
          RowBox[{"(", " ", 
           RowBox[{"$pVar", " ", ">", " ", 
            RowBox[{"$lowFold", " ", 
             RowBox[{"(", 
              RowBox[{"p2", "/.", 
               RowBox[{"res", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"$pVarNew", "=", 
           RowBox[{"$pVar", "*", " ", 
            RowBox[{"(", 
             RowBox[{"1", "-", 
              RowBox[{"deltaRel", "*", 
               RowBox[{"(", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
             ")"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
             RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", 
               RowBox[{"(", 
                RowBox[{"1", "-", "$minRelStep"}], ")"}]}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"p1", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{
                  RowBox[{"$initStep", "*", "p1"}], "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{"p1", ">", "0"}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p1", ",", 
                  RowBox[{"p1", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p1"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p3", ",", 
                  RowBox[{"p3", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p3"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p1", ">", "0"}], "&&", 
                RowBox[{"p1", ">", "0"}]}], "}"}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          "\[IndentingNewLine]", 
          RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "$pVar", 
                    ",", "0", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "$pVar", 
                    ",", "p3", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ",", " ", "resMon"}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "0", 
                ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "0", 
                 ",", "$deltaL"}], "}"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                RowBox[{"p3", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd", 
                ",", "\"\<SSR\>\""}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$deltaL"}], 
                "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"monRes", "=", 
           RowBox[{"{", 
            RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
          ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"p2$lowLim", "=", "$pVar"}], ";", 
     RowBox[{"p2$lowConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"$deltaL", ">=", "confLim"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
     RowBox[{"$modErr", "=", "0"}], ";", 
     RowBox[{"$pVar", "=", 
      RowBox[{"(", 
       RowBox[{"p2", "/.", 
        RowBox[{"res", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"While", " ", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
          RowBox[{"(", " ", 
           RowBox[{"$pVar", " ", "<", 
            RowBox[{"$highFold", " ", 
             RowBox[{"(", 
              RowBox[{"p2", "/.", 
               RowBox[{"res", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"$pVarNew", "=", 
           RowBox[{"$pVar", "*", " ", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              RowBox[{"deltaRel", "*", 
               RowBox[{"(", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                  RowBox[{"(", 
                   RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
             ")"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
             RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", 
               RowBox[{"(", 
                RowBox[{"1", "+", "$minRelStep"}], ")"}]}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{"p1", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{
                  RowBox[{"$initStep", "*", "p1"}], "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{"p1", ">", "0"}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p1", ",", 
                  RowBox[{"p1", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p1"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p3", ",", 
                  RowBox[{"p3", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p3"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p1", ">", "0"}], "&&", 
                RowBox[{"p1", ">", "0"}]}], "}"}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "$pVar", 
                    ",", "0", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"$optParLR", "=", 
               RowBox[{"NMinimize", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "$pVar", 
                    ",", "p3", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                 RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                 RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                 RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                 RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ",", "resMon"}], "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "0", 
                ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "0", 
                 ",", "$deltaL"}], "}"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                RowBox[{"p3", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "BGM", ",", "tEnd", 
                ",", "\"\<SSR\>\""}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$deltaL"}], 
                "}"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"monRes", "=", 
           RowBox[{"{", 
            RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
          ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"p2$upLim", "=", "$pVar"}], ";", 
     RowBox[{"p2$upConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", " ", "[", 
      RowBox[{
       RowBox[{"$deltaL", ">=", "confLim"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence :\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$allModErrList", ",", "$modErrList"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"traceDat", "=", 
      RowBox[{"SOT$globResMonDat", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"SOT$gmonix", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"totalProfile", ",", "traceDat"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"--", 
        RowBox[{"--", 
         RowBox[{"--", 
          RowBox[{"--", 
           RowBox[{"--", 
            RowBox[{"-", " ", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{
                    RowBox[{"p3", " ", "--"}], "--"}], "--"}], "--"}], "--"}],
                 "--"}], "--"}], "--"}]}]}]}]}]}]}], "-"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"SOT$gmonix", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"SOT$globResMonDat", "=", 
      RowBox[{"Table", "[", 
       RowBox[{"\"\<\>\"", ",", 
        RowBox[{"{", "100000", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{"$p3Out", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
        "Print", "[", 
         "\"\<Profiling parameter p3 not applicable, best p3 value = 0\>\"", 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"$modErrList", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"$modErrList", ",", 
          RowBox[{"{", 
           RowBox[{"$bestErr", ",", 
            RowBox[{"p1", "/.", 
             RowBox[{"res", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", 
            RowBox[{"p2", "/.", 
             RowBox[{"res", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", "0", ",", "confLim", 
            ","}], "}"}]}], "]"}]}], "\[IndentingNewLine]", ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Print", "[", "\"\<Profiling parameter p3\>\"", "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"$pVar", "=", 
         RowBox[{"(", 
          RowBox[{"p3", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"$bestPVar", "=", 
         RowBox[{"(", 
          RowBox[{"p3", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Optimal p3 val: \>\"", "<>", 
          RowBox[{"ToString", "[", "$pVar", "]"}], "<>", 
          "\"\<, best error: \>\"", "<>", 
          RowBox[{"ToString", "[", "$bestErr", "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"\"\<Confidence level to be reached:\>\"", "<>", 
          RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"$modErrList", "=", 
         RowBox[{"{", "}"}]}], ";", 
        RowBox[{"globC", "=", "0"}], ";", "\[IndentingNewLine]", 
        RowBox[{"monRes", "=", 
         RowBox[{"{", 
          RowBox[{"$pVar", ",", "$bestErr", ",", "0"}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
        RowBox[{"$modErr", "=", "0"}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"$modErrList", ",", 
          RowBox[{"{", 
           RowBox[{"$bestErr", ",", 
            RowBox[{"p1", "/.", 
             RowBox[{"res", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", 
            RowBox[{"p2", "/.", 
             RowBox[{"res", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
            "confLim"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"Monitor", "[", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"While", " ", "[", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
             RowBox[{"(", " ", 
              RowBox[{"$pVar", " ", ">", " ", 
               RowBox[{"$lowFold", " ", 
                RowBox[{"(", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", " ", 
               RowBox[{"(", 
                RowBox[{"1", "-", 
                 RowBox[{"deltaRel", "*", 
                  RowBox[{"(", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                    RowBox[{"(", 
                    RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
                ")"}]}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
                RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"$pVarNew", "=", 
                 RowBox[{"$pVar", "*", 
                  RowBox[{"(", 
                   RowBox[{"1", "-", "$minRelStep"}], ")"}]}]}], ";"}]}], 
              "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p1", ">", "0"}], "&&", 
                RowBox[{"p2", ">", "0"}]}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p1", ",", 
                  RowBox[{"p1", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p1"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p2", ",", 
                  RowBox[{"p2", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p2"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"$optParLR", "=", 
                RowBox[{"NMinimize", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "p2", ",",
                     "$pVar", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                  RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                  RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                  RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                  RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", "resMon"}], 
              "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "BGM", 
                ",", "tEnd", ",", "\"\<SSR\>\""}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                 "$deltaL"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"monRes", "=", 
              RowBox[{"{", 
               RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
             ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"p3$lowLim", "=", "$pVar"}], ";", 
        RowBox[{"p3$lowConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", " ", "[", 
         RowBox[{
          RowBox[{"$deltaL", ">=", "confLim"}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<Confidence :\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], 
         "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"$deltaL", "=", "$initDeltaL"}], ";", 
        RowBox[{"$modErr", "=", "0"}], ";", 
        RowBox[{"$pVar", "=", 
         RowBox[{"(", 
          RowBox[{"p3", "/.", 
           RowBox[{"res", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"While", " ", "[", " ", 
           RowBox[{
            RowBox[{
             RowBox[{"$deltaL", "<", "confLim"}], " ", "&&", " ", 
             RowBox[{"(", " ", 
              RowBox[{"$pVar", " ", "<", " ", 
               RowBox[{"$highFold", " ", 
                RowBox[{"(", 
                 RowBox[{"p3", "/.", 
                  RowBox[{"res", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ")"}]}], ",", 
            "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"$pVarNew", "=", 
              RowBox[{"$pVar", "*", " ", 
               RowBox[{"(", 
                RowBox[{"1", "+", 
                 RowBox[{"deltaRel", "*", 
                  RowBox[{"(", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"confLim", "-", "$deltaL"}], ")"}], "/", 
                    RowBox[{"(", 
                    RowBox[{"confLim", "-", "1"}], ")"}]}], ")"}], ")"}]}]}], 
                ")"}]}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"$pVarNew", "-", "$pVar"}], "]"}], "<", 
                RowBox[{"$minRelStep", "*", "$bestPVar"}]}], ",", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"$pVarNew", "=", 
                 RowBox[{"$pVar", "*", 
                  RowBox[{"(", 
                   RowBox[{"1", "+", "$minRelStep"}], ")"}]}]}], ";"}]}], 
              "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"$pVar", "=", "$pVarNew"}], ";", "\[IndentingNewLine]", 
             "\[IndentingNewLine]", 
             RowBox[{"$cons", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"p1", ">", "0"}], "&&", 
                RowBox[{"p2", ">", "0"}]}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$startVector", "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"p1", ",", 
                  RowBox[{"p1", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p1"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"p2", ",", 
                  RowBox[{"p2", "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                  RowBox[{
                   RowBox[{"$initStep", "*", "p2"}], "/.", 
                   RowBox[{"res", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
              "\[IndentingNewLine]", 
             RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"$optParLR", "=", 
                RowBox[{"NMinimize", "[", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"goalFunc$treatments", "[", " ", 
                    RowBox[{
                    "data", ",", "model", ",", "0", ",", "p1", ",", "p2", ",",
                     "$pVar", ",", "BGM", ",", "tEnd", ",", "\"\<SSR\>\""}], 
                    "]"}], ",", "$cons"}], "}"}], ",", "$startVector", ",", 
                  RowBox[{"Method", "\[Rule]", "minMethod"}], ",", 
                  RowBox[{"MaxIterations", "\[Rule]", "$maxIter"}], ",", 
                  RowBox[{"WorkingPrecision", "\[Rule]", "$WPrec"}], ",", 
                  RowBox[{"EvaluationMonitor", "\[RuleDelayed]", " ", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"SOT$globResMonDat", "[", 
                    RowBox[{"[", 
                    RowBox[{"SOT$gmonix", "++"}], "]"}], "]"}], "=", 
                    "resMonDat"}], "}"}]}]}], "]"}]}], ",", "resMon"}], 
              "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
             "\[IndentingNewLine]", 
             RowBox[{"$modErr", "=", 
              RowBox[{"goalFunc$treatments", "[", " ", 
               RowBox[{"data", ",", "model", ",", "0", ",", 
                RowBox[{"p1", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                RowBox[{"p2", "/.", 
                 RowBox[{"$optParLR", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", "BGM", 
                ",", "tEnd", ",", "\"\<SSR\>\""}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"$deltaL", "=", 
              RowBox[{"$modErr", "/", "$bestErr"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"$modErrList", ",", 
               RowBox[{"{", 
                RowBox[{"$modErr", ",", 
                 RowBox[{"p1", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", 
                 RowBox[{"p2", "/.", 
                  RowBox[{"$optParLR", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ",", "$pVar", ",", 
                 "$deltaL"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"monRes", "=", 
              RowBox[{"{", 
               RowBox[{"$pVar", ",", "$modErr", ",", "$deltaL"}], "}"}]}], 
             ";"}]}], "\[IndentingNewLine]", "]"}], ",", "monRes"}], "]"}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"p3$upLim", "=", "$pVar"}], ";", 
        RowBox[{"p3$upConf", "=", "$deltaL"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", " ", "[", 
         RowBox[{
          RowBox[{"$deltaL", ">=", "confLim"}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<Confidence :\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}], ",", 
          RowBox[{"Print", "[", 
           RowBox[{"\"\<Confidence not reached:\>\"", "<>", 
            RowBox[{"ToString", "[", 
             RowBox[{"AccountingForm", "@", "$pVar"}], "]"}]}], "]"}]}], 
         "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"$allModErrList", ",", "$modErrList"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"traceDat", "=", 
      RowBox[{"SOT$globResMonDat", "[", 
       RowBox[{"[", 
        RowBox[{"1", ";;", 
         RowBox[{"SOT$gmonix", "-", "1"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AppendTo", "[", 
      RowBox[{"totalProfile", ",", "traceDat"}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Best likelihood value (Chi-squared test): \>\"", ",", " ", 
       RowBox[{"ToString", "[", "$bestErr", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Confidence limit: \>\"", "<>", 
       RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
     "Print", "[", 
      "\"\<Parameter | lower limit| best value | upper limit\>\"", "]"}], ";",
      "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<p1       | \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p1$lowLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p1$lowConf"}], "]"}], "<>", 
       "\"\<)\\t| \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{
         RowBox[{"AccountingForm", "@", "p1"}], "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "<>", "\"\<\\t| \>\"", 
       "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p1$upLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p1$upConf"}], "]"}], "<>", 
       "\"\<)\>\""}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<p2       | \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p2$lowLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p2$lowConf"}], "]"}], "<>", 
       "\"\<)\\t| \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{
         RowBox[{"AccountingForm", "@", "p2"}], "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "<>", "\"\<\\t| \>\"", 
       "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p2$upLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p2$upConf"}], "]"}], "<>", 
       "\"\<)\>\""}], "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<p3       | \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p3$lowLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p3$lowConf"}], "]"}], "<>", 
       "\"\<)\\t| \>\"", "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{
         RowBox[{"AccountingForm", "@", "p3"}], "/.", 
         RowBox[{"res", "[", 
          RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "<>", "\"\<\\t| \>\"", 
       "\[IndentingNewLine]", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p3$upLim"}], "]"}], "<>", 
       "\"\< (\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"AccountingForm", "@", "p3$upConf"}], "]"}], "<>", 
       "\"\<)\>\""}], "]"}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"Return", "[", "$allModErrList", "]"}], ";"}]}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.601636606878379*^9, 3.6016366768390784`*^9}, {
   3.601636708189392*^9, 3.6016368304606147`*^9}, {3.601636862180932*^9, 
   3.60163712197453*^9}, {3.6016372982362947`*^9, 3.601637329156604*^9}, {
   3.601637366116973*^9, 3.601637792391236*^9}, {3.6016378345416574`*^9, 
   3.601637927132583*^9}, {3.6016380383646975`*^9, 3.601638099805312*^9}, {
   3.6016388306958647`*^9, 3.6016389330278883`*^9}, {3.6016390489190474`*^9, 
   3.6016390814693727`*^9}, {3.6016391187097454`*^9, 
   3.6016391269598274`*^9}, {3.601639170060259*^9, 3.6016392148507066`*^9}, {
   3.6016395512240705`*^9, 3.6016395933444915`*^9}, {3.601639706885627*^9, 
   3.601639738485943*^9}, {3.6016398284268427`*^9, 3.6016398454370127`*^9}, {
   3.6016399424479823`*^9, 3.601639945558014*^9}, {3.6016400097096553`*^9, 
   3.6016400191397495`*^9}, {3.6016401125806837`*^9, 
   3.6016402082116404`*^9}, {3.601640294892507*^9, 3.6016403125426836`*^9}, {
   3.601640345483013*^9, 3.601640384083399*^9}, {3.6016405269448276`*^9, 
   3.6016405320148783`*^9}, {3.6016406386459446`*^9, 3.601640676896327*^9}, {
   3.601656179769906*^9, 3.6016562035182805`*^9}, {3.6016563848794146`*^9, 
   3.6016564436712933`*^9}, {3.6016564889088163`*^9, 
   3.6016565669926243`*^9}, {3.6016566336402884`*^9, 
   3.6016567152104445`*^9}, {3.6016568319711194`*^9, 3.601656867599682*^9}, {
   3.601657261227041*^9, 3.6016573928001966`*^9}, {3.6016574457914953`*^9, 
   3.601657550508966*^9}, {3.601657596274542*^9, 3.6016577336432776`*^9}, {
   3.6016577754064536`*^9, 3.60165782107802*^9}, {3.6016578532022324`*^9, 
   3.6016579315990715`*^9}, {3.6016583234232492`*^9, 3.601658331440051*^9}, {
   3.6016675829761715`*^9, 3.601667601028977*^9}, {3.601667676949568*^9, 
   3.601667894429742*^9}, {3.6016680728515263`*^9, 3.6016682333631315`*^9}, {
   3.6016683503843017`*^9, 3.6016683541343393`*^9}, {3.6016930099406333`*^9, 
   3.6016930947414813`*^9}, {3.601693157702111*^9, 3.601693394694481*^9}, {
   3.601693437278985*^9, 3.6016936139190087`*^9}, {3.6016936717388363`*^9, 
   3.6016937097552977`*^9}, {3.6016948978086033`*^9, 
   3.6016949211689777`*^9}, {3.6016949567293334`*^9, 3.601695103690803*^9}, {
   3.601695203011796*^9, 3.601695221571982*^9}, {3.6016952591223574`*^9, 
   3.601695276152528*^9}, {3.601696449865265*^9, 3.601696486785634*^9}, {
   3.6016965580863466`*^9, 3.601696664778414*^9}, 3.601696715098917*^9, {
   3.601697967048566*^9, 3.6016979672885685`*^9}, {3.601699329059886*^9, 
   3.601699516081756*^9}, {3.601700644174037*^9, 3.6017006854844503`*^9}, {
   3.6017023005084276`*^9, 3.601702344159737*^9}, {3.6017441783373013`*^9, 
   3.601744178717305*^9}, {3.6017470075943704`*^9, 3.6017470298005905`*^9}, {
   3.6041213826922083`*^9, 3.6041213850984488`*^9}, {3.6041245658424916`*^9, 
   3.6041246951634226`*^9}, {3.604124744193325*^9, 3.6041247871546206`*^9}, {
   3.6041248460495095`*^9, 3.604124992848188*^9}, {3.6041250657224746`*^9, 
   3.6041251342373257`*^9}, {3.604125349691869*^9, 3.604125584407338*^9}, {
   3.6041256274266396`*^9, 3.604125946343528*^9}, {3.6041262992488146`*^9, 
   3.604126299430833*^9}, {3.604126360944984*^9, 3.6041263842533145`*^9}, {
   3.6041368769924836`*^9, 3.604136880995884*^9}, {3.6049765310293493`*^9, 
   3.6049765417394567`*^9}, {3.6053234323455086`*^9, 3.605323434385529*^9}, {
   3.618906340543769*^9, 3.6189063740981236`*^9}, {3.6189064177814913`*^9, 
   3.618906441556869*^9}, {3.618918215512509*^9, 3.61891827992795*^9}, {
   3.6189307625418453`*^9, 3.618930882647015*^9}, {3.6189380781378145`*^9, 
   3.6189382848304815`*^9}, {3.6189384318761845`*^9, 
   3.6189384499129887`*^9}, {3.618938547663762*^9, 3.6189386973146095`*^9}, {
   3.6189767296943808`*^9, 3.618976751508103*^9}, {3.6189767912850075`*^9, 
   3.618976892804599*^9}, {3.6189790417275248`*^9, 3.6189790770463777`*^9}, {
   3.6192406428687553`*^9, 3.6192407692919655`*^9}, {3.6192418467956705`*^9, 
   3.6192418523181067`*^9}, {3.619244007609088*^9, 3.619244069323084*^9}, {
   3.619244207884172*^9, 3.6192443040743885`*^9}, {3.619244342232233*^9, 
   3.61924434795747*^9}, {3.6192444545373535`*^9, 3.6192444935688033`*^9}, {
   3.619244548459358*^9, 3.6192445497541666`*^9}, {3.6192504737274714`*^9, 
   3.6192505432984276`*^9}, 3.619253900598124*^9, {3.6193216648390584`*^9, 
   3.6193220582303934`*^9}, {3.61932209619619*^9, 3.619322636482213*^9}, {
   3.6193227335059147`*^9, 3.619322805224086*^9}, {3.6193228416517277`*^9, 
   3.6193229054951115`*^9}, {3.619440016160529*^9, 3.619440019012814*^9}, {
   3.619440101010013*^9, 3.619440111060018*^9}, {3.619440201867098*^9, 
   3.619440276241534*^9}, {3.619440308818792*^9, 3.6194403466305723`*^9}, 
   3.61944373562407*^9, 3.6194438451280193`*^9, {3.6194450892944236`*^9, 
   3.6194452716116533`*^9}, {3.6194453613596277`*^9, 
   3.6194453632228136`*^9}, {3.619795614071819*^9, 3.6197956734968185`*^9}, {
   3.6197957117655935`*^9, 3.619795711955612*^9}, {3.619795750953512*^9, 
   3.619795762017618*^9}, {3.6197986875646267`*^9, 3.6197986983910265`*^9}, {
   3.619834882563918*^9, 3.6198348834999247`*^9}, {3.6198349431079063`*^9, 
   3.619834950720755*^9}, {3.6198350089871287`*^9, 3.61983500914313*^9}, 
   3.6198359586740665`*^9, {3.619844500638008*^9, 3.619844513196089*^9}, {
   3.619844635097681*^9, 3.6198446529441953`*^9}, {3.619844727107071*^9, 
   3.6198447478552036`*^9}, {3.6198447794922066`*^9, 3.619844810399012*^9}, {
   3.619844847652051*^9, 3.6198450220915823`*^9}, {3.619849718088818*^9, 
   3.6198498194890895`*^9}, 3.6198519850998545`*^9, {3.6198562081284094`*^9, 
   3.6198562212324934`*^9}, {3.6198567933775063`*^9, 
   3.6198567960451236`*^9}, {3.6198568407770176`*^9, 
   3.6198568442090397`*^9}, {3.619857003357667*^9, 3.6198570766845446`*^9}, {
   3.6198830661416907`*^9, 3.6198830701200886`*^9}, {3.6198850876098175`*^9, 
   3.619885101627219*^9}, {3.6200301857397013`*^9, 3.620030189666094*^9}, {
   3.62003032363949*^9, 3.6200304938765116`*^9}, {3.620030528711995*^9, 
   3.6200307034364653`*^9}, {3.62003073598372*^9, 3.620030780620183*^9}, {
   3.620030812878409*^9, 3.62003084819594*^9}, {3.6200308904381638`*^9, 
   3.620030891677288*^9}, {3.6200309736894884`*^9, 3.620030999299049*^9}, {
   3.6237285638085127`*^9, 3.623728568798562*^9}, {3.623728604448919*^9, 
   3.623728604778922*^9}, {3.6237291000838747`*^9, 3.62372912557413*^9}, {
   3.623729159105465*^9, 3.6237291945558195`*^9}, {3.6237292828967032`*^9, 
   3.623729283806712*^9}, {3.6237293873677483`*^9, 3.6237295764596386`*^9}, {
   3.623730124169609*^9, 3.6237301483398514`*^9}, {3.6237335879016104`*^9, 
   3.623733640072132*^9}, {3.6237342299010324`*^9, 3.6237343073018064`*^9}, {
   3.6237346970667048`*^9, 3.623734715786892*^9}, {3.623734784837582*^9, 
   3.6237347862275963`*^9}, 3.623734864328377*^9, {3.6237349184489183`*^9, 
   3.6237349384491177`*^9}, {3.6237350195099287`*^9, 
   3.6237350281800156`*^9}, {3.6237355649096165`*^9, 
   3.6237356235044756`*^9}, {3.6237356595180764`*^9, 3.623735689495074*^9}, {
   3.623741623338399*^9, 3.623741710120076*^9}, {3.623743922228265*^9, 
   3.6237439329103327`*^9}, {3.623744854203453*^9, 3.623744854735506*^9}, {
   3.623744887567789*^9, 3.623744889672*^9}, {3.623744971337165*^9, 
   3.6237450360226336`*^9}, {3.6246759688218446`*^9, 3.624675975933551*^9}, {
   3.624676568965971*^9, 3.6246765693059845`*^9}, {3.6333550563398156`*^9, 
   3.633355057224904*^9}, 3.6333550886410456`*^9, {3.6346235139651833`*^9, 
   3.6346235156403513`*^9}, {3.6346235956718235`*^9, 3.634623683438549*^9}, {
   3.634631739651836*^9, 3.6346317933502054`*^9}, {3.6346318304979196`*^9, 
   3.6346318581006794`*^9}, {3.634631892329102*^9, 3.6346320767710133`*^9}, 
   3.6349985037975616`*^9, {3.647697549405102*^9, 3.6476975945047812`*^9}, {
   3.647697654018886*^9, 3.6476976566552906`*^9}, {3.6476977619350758`*^9, 
   3.647697833149201*^9}, 3.6476981278745193`*^9, {3.647828209012164*^9, 
   3.6478282289967995`*^9}, {3.64828807332666*^9, 3.6482880880561333`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dealing with uncertainty ", "Title",
 CellChangeTimes->{{3.6362707986775084`*^9, 3.6362708082295218`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "extract", " ", "data", " ", "for", " ", "the", " ", "pulsed", " ", 
    "scenarios"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"extractSOTdataseries", "[", 
     RowBox[{"dataSet_", ",", "tEnd_", ",", "replicates_", ",", "nrTr_"}], 
     "]"}], ":=", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"trix", ",", "tix", ",", "TSdata", ",", "DATAvalsAbs"}], "}"}],
       ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"DATAvalsAbs", "=", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"tEnd", "+", "1"}], "}"}]}], "]"}], ",", 
          RowBox[{"{", "nrTr", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"TSdata", "=", "\[IndentingNewLine]", 
        RowBox[{"createModelTimeSeries1M", "[", 
         RowBox[{"ITmodel", " ", 
          RowBox[{"(*", 
           RowBox[{"not", " ", "relevant"}], "*)"}], ",", 
          "\[IndentingNewLine]", "dataSet", ",", "0", ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"p1", "\[Rule]", "1"}], ",", 
             RowBox[{"p2", "\[Rule]", "1"}], ",", 
             RowBox[{"p3", "\[Rule]", "1"}]}], "}"}], "}"}], ",", 
          "\[IndentingNewLine]", "tEnd", ",", "replicates", ",", "1.0"}], 
         "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"trix", "=", "1"}], ",", 
         RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
         RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"For", "[", 
           RowBox[{
            RowBox[{"tix", "=", "1"}], ",", 
            RowBox[{"tix", "<=", 
             RowBox[{"tEnd", "+", "1"}]}], ",", 
            RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Length", "[", 
                 RowBox[{
                  RowBox[{"Drop", "[", 
                   RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                  RowBox[{"[", "tix", "]"}], "]"}], "]"}], ">", "2"}], ",", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"DATAvalsAbs", "[", 
                 RowBox[{"[", 
                  RowBox[{"trix", ",", "tix"}], "]"}], "]"}], "=", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"tix", "-", "1"}], ",", 
                  RowBox[{
                   RowBox[{"dataSet", "[", 
                    RowBox[{"[", 
                    RowBox[{"2", ",", "trix"}], "]"}], "]"}], "*", 
                   RowBox[{
                    RowBox[{"Drop", "[", 
                    RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                    RowBox[{"[", 
                    RowBox[{"tix", ",", "3"}], "]"}], "]"}]}]}], "}"}]}], ",",
                "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"DATAvalsAbs", "[", 
                 RowBox[{"[", 
                  RowBox[{"trix", ",", "tix"}], "]"}], "]"}], "=", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"tix", "-", "1"}], ",", "\"\<\>\""}], "}"}]}]}], 
              "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
        "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       "DATAvalsAbs"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.6362614848885126`*^9, 3.636261498991319*^9}, {
   3.6362615972549396`*^9, 3.636261666998929*^9}, {3.6362617326476836`*^9, 
   3.636261796703347*^9}, {3.6362618359985948`*^9, 3.636261842077943*^9}, {
   3.6362620650486956`*^9, 3.6362620659347467`*^9}, {3.6362622057837453`*^9, 
   3.636262206510787*^9}, {3.63626494670434*^9, 3.636265068524729*^9}, {
   3.6362677903346357`*^9, 3.636267792134638*^9}, 3.6362707226713996`*^9, {
   3.6362718145810986`*^9, 3.636271840267135*^9}, {3.6392518488033752`*^9, 
   3.639251863844879*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"extractSOTdataseriesRel", "[", 
    RowBox[{"dataSet_", ",", "tEnd_", ",", "nrTr_", ",", "Ninit_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"trix", ",", "tix", ",", "TSdata", ",", "DATAvalsAbs"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"DATAvalsAbs", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"tEnd", "+", "1"}], "}"}]}], "]"}], ",", 
         RowBox[{"{", "nrTr", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"TSdata", "=", "\[IndentingNewLine]", 
       RowBox[{"createModelTimeSeries1M", "[", 
        RowBox[{"ITmodel", " ", 
         RowBox[{"(*", 
          RowBox[{"not", " ", "relevant"}], "*)"}], ",", 
         "\[IndentingNewLine]", "dataSet", ",", "0", ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"p1", "\[Rule]", "1"}], ",", 
            RowBox[{"p2", "\[Rule]", "1"}], ",", 
            RowBox[{"p3", "\[Rule]", "1"}]}], "}"}], "}"}], ",", 
         "\[IndentingNewLine]", "tEnd", ",", "1", ",", "1.0"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"trix", "=", "1"}], ",", 
        RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
        RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"tix", "=", "1"}], ",", 
           RowBox[{"tix", "<=", 
            RowBox[{"tEnd", "+", "1"}]}], ",", 
           RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Length", "[", 
                RowBox[{
                 RowBox[{"Drop", "[", 
                  RowBox[{
                   RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                 RowBox[{"[", "tix", "]"}], "]"}], "]"}], ">", "2"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"DATAvalsAbs", "[", 
                RowBox[{"[", 
                 RowBox[{"trix", ",", "tix"}], "]"}], "]"}], "=", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"tix", "-", "1"}], ",", 
                 RowBox[{
                  RowBox[{"1", "/", "Ninit"}], "*", 
                  RowBox[{"dataSet", "[", 
                   RowBox[{"[", 
                    RowBox[{"2", ",", "trix"}], "]"}], "]"}], "*", 
                  RowBox[{
                   RowBox[{"Drop", "[", 
                    RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                   RowBox[{"[", 
                    RowBox[{"tix", ",", "3"}], "]"}], "]"}]}]}], "}"}]}], ",",
               "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"DATAvalsAbs", "[", 
                RowBox[{"[", 
                 RowBox[{"trix", ",", "tix"}], "]"}], "]"}], "=", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"tix", "-", "1"}], ",", "\"\<\>\""}], "}"}]}]}], 
             "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      "DATAvalsAbs"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6382620024084196`*^9, 3.6382620548374195`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"create", " ", 
    StyleBox["nRnd",
     FontSlant->"Italic"], " ", "random", " ", "variates", " ", "with", " ", 
    "absolute", " ", "numbers", " ", "of", " ", "living", " ", "individuals", 
    " ", "at", " ", "times", " ", "t", "\[IndentingNewLine]", "  ", "Return", 
    " ", "median", " ", "and", " ", "lower", " ", "and", " ", "upper", " ", 
    "precentiles", " ", "in", " ", "ordered", " ", 
    RowBox[{"lists", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Clear", "[", "SOT$prediction$Uncertainty$Stochasticity", "]"}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"SOT$prediction$Uncertainty$Stochasticity", "[", 
      RowBox[{
      "data_", ",", "model_", ",", "parSet_", ",", "BG$mort_", ",", 
       "replicates_", ",", "tEnd_", ",", "Ninit_", ",", "nRnd_", ",", "nrTr_",
        ",", 
       RowBox[{"lcl_:", "0.1"}], ",", 
       RowBox[{"ucl_:", "0.9"}], ",", 
       RowBox[{"deltaT_:", "1.0"}]}], "]"}], ":=", 
     RowBox[{"Block", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "tixMax", ",", "NLOlist", ",", "allNLO", ",", "TSdata", ",", "rix", 
         ",", "trix", ",", "tix", ",", "N$t", ",", "NLOmedians", ",", 
         "allNLOmedians", ",", "allNLOLperc", ",", "NLOLperc", ",", 
         "allNLOUperc", ",", "NLOUperc", ",", "allOptDat"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"tixMax", "=", 
         RowBox[{"Round", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"(", "tEnd", ")"}], "/", "deltaT"}], "+", "1"}], "]"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"NLOlist", "=", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{"{", "}"}], ",", 
             RowBox[{"{", "tixMax", "}"}]}], "]"}], ",", 
           RowBox[{"{", "nrTr", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"allNLO", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"TSdata", "=", 
         RowBox[{"createModelTimeSeries1M", "[", 
          RowBox[{"model", ",", "data", ",", "BG$mort", ",", 
           RowBox[{"{", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"p1", "\[Rule]", 
               RowBox[{"parSet", "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ",", 
              RowBox[{"p2", "\[Rule]", 
               RowBox[{"parSet", "[", 
                RowBox[{"[", "2", "]"}], "]"}]}], ",", 
              RowBox[{"p3", "->", 
               RowBox[{"parSet", "[", 
                RowBox[{"[", "3", "]"}], "]"}]}]}], "}"}], "}"}], ",", "tEnd",
            ",", "replicates", ",", "deltaT"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"For", "[", 
           RowBox[{
            RowBox[{"rix", "=", "1"}], ",", 
            RowBox[{"rix", "<=", "nRnd"}], ",", 
            RowBox[{"rix", "++"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"For", "[", 
              RowBox[{
               RowBox[{"trix", "=", "1"}], ",", 
               RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
               RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"N$t", "=", "Ninit"}], ";", "\[IndentingNewLine]", 
                RowBox[{"For", "[", 
                 RowBox[{
                  RowBox[{"tix", "=", "1"}], ",", 
                  RowBox[{"tix", "<=", "tixMax"}], ",", 
                  RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"Sval", "=", 
                    RowBox[{
                    RowBox[{"Drop", "[", 
                    RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                    RowBox[{"[", 
                    RowBox[{"tix", ",", "2"}], "]"}], "]"}]}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"Sval$old", "=", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"tix", ">", "1"}], ",", 
                    RowBox[{
                    RowBox[{"Drop", "[", 
                    RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"tix", "-", "1"}], ",", "2"}], "]"}], "]"}], ",", 
                    "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
                   RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"Sval$old", ">", "0"}], ",", 
                    RowBox[{"pCond", "=", 
                    RowBox[{"Sval", "/", "Sval$old"}]}], ",", 
                    RowBox[{"pCond", "=", "0"}]}], "]"}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"pCond", ">", "1"}], ",", 
                    RowBox[{"pCond", "=", "1"}]}], "]"}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{"N$t", "=", 
                    RowBox[{
                    RowBox[{"RandomVariate", "[", 
                    RowBox[{
                    RowBox[{"BinomialDistribution", "[", 
                    RowBox[{"N$t", ",", "pCond"}], "]"}], ",", "1"}], "]"}], 
                    "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], ";", 
                   "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"NLOlist", "[", 
                    RowBox[{"[", 
                    RowBox[{"trix", ",", "tix"}], "]"}], "]"}], "=", "N$t"}], 
                   ";"}]}], "\[IndentingNewLine]", "]"}]}]}], 
              "\[IndentingNewLine]", "]"}], ";", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"allNLO", ",", "NLOlist"}], "]"}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", ",", 
          RowBox[{
           RowBox[{"ToString", "[", 
            RowBox[{"Length", "@", "allNLO"}], "]"}], "<>", "\"\< of \>\"", "<>", 
           RowBox[{"ToString", "[", "nRnd", "]"}], "<>", 
           "\"\< parameter samples\>\""}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "extract", " ", "quantiles", " ", "of", " ", "the", " ", 
          "distribution"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"allNLOmedians", "=", 
         RowBox[{"{", "}"}]}], ";", " ", 
        RowBox[{"allNLOLperc", "=", 
         RowBox[{"{", "}"}]}], ";", 
        RowBox[{"allNLOUperc", "=", 
         RowBox[{"{", "}"}]}], ";", 
        RowBox[{"allOptDat", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"trix", "=", "1"}], ",", 
          RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
          RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"NLOmedians", "=", 
            RowBox[{"{", "}"}]}], ";", " ", 
           RowBox[{"NLOLperc", "=", 
            RowBox[{"{", "}"}]}], ";", " ", 
           RowBox[{"NLOUperc", "=", 
            RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"trNLO", "=", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               RowBox[{"allNLO", "[", 
                RowBox[{"[", 
                 RowBox[{"#", ",", "trix"}], "]"}], "]"}], "&"}], "/@", 
              RowBox[{"Range", "[", 
               RowBox[{"nRnd", "-", "1"}], "]"}]}], ")"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"tix", "=", "1"}], ",", 
             RowBox[{"tix", "<=", "tixMax"}], ",", 
             RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"AppendTo", "[", 
               RowBox[{"NLOmedians", ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"tix", "-", "1"}], ")"}], "*", "deltaT"}], ",", 
                  RowBox[{"Median", "[", 
                   RowBox[{
                    RowBox[{"Transpose", "[", "trNLO", "]"}], "[", 
                    RowBox[{"[", "tix", "]"}], "]"}], "]"}]}], "}"}]}], "]"}],
               ";", "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"NLOLperc", ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"tix", "-", "1"}], ")"}], "*", "deltaT"}], ",", 
                  RowBox[{"Quantile", "[", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Transpose", "[", "trNLO", "]"}], "[", 
                    RowBox[{"[", "tix", "]"}], "]"}], ",", "lcl"}], "]"}]}], 
                 "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"NLOUperc", ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"tix", "-", "1"}], ")"}], "*", "deltaT"}], ",", 
                  RowBox[{"Quantile", "[", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Transpose", "[", "trNLO", "]"}], "[", 
                    RowBox[{"[", "tix", "]"}], "]"}], ",", "ucl"}], "]"}]}], 
                 "}"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"tmpDat", "=", 
            RowBox[{"Drop", "[", 
             RowBox[{
              RowBox[{"TSdata", "[", 
               RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"optDat", "=", 
            RowBox[{"Table", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{
                RowBox[{"tmpDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
                RowBox[{"Ninit", "*", 
                 RowBox[{"tmpDat", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "2"}], "]"}], "]"}]}]}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "1", ",", 
                RowBox[{"Length", "[", "tmpDat", "]"}]}], "}"}]}], "]"}]}], 
           ";", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"allOptDat", ",", "optDat"}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"allNLOmedians", ",", "NLOmedians"}], "]"}], ";", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"allNLOLperc", ",", "NLOLperc"}], "]"}], ";", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"allNLOUperc", ",", "NLOUperc"}], "]"}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Returns", " ", "lists", " ", "in", " ", "order", " ", "median", " ",
           "and", " ", "lower", " ", "and", " ", "upper", " ", "percentile"}],
          "*)"}], "\[IndentingNewLine]", 
        RowBox[{"Return", "[", 
         RowBox[{"{", 
          RowBox[{
          "allNLOmedians", ",", "allNLOLperc", ",", "allNLOUperc", ",", 
           "allOptDat"}], "}"}], "]"}], ";"}]}], "\[IndentingNewLine]", 
      "]"}]}], ";"}]}]}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellChangeTimes->{{3.6362217527309585`*^9, 3.6362217940322313`*^9}, {
   3.6362218347115035`*^9, 3.6362218580229445`*^9}, {3.636221942618294*^9, 
   3.636221963950532*^9}, {3.636222124663816*^9, 3.63622238172727*^9}, {
   3.6362224410183754`*^9, 3.6362224699030266`*^9}, {3.6362225741906104`*^9, 
   3.6362226433147326`*^9}, {3.636222712980856*^9, 3.6362227272910814`*^9}, {
   3.63622276044514*^9, 3.636222815070836*^9}, {3.636222881650154*^9, 
   3.6362228956453786`*^9}, {3.6362230010445647`*^9, 3.636223026288409*^9}, {
   3.6362230574476643`*^9, 3.6362231196589746`*^9}, {3.636223166624058*^9, 
   3.636223184536889*^9}, {3.6362238329476357`*^9, 3.6362238368944426`*^9}, 
   3.6362573787705007`*^9, 3.636257496661667*^9, {3.636260569264142*^9, 
   3.6362606043751497`*^9}, 3.636260913934856*^9, {3.6362610393040266`*^9, 
   3.636261040518096*^9}, {3.6362610803583746`*^9, 3.636261085766684*^9}, {
   3.6362618523425293`*^9, 3.6362618600459704`*^9}, 3.6362622177044272`*^9, {
   3.636265111223789*^9, 3.636265113673793*^9}, {3.636267900041791*^9, 
   3.636268012305952*^9}, {3.6362680573540154`*^9, 3.636268111416093*^9}, {
   3.6362684898476295`*^9, 3.636268670725885*^9}, {3.6362687822980423`*^9, 
   3.636268785869047*^9}, {3.63626915672658*^9, 3.636269178780611*^9}, 
   3.6362695859591856`*^9, {3.6362696452632694`*^9, 3.63626968802433*^9}, {
   3.636270740402425*^9, 3.63627076503146*^9}, {3.6362718524311523`*^9, 
   3.6362719132562385`*^9}, 3.6362738171504073`*^9, {3.638074855660055*^9, 
   3.638074867672072*^9}, {3.638254134710449*^9, 3.638254155710549*^9}, {
   3.6382568552724924`*^9, 3.6382569018408246`*^9}, {3.6382569360068245`*^9, 
   3.6382569714358244`*^9}, 3.6382570440648246`*^9, {3.638257166224825*^9, 
   3.6382571707848244`*^9}, {3.6382572330058246`*^9, 
   3.6382572577558246`*^9}, {3.638257488984866*^9, 3.6382574942323914`*^9}, {
   3.638258878005769*^9, 3.6382589046874366`*^9}, {3.6382591116166277`*^9, 
   3.6382591137036276`*^9}, {3.6382591535026274`*^9, 
   3.6382591538986273`*^9}, {3.6382592017636275`*^9, 3.638259272572226*^9}, {
   3.638260088670828*^9, 3.6382600895899196`*^9}, {3.638260327296688*^9, 
   3.6382603290198603`*^9}, 3.63826035931689*^9, {3.6382606195049057`*^9, 
   3.6382606648754425`*^9}, {3.6382607045004044`*^9, 
   3.6382607238273373`*^9}, {3.6382607758535395`*^9, 3.638260794534407*^9}, {
   3.638260871241077*^9, 3.638260872826236*^9}, {3.6382609268696394`*^9, 
   3.6382609300599585`*^9}, 3.638273105152384*^9, {3.63844212355539*^9, 
   3.6384421299310274`*^9}, {3.638442468641019*^9, 3.6384424976599207`*^9}, {
   3.6384425624894032`*^9, 3.638442586708825*^9}, {3.638461542083713*^9, 
   3.6384615423277373`*^9}}],

Cell[BoxData[{
 RowBox[{"Clear", "[", "SOT$prediction$Uncertainty$Param", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SOT$prediction$Uncertainty$Param", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "parSetList_", ",", "optParSet_", ",", 
     "BG$mort_", ",", "replicates_", ",", "tEnd_", ",", "nrTr_", ",", 
     RowBox[{"lcl_:", "0.1"}], ",", 
     RowBox[{"ucl_:", "0.9"}], ",", 
     RowBox[{"deltaT_:", "1.0"}], ",", 
     RowBox[{"maxIT_:", "0"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "N$t", ",", "tixMax", ",", "MODvals", ",", "TSdata", ",", "trIT", ",", 
       "predMOD", ",", "trix", ",", "tix", ",", "rix", ",", "meanDat", ",", 
       "llDat", ",", "ulDat", ",", "optDat", ",", "tmpDat", ",", "allMeanDat",
        ",", "allLLDat", ",", "allULDat", ",", "allOptDat", ",", "ix", ",", 
       "maxIX", ",", "parSetRandom"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"tixMax", "=", 
       RowBox[{"Round", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"(", "tEnd", ")"}], "/", "deltaT"}], "+", "1"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"MODvals", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", 
           RowBox[{"{", "tixMax", "}"}]}], "]"}], ",", 
         RowBox[{"{", "nrTr", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"maxIT", "\[Equal]", "0"}], ",", 
        RowBox[{"maxIX", "=", 
         RowBox[{"Length", "[", "parSetList", "]"}]}], ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"maxIT", "\[LessEqual]", 
           RowBox[{"Length", "[", "parSetList", "]"}]}], ",", 
          RowBox[{"maxIX", "=", "maxIT"}], ",", 
          RowBox[{"maxIX", "=", 
           RowBox[{"Length", "[", "parSetList", "]"}]}]}], "]"}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"parSetRandom", "=", 
       RowBox[{"RandomSample", "[", "parSetList", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"ix", "=", "1"}], ",", 
          RowBox[{"ix", "\[LessEqual]", "maxIX"}], ",", 
          RowBox[{"ix", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"TSdata", "=", "\[IndentingNewLine]", 
            RowBox[{"createModelTimeSeries1M", "[", 
             RowBox[{"model", ",", "data", ",", "BG$mort", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"p1", "\[Rule]", 
                  RowBox[{"parSetRandom", "[", 
                   RowBox[{"[", 
                    RowBox[{"ix", ",", "2"}], "]"}], "]"}]}], ",", 
                 RowBox[{"p2", "\[Rule]", 
                  RowBox[{"parSetRandom", "[", 
                   RowBox[{"[", 
                    RowBox[{"ix", ",", "3"}], "]"}], "]"}]}], ",", 
                 RowBox[{"p3", "->", 
                  RowBox[{"parSetRandom", "[", 
                   RowBox[{"[", 
                    RowBox[{"ix", ",", "4"}], "]"}], "]"}]}]}], "}"}], "}"}], 
              ",", "tEnd", ",", "replicates", ",", "deltaT"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"trix", "=", "1"}], ",", 
             RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
             RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"N$t", "=", "Ninit"}], ";", "\[IndentingNewLine]", 
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"tix", "=", "1"}], ",", 
                RowBox[{"tix", "<=", "tixMax"}], ",", 
                RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"AppendTo", "[", 
                  RowBox[{
                   RowBox[{"MODvals", "[", 
                    RowBox[{"[", 
                    RowBox[{"trix", ",", "tix"}], "]"}], "]"}], ",", 
                   RowBox[{
                    RowBox[{"Drop", "[", 
                    RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                    RowBox[{"[", 
                    RowBox[{"tix", ",", "2"}], "]"}], "]"}]}], "]"}], ";"}]}],
                "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
            "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ",", 
        RowBox[{
         RowBox[{"ToString", "[", "ix", "]"}], "<>", "\"\< of \>\"", "<>", 
         RowBox[{"ToString", "[", "maxIX", "]"}], "<>", 
         "\"\< parameter samples\>\""}]}], "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"predMOD", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"trix", "=", "1"}], ",", 
        RowBox[{"trix", "<=", "nrTr"}], ",", 
        RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"trIT", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"tix", "=", "1"}], ",", 
           RowBox[{"tix", "\[LessEqual]", "tixMax"}], ",", 
           RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"trIT", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"tix", "-", "1"}], ")"}], "*", "deltaT"}], ",", 
                RowBox[{"Median", "[", 
                 RowBox[{"MODvals", "[", 
                  RowBox[{"[", 
                   RowBox[{"trix", ",", "tix"}], "]"}], "]"}], "]"}], ",", 
                RowBox[{"Quantile", "[", 
                 RowBox[{
                  RowBox[{"MODvals", "[", 
                   RowBox[{"[", 
                    RowBox[{"trix", ",", "tix"}], "]"}], "]"}], ",", "lcl"}], 
                 "]"}], ",", 
                RowBox[{"Quantile", "[", 
                 RowBox[{
                  RowBox[{"MODvals", "[", 
                   RowBox[{"[", 
                    RowBox[{"trix", ",", "tix"}], "]"}], "]"}], ",", "ucl"}], 
                 "]"}]}], "}"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", 
          "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"predMOD", ",", "trIT"}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"TSdataOPTIT", "=", 
       RowBox[{"createModelTimeSeries1M", "[", 
        RowBox[{"model", ",", "data", ",", "BG$mort", ",", 
         RowBox[{"{", 
          RowBox[{"optParSet", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "}"}], ",", "tEnd", ",", 
         "replicates", ",", "deltaT"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"allMeanDat", "=", 
       RowBox[{"{", "}"}]}], ";", 
      RowBox[{"allLLDat", "=", 
       RowBox[{"{", "}"}]}], ";", 
      RowBox[{"allULDat", "=", 
       RowBox[{"{", "}"}]}], ";", 
      RowBox[{"allOptDat", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"trix", "=", "1"}], ",", 
        RowBox[{"trix", "<=", "nrTr"}], ",", 
        RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"meanDat", "=", 
          RowBox[{"Transpose", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"Transpose", "[", 
               RowBox[{"predMOD", "[", 
                RowBox[{"[", "trix", "]"}], "]"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{
              RowBox[{"Transpose", "[", 
               RowBox[{"predMOD", "[", 
                RowBox[{"[", "trix", "]"}], "]"}], "]"}], "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"llDat", "=", 
          RowBox[{"Transpose", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"Transpose", "[", 
               RowBox[{"predMOD", "[", 
                RowBox[{"[", "trix", "]"}], "]"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{
              RowBox[{"Transpose", "[", 
               RowBox[{"predMOD", "[", 
                RowBox[{"[", "trix", "]"}], "]"}], "]"}], "[", 
              RowBox[{"[", "3", "]"}], "]"}]}], "}"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"ulDat", "=", 
          RowBox[{"Transpose", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"Transpose", "[", 
               RowBox[{"predMOD", "[", 
                RowBox[{"[", "trix", "]"}], "]"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{
              RowBox[{"Transpose", "[", 
               RowBox[{"predMOD", "[", 
                RowBox[{"[", "trix", "]"}], "]"}], "]"}], "[", 
              RowBox[{"[", "4", "]"}], "]"}]}], "}"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"tmpDat", "=", 
          RowBox[{"Drop", "[", 
           RowBox[{
            RowBox[{"TSdataOPTIT", "[", 
             RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"optDat", "=", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"tmpDat", "[", 
               RowBox[{"[", 
                RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
              RowBox[{"tmpDat", "[", 
               RowBox[{"[", 
                RowBox[{"i", ",", "2"}], "]"}], "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "1", ",", 
              RowBox[{"Length", "[", "tmpDat", "]"}]}], "}"}]}], "]"}]}], ";",
          "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allMeanDat", ",", "meanDat"}], "]"}], ";", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allLLDat", ",", "llDat"}], "]"}], ";", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allULDat", ",", "ulDat"}], "]"}], ";", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allOptDat", ",", "optDat"}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
        "allMeanDat", ",", "allLLDat", ",", "allULDat", ",", "allOptDat"}], 
        "}"}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.634301290309371*^9, 3.6343013945477934`*^9}, {
   3.6343014485911975`*^9, 3.634301451494488*^9}, {3.6343018667520094`*^9, 
   3.6343018718555193`*^9}, {3.6343028316694913`*^9, 
   3.6343028331126356`*^9}, {3.634721874974286*^9, 3.634721922605048*^9}, {
   3.63472210443723*^9, 3.634722114477234*^9}, {3.6347221729820833`*^9, 
   3.634722191615947*^9}, {3.6347222263064156`*^9, 3.634722287035488*^9}, {
   3.6347225203198137`*^9, 3.63472254628841*^9}, {3.6347225851132927`*^9, 
   3.634722620475828*^9}, {3.6347228425581274`*^9, 3.634723109291417*^9}, 
   3.6349997040633717`*^9, {3.6349998940073643`*^9, 3.6349998940393677`*^9}, {
   3.634999926656629*^9, 3.63499996366833*^9}, {3.6351675985212183`*^9, 
   3.635167614912857*^9}, {3.635167729175282*^9, 3.6351677709214563`*^9}, {
   3.63517004968831*^9, 3.635170058607202*^9}, {3.635170157036044*^9, 
   3.6351701573710775`*^9}, {3.635170252666606*^9, 3.6351702899173307`*^9}, {
   3.6351703701623545`*^9, 3.635170371069445*^9}, {3.6351706069070263`*^9, 
   3.6351706121785536`*^9}, 3.635170653365672*^9, 3.635170819972331*^9, {
   3.635170856638997*^9, 3.635170858061139*^9}, {3.635170943336666*^9, 
   3.635170945656898*^9}, 3.635171031816762*^9, {3.6351711387444034`*^9, 
   3.635171147345619*^9}, {3.635171203709293*^9, 3.6351712063360834`*^9}, {
   3.63517170973343*^9, 3.6351717308883142`*^9}, {3.6351718645749445`*^9, 
   3.635171870346367*^9}, {3.635171913132088*^9, 3.6351719165959206`*^9}, {
   3.635171948618921*^9, 3.6351719488179207`*^9}, {3.6351727162390656`*^9, 
   3.6351727595393953`*^9}, {3.6351746001074333`*^9, 3.6351746287773*^9}, {
   3.6362239998365307`*^9, 3.636224005905941*^9}, 3.636260712263321*^9, {
   3.6362643179779444`*^9, 3.636264339976976*^9}, 3.636271255358307*^9, {
   3.6362713729024734`*^9, 3.636271458304594*^9}, 3.636271534859703*^9, {
   3.636271599093794*^9, 3.6362716904429235`*^9}, {3.6362717223889685`*^9, 
   3.636271723918971*^9}, {3.6362717745810423`*^9, 3.6362718010190797`*^9}, {
   3.6362719296492615`*^9, 3.6362720465554266`*^9}, {3.636272136181554*^9, 
   3.6362721362415533`*^9}, {3.63627221407578*^9, 3.63627232655752*^9}, {
   3.6362724011486254`*^9, 3.636272465856718*^9}, {3.6362724994417663`*^9, 
   3.6362726233939433`*^9}, {3.636272765057143*^9, 3.636272803029197*^9}, 
   3.6362728661332874`*^9, {3.6362729031703396`*^9, 3.636273051402549*^9}, 
   3.636273833905431*^9, {3.6382541095509334`*^9, 3.638254118150793*^9}, {
   3.6382569186478243`*^9, 3.6382569204638243`*^9}, {3.6382589125782256`*^9, 
   3.638258914954463*^9}, {3.6382591269716277`*^9, 3.6382591472806273`*^9}, {
   3.638259318359804*^9, 3.6382593188738556`*^9}, {3.63826019610857*^9, 
   3.6382601963085904`*^9}, 3.6382612014677343`*^9, {3.638261237998081*^9, 
   3.6382612948384194`*^9}, {3.6382615901114197`*^9, 
   3.6382615965424194`*^9}, {3.6382619014704194`*^9, 
   3.6382619029644194`*^9}, {3.6382621558864193`*^9, 
   3.6382622118254194`*^9}, {3.6384425910362577`*^9, 3.638442666688822*^9}, {
   3.638442778302333*^9, 3.638442779269429*^9}, {3.638496317630453*^9, 
   3.638496335580474*^9}, {3.638525050418002*^9, 3.6385250549614563`*^9}, {
   3.639105322788162*^9, 3.639105333848268*^9}}],

Cell[BoxData[{
 RowBox[{"Clear", "[", "SOT$prediction$Uncertainty$Total", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SOT$prediction$Uncertainty$Total", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "parSetList_", ",", "optParSet_", ",", 
     "BG$mort_", ",", "replicates_", ",", "tEnd_", ",", "Ninit_", ",", 
     "nRand_", ",", "nrTr_", ",", 
     RowBox[{"lcl_:", "0.1"}], ",", 
     RowBox[{"ucl_:", "0.9"}], ",", 
     RowBox[{"deltaT_:", "1.0"}], ",", 
     RowBox[{"maxIT_:", "0"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "tixMax", ",", "MODvals", ",", "TSdata", ",", "NLOlist", ",", "N$t", 
       ",", "trIT", ",", "predMOD", ",", "trix", ",", "tix", ",", "rix", ",", 
       "meanDat", ",", "llDat", ",", "ulDat", ",", "optDat", ",", "tmpDat", 
       ",", "allOptDat", ",", "allNLOmedians", ",", "allNLOLperc", ",", 
       "allNLOUperc", ",", "NLOmedians", ",", "NLOLperc", ",", "NLOUperc", 
       ",", "ix", ",", "maxIX", ",", "parSetRandom"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"tixMax", "=", 
       RowBox[{"Round", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"(", "tEnd", ")"}], "/", "deltaT"}], "+", "1"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"MODvals", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", 
           RowBox[{"{", "tixMax", "}"}]}], "]"}], ",", 
         RowBox[{"{", "nrTr", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"NLOlist", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", 
           RowBox[{"{", "tixMax", "}"}]}], "]"}], ",", 
         RowBox[{"{", "nrTr", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"maxIT", "\[Equal]", "0"}], ",", 
        RowBox[{"maxIX", "=", 
         RowBox[{"Length", "[", "parSetList", "]"}]}], ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"maxIT", "\[LessEqual]", 
           RowBox[{"Length", "[", "parSetList", "]"}]}], ",", 
          RowBox[{"maxIX", "=", "maxIT"}], ",", 
          RowBox[{"maxIX", "=", 
           RowBox[{"Length", "[", "parSetList", "]"}]}]}], "]"}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"parSetRandom", "=", 
       RowBox[{"RandomSample", "[", "parSetList", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"allNLO", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Monitor", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"ix", "=", "1"}], ",", 
          RowBox[{"ix", "\[LessEqual]", "maxIX"}], ",", 
          RowBox[{"ix", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"TSdata", "=", "\[IndentingNewLine]", 
            RowBox[{"createModelTimeSeries1M", "[", 
             RowBox[{"model", ",", "data", ",", "BG$mort", ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"p1", "\[Rule]", 
                  RowBox[{"parSetRandom", "[", 
                   RowBox[{"[", 
                    RowBox[{"ix", ",", "2"}], "]"}], "]"}]}], ",", 
                 RowBox[{"p2", "\[Rule]", 
                  RowBox[{"parSetRandom", "[", 
                   RowBox[{"[", 
                    RowBox[{"ix", ",", "3"}], "]"}], "]"}]}], ",", 
                 RowBox[{"p3", "->", 
                  RowBox[{"parSetRandom", "[", 
                   RowBox[{"[", 
                    RowBox[{"ix", ",", "4"}], "]"}], "]"}]}]}], "}"}], "}"}], 
              ",", "tEnd", ",", "replicates", ",", "deltaT"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"rix", "=", "1"}], ",", 
             RowBox[{"rix", "<=", "nRand"}], ",", 
             RowBox[{"rix", "++"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"For", "[", 
               RowBox[{
                RowBox[{"trix", "=", "1"}], ",", 
                RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
                RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"N$t", "=", "Ninit"}], ";", "\[IndentingNewLine]", 
                 RowBox[{"For", "[", 
                  RowBox[{
                   RowBox[{"tix", "=", "1"}], ",", 
                   RowBox[{"tix", "<=", "tixMax"}], ",", 
                   RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"AppendTo", "[", 
                    RowBox[{
                    RowBox[{"MODvals", "[", 
                    RowBox[{"[", 
                    RowBox[{"trix", ",", "tix"}], "]"}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Drop", "[", 
                    RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                    RowBox[{"[", 
                    RowBox[{"tix", ",", "2"}], "]"}], "]"}]}], "]"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"Sval", "=", 
                    RowBox[{
                    RowBox[{"Drop", "[", 
                    RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                    RowBox[{"[", 
                    RowBox[{"tix", ",", "2"}], "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"Sval$old", "=", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"tix", ">", "1"}], ",", 
                    RowBox[{
                    RowBox[{"Drop", "[", 
                    RowBox[{
                    RowBox[{"TSdata", "[", 
                    RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}], "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"tix", "-", "1"}], ",", "2"}], "]"}], "]"}], ",", 
                    "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"Sval$old", ">", "0"}], ",", 
                    RowBox[{"pCond", "=", 
                    RowBox[{"Sval", "/", "Sval$old"}]}], ",", 
                    RowBox[{"pCond", "=", "0"}]}], "]"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"pCond", ">", "1"}], ",", 
                    RowBox[{"pCond", "=", "1"}]}], "]"}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"N$t", "=", 
                    RowBox[{
                    RowBox[{"RandomVariate", "[", 
                    RowBox[{
                    RowBox[{"BinomialDistribution", "[", 
                    RowBox[{"N$t", ",", "pCond"}], "]"}], ",", "1"}], "]"}], 
                    "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"NLOlist", "[", 
                    RowBox[{"[", 
                    RowBox[{"trix", ",", "tix"}], "]"}], "]"}], "=", "N$t"}], 
                    ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
               "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"allNLO", ",", "NLOlist"}], "]"}], ";"}]}], 
            "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", 
         "]"}], ",", 
        RowBox[{
         RowBox[{"ToString", "[", 
          RowBox[{"Length", "@", "allNLO"}], "]"}], "<>", "\"\< of \>\"", "<>", 
         RowBox[{"ToString", "[", 
          RowBox[{"nRand", "*", "maxIX"}], "]"}], "<>", 
         "\"\< parameter samples\>\""}]}], "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"TSdataOPTIT", "=", 
       RowBox[{"createModelTimeSeries1M", "[", 
        RowBox[{"model", ",", "data", ",", "BG$mort", ",", 
         RowBox[{"{", 
          RowBox[{"optParSet", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "}"}], ",", "tEnd", ",", 
         "replicates", ",", "deltaT"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"allNLOmedians", "=", 
       RowBox[{"{", "}"}]}], ";", " ", 
      RowBox[{"allNLOLperc", "=", 
       RowBox[{"{", "}"}]}], ";", 
      RowBox[{"allNLOUperc", "=", 
       RowBox[{"{", "}"}]}], ";", 
      RowBox[{"allOptDat", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"trix", "=", "1"}], ",", 
        RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
        RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"NLOmedians", "=", 
          RowBox[{"{", "}"}]}], ";", " ", 
         RowBox[{"NLOLperc", "=", 
          RowBox[{"{", "}"}]}], ";", " ", 
         RowBox[{"NLOUperc", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"trNLO", "=", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"allNLO", "[", 
              RowBox[{"[", 
               RowBox[{"#", ",", "trix"}], "]"}], "]"}], "&"}], "/@", 
            RowBox[{"Range", "[", 
             RowBox[{
              RowBox[{"Length", "[", "allNLO", "]"}], "-", "1"}], "]"}]}], 
           ")"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"tix", "=", "1"}], ",", 
           RowBox[{"tix", "<=", "tixMax"}], ",", 
           RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"NLOmedians", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"tix", "-", "1"}], ")"}], "*", "deltaT"}], ",", 
                RowBox[{"Median", "[", 
                 RowBox[{
                  RowBox[{"Transpose", "[", "trNLO", "]"}], "[", 
                  RowBox[{"[", "tix", "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"NLOLperc", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"tix", "-", "1"}], ")"}], "*", "deltaT"}], ",", 
                RowBox[{"Min", "[", 
                 RowBox[{
                  RowBox[{"Transpose", "[", "trNLO", "]"}], "[", 
                  RowBox[{"[", "tix", "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"NLOUperc", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"tix", "-", "1"}], ")"}], "*", "deltaT"}], ",", 
                RowBox[{"Max", "[", 
                 RowBox[{
                  RowBox[{"Transpose", "[", "trNLO", "]"}], "[", 
                  RowBox[{"[", "tix", "]"}], "]"}], "]"}]}], "}"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"tmpDat", "=", 
          RowBox[{"Drop", "[", 
           RowBox[{
            RowBox[{"TSdataOPTIT", "[", 
             RowBox[{"[", "trix", "]"}], "]"}], ",", "2"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"optDat", "=", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"tmpDat", "[", 
               RowBox[{"[", 
                RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
              RowBox[{"Ninit", "*", 
               RowBox[{"tmpDat", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "2"}], "]"}], "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "1", ",", 
              RowBox[{"Length", "[", "tmpDat", "]"}]}], "}"}]}], "]"}]}], ";",
          "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allOptDat", ",", "optDat"}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allNLOmedians", ",", "NLOmedians"}], "]"}], ";", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allNLOLperc", ",", "NLOLperc"}], "]"}], ";", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allNLOUperc", ",", "NLOUperc"}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{
        "allNLOmedians", ",", "allNLOLperc", ",", "allNLOUperc", ",", 
         "allOptDat"}], "}"}], "]"}], ";"}]}], "\[IndentingNewLine]", 
    RowBox[{"(*", " ", 
     RowBox[{
     "Returns", " ", "lists", " ", "in", " ", "order", " ", "median", " ", 
      "and", " ", "lower", " ", "and", " ", "upper", " ", "percentile"}], 
     "*)"}], "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.634301290309371*^9, 3.6343013945477934`*^9}, {
   3.6343014485911975`*^9, 3.634301451494488*^9}, {3.6343018667520094`*^9, 
   3.6343018718555193`*^9}, {3.6343028316694913`*^9, 
   3.6343028331126356`*^9}, {3.634721874974286*^9, 3.634721922605048*^9}, {
   3.63472210443723*^9, 3.634722114477234*^9}, {3.6347221729820833`*^9, 
   3.634722191615947*^9}, {3.6347222263064156`*^9, 3.634722287035488*^9}, {
   3.6347225203198137`*^9, 3.63472254628841*^9}, {3.6347225851132927`*^9, 
   3.634722620475828*^9}, {3.6347228425581274`*^9, 3.634723109291417*^9}, 
   3.6349997040633717`*^9, {3.6349998940073643`*^9, 3.6349998940393677`*^9}, {
   3.634999926656629*^9, 3.63499996366833*^9}, {3.6351675985212183`*^9, 
   3.635167614912857*^9}, {3.635167729175282*^9, 3.6351677709214563`*^9}, {
   3.63517004968831*^9, 3.635170058607202*^9}, {3.635170157036044*^9, 
   3.6351701573710775`*^9}, {3.635170252666606*^9, 3.6351702899173307`*^9}, {
   3.6351703701623545`*^9, 3.635170371069445*^9}, {3.6351706069070263`*^9, 
   3.6351706121785536`*^9}, 3.635170653365672*^9, 3.635170819972331*^9, {
   3.635170856638997*^9, 3.635170858061139*^9}, {3.635170943336666*^9, 
   3.635170945656898*^9}, 3.635171031816762*^9, {3.6351711387444034`*^9, 
   3.635171147345619*^9}, {3.635171203709293*^9, 3.6351712063360834`*^9}, {
   3.63517170973343*^9, 3.6351717308883142`*^9}, {3.6351718645749445`*^9, 
   3.635171870346367*^9}, {3.635171913132088*^9, 3.6351719165959206`*^9}, {
   3.635171948618921*^9, 3.6351719488179207`*^9}, {3.6351727162390656`*^9, 
   3.6351727595393953`*^9}, {3.6351746001074333`*^9, 3.6351746287773*^9}, {
   3.6362239998365307`*^9, 3.636224005905941*^9}, 3.636260712263321*^9, {
   3.6362643179779444`*^9, 3.636264339976976*^9}, 3.636271255358307*^9, {
   3.6362713729024734`*^9, 3.636271458304594*^9}, 3.636271534859703*^9, {
   3.636271599093794*^9, 3.6362716904429235`*^9}, {3.6362717223889685`*^9, 
   3.636271723918971*^9}, {3.6362717745810423`*^9, 3.6362718010190797`*^9}, {
   3.6362719296492615`*^9, 3.6362720465554266`*^9}, {3.636272136181554*^9, 
   3.6362721362415533`*^9}, {3.63627221407578*^9, 3.63627232655752*^9}, {
   3.6362724011486254`*^9, 3.636272465856718*^9}, {3.6362724994417663`*^9, 
   3.6362726233939433`*^9}, {3.636272765057143*^9, 3.636272803029197*^9}, 
   3.6362728661332874`*^9, {3.6362729031703396`*^9, 3.636273051402549*^9}, {
   3.636273891988513*^9, 3.636273895570518*^9}, {3.6380753273300076`*^9, 
   3.6380753356500196`*^9}, {3.6380765324757013`*^9, 3.638076533025702*^9}, {
   3.6380765932717867`*^9, 3.63807659547179*^9}, 3.638076639575852*^9, {
   3.6380768343203287`*^9, 3.6380768346903286`*^9}, 3.6380770928856916`*^9, {
   3.638077329841529*^9, 3.6380773509707375`*^9}, 3.638077488804302*^9, {
   3.6380775467813835`*^9, 3.638077597335455*^9}, {3.63807779757275*^9, 
   3.638077802582757*^9}, {3.6380778395068088`*^9, 3.6380778617988405`*^9}, {
   3.638077899074893*^9, 3.63807791872692*^9}, {3.63807854684181*^9, 
   3.6380785776348534`*^9}, {3.6380788209841957`*^9, 3.638078827645205*^9}, {
   3.6380789439263687`*^9, 3.6380789527273808`*^9}, {3.638079060018532*^9, 
   3.638079194821721*^9}, {3.6380793170588937`*^9, 3.638079319218897*^9}, {
   3.6380814988869815`*^9, 3.63808155491306*^9}, 3.6380824146152773`*^9, {
   3.6380824911443853`*^9, 3.6380824963153925`*^9}, {3.638086667046713*^9, 
   3.6380866942507515`*^9}, {3.638111086476127*^9, 3.638111087676129*^9}, {
   3.638254052970276*^9, 3.6382540672077*^9}, {3.6382569243518248`*^9, 
   3.6382569249528246`*^9}, {3.6382589226742353`*^9, 
   3.6382589267626276`*^9}, {3.638259130206628*^9, 3.6382591364556274`*^9}, {
   3.6382593230502734`*^9, 3.6382593235133195`*^9}, {3.6382622759594193`*^9, 
   3.6382623920154195`*^9}, {3.6382624455404196`*^9, 
   3.6382624458684196`*^9}, {3.638263458412986*^9, 3.6382634585810027`*^9}, 
   3.638263840102151*^9, {3.6382638830244427`*^9, 3.638263917208861*^9}, {
   3.6382707607012887`*^9, 3.6382707632454343`*^9}, 3.638272966066428*^9, 
   3.6382730962828765`*^9, {3.6383600111937895`*^9, 3.6383600142250924`*^9}, {
   3.6383615839610505`*^9, 3.638361631041758*^9}, 3.6383617055302057`*^9, {
   3.638442355699726*^9, 3.6384423623073874`*^9}, {3.638442406981854*^9, 
   3.6384424521823735`*^9}, {3.6384426751454563`*^9, 3.63844271019453*^9}, {
   3.6384427552720375`*^9, 3.6384427558920994`*^9}, {3.6384465335680113`*^9, 
   3.638446539510606*^9}, {3.6384615235158567`*^9, 3.638461532996804*^9}, {
   3.638495948475746*^9, 3.63849601865336*^9}, {3.6384960629628506`*^9, 
   3.6384963057583923`*^9}, {3.6385247834933424`*^9, 
   3.6385247892869215`*^9}, {3.6385249920951705`*^9, 
   3.6385250414931097`*^9}, {3.639105275510435*^9, 3.6391053091617994`*^9}, {
   3.648272385207349*^9, 3.64827238549332*^9}, {3.648272424247444*^9, 
   3.648272429919877*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Goodness-of-fit measures", "Title",
 CellChangeTimes->{{3.6446650814402013`*^9, 3.644665086167674*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.6446576987340045`*^9, 3.6446577201811485`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"prepareDataAndMedianExtracts", "[", 
   RowBox[{
   "data_", ",", "tEnd_", ",", "repl_", ",", "nrTr_", ",", "probData_"}], 
   "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "datPure", ",", "ts", ",", "tsPure", ",", "datList", ",", "modList", ",",
       "trix", ",", "tix", ",", "trListMod", ",", "trListDat", ",", "elem"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"datPure", "=", 
      RowBox[{"extractSOTdataseries", "[", 
       RowBox[{"data", ",", "tEnd", ",", "repl", ",", "nrTr"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"tsPure", "=", 
      RowBox[{"probData", "[", 
       RowBox[{"[", "1", 
        RowBox[{"(*", "med", "*)"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"datList", "=", 
      RowBox[{"{", "}"}]}], ";", 
     RowBox[{"modList", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"trix", "=", "1"}], ",", 
       RowBox[{"trix", "<=", "nrTr"}], ",", 
       RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"trListMod", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"trListDat", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"tix", "=", "1"}], ",", 
          RowBox[{"tix", "<=", 
           RowBox[{"Length", "[", 
            RowBox[{"datPure", "[", 
             RowBox[{"[", "trix", "]"}], "]"}], "]"}]}], ",", 
          RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"elem", "=", 
            RowBox[{"Select", "[", 
             RowBox[{
              RowBox[{"tsPure", "[", 
               RowBox[{"[", "trix", "]"}], "]"}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], "==", 
                RowBox[{"datPure", "[", 
                 RowBox[{"[", 
                  RowBox[{"trix", ",", "tix", ",", "1"}], "]"}], "]"}]}], 
               "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "[", "elem", "]"}], ">", "0"}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"AppendTo", "[", 
               RowBox[{"trListMod", ",", 
                RowBox[{"elem", "[", 
                 RowBox[{"[", 
                  RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "]"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"trListDat", ",", 
                RowBox[{"datPure", "[", 
                 RowBox[{"[", 
                  RowBox[{"trix", ",", "tix", ",", "2"}], "]"}], "]"}]}], 
               "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"datList", ",", "trListDat"}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"modList", ",", "trListMod"}], "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Return", "[", 
      RowBox[{"{", 
       RowBox[{"datList", ",", "modList"}], "}"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.644674672666228*^9, 3.644674675609522*^9}, {
  3.64467478590355*^9, 3.6446748404480047`*^9}, {3.6446748954125004`*^9, 
  3.6446749004350023`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"prepareDataAndModelExtracts", "[", 
    RowBox[{
    "data_", ",", "tEnd_", ",", "repl_", ",", "nrTr_", ",", "model_", ",", 
     "kBG_", ",", "parms_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "datPure", ",", "ts", ",", "tsPure", ",", "datList", ",", "modList", 
       ",", "trix", ",", "tix", ",", "trListMod", ",", "trListDat", ",", 
       "elem"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"datPure", "=", 
       RowBox[{"extractSOTdataseries", "[", 
        RowBox[{"data", ",", "tEnd", ",", "repl", ",", "nrTr"}], "]"}]}], ";",
       "\[IndentingNewLine]", 
      RowBox[{"ts", "=", 
       RowBox[{"createModelTimeSeries1M", "[", 
        RowBox[{
        "model", ",", "data", ",", "kBG", ",", "parms", ",", "tEnd", ",", 
         "repl", ",", 
         RowBox[{"deltaT", "=", "1"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"tsPure", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Transpose", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"Transpose", "[", 
              RowBox[{"Drop", "[", 
               RowBox[{
                RowBox[{"ts", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", "2"}], "]"}], "]"}], 
             "[", 
             RowBox[{"[", "1", "]"}], "]"}], ",", 
            RowBox[{
             RowBox[{"Transpose", "[", 
              RowBox[{"Drop", "[", 
               RowBox[{
                RowBox[{"ts", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], ",", "2"}], "]"}], "]"}], 
             "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "nrTr"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"datList", "=", 
       RowBox[{"{", "}"}]}], ";", 
      RowBox[{"modList", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"trix", "=", "1"}], ",", 
        RowBox[{"trix", "<=", "nrTr"}], ",", 
        RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"trListMod", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"trListDat", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"InitAbundance", "=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{"2", ",", "trix"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"tix", "=", "1"}], ",", 
           RowBox[{"tix", "<=", 
            RowBox[{"Length", "[", 
             RowBox[{"datPure", "[", 
              RowBox[{"[", "trix", "]"}], "]"}], "]"}]}], ",", 
           RowBox[{"tix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"elem", "=", 
             RowBox[{"Select", "[", 
              RowBox[{
               RowBox[{"tsPure", "[", 
                RowBox[{"[", "trix", "]"}], "]"}], ",", 
               RowBox[{
                RowBox[{
                 RowBox[{"#", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], "==", 
                 RowBox[{"datPure", "[", 
                  RowBox[{"[", 
                   RowBox[{"trix", ",", "tix", ",", "1"}], "]"}], "]"}]}], 
                "&"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Length", "[", "elem", "]"}], ">", "0"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"AppendTo", "[", 
                RowBox[{"trListMod", ",", 
                 RowBox[{"InitAbundance", "*", 
                  RowBox[{"elem", "[", 
                   RowBox[{"[", 
                    RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], "]"}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"AppendTo", "[", 
                RowBox[{"trListDat", ",", 
                 RowBox[{"datPure", "[", 
                  RowBox[{"[", 
                   RowBox[{"trix", ",", "tix", ",", "2"}], "]"}], "]"}]}], 
                "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"datList", ",", "trListDat"}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"modList", ",", "trListMod"}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", 
       RowBox[{"{", 
        RowBox[{"datList", ",", "modList"}], "}"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6446632794800234`*^9, 3.644663518932966*^9}, 
   3.6446641213392005`*^9, {3.644664196761742*^9, 3.6446642148095465`*^9}, {
   3.6446647716352234`*^9, 3.644664790772137*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"GOF$EF", "[", 
    RowBox[{
    "data_", ",", "tEnd_", ",", "repl_", ",", "nrTr_", ",", "model_", ",", 
     "kBG_", ",", "parms_", ",", 
     RowBox[{"total_:", "\"\<Y\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"trix", ",", "retDat", ",", "val"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"retDat", "=", 
       RowBox[{"prepareDataAndModelExtracts", "[", 
        RowBox[{
        "data", ",", "tEnd", ",", "repl", ",", "nrTr", ",", "model", ",", 
         "kBG", ",", "parms"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"total", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", "0"}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"val", "+=", " ", 
             RowBox[{"calculateGOF$EF", "[", 
              RowBox[{
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], ";"}]}],
           "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"val", ",", 
              RowBox[{"calculateGOF$EF", "[", 
               RowBox[{
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "val", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"GOF$NRMSD", "[", 
    RowBox[{
    "data_", ",", "tEnd_", ",", "repl_", ",", "nrTr_", ",", "model_", ",", 
     "kBG_", ",", "parms_", ",", 
     RowBox[{"total_:", "\"\<Y\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"trix", ",", "retDat", ",", "val"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"retDat", "=", 
       RowBox[{"prepareDataAndModelExtracts", "[", 
        RowBox[{
        "data", ",", "tEnd", ",", "repl", ",", "nrTr", ",", "model", ",", 
         "kBG", ",", "parms"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"total", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", "0"}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"val", "+=", " ", 
             RowBox[{"calculateGOF$NRMSD", "[", 
              RowBox[{
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], ";"}]}],
           "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"val", ",", 
              RowBox[{"calculateGOF$NRMSD", "[", 
               RowBox[{
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "val", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.644664250525118*^9, 3.644664468098873*^9}, {
   3.644665450402094*^9, 3.6446654918042336`*^9}, {3.644668952434262*^9, 
   3.6446689740124197`*^9}, 3.6446750281757755`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GOF$NRMSD$MED", "[", 
    RowBox[{
    "data_", ",", "tEnd_", ",", "repl_", ",", "nrTr_", ",", "probData_", ",", 
     RowBox[{"total_:", "\"\<Y\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"trix", ",", "retDat", ",", "val"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"retDat", "=", 
       RowBox[{"prepareDataAndMedianExtracts", "[", 
        RowBox[{
        "data", ",", "tEnd", ",", "repl", ",", "nrTr", ",", "probData"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"total", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", "0"}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"val", "+=", " ", 
             RowBox[{"calculateGOF$NRMSD", "[", 
              RowBox[{
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], ";"}]}],
           "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"val", ",", 
              RowBox[{"calculateGOF$NRMSD", "[", 
               RowBox[{
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "val", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.644674994028361*^9, 3.64467502532549*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GOF$Chi2$MED", "[", 
    RowBox[{
    "data_", ",", "tEnd_", ",", "repl_", ",", "nrTr_", ",", "probData_", ",", 
     RowBox[{"total_:", "\"\<Y\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"trix", ",", "retDat", ",", "val"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"retDat", "=", 
       RowBox[{"prepareDataAndMedianExtracts", "[", 
        RowBox[{
        "data", ",", "tEnd", ",", "repl", ",", "nrTr", ",", "probData"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"total", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", "0"}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"val", "+=", " ", 
             RowBox[{"calculateGOF$ChiSq", "[", 
              RowBox[{
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], ";"}]}],
           "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"val", ",", 
              RowBox[{"calculateGOF$ChiSq", "[", 
               RowBox[{
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "val", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6446750405740147`*^9, 3.6446750483427916`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GOF$Chi2", "[", 
    RowBox[{
    "data_", ",", "tEnd_", ",", "repl_", ",", "nrTr_", ",", "model_", ",", 
     "kBG_", ",", "parms_", ",", 
     RowBox[{"total_:", "\"\<Y\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"trix", ",", "retDat", ",", "val"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"retDat", "=", 
       RowBox[{"prepareDataAndModelExtracts", "[", 
        RowBox[{
        "data", ",", "tEnd", ",", "repl", ",", "nrTr", ",", "model", ",", 
         "kBG", ",", "parms"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"total", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", "0"}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"val", "+=", " ", 
             RowBox[{"calculateGOF$ChiSq", "[", 
              RowBox[{
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
               RowBox[{"retDat", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], ";"}]}],
           "\[IndentingNewLine]", "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"val", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"trix", "=", "1"}], ",", 
           RowBox[{"trix", "\[LessEqual]", "nrTr"}], ",", 
           RowBox[{"trix", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"val", ",", 
              RowBox[{"calculateGOF$ChiSq", "[", 
               RowBox[{
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"2", ",", "trix"}], "]"}], "]"}], ",", 
                RowBox[{"retDat", "[", 
                 RowBox[{"[", 
                  RowBox[{"1", ",", "trix"}], "]"}], "]"}]}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "val", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6446645031223755`*^9, 3.6446645453025928`*^9}, {
  3.6446655033673897`*^9, 3.644665511326185*^9}, {3.6446660495089984`*^9, 
  3.644666054933541*^9}, {3.6446668418962293`*^9, 3.644666842063246*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "calculateGOF$EF", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"calculateGOF$EF", "[", 
    RowBox[{"model_", ",", "data_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"n", ",", "meanData", ",", "res"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "model", "]"}], "\[NotEqual]", 
         RowBox[{"Length", "[", "data", "]"}]}], ",", 
        RowBox[{"Return", "[", "NaN", "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"n", "=", 
          RowBox[{"Length", "[", "data", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"meanData", "=", 
          RowBox[{"Mean", "[", "data", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "-", "meanData"}], ")"}], 
               "^", "2"}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}], "\[Equal]", 
            "0"}], ",", "\[IndentingNewLine]", 
           RowBox[{"Return", "[", 
            RowBox[{"-", "Infinity"}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"res", "=", 
             RowBox[{"1", "-", 
              FractionBox[
               RowBox[{"Sum", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"model", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "-", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], ")"}], "^", "2"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}], 
               RowBox[{"Sum", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "-", "meanData"}], ")"}], 
                  "^", "2"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}]]}]}], ";",
             "\[IndentingNewLine]", 
            RowBox[{"Return", "[", 
             RowBox[{"N", "@", "res"}], "]"}]}]}], "]"}]}]}], 
       "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Clear", "[", "calculateGOF$NRMSD", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"calculateGOF$NRMSD", "[", 
    RowBox[{"model_", ",", "data_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"num", ",", "meanData", ",", "res"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "model", "]"}], "\[NotEqual]", 
         RowBox[{"Length", "[", "data", "]"}]}], ",", 
        RowBox[{"Return", "[", "NaN", "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"num", "=", 
          RowBox[{"Length", "[", "data", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"meanData", "=", 
          RowBox[{"Mean", "[", "data", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"res", "=", 
          RowBox[{
           FractionBox["1", "meanData"], "*", 
           RowBox[{"Sqrt", "[", 
            FractionBox[
             RowBox[{"Sum", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"model", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "-", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", "i", "]"}], "]"}]}], ")"}], "^", "2"}], ",", 
               RowBox[{"{", 
                RowBox[{"i", ",", "1", ",", "num"}], "}"}]}], "]"}], "num"], 
            "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"Return", "[", 
          RowBox[{"N", "@", "res"}], "]"}]}]}], "\[IndentingNewLine]", "]"}], 
      ";"}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6446577329494257`*^9, 3.64465777060019*^9}, {
  3.644657858155945*^9, 3.6446580449596233`*^9}, {3.6446580789690237`*^9, 
  3.6446581087610025`*^9}, {3.644658239351061*^9, 3.6446583446125855`*^9}, {
  3.6446586467407956`*^9, 3.6446586572408457`*^9}, {3.6446625399870815`*^9, 
  3.644662594032485*^9}, {3.644668989445963*^9, 3.6446689951965375`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "calculateGOF$ChiSq", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"calculateGOF$ChiSq", "[", 
    RowBox[{"model_", ",", "data_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"num", ",", "meanData", ",", "res"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "model", "]"}], "\[NotEqual]", 
         RowBox[{"Length", "[", "data", "]"}]}], ",", 
        RowBox[{"Return", "[", "NaN", "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"num", "=", 
          RowBox[{"Length", "[", "data", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"meanData", "=", 
          RowBox[{"Mean", "[", "data", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"res", "=", 
          RowBox[{"Sum", "[", 
           RowBox[{
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"model", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "-", 
                RowBox[{"data", "[", 
                 RowBox[{"[", "i", "]"}], "]"}]}], ")"}], "^", "2"}], 
             RowBox[{"model", "[", 
              RowBox[{"[", "i", "]"}], "]"}]], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "1", ",", "num"}], "}"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"Return", "[", 
          RowBox[{"N", "@", "res"}], "]"}]}]}], "\[IndentingNewLine]", "]"}], 
      ";"}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6446585860707293`*^9, 3.6446586608642073`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"modDat", "=", 
   RowBox[{"{", 
    RowBox[{
    "10", ",", "9", ",", "8", ",", "7", ",", "6", ",", "5", ",", "4", ",", 
     "3", ",", "2", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"obsDat", "=", 
   RowBox[{"{", 
    RowBox[{
    "10", ",", "9", ",", "8", ",", "7", ",", "6", ",", "5", ",", "4", ",", 
     "3", ",", "2.2", ",", "1"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"calculateGOF$EF", "[", 
  RowBox[{"modDat", ",", "obsDat"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"calculateGOF$NRMSD", "[", 
  RowBox[{"modDat", ",", "obsDat"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"calculateGOF$ChiSq", "[", 
  RowBox[{"modDat", ",", "obsDat"}], "]"}]}], "Input",
 CellChangeTimes->{{3.644658055564684*^9, 3.6446581201681433`*^9}, {
  3.6446583725423784`*^9, 3.6446584350766315`*^9}, {3.6446586671208334`*^9, 
  3.644658691920313*^9}, {3.6446626440504866`*^9, 3.6446627174288235`*^9}, {
  3.6484417565239415`*^9, 3.648441760827372*^9}}],

Cell[BoxData["0.9995070005915992`"], "Output",
 CellChangeTimes->{{3.644658409676091*^9, 3.644658435494673*^9}, {
   3.6446586698641076`*^9, 3.6446586921553364`*^9}, {3.644662656755757*^9, 
   3.644662717620843*^9}, 3.644664679128974*^9, 3.6446647208481455`*^9, {
   3.648441737276017*^9, 3.648441761889478*^9}, 3.648450622658589*^9, 
   3.648472256206727*^9, 3.6485242525097365`*^9, 3.6487776368908515`*^9, 
   3.648782607078575*^9, 3.648787316792289*^9, 3.653790163178257*^9}],

Cell[BoxData["0.011457527754233266`"], "Output",
 CellChangeTimes->{{3.644658409676091*^9, 3.644658435494673*^9}, {
   3.6446586698641076`*^9, 3.6446586921553364`*^9}, {3.644662656755757*^9, 
   3.644662717620843*^9}, 3.644664679128974*^9, 3.6446647208481455`*^9, {
   3.648441737276017*^9, 3.648441761889478*^9}, 3.648450622658589*^9, 
   3.648472256206727*^9, 3.6485242525097365`*^9, 3.6487776368908515`*^9, 
   3.648782607078575*^9, 3.648787316792289*^9, 3.6537901631792574`*^9}],

Cell[BoxData["0.020000000000000035`"], "Output",
 CellChangeTimes->{{3.644658409676091*^9, 3.644658435494673*^9}, {
   3.6446586698641076`*^9, 3.6446586921553364`*^9}, {3.644662656755757*^9, 
   3.644662717620843*^9}, 3.644664679128974*^9, 3.6446647208481455`*^9, {
   3.648441737276017*^9, 3.648441761889478*^9}, 3.648450622658589*^9, 
   3.648472256206727*^9, 3.6485242525097365`*^9, 3.6487776368908515`*^9, 
   3.648782607078575*^9, 3.648787316792289*^9, 3.6537901631802573`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{"*", " ", "Function", " ", "returns", " ", "chi"}], "-", 
     RowBox[{
     "squared", " ", "statistics", " ", "and", " ", "normalised", " ", "root",
       " ", "mean", " ", "square", " ", "deviations", " ", "for", " ", 
      "pulse", " ", "A", " ", "and", " ", "B"}]}], ",", " ", 
    RowBox[{"opt", " ", "and", " ", "median", " ", "prediction"}]}], "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"GOFPredictions", "[", 
     RowBox[{"yData_", ",", "predData_", ",", 
      RowBox[{"verbose_:", "0"}]}], "]"}], ":=", 
    RowBox[{"Block", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "timeD", ",", "num", ",", "sData", ",", "obsD", ",", "sumOpt", ",", 
        "sumOptN", ",", "sumMed", ",", "sumMedN", ",", "optEl", ",", "medEl", 
        ",", "sumOptElem", ",", "sumMedElem", ",", "retVals", ",", "Ninit"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"timeD", "=", 
        RowBox[{"Drop", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Transpose", "[", "yData", "]"}], "[", 
           RowBox[{"[", "1", "]"}], "]"}], ",", "1"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"num", "=", 
        RowBox[{
         RowBox[{"Length", "[", "timeD", "]"}], "*", "2"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"retVals", "=", 
        RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"szix", "=", "1"}], ",", 
         RowBox[{"szix", "\[LessEqual]", "2"}], ",", 
         RowBox[{"szix", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Ninit", "=", 
           RowBox[{"predData", "[", 
            RowBox[{"[", 
             RowBox[{"szix", ",", "1", ",", "2"}], "]"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"sData", "=", 
           RowBox[{"predData", "[", 
            RowBox[{"[", "szix", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"obsD", "=", 
           RowBox[{"Drop", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Transpose", "[", "yData", "]"}], "[", 
              RowBox[{"[", 
               RowBox[{"szix", "+", "1"}], "]"}], "]"}], ",", "1"}], "]"}]}], 
          ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"verbose", "\[Equal]", "1"}], ",", 
            RowBox[{
            "Print", "[", 
             "\"\<t  | obs | Sopt  | Smed |  chi-sq-opt  |  chi-sq-med\>\"", 
             "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"sumOpt", "=", "0"}], ";", 
          RowBox[{"sumMed", "=", "0"}], ";", 
          RowBox[{"sumOptN", "=", "0"}], ";", 
          RowBox[{"sumMedN", "=", "0"}], ";", "\[IndentingNewLine]", 
          RowBox[{"For", "[", 
           RowBox[{
            RowBox[{"i", "=", "1"}], ",", 
            RowBox[{"i", "\[LessEqual]", 
             RowBox[{"Length", "[", "timeD", "]"}]}], ",", 
            RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"elem", "=", 
              RowBox[{
               RowBox[{"Select", "[", 
                RowBox[{"sData", ",", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"#", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", 
                   RowBox[{"timeD", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "&"}]}], "]"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"sumOptElem", "=", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"obsD", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "-", 
                  RowBox[{"elem", "[", 
                   RowBox[{"[", "5", "]"}], "]"}]}], ")"}], "^", "2"}], "/", 
               RowBox[{"(", 
                RowBox[{"1", "+", 
                 RowBox[{"elem", "[", 
                  RowBox[{"[", "5", "]"}], "]"}]}], ")"}]}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"sumMedElem", "=", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"obsD", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "-", 
                  RowBox[{"elem", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], ")"}], "^", "2"}], "/", 
               RowBox[{"(", 
                RowBox[{"1", "+", 
                 RowBox[{"elem", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"optEl", "=", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"obsD", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "-", 
                 RowBox[{"elem", "[", 
                  RowBox[{"[", "5", "]"}], "]"}]}], ")"}], "^", "2"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"medEl", "=", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"obsD", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "-", 
                 RowBox[{"elem", "[", 
                  RowBox[{"[", "2", "]"}], "]"}]}], ")"}], "^", "2"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"verbose", "\[Equal]", "1"}], ",", 
               RowBox[{"Print", "[", 
                RowBox[{
                 RowBox[{"ToString", "[", 
                  RowBox[{"timeD", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}], "<>", 
                 "\"\< |  \>\"", "<>", 
                 RowBox[{"ToString", "[", 
                  RowBox[{"obsD", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}], "<>", 
                 "\"\< |  \>\"", "<>", 
                 RowBox[{"ToString", "[", 
                  RowBox[{"elem", "[", 
                   RowBox[{"[", "5", "]"}], "]"}], "]"}], "<>", 
                 "\"\< |  \>\"", "<>", 
                 RowBox[{"ToString", "[", 
                  RowBox[{"elem", "[", 
                   RowBox[{"[", "2", "]"}], "]"}], "]"}], "<>", 
                 "\"\< |  \>\"", "<>", 
                 RowBox[{"ToString", "[", "sumOptElem", "]"}], "<>", 
                 "\"\< |  \>\"", "<>", 
                 RowBox[{"ToString", "[", "sumMedElem", "]"}], "<>", 
                 "\"\< |  \>\""}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"sumOpt", "+=", "sumOptElem"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"sumMed", "+=", "sumMedElem"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"sumOptN", "+=", "optEl"}], ";", "\[IndentingNewLine]", 
             RowBox[{"sumMedN", "+=", "medEl"}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"verbose", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"Print", "[", 
              RowBox[{"\"\<chi-sq opt:\>\"", "<>", 
               RowBox[{"ToString", "[", "sumOpt", "]"}]}], "]"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"Print", "[", 
              RowBox[{"\"\<chi-sq Med:\>\"", "<>", 
               RowBox[{"ToString", "[", "sumMed", "]"}]}], "]"}]}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"sumOptN", "=", 
           RowBox[{
            RowBox[{"Sqrt", "[", 
             RowBox[{"sumOptN", "/", "num"}], "]"}], "/", "Ninit"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"sumMedN", "=", 
           RowBox[{
            RowBox[{"Sqrt", "[", 
             RowBox[{"sumMedN", "/", "num"}], "]"}], "/", "Ninit"}]}], ";", 
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"retVals", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"sumOpt", ",", "sumMed"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"sumOptN", ",", "sumMedN"}], "}"}]}], "}"}]}], "]"}], 
          ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "retVals", "]"}]}]}], "\[IndentingNewLine]", 
     "]"}]}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.643686189780929*^9, 3.643686218079379*^9}, 
   3.643686301383525*^9, {3.643686381645666*^9, 3.6436863879636774`*^9}, {
   3.643686427837347*^9, 3.6436869345106373`*^9}, {3.6436871247851715`*^9, 
   3.643687336914344*^9}, {3.643687439344124*^9, 3.6436875545971265`*^9}, {
   3.6436893683176937`*^9, 3.643689533177148*^9}, {3.6436895802239695`*^9, 
   3.6436896017852025`*^9}, {3.64368965524726*^9, 3.6436897382950106`*^9}, {
   3.643689825900021*^9, 3.643689888731615*^9}, {3.6436899270088043`*^9, 
   3.643690018444034*^9}, {3.6436900669028053`*^9, 3.6436900673348303`*^9}, {
   3.6436910721092997`*^9, 3.643691099154847*^9}, {3.6436914411454077`*^9, 
   3.6436914450366306`*^9}, {3.6445611970904517`*^9, 
   3.6445611992904515`*^9}, {3.6445637247824516`*^9, 3.6445637471754513`*^9}, 
   3.644580727401452*^9, {3.648442019084195*^9, 3.6484420771169977`*^9}, {
   3.6484730855846567`*^9, 3.6484730859446926`*^9}, {3.648473296379734*^9, 
   3.6484732983449306`*^9}, {3.64847333573967*^9, 3.648473387911886*^9}, {
   3.6484734540575004`*^9, 3.6484734550505996`*^9}, {3.648473608223915*^9, 
   3.6484736091080036`*^9}, {3.6484764836524296`*^9, 3.648476504450509*^9}}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{3.6446648982648854`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NotebookDirectory", "[", "]"}]], "Input",
 CellChangeTimes->{{3.644721698449361*^9, 3.644721703846961*^9}}],

Cell[BoxData["\<\"D:\\\\GUTS-PC\\\\\"\>"], "Output",
 CellChangeTimes->{{3.644721704034161*^9, 3.644721708495761*^9}, 
   3.6484417373580256`*^9, 3.648450622839607*^9, 3.648472256282735*^9, 
   3.648524252562742*^9, 3.6487776369532523`*^9, 3.6487826071435814`*^9, 
   3.6487873168390894`*^9, 3.6537901632172613`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "calculateGOF$ChiSq$Mod", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"calculateGOF$ChiSq$Mod", "[", " ", 
    RowBox[{"data_", ",", "model_", ",", "Ci0_", ",", 
     RowBox[{"p1_", "?", "NumericQ"}], ",", 
     RowBox[{"p2_", "?", "NumericQ"}], ",", 
     RowBox[{"p3_", "?", "NumericQ"}], ",", "Haz$BG_", ",", "tEnd_"}], "]"}], 
   ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "data$days", ",", "data$initAbundance", ",", "data$conc", ",", 
       "modelrun", ",", "N$of$treatments", ",", "N$of$days", ",", "ges$err", 
       ",", "step$err", ",", "data$currentAbundances", ",", "SOT", ",", 
       "Cw$temp", ",", "tr$i", ",", "di", ",", "numAcc"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"numAcc", "=", 
       RowBox[{"10", "^", 
        RowBox[{"-", "5"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"data$days", "=", 
       RowBox[{"data", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", " ", 
      RowBox[{"data$initAbundance", "=", 
       RowBox[{"data", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", " ", 
      RowBox[{"data$conc", "=", 
       RowBox[{"data", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"N$of$treatments", "=", 
       RowBox[{"Length", "[", "data$initAbundance", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"N$of$days", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"data", "[", 
         RowBox[{"[", "4", "]"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"ges$err", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"tr$i", "=", "1"}], ",", 
        RowBox[{"tr$i", "<=", "N$of$treatments"}], ",", 
        RowBox[{"tr$i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "tr$i"}], "]"}], "]"}]}], ";", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
           RowBox[{"data", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"-", "1"}], ",", 
              RowBox[{"tr$i", "+", "1"}]}], "]"}], "]"}]}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"data$currentAbundances", "=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{"4", "+", "tr$i", "-", "1"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{"modelrun", "=", 
            RowBox[{"Flatten", "[", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"model", "[", 
                 RowBox[{"Ci0", ",", 
                  RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"Haz$BG", ",", "p1", ",", "p2", ",", "p3"}], "}"}],
                   ",", "tEnd"}], "]"}], "[", 
                RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"modelrun", "=", 
            RowBox[{"Flatten", "[", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"model", "[", 
                 RowBox[{"Ci0", ",", "Cw$temp", ",", 
                  RowBox[{"{", 
                   RowBox[{"Haz$BG", ",", "p1", ",", "p2", ",", "p3"}], "}"}],
                   ",", "tEnd"}], "]"}], "[", 
                RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"di", "=", "1"}], ",", 
             RowBox[{"di", "\[LessEqual]", 
              RowBox[{"Length", "[", "data$days", "]"}]}], ",", 
             RowBox[{"di", "++"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"modelrun", "[", 
                 RowBox[{"[", "di", "]"}], "]"}], "<", "numAcc"}], ",", 
               RowBox[{
                RowBox[{"modelrun", "[", 
                 RowBox[{"[", "di", "]"}], "]"}], "=", 
                RowBox[{"numAcc", "/", "di"}]}]}], "]"}]}], "]"}], ";"}], 
          "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"step$err", "=", 
          RowBox[{"Sum", "[", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"NumberQ", "[", 
                RowBox[{"data$currentAbundances", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "]"}], " ", "&&", " ", 
               RowBox[{
                RowBox[{"data$currentAbundances", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], ">", "0"}]}], ",", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"data$currentAbundances", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "-", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"data$initAbundance", "[", 
                    RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
                    RowBox[{"modelrun", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], ")"}]}], ")"}], ")"}], 
                "^", "2"}], "/", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"data$initAbundance", "[", 
                  RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
                 RowBox[{"modelrun", "[", 
                  RowBox[{"[", "i", "]"}], "]"}]}], ")"}]}], ",", "0"}], 
             "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "1", ",", 
              RowBox[{"Length", "[", "data$currentAbundances", "]"}]}], 
             "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"ges$err", "+=", "step$err"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"ges$err", "=", "ges$err"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "ges$err", "]"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], "\[IndentingNewLine]"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.6445460877684517`*^9, 3.644546108530452*^9}, {
  3.644546177337452*^9, 3.6445461985704517`*^9}, {3.6445462319134517`*^9, 
  3.6445463391184516`*^9}, {3.6445464558314514`*^9, 3.644546479233452*^9}, {
  3.6445467536454515`*^9, 3.6445468652884517`*^9}, {3.6445472277894516`*^9, 
  3.644547229009452*^9}, {3.6445475086374516`*^9, 3.6445475294094515`*^9}, {
  3.6445476425524516`*^9, 3.6445476445724516`*^9}, {3.6445806996074524`*^9, 
  3.6445806998174524`*^9}, {3.6446647061136723`*^9, 3.6446647123943005`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "calculateGOF$NRMSD$Mod", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"calculateGOF$NRMSD$Mod", "[", " ", 
    RowBox[{"data_", ",", "model_", ",", "Ci0_", ",", 
     RowBox[{"p1_", "?", "NumericQ"}], ",", 
     RowBox[{"p2_", "?", "NumericQ"}], ",", 
     RowBox[{"p3_", "?", "NumericQ"}], ",", "Haz$BG_", ",", "tEnd_"}], "]"}], 
   ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "data$days", ",", "data$initAbundance", ",", "data$conc", ",", 
       "modelrun", ",", "N$of$treatments", ",", "N$of$days", ",", "ges$err", 
       ",", "step$err", ",", "data$currentAbundances", ",", "SOT", ",", 
       "Cw$temp", ",", "tr$i", ",", "di", ",", "numAcc", ",", "normFac"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"numAcc", "=", 
       RowBox[{"10", "^", 
        RowBox[{"-", "5"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"data$days", "=", 
       RowBox[{"data", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", " ", 
      RowBox[{"data$initAbundance", "=", 
       RowBox[{"data", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", " ", 
      RowBox[{"data$conc", "=", 
       RowBox[{"data", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"N$of$treatments", "=", 
       RowBox[{"Length", "[", "data$initAbundance", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"N$of$days", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"data", "[", 
         RowBox[{"[", "4", "]"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"ges$err", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"tr$i", "=", "1"}], ",", 
        RowBox[{"tr$i", "<=", "N$of$treatments"}], ",", 
        RowBox[{"tr$i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "tr$i"}], "]"}], "]"}]}], ";", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
           RowBox[{"data", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"-", "1"}], ",", 
              RowBox[{"tr$i", "+", "1"}]}], "]"}], "]"}]}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"data$currentAbundances", "=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{"4", "+", "tr$i", "-", "1"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{"modelrun", "=", 
            RowBox[{"Flatten", "[", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"model", "[", 
                 RowBox[{"Ci0", ",", 
                  RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"Haz$BG", ",", "p1", ",", "p2", ",", "p3"}], "}"}],
                   ",", "tEnd"}], "]"}], "[", 
                RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"modelrun", "=", 
            RowBox[{"Flatten", "[", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"model", "[", 
                 RowBox[{"Ci0", ",", "Cw$temp", ",", 
                  RowBox[{"{", 
                   RowBox[{"Haz$BG", ",", "p1", ",", "p2", ",", "p3"}], "}"}],
                   ",", "tEnd"}], "]"}], "[", 
                RowBox[{"[", "3", "]"}], "]"}], ")"}], "/.", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"t", "\[Rule]", "data$days"}], "}"}], "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"di", "=", "1"}], ",", 
           RowBox[{"di", "\[LessEqual]", 
            RowBox[{"Length", "[", "data$days", "]"}]}], ",", 
           RowBox[{"di", "++"}], ",", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"modelrun", "[", 
               RowBox[{"[", "di", "]"}], "]"}], "<", "numAcc"}], ",", 
             RowBox[{
              RowBox[{"modelrun", "[", 
               RowBox[{"[", "di", "]"}], "]"}], "=", 
              RowBox[{"numAcc", "/", "di"}]}]}], "]"}]}], "]"}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"step$err", "=", 
          RowBox[{"Sum", "[", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"NumberQ", "[", 
                RowBox[{"data$currentAbundances", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "]"}], " ", "&&", " ", 
               RowBox[{
                RowBox[{"data$currentAbundances", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], ">", "0"}]}], ",", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"data$currentAbundances", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "-", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"data$initAbundance", "[", 
                    RowBox[{"[", "tr$i", "]"}], "]"}], "*", 
                    RowBox[{"modelrun", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], ")"}]}], ")"}], ")"}], 
               "^", "2"}], ",", "0"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "1", ",", 
              RowBox[{"Length", "[", "data$currentAbundances", "]"}]}], 
             "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"ges$err", "+=", "step$err"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"normFac", "=", 
       RowBox[{"Max", "[", "data$initAbundance", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ges$err", "=", 
       RowBox[{
        RowBox[{"Sqrt", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "/", 
            RowBox[{"(", 
             RowBox[{"N$of$treatments", "*", "N$of$days"}], ")"}]}], ")"}], 
          "*", "ges$err"}], "]"}], "/", "normFac"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Return", "[", "ges$err", "]"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.644546879089452*^9, 3.6445469594194517`*^9}, 
   3.6445475408404517`*^9, {3.644547575594452*^9, 3.644547600938452*^9}, {
   3.6445476309914513`*^9, 3.6445476325814514`*^9}, {3.6446690125152693`*^9, 
   3.644669018194837*^9}, 3.6487273271953764`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"extractConfLimits", "[", 
    RowBox[{"LLprofData_", ",", "optTraceData_", ",", "confLim_"}], "]"}], ":=", 
   RowBox[{"Block", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "sel", ",", "shix", ",", "parix", ",", "lcl", ",", "ucl", ",", 
       "optVal"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
      "Print", "[", "\"\<---------------------------------------------\>\"", 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", "\"\<Extracted parameter information\>\"", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<Single parameter confidence intervals based on critical value :\>\
\"", "<>", 
        RowBox[{"ToString", "[", "confLim", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Optimal solution:\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"TraditionalForm", "[", 
          RowBox[{"optTraceData", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", "\"\<---------------------------------------------\>\"", 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", "\"\<Likelihood profiling:\>\"", "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Best error: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"LLprofData", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "1", ",", "1"}], "]"}], "]"}], "]"}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", 
       "\"\<Parameter: lower confidence limit  |  opt value  |  upper \
confidence limit:\>\"", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"parix", "=", "2"}], ";", "\[IndentingNewLine]", 
      RowBox[{"shix", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"sel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Drop", "[", 
          RowBox[{
           RowBox[{"LLprofData", "[", 
            RowBox[{"[", "shix", "]"}], "]"}], ",", "1"}], "]"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "5", "]"}], "]"}], ">", "confLim"}], "&"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"optVal", "=", 
       RowBox[{"LLprofData", "[", 
        RowBox[{"[", 
         RowBox[{"shix", ",", "1", ",", "parix"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "sel", "]"}], "==", "0"}], ",", 
        RowBox[{
         RowBox[{"lcl", "=", "\"\<-\>\""}], ";", 
         RowBox[{"ucl", "=", "\"\<-\>\""}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "sel", "]"}], "==", "1"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"If", " ", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"sel", "[", 
              RowBox[{"[", 
               RowBox[{"1", ",", "parix"}], "]"}], "]"}], "<", "optVal"}], 
            ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"lcl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"ucl", "=", "\"\<-\>\""}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"ucl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"lcl", "=", "\"\<-\>\""}]}]}], "]"}], 
          "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "sel", "]"}], "\[Equal]", "2"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"lcl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"ucl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"2", ",", "parix"}], "]"}], "]"}]}]}], ","}], 
           "\[IndentingNewLine]", "]"}]}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<par\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"parix", "-", "1"}], "]"}], "<>", " ", "\"\<: \>\"", "<>", 
        RowBox[{"ToString", "[", "lcl", "]"}], "<>", "\"\<| \>\"", "<>", 
        RowBox[{"ToString", "[", "optVal", "]"}], "<>", "\"\<| \>\"", "<>", 
        RowBox[{"ToString", "[", "ucl", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"parix", "=", "3"}], ";", "\[IndentingNewLine]", 
      RowBox[{"shix", "=", "2"}], ";", "\[IndentingNewLine]", 
      RowBox[{"sel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Drop", "[", 
          RowBox[{
           RowBox[{"LLprofData", "[", 
            RowBox[{"[", "shix", "]"}], "]"}], ",", "1"}], "]"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "5", "]"}], "]"}], ">", "confLim"}], "&"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"optVal", "=", 
       RowBox[{"LLprofData", "[", 
        RowBox[{"[", 
         RowBox[{"shix", ",", "1", ",", "parix"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "sel", "]"}], "==", "0"}], ",", 
        RowBox[{
         RowBox[{"lcl", "=", "\"\<-\>\""}], ";", 
         RowBox[{"ucl", "=", "\"\<-\>\""}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "sel", "]"}], "==", "1"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"If", " ", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"sel", "[", 
              RowBox[{"[", 
               RowBox[{"1", ",", "parix"}], "]"}], "]"}], "<", "optVal"}], 
            ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"lcl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"ucl", "=", "\"\<-\>\""}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"ucl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"lcl", "=", "\"\<-\>\""}]}]}], "]"}], 
          "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "sel", "]"}], "\[Equal]", "2"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"lcl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"ucl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"2", ",", "parix"}], "]"}], "]"}]}]}], ","}], 
           "\[IndentingNewLine]", "]"}]}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<par\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"parix", "-", "1"}], "]"}], "<>", " ", "\"\<: \>\"", "<>", 
        RowBox[{"ToString", "[", "lcl", "]"}], "<>", "\"\<| \>\"", "<>", 
        RowBox[{"ToString", "[", "optVal", "]"}], "<>", "\"\<| \>\"", "<>", 
        RowBox[{"ToString", "[", "ucl", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"parix", "=", "4"}], ";", "\[IndentingNewLine]", 
      RowBox[{"shix", "=", "3"}], ";", "\[IndentingNewLine]", 
      RowBox[{"sel", "=", 
       RowBox[{"Select", "[", 
        RowBox[{
         RowBox[{"Drop", "[", 
          RowBox[{
           RowBox[{"LLprofData", "[", 
            RowBox[{"[", "shix", "]"}], "]"}], ",", "1"}], "]"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "5", "]"}], "]"}], ">", "confLim"}], "&"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"optVal", "=", 
       RowBox[{"LLprofData", "[", 
        RowBox[{"[", 
         RowBox[{"shix", ",", "1", ",", "parix"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "sel", "]"}], "==", "0"}], ",", 
        RowBox[{
         RowBox[{"lcl", "=", "\"\<-\>\""}], ";", 
         RowBox[{"ucl", "=", "\"\<-\>\""}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "sel", "]"}], "==", "1"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"If", " ", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"sel", "[", 
              RowBox[{"[", 
               RowBox[{"1", ",", "parix"}], "]"}], "]"}], "<", "optVal"}], 
            ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"lcl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"ucl", "=", "\"\<-\>\""}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"ucl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"lcl", "=", "\"\<-\>\""}]}]}], "]"}], 
          "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "sel", "]"}], "\[Equal]", "2"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"lcl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "parix"}], "]"}], "]"}]}], ";", 
             RowBox[{"ucl", "=", 
              RowBox[{"sel", "[", 
               RowBox[{"[", 
                RowBox[{"2", ",", "parix"}], "]"}], "]"}]}]}], ","}], 
           "\[IndentingNewLine]", "]"}]}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<par\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"parix", "-", "1"}], "]"}], "<>", " ", "\"\<: \>\"", "<>", 
        RowBox[{"ToString", "[", "lcl", "]"}], "<>", "\"\<| \>\"", "<>", 
        RowBox[{"ToString", "[", "optVal", "]"}], "<>", "\"\<| \>\"", "<>", 
        RowBox[{"ToString", "[", "ucl", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", "\"\<---------------------------------------------\>\"", 
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6435145903039637`*^9, 3.6435148671006413`*^9}, {
   3.643514905407471*^9, 3.643514922634194*^9}, {3.643515222478175*^9, 
   3.6435152243953667`*^9}, {3.643515369780904*^9, 3.6435156669556184`*^9}, {
   3.6435157067615986`*^9, 3.6435157578567076`*^9}, {3.643515816153537*^9, 
   3.6435159818791075`*^9}, {3.643516064393358*^9, 3.643516085350454*^9}, {
   3.643516129276846*^9, 3.643516132966215*^9}, {3.643516514984413*^9, 
   3.6435165232312374`*^9}, {3.6435180165035496`*^9, 
   3.6435180188797874`*^9}, {3.6447427082063837`*^9, 
   3.6447428537783833`*^9}, {3.6447430068663836`*^9, 3.6447430070363836`*^9}, 
   3.6447599971324472`*^9, {3.6448046339253674`*^9, 3.644804634195367*^9}, {
   3.648727430611717*^9, 3.6487274354652023`*^9}, {3.648777340041207*^9, 
   3.6487773717103734`*^9}, {3.648777499737175*^9, 3.648777507880989*^9}, {
   3.648778035507162*^9, 3.6487780694219966`*^9}, {3.648778101870413*^9, 
   3.6487781035396338`*^9}, {3.648778154458687*^9, 3.6487781567207155`*^9}, {
   3.6487784493492675`*^9, 3.6487785001279182`*^9}, {3.6487792838470726`*^9, 
   3.648779310336212*^9}, {3.6487793618168726`*^9, 3.6487793639228992`*^9}, {
   3.648782871535018*^9, 3.648782893063171*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"loadParameterProfiles", "[", 
   RowBox[{
   "speciesName_", ",", "SRTFileName_", ",", "profileFileName_", ",", 
    "allProfileFileName_", ",", "profileFileName$CI_", ",", "SCIconfLim_", 
    ",", "JCIconfLim_"}], "]"}], ":=", 
  RowBox[{"Block", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "SRT", ",", "SRTSD", ",", "optErr", ",", "lprofDataAll", ",", 
      "lprofData", ",", "LRdata", ",", "LRAdatatmp", ",", "LRAdata", ",", 
      "SRTextSig", ",", "SRTextNS", ",", "LRTextSig", ",", "LRTextNS", ",", 
      "LRATextSig", ",", "LRATextNS", ",", "nonNS", ",", "paramSamples", ",", 
      "dataSets", ",", "lprofDataSCI", ",", "optPar"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Print", "[", 
      RowBox[{
      "\"\<Loading parameter profiles for \>\"", "<>", "speciesName"}], "]"}],
      ";", "\[IndentingNewLine]", 
     RowBox[{"SRT", "=", 
      RowBox[{
       RowBox[{"Import", "[", "SRTFileName", "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"optPar", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SRT", "[", 
         RowBox[{"[", 
          RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"p1", "\[Rule]", 
           RowBox[{"SRT", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "2"}], "]"}], "]"}]}], ",", 
          RowBox[{"p2", "\[Rule]", 
           RowBox[{"SRT", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "3"}], "]"}], "]"}]}], ",", 
          RowBox[{"p3", "\[Rule]", 
           RowBox[{"SRT", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "4"}], "]"}], "]"}]}]}], "}"}]}], "}"}]}], ";",
      "\[IndentingNewLine]", 
     RowBox[{"optErr", "=", 
      RowBox[{"SRT", "[", 
       RowBox[{"[", 
        RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SRTSD", "=", 
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Transpose", "[", "SRT", "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Transpose", "[", "SRT", "]"}], "[", 
          RowBox[{"[", "2", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Transpose", "[", "SRT", "]"}], "[", 
          RowBox[{"[", "3", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Transpose", "[", "SRT", "]"}], "[", 
          RowBox[{"[", "4", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Transpose", "[", "SRT", "]"}], "[", 
           RowBox[{"[", "1", "]"}], "]"}], "-", "optErr"}]}], "}"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"SRTextSig", "=", 
      RowBox[{"Select", "[", 
       RowBox[{"SRTSD", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "5", "]"}], "]"}], "<=", "JCIconfLim"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"SRTextNS", "=", 
      RowBox[{"Select", "[", 
       RowBox[{"SRTSD", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "5", "]"}], "]"}], ">", "JCIconfLim"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"lprofDataAll", "=", 
        RowBox[{"Import", "[", "allProfileFileName", "]"}]}], ";"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"TrueQ", "[", " ", 
         RowBox[{
          RowBox[{"Head", "[", "allProfileFileName", "]"}], "\[Equal]", 
          "List"}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"LRAdatatmp", "=", 
         RowBox[{"Import", "[", "allProfileFileName", "]"}]}], ";"}], 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"lprofDataAll", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", 
           RowBox[{"lprofDataAll", "[", 
            RowBox[{"[", "2", "]"}], "]"}], ",", 
           RowBox[{"lprofDataAll", "[", 
            RowBox[{"[", "3", "]"}], "]"}]}], "]"}], ";"}], "*)"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"dataSets", "=", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"Import", "[", 
            RowBox[{"allProfileFileName", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "allProfileFileName", "]"}]}], "}"}]}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"LRAdatatmp", "=", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"dataSets", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", 
           RowBox[{"dataSets", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"LRAdata", "=", 
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"Transpose", "[", "LRAdatatmp", "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Transpose", "[", "LRAdatatmp", "]"}], "[", 
          RowBox[{"[", "2", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Transpose", "[", "LRAdatatmp", "]"}], "[", 
          RowBox[{"[", "3", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Transpose", "[", "LRAdatatmp", "]"}], "[", 
          RowBox[{"[", "4", "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Transpose", "[", "LRAdatatmp", "]"}], "[", 
           RowBox[{"[", "1", "]"}], "]"}], "-", "optErr"}]}], "}"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"LRATextSig", "=", 
      RowBox[{"Select", "[", 
       RowBox[{"LRAdata", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "5", "]"}], "]"}], "<=", "JCIconfLim"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"LRATextNS", "=", 
      RowBox[{"Select", "[", 
       RowBox[{"LRAdata", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "5", "]"}], "]"}], ">", "JCIconfLim"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"TrueQ", "[", " ", 
         RowBox[{
          RowBox[{"Head", "[", "profileFileName", "]"}], "\[Equal]", "List"}],
          "]"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"lprofData", "=", 
        RowBox[{"Import", "[", "profileFileName", "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"dataSets", "=", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"Import", "[", 
            RowBox[{"profileFileName", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "allProfileFileName", "]"}]}], "}"}]}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"lprofData", "=", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"dataSets", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", 
           RowBox[{"dataSets", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"LRdata", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"lprofData", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", 
        RowBox[{"lprofData", "[", 
         RowBox[{"[", "2", "]"}], "]"}], ",", 
        RowBox[{"lprofData", "[", 
         RowBox[{"[", "3", "]"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"LRTextSig", "=", 
      RowBox[{"Select", "[", 
       RowBox[{"LRdata", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "5", "]"}], "]"}], "<=", "JCIconfLim"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"LRTextNS", "=", 
      RowBox[{"Select", "[", 
       RowBox[{"LRdata", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "5", "]"}], "]"}], ">", "JCIconfLim"}], "&"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Joint confidence region based on critical value :\>\"", "<>", 
       RowBox[{"ToString", "[", "JCIconfLim", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Selected \>\"", "<>", 
       RowBox[{"ToString", "@", 
        RowBox[{"Length", "[", "SRTextSig", "]"}]}], "<>", 
       "\"\< parameter sets out of \>\"", "<>", 
       RowBox[{"ToString", "@", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Length", "[", "SRTextNS", "]"}], "+", 
          RowBox[{"Length", "[", "SRTextSig", "]"}]}], ")"}]}], "<>", 
       "\"\< from the parameter optimisation routine\>\""}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Selected \>\"", "<>", 
       RowBox[{"ToString", "@", 
        RowBox[{"Length", "[", "LRTextSig", "]"}]}], "<>", 
       "\"\< parameter sets out of \>\"", "<>", 
       RowBox[{"ToString", "@", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Length", "[", "LRTextSig", "]"}], "+", 
          RowBox[{"Length", "[", "LRTextNS", "]"}]}], ")"}]}], "<>", 
       "\"\< from the likelihood profiling directly\>\""}], "]"}], ";", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<Selected \>\"", "<>", 
       RowBox[{"ToString", "@", 
        RowBox[{"Length", "[", "LRATextSig", "]"}]}], "<>", 
       "\"\< parameter sets out of \>\"", "<>", 
       RowBox[{"ToString", "@", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Length", "[", "LRATextSig", "]"}], "+", 
          RowBox[{"Length", "[", "LRATextNS", "]"}]}], ")"}]}], "<>", 
       "\"\< from the likelihood profiling.\>\""}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"nonNS", "=", 
      RowBox[{
       RowBox[{"Length", "[", "SRTextNS", "]"}], "+", 
       RowBox[{"Length", "[", "SRTextSig", "]"}], "+", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Length", "[", "LRTextSig", "]"}], "+", 
         RowBox[{"Length", "[", "LRTextNS", "]"}]}], ")"}], "+", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Length", "[", "LRATextSig", "]"}], "+", 
         RowBox[{"Length", "[", "LRATextNS", "]"}]}], ")"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"paramSamples", "=", 
      RowBox[{"Join", "[", 
       RowBox[{"SRTextSig", ",", 
        RowBox[{"LRTextSig", "[", 
         RowBox[{"[", 
          RowBox[{"1", ";;", "5"}], "]"}], "]"}], ",", "LRATextSig"}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<In total \>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"Length", "[", "paramSamples", "]"}], "]"}], "<>", 
       "\"\<  parameter sets of total \>\"", "<>", 
       RowBox[{"ToString", "[", "nonNS", "]"}], "<>", 
       "\"\< within confidence limit.\>\""}], "]"}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"lprofDataSCI", "=", 
      RowBox[{"Import", "[", "profileFileName$CI", "]"}]}], ";", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"extractConfLimits", "[", 
      RowBox[{"lprofDataSCI", ",", "SRT", ",", "SCIconfLim"}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Return", "[", 
      RowBox[{"{", 
       RowBox[{"optErr", ",", "optPar", ",", "paramSamples"}], "}"}], "]"}], 
     ";"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.64476001906479*^9, 3.6447600197157707`*^9}, {
   3.648727558723678*^9, 3.6487276093307266`*^9}, {3.6487278647684016`*^9, 
   3.6487279289788246`*^9}, {3.6487282646639285`*^9, 
   3.6487284827580833`*^9}, {3.648728645791385*^9, 3.648728645909397*^9}, {
   3.648728697373543*^9, 3.6487287177825837`*^9}, {3.6487778196471944`*^9, 
   3.648777871018653*^9}, {3.6487779837924986`*^9, 3.6487780162097144`*^9}, {
   3.648778112509749*^9, 3.648778114007368*^9}, {3.6487791337886267`*^9, 
   3.648779207889699*^9}, 3.64878226243124*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions vor visualisation", "Title",
 CellChangeTimes->{{3.6278883645182505`*^9, 3.627888369675766*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"MyMarkers", "=", 
    RowBox[{"Graphics", "/@", 
     RowBox[{"{", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Lighter", "[", 
            RowBox[{"Green", ",", "0"}], "]"}], ",", 
           RowBox[{"Disk", "[", "]"}]}], "}"}]}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Darker", "[", 
            RowBox[{"Green", ",", "0.8"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0.2", ",", "0.2"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.2", ",", "0.8"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.8", ",", "0.8"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.8", ",", "0.2"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
        "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Darker", "[", 
            RowBox[{"Brown", ",", "0.2"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0.5", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0.5"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.5", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0.5"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
        "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Lighter", "[", 
            RowBox[{"Orange", ",", "0"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.5", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
        "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Lighter", "[", 
            RowBox[{"Red", ",", "0.5"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0.5", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
        "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Darker", "[", 
            RowBox[{"Red", ",", "0.5"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}]}], 
        "}"}]}], "\[IndentingNewLine]", "}"}]}]}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"Funktion", ",", " ", 
    RowBox[{"die", " ", "Skalierte", " ", "Symbole", " ", "liefert"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"MarkersScaled", "[", 
     RowBox[{
      RowBox[{"size_:", "0.025"}], ",", 
      RowBox[{"markers_:", "MyMarkers"}]}], "]"}], ":=", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"markers", "[", 
         RowBox[{"[", "i", "]"}], "]"}], ",", "size"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", 
        RowBox[{"Length", "[", "markers", "]"}]}], "}"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"Farben", " ", "f\[UDoubleDot]r", " ", "Plots"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", "MyMarkers"}], "Input",
 CellChangeTimes->{{3.5944622958533363`*^9, 3.5944624048955173`*^9}, {
  3.594462456586551*^9, 3.5944627106616325`*^9}, {3.6278884764534426`*^9, 
  3.6278884807628736`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Opacity[0.8], 
     {RGBColor[0, 1, 0], DiskBox[{0, 0}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {RGBColor[0., 0.19999999999999996`, 0.], 
      PolygonBox[{{0.2, 0.2}, {0.2, 0.8}, {0.8, 0.8}, {0.8, 0.2}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {RGBColor[0.48, 0.32, 0.16], 
      PolygonBox[{{0.5, 0}, {0, 0.5}, {0.5, 1}, {1, 0.5}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {RGBColor[1, 0.5, 0], PolygonBox[{{0, 0}, {0.5, 1}, {1, 0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {RGBColor[1., 0.5, 0.5], PolygonBox[{{0.5, 0}, {0, 1}, {1, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {RGBColor[0.5, 0., 0.], 
      PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}]}}]}], "}"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwdzl1I02EYBfCJlWPtH6ZzE79SyrByU2eKs2G9w7GSsM0wln1gC0UUP6hZ
FmkWYmVsNrWYy1ZoCxdaYFKmCZtN2U06oVY4mRXb0L1DUrExR9bzdvHwuzic
w5OkqC0qC6HRaMVwY9/4PkMYRjcGhtaJDYUxG0ST4zTrBbjZw44hmvwrrn6Q
Tf0uMIL25aNniMmBkalPpDfN6FjbjpGTqnhHvFthshC/i6X+ALhY+KZplYmR
XDmT5qVgt94hnQvHSFebOPloJ0aZms4tZpBpaKRHRGBkkIY/JM4JXzEiwV/2
gh9ZkRgdYCtYMhZGba72qpugqnRWnxWF0c/DV73E5MqRujugp3jh/j3iujRK
DfoZaantYMdQa/oo2BOfPVHCxig6YbzoEmin0HI9OK+59bGNg5HSrORGxkIe
KtZXgy1e7Uui54mP0wCqukI6NeCRys0PevC4q8X2FDxJO+a2gtp59QoVh1G/
UbBtB8g4n+nJA/2KMIkIZCaKVBZw/+CFJUY8RjbDn5J8sEnYZCFeS+e+l4CC
7mdKGTgoiZWXgnXV6nPEU0bntCoBo9XLOXwN6BMJTxD3WFN4fWBG2eezXbsw
ki1MoWou/Luv70oNeP2gT0PM+FKiI27yfeNEzoPW51bQuXWvixiXR0v5CtJN
oYL/psyuEf29lhwHuBFU5i2AjyfGZtygzixPvMjDKDg8mdsM7j7Efa0Gy6Pp
qToQBd7aBsDRfHeYE5wPUNpF0BvsCoakYZT9t3e4EeQJl5KawXK5vuY2aBdX
5VICjP4BPUJIBg==
  "]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"MyMarkers", "=", 
   RowBox[{"Graphics", "/@", 
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Lighter", "[", 
               RowBox[{"Blue", ",", "0"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Disk", "[", "]"}]}], "}"}], "}"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Darker", "[", 
               RowBox[{"Red", ",", "0.2"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0.3", ",", "0.3"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.3", ",", "0.7"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.7", ",", "0.7"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.7", ",", "0.3"}], "}"}]}], "}"}], "]"}]}], "}"}], 
         "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Darker", "[", 
               RowBox[{"Green", ",", "0.2"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0.5", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "0.5"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.5", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0.5"}], "}"}]}], "}"}], "]"}]}], "}"}], 
         "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Lighter", "[", 
               RowBox[{"Orange", ",", "0"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.5", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
         "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Lighter", "[", 
               RowBox[{"Blue", ",", "0"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
         "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Lighter", "[", 
               RowBox[{"Blue", ",", "0"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0.5", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}], 
         "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Lighter", "[", 
               RowBox[{"Blue", ",", "0"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0.5", ",", "1"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.35", ",", "0.65"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.0", ",", "0.65"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.27", ",", "0.4"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.5", ",", "0.25"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.9", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.73", ",", "0.4"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"1", ",", "0.65"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.65", ",", "0.65"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.5", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}], 
         "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Lighter", "[", 
               RowBox[{"Blue", ",", "0"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Disk", "[", "]"}]}], "}"}], "}"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"EdgeForm", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Thickness", "[", "0.045", "]"}], ",", 
              RowBox[{"Lighter", "[", 
               RowBox[{"Blue", ",", "0"}], "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"FaceForm", "[", 
            RowBox[{"{", 
             RowBox[{"White", ",", 
              RowBox[{"Opacity", "[", "0", "]"}]}], "}"}], "]"}], ",", 
           RowBox[{"Polygon", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"0.3", ",", "0.3"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.3", ",", "0.7"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.7", ",", "0.7"}], "}"}], ",", 
              RowBox[{"{", 
               RowBox[{"0.7", ",", "0.3"}], "}"}]}], "}"}], "]"}]}], "}"}], 
         "}"}]}], "}"}]}], "\[IndentingNewLine]", "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MarkersScaled", "[", 
    RowBox[{
     RowBox[{"size_:", "0.025"}], ",", 
     RowBox[{"markers_:", "MyMarkers"}]}], "]"}], ":=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"markers", "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", "size"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", 
       RowBox[{"Length", "[", "markers", "]"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", "MyMarkers", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Farben", " ", "f\[UDoubleDot]r", " ", "Plots"}], " ", 
   "*)"}]}]}], "Input",
 CellChangeTimes->{{3.5994672158206353`*^9, 3.5994672484509616`*^9}, {
  3.599467762356101*^9, 3.599467953119008*^9}, {3.599467988159359*^9, 
  3.599467989899376*^9}, {3.5994683193426704`*^9, 3.599468329372771*^9}, {
  3.599468448603963*^9, 3.5994684510039873`*^9}, {3.599468582066298*^9, 
  3.5994685824163017`*^9}, {3.6018912321788373`*^9, 3.601891282649342*^9}, {
  3.601891390780423*^9, 3.6018915711222267`*^9}, {3.6018916016635323`*^9, 
  3.6018918362758784`*^9}, {3.6018918975864916`*^9, 3.6018920208977246`*^9}, {
  3.601892439881914*^9, 3.6018924455019703`*^9}, {3.6085589448716993`*^9, 
  3.608559141603667*^9}, {3.627742157734465*^9, 3.6277421619198837`*^9}, {
  3.627888492766074*^9, 3.6278884986126585`*^9}, {3.627899567806467*^9, 
  3.6278995771704035`*^9}, {3.6278997072104063`*^9, 3.627899844141098*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 0, 1], Thickness[0.045]}], FaceForm[{GrayLevel[1],
       Opacity[0]}], DiskBox[{0, 0}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0.8, 0., 0.], Thickness[0.045]}], FaceForm[{
      GrayLevel[1], Opacity[0]}], 
      PolygonBox[{{0.3, 0.3}, {0.3, 0.7}, {0.7, 0.7}, {0.7, 0.3}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0., 0.8, 0.], Thickness[0.045]}], FaceForm[{
      GrayLevel[1], Opacity[0]}], 
      PolygonBox[{{0.5, 0}, {0, 0.5}, {0.5, 1}, {1, 0.5}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0.5, 0], Thickness[0.045]}], FaceForm[{GrayLevel[
      1], Opacity[0]}], PolygonBox[{{0, 0}, {0.5, 1}, {1, 0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 0, 1], Thickness[0.045]}], FaceForm[{GrayLevel[1],
       Opacity[0]}], PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 0, 1], Thickness[0.045]}], FaceForm[{GrayLevel[1],
       Opacity[0]}], PolygonBox[{{0.5, 0}, {0, 1}, {1, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 0, 1], Thickness[0.045]}], FaceForm[{GrayLevel[1],
       Opacity[0]}], 
      PolygonBox[{{0.5, 1}, {0.35, 0.65}, {0., 0.65}, {0.27, 0.4}, {0.1, 0}, {
        0.5, 0.25}, {0.9, 0}, {0.73, 0.4}, {1, 0.65}, {0.65, 0.65}, {
        0.5, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[0, 0, 1], Thickness[0.045]}], FaceForm[{GrayLevel[1],
       Opacity[0]}], DiskBox[{0, 0}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[0, 0, 1], Thickness[0.045]}], FaceForm[{GrayLevel[1],
       Opacity[0]}], 
      PolygonBox[{{0.3, 0.3}, {0.3, 0.7}, {0.7, 0.7}, {0.7, 0.3}}]}}]}], 
  "}"}]], "Output",
 CellChangeTimes->CompressedData["
1:eJwVz1FMklEYgGEol0L+lYBxIZZ6kzYFZblAm3bUcmktcdXKbGNgba2hroK0
LTRXhhUChayY061ouSJxi1GUOFE0Yqngmm6ZlS1cccJV1FTaqO9cvHuu33RZ
Q/VJOo1Gq4I0x2sVj+Mx0nIpFVF4LNBMzKNWxifAmDPxVmQ9RjcPn35G3Dng
9hD1WsnyKlgxaVf/SsTIzQ8IQhRGfOV81btNGL2uzxgzJWFUyuqKc4MV9MsJ
LBZG9kBSF7Fs2cZkg3nS/Qv5bIwG6XUcCQejJbnhTAv4RPa2Jz8Zo35mc4hY
POJsvAbKhj7d6ABjB6qTO4n23GwdOFynyX0B+niikZrNGA2kDlWfBcXakiUl
GJtuG73OxShqVeWwUzDaxdjbowA/W8yPiN3BMLcJtDSsvW0Ao+00Vw94IXTV
3wueOlIZ9IKcPfqfFA+jCKNw3QYww7FjsQgUjSaUl4BNtjKtB1RS8m/MVIyM
d//VlIE/Jlo8xO9uvrMcFFTeOy8BVx7yjkrBjjHdCSKvb2FSuwWjyqlCoQFM
yyo+SGwv386/DzqqZmqNWzHaPedFihyMXmY+UNWD75+GDUTjoVoz0SQMu4jR
eY3FCw6qt30h+vrpmbPgmqE4MfGKfjpCnEkZF82Bv1eVRR9Ba6lrKgi+Ga5J
k/Phm3pV0Ar2XeTbOkGelJFtBrP2PfdbQd3iYvwH0P9n452voGPc9JcuwMis
ttgvgW3nQumtoG+2t74NZCkbCygxRv8BpcQ0pQ==
  "]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "MyMarkersCol", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MyMarkersCol", "[", 
    RowBox[{"col_", ",", 
     RowBox[{"lig_:", "0"}], ",", 
     RowBox[{"thick_:", "0.045"}], ",", 
     RowBox[{"filled_:", "False"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Graphics", "/@", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Disk", "[", "]"}]}], "}"}], "}"}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.3", ",", "0.3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.3", ",", "0.7"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.7", ",", "0.7"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.7", ",", "0.3"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.5"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.5"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"GeometricTransformation", "[", 
              RowBox[{
               RowBox[{"Polygon", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "1"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "1"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}], ",", 
               RowBox[{"RotationTransform", "[", 
                RowBox[{"90", "Degree"}], "]"}]}], "]"}]}], "}"}], "}"}]}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.35", ",", "0.65"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.0", ",", "0.65"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.27", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.1", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "0.25"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.9", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.73", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.65"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.65", ",", "0.65"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", "\[IndentingNewLine]", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "0.4"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", "\[IndentingNewLine]", 
             RowBox[{"GeometricTransformation", "[", 
              RowBox[{
               RowBox[{"Polygon", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "0.4"}], "}"}]}], "}"}], "]"}], ",", 
               RowBox[{"RotationTransform", "[", 
                RowBox[{"60", "Degree"}], "]"}]}], "]"}]}], "}"}], "}"}]}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "0.4"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"GeometricTransformation", "[", 
              RowBox[{
               RowBox[{"Polygon", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "1"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.6", ",", "1"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.6", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.6", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.6", ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "0.4"}], "}"}]}], "}"}], "]"}], ",", 
               RowBox[{"RotationTransform", "[", 
                RowBox[{"45", "Degree"}], "]"}]}], "]"}]}], "}"}], "}"}]}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.2"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.8"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.6"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"FaceForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}], ",", 
                RowBox[{"Opacity", "[", 
                 RowBox[{"If", "[", 
                  RowBox[{"filled", ",", "1", ",", "0"}], "]"}], "]"}]}], 
               "}"}], "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.8"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.2"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}]}], "\[IndentingNewLine]", "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MyMarkersCol", "[", 
   RowBox[{"Green", ",", "0", ",", "0.06", ",", "True"}], "]"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Farben", " ", "f\[UDoubleDot]r", " ", "Plots"}], " ", 
   "*)"}]}]}], "Input",
 CellChangeTimes->{{3.627899858791563*^9, 3.6278999379834814`*^9}, {
  3.6278999692596083`*^9, 3.6279000781284943`*^9}, {3.6279003542341022`*^9, 
  3.6279006114818244`*^9}, {3.627900801942868*^9, 3.6279008034470186`*^9}, {
  3.627900888279501*^9, 3.627901033970069*^9}, {3.6279011051361847`*^9, 
  3.6279011684505157`*^9}, {3.627901201366807*^9, 3.6279012646321325`*^9}, {
  3.6279013926989384`*^9, 3.627901551361803*^9}, {3.6279016178814545`*^9, 
  3.6279016454432096`*^9}, {3.6279701120714417`*^9, 3.627970176443879*^9}, {
  3.6279710114473705`*^9, 3.627971058215047*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], DiskBox[{0, 0}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      PolygonBox[{{0.3, 0.3}, {0.3, 0.7}, {0.7, 0.7}, {0.7, 0.3}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      PolygonBox[{{0.5, 0}, {0, 0.5}, {0.5, 1}, {1, 0.5}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], PolygonBox[{{0, 0}, {0.5, 1}, {1, 0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}], {{{0, -1}, {1, 0}}, {0, 
        0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}]}}],
    ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], PolygonBox[{{0.5, 0}, {0, 1}, {1, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      PolygonBox[{{0.5, 1}, {0.35, 0.65}, {0., 0.65}, {0.27, 0.4}, {0.1, 0}, {
        0.5, 0.25}, {0.9, 0}, {0.73, 0.4}, {1, 0.65}, {0.65, 0.65}, {
        0.5, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      PolygonBox[{{0, 0.4}, {0, 0.6}, {1, 0.6}, {1, 0.4}, {0.4, 0.4}}]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0.4}, {0, 0.6}, {1, 0.6}, {1, 0.4}, {0.4, 0.4}}], 
       NCache[{{{Rational[1, 2], Rational[-1, 2] 3^Rational[1, 2]}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}}, {0, 0}}, {{{
         0.5, -0.8660254037844386}, {0.8660254037844386, 0.5}}, {0, 0}}]]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      PolygonBox[{{0, 0.4}, {0, 0.6}, {0.4, 0.6}, {0.4, 1}, {0.6, 1}, {0.6, 
        0.6}, {1, 0.6}, {1, 0.4}, {0.6, 0.4}, {0.6, 0}, {0.4, 0}, {0.4, 
        0.4}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0.4}, {0, 0.6}, {0.4, 0.6}, {0.4, 1}, {0.6, 1}, {0.6, 
         0.6}, {1, 0.6}, {1, 0.4}, {0.6, 0.4}, {0.6, 0}, {0.4, 0}, {0.4, 
         0.4}}], NCache[{{{2^Rational[-1, 2], -2^Rational[-1, 2]}, {
          2^Rational[-1, 2], 2^Rational[-1, 2]}}, {0, 0}}, {{{
         0.7071067811865475, -0.7071067811865475}, {0.7071067811865475, 
         0.7071067811865475}}, {0, 0}}]]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      PolygonBox[{{0, 0.2}, {0, 0.4}, {1, 0.8}, {1, 0.6}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[0, 1, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      0, 1, 0], Opacity[1]}], 
      PolygonBox[{{0, 0.6}, {0, 0.8}, {1, 0.4}, {1, 0.2}}]}}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.6279004286175394`*^9, {3.6279004634780254`*^9, 3.627900522557933*^9}, {
   3.6279005673894157`*^9, 3.627900611958872*^9}, {3.6279009711117835`*^9, 
   3.6279010177144437`*^9}, 3.6279011269303637`*^9, {3.627901174684139*^9, 
   3.627901265214191*^9}, 3.6279014009807663`*^9, {3.6279014335390215`*^9, 
   3.6279014401596837`*^9}, {3.6279014774414115`*^9, 
   3.6279014834210095`*^9}, {3.6279015229499617`*^9, 3.6279015646971364`*^9}, 
   3.6279016508167477`*^9, 3.6279017294616113`*^9, 3.6279701052867637`*^9, {
   3.6279701461288476`*^9, 3.6279701769609303`*^9}, {3.627971032484474*^9, 
   3.627971059031129*^9}, 3.6301446566777067`*^9, 3.630144955381694*^9, 
   3.6301450621327615`*^9, 3.6301577546285315`*^9, 3.630158723788335*^9, 
   3.6306667417593822`*^9, 3.631174014653969*^9, 3.632714375138955*^9, 
   3.6331003292008247`*^9, 3.6331868527534885`*^9, 3.6333527722604303`*^9, 
   3.633353896637857*^9, 3.633354837592943*^9, 3.6339504760499086`*^9, 
   3.6345304673652363`*^9, 3.63462537266825*^9, 3.634998518749057*^9, 
   3.634999123878564*^9, 3.6351660859049716`*^9, 3.6351703991132493`*^9, 
   3.6351796906590295`*^9, 3.635181921613103*^9, 3.6352700203522987`*^9, 
   3.635598865862715*^9, 3.635647126034724*^9, 3.6356545142045393`*^9, 
   3.636221468166256*^9, 3.638073707788615*^9, 3.638258930279628*^9, 
   3.6382590289036274`*^9, 3.6382803835102863`*^9, 3.6383325683653517`*^9, 
   3.638359528161491*^9, 3.638361732250878*^9, 3.638445866478309*^9, 
   3.638587276624444*^9, 3.638587441930973*^9, 3.6386875377402425`*^9, 
   3.638694456359831*^9, 3.6389612582484303`*^9, 3.639105355188402*^9, 
   3.6392230002025185`*^9, 3.6392238440778975`*^9, 3.63922989754418*^9, 
   3.639249829164631*^9, 3.6392803209855127`*^9, 3.6392803645858727`*^9, 
   3.639897719176572*^9, 3.639904658094557*^9, 3.6399048900829706`*^9, 
   3.6399414893836317`*^9, 3.6404354307873526`*^9, 3.6476953743777695`*^9, 
   3.647697845239222*^9, 3.647698063490606*^9, 3.6476981309633245`*^9, 
   3.647698415474825*^9, 3.64788253261213*^9, 3.6478830413142285`*^9, 
   3.647912256139595*^9, 3.647912392214201*^9, 3.647913886658149*^9, 
   3.6479205822709055`*^9, 3.6479288215919523`*^9, 3.6479422699965186`*^9, 
   3.6481989149608383`*^9, 3.648258518324357*^9, 3.6482881215374813`*^9, 
   3.6483116483819914`*^9, 3.64834982984894*^9, 3.648450623338657*^9, 
   3.6484722565997667`*^9, 3.6485242528877745`*^9, 3.648777637280856*^9, 
   3.6487826074126086`*^9, 3.6487873170730925`*^9, 3.653790163613301*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"MarkersScaledCol", "[", 
   RowBox[{"col_", ",", 
    RowBox[{"lig_:", "0"}], ",", 
    RowBox[{"thick_:", "0.045"}], ",", 
    RowBox[{"filled_:", "False"}], ",", 
    RowBox[{"size_:", "0.025"}]}], "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"MyMarkersCol", "[", 
        RowBox[{"col", ",", "lig", ",", "thick", ",", "filled"}], "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], ",", "size"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"MyMarkersCol", "[", 
        RowBox[{"col", ",", "lig", ",", "thick", ",", "filled"}], "]"}], 
       "]"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6279001086605473`*^9, 3.627900163073988*^9}, 
   3.627900236569337*^9, {3.6279017106557307`*^9, 3.6279017256262274`*^9}, {
   3.627901810483712*^9, 3.627901815527217*^9}, {3.627970933799607*^9, 
   3.6279709763508615`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"MarkersScaledCol", "[", 
   RowBox[{"Green", ",", "0.5", ",", "0.05", ",", "False", ",", "0.09"}], 
   "]"}], "[", 
  RowBox[{"[", "2", "]"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MarkersScaledCol", "[", 
   RowBox[{"Green", ",", "0.5", ",", "0.05", ",", "True", ",", "0.09"}], 
   "]"}], "[", 
  RowBox[{"[", "2", "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.627900169864667*^9, 3.6279001703937197`*^9}, {
  3.6279002460192814`*^9, 3.6279002598276625`*^9}, {3.6279015918208485`*^9, 
  3.6279015936390305`*^9}, {3.6279017358432493`*^9, 3.6279017385645213`*^9}, {
  3.627970946821909*^9, 3.627970955355762*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0., 0.5, 0.], Thickness[0.05]}], FaceForm[{RGBColor[
      0., 0.5, 0.], Opacity[0]}], 
      PolygonBox[{{0.3, 0.3}, {0.3, 0.7}, {0.7, 0.7}, {0.7, 0.3}}]}}], ",", 
   "0.09`"}], "}"}]], "Output",
 CellChangeTimes->{
  3.6279001708697677`*^9, {3.627900238516532*^9, 3.6279002600666866`*^9}, 
   3.6279015942620926`*^9, 3.62790173885655*^9, {3.6279709418334103`*^9, 
   3.6279709777830048`*^9}, {3.627971036699896*^9, 3.62797106224745*^9}, 
   3.6301446568477087`*^9, 3.630144955471695*^9, 3.6301450622327623`*^9, 
   3.630157754728532*^9, 3.630158726663622*^9, 3.6306667418529825`*^9, 
   3.6311740147469783`*^9, 3.632714375357357*^9, 3.633100329294425*^9, 
   3.633186853000513*^9, 3.633352772375442*^9, 3.6333538967378674`*^9, 
   3.633354837700954*^9, 3.633950476236927*^9, 3.6345304675382533`*^9, 
   3.6346253727618513`*^9, 3.6349985188970714`*^9, 3.6349991239755735`*^9, 
   3.6351660859989805`*^9, 3.63517039921826*^9, 3.63517969076204*^9, 
   3.635181921718113*^9, 3.6352700204322987`*^9, 3.6355988660031147`*^9, 
   3.6356471261127243`*^9, 3.63565451436054*^9, 3.636221468244256*^9, 
   3.638073707868615*^9, 3.6382589303616276`*^9, 3.6382590289816275`*^9, 
   3.6382803835922937`*^9, 3.6383325684623423`*^9, 3.638359528263501*^9, 
   3.6383617323378863`*^9, 3.6384458665733185`*^9, 3.638587276731455*^9, 
   3.6385874420369835`*^9, 3.638687537841253*^9, 3.6386944564628305`*^9, 
   3.6389612583514404`*^9, 3.6391053552914124`*^9, 3.6392230003115296`*^9, 
   3.6392238441869087`*^9, 3.639229897652191*^9, 3.6392498292725773`*^9, 
   3.6392803210875235`*^9, 3.639280364684882*^9, 3.6398977192857723`*^9, 
   3.6399046582649574`*^9, 3.639904890227872*^9, 3.639941489461632*^9, 
   3.6404354308573527`*^9, 3.6476953744657745`*^9, 3.647697845317222*^9, 
   3.647698063584206*^9, 3.647698131056925*^9, 3.647698415568425*^9, 
   3.6478825343933077`*^9, 3.647883041414239*^9, 3.6479122562416053`*^9, 
   3.6479123922992096`*^9, 3.6479138867561584`*^9, 3.647920582372916*^9, 
   3.6479288217359667`*^9, 3.6479422701025295`*^9, 3.6481989151168423`*^9, 
   3.6482585184453692`*^9, 3.648288121637491*^9, 3.6483116484820013`*^9, 
   3.6483498300169563`*^9, 3.64845062346967*^9, 3.6484722567017765`*^9, 
   3.6485242529837837`*^9, 3.6487776374680586`*^9, 3.648782607610628*^9, 
   3.6487873171666937`*^9, 3.653790163712311*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[0., 0.5, 0.], Thickness[0.05]}], FaceForm[{RGBColor[
      0., 0.5, 0.], Opacity[1]}], 
      PolygonBox[{{0.3, 0.3}, {0.3, 0.7}, {0.7, 0.7}, {0.7, 0.3}}]}}], ",", 
   "0.09`"}], "}"}]], "Output",
 CellChangeTimes->{
  3.6279001708697677`*^9, {3.627900238516532*^9, 3.6279002600666866`*^9}, 
   3.6279015942620926`*^9, 3.62790173885655*^9, {3.6279709418334103`*^9, 
   3.6279709777830048`*^9}, {3.627971036699896*^9, 3.62797106224745*^9}, 
   3.6301446568477087`*^9, 3.630144955471695*^9, 3.6301450622327623`*^9, 
   3.630157754728532*^9, 3.630158726663622*^9, 3.6306667418529825`*^9, 
   3.6311740147469783`*^9, 3.632714375357357*^9, 3.633100329294425*^9, 
   3.633186853000513*^9, 3.633352772375442*^9, 3.6333538967378674`*^9, 
   3.633354837700954*^9, 3.633950476236927*^9, 3.6345304675382533`*^9, 
   3.6346253727618513`*^9, 3.6349985188970714`*^9, 3.6349991239755735`*^9, 
   3.6351660859989805`*^9, 3.63517039921826*^9, 3.63517969076204*^9, 
   3.635181921718113*^9, 3.6352700204322987`*^9, 3.6355988660031147`*^9, 
   3.6356471261127243`*^9, 3.63565451436054*^9, 3.636221468244256*^9, 
   3.638073707868615*^9, 3.6382589303616276`*^9, 3.6382590289816275`*^9, 
   3.6382803835922937`*^9, 3.6383325684623423`*^9, 3.638359528263501*^9, 
   3.6383617323378863`*^9, 3.6384458665733185`*^9, 3.638587276731455*^9, 
   3.6385874420369835`*^9, 3.638687537841253*^9, 3.6386944564628305`*^9, 
   3.6389612583514404`*^9, 3.6391053552914124`*^9, 3.6392230003115296`*^9, 
   3.6392238441869087`*^9, 3.639229897652191*^9, 3.6392498292725773`*^9, 
   3.6392803210875235`*^9, 3.639280364684882*^9, 3.6398977192857723`*^9, 
   3.6399046582649574`*^9, 3.639904890227872*^9, 3.639941489461632*^9, 
   3.6404354308573527`*^9, 3.6476953744657745`*^9, 3.647697845317222*^9, 
   3.647698063584206*^9, 3.647698131056925*^9, 3.647698415568425*^9, 
   3.6478825343933077`*^9, 3.647883041414239*^9, 3.6479122562416053`*^9, 
   3.6479123922992096`*^9, 3.6479138867561584`*^9, 3.647920582372916*^9, 
   3.6479288217359667`*^9, 3.6479422701025295`*^9, 3.6481989151168423`*^9, 
   3.6482585184453692`*^9, 3.648288121637491*^9, 3.6483116484820013`*^9, 
   3.6483498300169563`*^9, 3.64845062346967*^9, 3.6484722567017765`*^9, 
   3.6485242529837837`*^9, 3.6487776374680586`*^9, 3.648782607610628*^9, 
   3.6487873171666937`*^9, 3.6537901637953186`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MyMarkersCol", "[", "Red", "]"}]], "Input",
 CellChangeTimes->{{3.627900055163198*^9, 3.6279000882515063`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], DiskBox[{0, 0}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      PolygonBox[{{0.3, 0.3}, {0.3, 0.7}, {0.7, 0.7}, {0.7, 0.3}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      PolygonBox[{{0.5, 0}, {0, 0.5}, {0.5, 1}, {1, 0.5}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], PolygonBox[{{0, 0}, {0.5, 1}, {1, 0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}], {{{0, -1}, {1, 0}}, {0, 
        0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}]}}],
    ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], PolygonBox[{{0.5, 0}, {0, 1}, {1, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      PolygonBox[{{0.5, 1}, {0.35, 0.65}, {0., 0.65}, {0.27, 0.4}, {0.1, 0}, {
        0.5, 0.25}, {0.9, 0}, {0.73, 0.4}, {1, 0.65}, {0.65, 0.65}, {
        0.5, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      PolygonBox[{{0, 0.4}, {0, 0.6}, {1, 0.6}, {1, 0.4}, {0.4, 0.4}}]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0.4}, {0, 0.6}, {1, 0.6}, {1, 0.4}, {0.4, 0.4}}], 
       NCache[{{{Rational[1, 2], Rational[-1, 2] 3^Rational[1, 2]}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}}, {0, 0}}, {{{
         0.5, -0.8660254037844386}, {0.8660254037844386, 0.5}}, {0, 0}}]]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      PolygonBox[{{0, 0.4}, {0, 0.6}, {0.4, 0.6}, {0.4, 1}, {0.6, 1}, {0.6, 
        0.6}, {1, 0.6}, {1, 0.4}, {0.6, 0.4}, {0.6, 0}, {0.4, 0}, {0.4, 
        0.4}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0.4}, {0, 0.6}, {0.4, 0.6}, {0.4, 1}, {0.6, 1}, {0.6, 
         0.6}, {1, 0.6}, {1, 0.4}, {0.6, 0.4}, {0.6, 0}, {0.4, 0}, {0.4, 
         0.4}}], NCache[{{{2^Rational[-1, 2], -2^Rational[-1, 2]}, {
          2^Rational[-1, 2], 2^Rational[-1, 2]}}, {0, 0}}, {{{
         0.7071067811865475, -0.7071067811865475}, {0.7071067811865475, 
         0.7071067811865475}}, {0, 0}}]]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      PolygonBox[{{0, 0.2}, {0, 0.4}, {1, 0.8}, {1, 0.6}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.045]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0]}], 
      PolygonBox[{{0, 0.6}, {0, 0.8}, {1, 0.4}, {1, 0.2}}]}}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.6278999616778507`*^9, {3.627900023451027*^9, 3.6279000886515465`*^9}, 
   3.6301446569577093`*^9, 3.630144955581696*^9, 3.630145062342764*^9, 
   3.630157754828533*^9, 3.630158728552811*^9, 3.6306667420089827`*^9, 
   3.631174014915995*^9, 3.6327143754665575`*^9, 3.6331003295128264`*^9, 
   3.6331868531305265`*^9, 3.633352772482453*^9, 3.6333538969668903`*^9, 
   3.6333548378319674`*^9, 3.6339504763619394`*^9, 3.634530467653265*^9, 
   3.634625372980254*^9, 3.634998519024084*^9, 3.6349991240905848`*^9, 
   3.635166086106992*^9, 3.635170399332271*^9, 3.635179690874051*^9, 
   3.6351819218491263`*^9, 3.635270020512299*^9, 3.6355988662683153`*^9, 
   3.635647126206324*^9, 3.63565451446974*^9, 3.6362214683378563`*^9, 
   3.6380737079686155`*^9, 3.638258930444628*^9, 3.638259029062628*^9, 
   3.638280383673302*^9, 3.6383325686513233`*^9, 3.6383595283835135`*^9, 
   3.638361732440897*^9, 3.63844586668533*^9, 3.6385872768454657`*^9, 
   3.6385874421469946`*^9, 3.638687537953264*^9, 3.6386944565738306`*^9, 
   3.638961258586464*^9, 3.639105355402423*^9, 3.6392230004265413`*^9, 
   3.639223844308921*^9, 3.639229897772203*^9, 3.639249829398514*^9, 
   3.6392803212025347`*^9, 3.6392803647978935`*^9, 3.639897719488572*^9, 
   3.6399046585008583`*^9, 3.639904890350273*^9, 3.639941489570832*^9, 
   3.640435430937353*^9, 3.6476953745607805`*^9, 3.6476978454108224`*^9, 
   3.647698063677806*^9, 3.647698131150525*^9, 3.6476984156464252`*^9, 
   3.6478825344593143`*^9, 3.64788304152925*^9, 3.647912256357617*^9, 
   3.6479123925072303`*^9, 3.6479138868771706`*^9, 3.647920582492928*^9, 
   3.647928821835977*^9, 3.6479422703245516`*^9, 3.648198915241645*^9, 
   3.6482585186433887`*^9, 3.648288121833511*^9, 3.648311648589012*^9, 
   3.6483498300579605`*^9, 3.6484506236006827`*^9, 3.648472256814788*^9, 
   3.648524253096795*^9, 3.64877763757726*^9, 3.648782607720639*^9, 
   3.6487873173694963`*^9, 3.653790163827322*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"MarkersScaled", "[", 
   RowBox[{
    RowBox[{"size_:", "0.025"}], ",", 
    RowBox[{"markers_:", 
     RowBox[{"MyMarkersCol", "[", "Red", "]"}]}]}], "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"markers", "[", 
       RowBox[{"[", "i", "]"}], "]"}], ",", "size"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"Length", "[", "markers", "]"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6278999478624687`*^9, 3.627899952303913*^9}}],

Cell[BoxData["\[IndentingNewLine]"], "Input",
 CellChangeTimes->{3.6306785063010235`*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "MyMarkersCol2", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MyMarkersCol2", "[", 
    RowBox[{"col_", ",", 
     RowBox[{"lig_:", "0"}], ",", 
     RowBox[{"thick_:", "0.045"}], ",", 
     RowBox[{"filled_:", "1"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Graphics", "/@", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Disk", "[", "]"}]}], "}"}], "}"}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.3", ",", "0.3"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.3", ",", "0.7"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.7", ",", "0.7"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.7", ",", "0.3"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.5"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.5"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"GeometricTransformation", "[", 
              RowBox[{
               RowBox[{"Polygon", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "1"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "1"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}], ",", 
               RowBox[{"RotationTransform", "[", 
                RowBox[{"90", "Degree"}], "]"}]}], "]"}]}], "}"}], "}"}]}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.8", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.35", ",", "0.65"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.0", ",", "0.65"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.27", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.1", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "0.25"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.9", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.73", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.65"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.65", ",", "0.65"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.5", ",", "1"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", "\[IndentingNewLine]", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "0.4"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", "\[IndentingNewLine]", 
             RowBox[{"GeometricTransformation", "[", 
              RowBox[{
               RowBox[{"Polygon", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "0.4"}], "}"}]}], "}"}], "]"}], ",", 
               RowBox[{"RotationTransform", "[", 
                RowBox[{"60", "Degree"}], "]"}]}], "]"}]}], "}"}], "}"}]}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.4", ",", "0.4"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"GeometricTransformation", "[", 
              RowBox[{
               RowBox[{"Polygon", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "1"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.6", ",", "1"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.6", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0.6"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.6", ",", "0.4"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.6", ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"0.4", ",", "0.4"}], "}"}]}], "}"}], "]"}], ",", 
               RowBox[{"RotationTransform", "[", 
                RowBox[{"45", "Degree"}], "]"}]}], "]"}]}], "}"}], "}"}]}], 
         "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.2"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.8"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.6"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Opacity", "[", "0.66", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"EdgeForm", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "thick", "]"}], ",", 
                RowBox[{"Darker", "[", 
                 RowBox[{"col", ",", "lig"}], "]"}]}], "}"}], "]"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"filled", "\[Equal]", "0"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", "White", "}"}], "]"}], ",", 
               RowBox[{"FaceForm", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Darker", "[", 
                   RowBox[{"col", ",", "lig"}], "]"}], ",", 
                  RowBox[{"Opacity", "[", "filled", "]"}]}], "}"}], "]"}]}], 
              "]"}], ",", 
             RowBox[{"Polygon", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.6"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "0.8"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.4"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"1", ",", "0.2"}], "}"}]}], "}"}], "]"}]}], "}"}], 
           "}"}]}], "}"}]}], "\[IndentingNewLine]", "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MyMarkersCol2", "[", 
   RowBox[{"Red", ",", "0", ",", "0.06", ",", "0.0"}], "]"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{"MyMarkersCol2", "[", 
  RowBox[{"Red", ",", "0", ",", "0.06", ",", "0.21"}], 
  "]"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.6306785109964933`*^9, 3.6306785531327066`*^9}, {
   3.633332116856982*^9, 3.633332117229019*^9}, 3.653796491314518*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      DiskBox[{0, 0}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0.3, 0.3}, {0.3, 0.7}, {0.7, 0.7}, {0.7, 0.3}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0.5, 0}, {0, 0.5}, {0.5, 1}, {1, 0.5}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0, 0}, {0.5, 1}, {1, 0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}], {{{0, -1}, {1, 0}}, {0, 
        0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0.5, 0}, {0, 1}, {1, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0.5, 1}, {0.35, 0.65}, {0., 0.65}, {0.27, 0.4}, {0.1, 0}, {
        0.5, 0.25}, {0.9, 0}, {0.73, 0.4}, {1, 0.65}, {0.65, 0.65}, {
        0.5, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0, 0.4}, {0, 0.6}, {1, 0.6}, {1, 0.4}, {0.4, 0.4}}]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0.4}, {0, 0.6}, {1, 0.6}, {1, 0.4}, {0.4, 0.4}}], 
       NCache[{{{Rational[1, 2], Rational[-1, 2] 3^Rational[1, 2]}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}}, {0, 0}}, {{{
         0.5, -0.8660254037844386}, {0.8660254037844386, 0.5}}, {0, 0}}]]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0, 0.4}, {0, 0.6}, {0.4, 0.6}, {0.4, 1}, {0.6, 1}, {0.6, 
        0.6}, {1, 0.6}, {1, 0.4}, {0.6, 0.4}, {0.6, 0}, {0.4, 0}, {0.4, 
        0.4}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0.4}, {0, 0.6}, {0.4, 0.6}, {0.4, 1}, {0.6, 1}, {0.6, 
         0.6}, {1, 0.6}, {1, 0.4}, {0.6, 0.4}, {0.6, 0}, {0.4, 0}, {0.4, 
         0.4}}], NCache[{{{2^Rational[-1, 2], -2^Rational[-1, 2]}, {
          2^Rational[-1, 2], 2^Rational[-1, 2]}}, {0, 0}}, {{{
         0.7071067811865475, -0.7071067811865475}, {0.7071067811865475, 
         0.7071067811865475}}, {0, 0}}]]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0, 0.2}, {0, 0.4}, {1, 0.8}, {1, 0.6}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[GrayLevel[1]], 
      PolygonBox[{{0, 0.6}, {0, 0.8}, {1, 0.4}, {1, 0.2}}]}}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.630678547109104*^9, 3.6306785538747807`*^9}, 
   3.631174015026006*^9, 3.632714375591358*^9, 3.6331003296532273`*^9, 
   3.633186853257539*^9, 3.633332117576054*^9, 3.6333527726004643`*^9, 
   3.6333538970608993`*^9, 3.6333548379389777`*^9, 3.6339504764509487`*^9, 
   3.634530467776277*^9, 3.6346253730894556`*^9, 3.6349985191580973`*^9, 
   3.6349991242095966`*^9, 3.635166086232004*^9, 3.6351703994392815`*^9, 
   3.6351796909660606`*^9, 3.635181921964138*^9, 3.635270020622299*^9, 
   3.635598866533516*^9, 3.6356471262999244`*^9, 3.6356545146569405`*^9, 
   3.6362214684470563`*^9, 3.638073708068616*^9, 3.638258930527628*^9, 
   3.6382590291566277`*^9, 3.6382803837663116`*^9, 3.6383325688173065`*^9, 
   3.6383595284785223`*^9, 3.638361732509904*^9, 3.638445866872349*^9, 
   3.638587277094491*^9, 3.638587442334013*^9, 3.6386875380562744`*^9, 
   3.6386944566708307`*^9, 3.638961258735479*^9, 3.639105355498433*^9, 
   3.63922300051155*^9, 3.6392238443989296`*^9, 3.639229897862212*^9, 
   3.639249830981722*^9, 3.6392803213115454`*^9, 3.6392803648929033`*^9, 
   3.6398977196445723`*^9, 3.6399046586051593`*^9, 3.639904890493874*^9, 
   3.6399414896488323`*^9, 3.640435431028353*^9, 3.647695374675787*^9, 
   3.6476978454732227`*^9, 3.6476980637558064`*^9, 3.647698131228525*^9, 
   3.6476984157244253`*^9, 3.6478825345463233`*^9, 3.6478830416152587`*^9, 
   3.647912256441625*^9, 3.647912392577237*^9, 3.647913887064189*^9, 
   3.6479205826819468`*^9, 3.647928821903984*^9, 3.6479422704135604`*^9, 
   3.648198915350848*^9, 3.6482585187473993`*^9, 3.64828812193052*^9, 
   3.6483116486830215`*^9, 3.648349830216976*^9, 3.6484506237676997`*^9, 
   3.6484722569488015`*^9, 3.648524253213807*^9, 3.6487776376708612`*^9, 
   3.648782607815649*^9, 3.648787317463097*^9, 3.653790163935333*^9, {
   3.6537964921486015`*^9, 3.653796495160903*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], DiskBox[{0, 0}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      PolygonBox[{{0.3, 0.3}, {0.3, 0.7}, {0.7, 0.7}, {0.7, 0.3}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      PolygonBox[{{0.5, 0}, {0, 0.5}, {0.5, 1}, {1, 0.5}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], PolygonBox[{{0, 0}, {0.5, 1}, {1, 0}}]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}], {{{0, -1}, {1, 0}}, {0, 
        0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      PolygonBox[{{0, 0}, {1, 1}, {0, 1}, {1, 0}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], PolygonBox[{{0.5, 0}, {0, 1}, {1, 1}}]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.8], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      PolygonBox[{{0.5, 1}, {0.35, 0.65}, {0., 0.65}, {0.27, 0.4}, {0.1, 0}, {
        0.5, 0.25}, {0.9, 0}, {0.73, 0.4}, {1, 0.65}, {0.65, 0.65}, {
        0.5, 1}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      PolygonBox[{{0, 0.4}, {0, 0.6}, {1, 0.6}, {1, 0.4}, {0.4, 0.4}}]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0.4}, {0, 0.6}, {1, 0.6}, {1, 0.4}, {0.4, 0.4}}], 
       NCache[{{{Rational[1, 2], Rational[-1, 2] 3^Rational[1, 2]}, {
          Rational[1, 2] 3^Rational[1, 2], Rational[1, 2]}}, {0, 0}}, {{{
         0.5, -0.8660254037844386}, {0.8660254037844386, 0.5}}, {0, 0}}]]}}], 
   ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      PolygonBox[{{0, 0.4}, {0, 0.6}, {0.4, 0.6}, {0.4, 1}, {0.6, 1}, {0.6, 
        0.6}, {1, 0.6}, {1, 0.4}, {0.6, 0.4}, {0.6, 0}, {0.4, 0}, {0.4, 
        0.4}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      GeometricTransformationBox[
       PolygonBox[{{0, 0.4}, {0, 0.6}, {0.4, 0.6}, {0.4, 1}, {0.6, 1}, {0.6, 
         0.6}, {1, 0.6}, {1, 0.4}, {0.6, 0.4}, {0.6, 0}, {0.4, 0}, {0.4, 
         0.4}}], NCache[{{{2^Rational[-1, 2], -2^Rational[-1, 2]}, {
          2^Rational[-1, 2], 2^Rational[-1, 2]}}, {0, 0}}, {{{
         0.7071067811865475, -0.7071067811865475}, {0.7071067811865475, 
         0.7071067811865475}}, {0, 0}}]]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      PolygonBox[{{0, 0.2}, {0, 0.4}, {1, 0.8}, {1, 0.6}}]}}], ",", 
   GraphicsBox[
    {Opacity[0.66], 
     {EdgeForm[{RGBColor[1, 0, 0], Thickness[0.06]}], FaceForm[{RGBColor[
      1, 0, 0], Opacity[0.21]}], 
      PolygonBox[{{0, 0.6}, {0, 0.8}, {1, 0.4}, {1, 0.2}}]}}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.630678547109104*^9, 3.6306785538747807`*^9}, 
   3.631174015026006*^9, 3.632714375591358*^9, 3.6331003296532273`*^9, 
   3.633186853257539*^9, 3.633332117576054*^9, 3.6333527726004643`*^9, 
   3.6333538970608993`*^9, 3.6333548379389777`*^9, 3.6339504764509487`*^9, 
   3.634530467776277*^9, 3.6346253730894556`*^9, 3.6349985191580973`*^9, 
   3.6349991242095966`*^9, 3.635166086232004*^9, 3.6351703994392815`*^9, 
   3.6351796909660606`*^9, 3.635181921964138*^9, 3.635270020622299*^9, 
   3.635598866533516*^9, 3.6356471262999244`*^9, 3.6356545146569405`*^9, 
   3.6362214684470563`*^9, 3.638073708068616*^9, 3.638258930527628*^9, 
   3.6382590291566277`*^9, 3.6382803837663116`*^9, 3.6383325688173065`*^9, 
   3.6383595284785223`*^9, 3.638361732509904*^9, 3.638445866872349*^9, 
   3.638587277094491*^9, 3.638587442334013*^9, 3.6386875380562744`*^9, 
   3.6386944566708307`*^9, 3.638961258735479*^9, 3.639105355498433*^9, 
   3.63922300051155*^9, 3.6392238443989296`*^9, 3.639229897862212*^9, 
   3.639249830981722*^9, 3.6392803213115454`*^9, 3.6392803648929033`*^9, 
   3.6398977196445723`*^9, 3.6399046586051593`*^9, 3.639904890493874*^9, 
   3.6399414896488323`*^9, 3.640435431028353*^9, 3.647695374675787*^9, 
   3.6476978454732227`*^9, 3.6476980637558064`*^9, 3.647698131228525*^9, 
   3.6476984157244253`*^9, 3.6478825345463233`*^9, 3.6478830416152587`*^9, 
   3.647912256441625*^9, 3.647912392577237*^9, 3.647913887064189*^9, 
   3.6479205826819468`*^9, 3.647928821903984*^9, 3.6479422704135604`*^9, 
   3.648198915350848*^9, 3.6482585187473993`*^9, 3.64828812193052*^9, 
   3.6483116486830215`*^9, 3.648349830216976*^9, 3.6484506237676997`*^9, 
   3.6484722569488015`*^9, 3.648524253213807*^9, 3.6487776376708612`*^9, 
   3.648782607815649*^9, 3.648787317463097*^9, 3.653790163935333*^9, {
   3.6537964921486015`*^9, 3.6537964951689034`*^9}}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"MarkersScaledCol2", "[", 
   RowBox[{"col_", ",", 
    RowBox[{"lig_:", "0"}], ",", 
    RowBox[{"thick_:", "0.045"}], ",", 
    RowBox[{"filled_:", "1"}], ",", 
    RowBox[{"size_:", "0.025"}]}], "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"MyMarkersCol2", "[", 
        RowBox[{"col", ",", "lig", ",", "thick", ",", "filled"}], "]"}], "[", 
       RowBox[{"[", "i", "]"}], "]"}], ",", "size"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"Length", "[", 
       RowBox[{"MyMarkersCol", "[", 
        RowBox[{"col", ",", "lig", ",", "thick", ",", "filled"}], "]"}], 
       "]"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6306785720685997`*^9, 3.6306785779491878`*^9}}],

Cell[BoxData["\[IndentingNewLine]"], "Input",
 CellChangeTimes->{3.6306785071401076`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenarios", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "opt$Haz$BG_", ",", "optRes_", ",", 
     "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p1loc", ",", "p2loc", ",", "p3loc"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"iz", "=", "1"}], ",", 
        RowBox[{"iz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"iz", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "iz"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{"s1", "=", 
            RowBox[{"model", "[", 
             RowBox[{"0", ",", 
              RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
              RowBox[{"{", 
               RowBox[{
               "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
               "}"}], ",", "tEnd"}], "]"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"s1", "=", 
            RowBox[{"model", "[", 
             RowBox[{"0", ",", "Cw$temp", ",", 
              RowBox[{"{", 
               RowBox[{
               "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
               "}"}], ",", "tEnd"}], "]"}]}]}], "\[IndentingNewLine]", "]"}], 
         ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"ga", ",", 
           RowBox[{"Show", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Plot", "[", 
               RowBox[{
                RowBox[{"s1", "[", 
                 RowBox[{"[", "3", "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                  "}"}]}], ",", 
                RowBox[{"BaseStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                   RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                RowBox[{"FrameLabel", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                  "\"\<time (days)\>\"", ",", 
                   "\"\<Survival over time (%)\>\"", ",", 
                   RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"Cw$temp", "[", "t", "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                RowBox[{"PlotRange", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"Directive", "[", 
                  RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
              RowBox[{"ListPlot", "[", 
               RowBox[{
                RowBox[{"Transpose", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"data", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", "+", "iz"}], "]"}], "]"}]}], "}"}], "]"}], 
                ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotMarkers", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], "}"}],
             "]"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<log-Likelihood for this parameter setting: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "opt$Haz$BG", ",", "tEnd"}], "]"}], "]"}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<\!\(\*SuperscriptBox[\(Chi\), \(2\)]\) for this parameter \
setting: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "opt$Haz$BG", ",", "tEnd", ",", "\"\<SSR\>\""}], 
          "]"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", "ga"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.58831838949117*^9, 3.588318531265423*^9}, {
  3.5883188529960747`*^9, 3.588318857486209*^9}, {3.588320529756171*^9, 
  3.588320537187914*^9}, {3.5883283536064777`*^9, 3.5883284230734243`*^9}, {
  3.588328508588975*^9, 3.588328513707486*^9}, {3.5958453603523684`*^9, 
  3.5958453931226454`*^9}, {3.5958454975040827`*^9, 3.595845503390671*^9}, {
  3.618655220163782*^9, 3.6186552203238173`*^9}, {3.627806422908594*^9, 
  3.6278064295612593`*^9}, {3.6278305497360916`*^9, 3.62783059652077*^9}, {
  3.6278947148922243`*^9, 3.6278947151522503`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowResultScenarios", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenarios", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "opt$Haz$BG_", ",", "optRes_", ",", "tEnd_", 
     ",", "replicates_", ",", 
     RowBox[{"mean_:", "\"\<N\>\""}], ",", 
     RowBox[{"plotOptions_:", "None"}], ",", 
     RowBox[{"fsize_:", "18"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "rList", ",", "lz", ",", 
       "izz", ",", "Cw$temp", ",", "plotStyle", ",", "plotMarkers"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"symix", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Cw$temp", "[", "t_", "]"}], "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
            RowBox[{"s1", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", 
               RowBox[{"tEnd", "+", "0.1"}]}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
            RowBox[{"s1", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", 
               RowBox[{"tEnd", "+", "0.1"}]}], "]"}]}]}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"allSols", ",", "s1"}], "]"}], ";"}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"s1", "=", 
            RowBox[{"model", "[", 
             RowBox[{"0", ",", "Cw$temp", ",", 
              RowBox[{"{", 
               RowBox[{
               "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
               "}"}], ",", "tEnd"}], "]"}]}], ";"}], "*)"}], 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"replData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"Transpose", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", 
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
                "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"replData", ",", 
              RowBox[{"data", "[", 
               RowBox[{"[", 
                RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Length", "[", "plotOptions", "]"}], ">", "0"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"plotMarkers", "=", 
             RowBox[{
              RowBox[{"plotOptions", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"[", 
               RowBox[{"symix", "++"}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"plotStyle", "=", 
             RowBox[{"plotOptions", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Length", "@", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], "/.", 
                  RowBox[{"{", 
                   RowBox[{"t", "\[Rule]", "0"}], "}"}]}], ")"}]}], ">", 
               "0"}], ",", "\[IndentingNewLine]", 
              RowBox[{"funcVals", "=", 
               RowBox[{"Table", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"t", ",", 
                   RowBox[{
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"t", ",", "0", ",", 
                   RowBox[{"tEnd", "+", "0.1"}], ",", "0.01"}], "}"}]}], 
                "]"}]}], ",", "\[IndentingNewLine]", 
              RowBox[{"funcVals", "=", 
               RowBox[{"Table", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"t", ",", 
                   RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"t", ",", "0", ",", 
                   RowBox[{"tEnd", "+", "0.1"}], ",", "0.01"}], "}"}]}], 
                "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
            "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"StringLength", "[", 
                RowBox[{"plotOptions", "[", 
                 RowBox[{"[", "3", "]"}], "]"}], "]"}], ">", "0"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"plotLabel", "=", 
               RowBox[{"plotOptions", "[", 
                RowBox[{"[", "3", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
              RowBox[{"plotLabel", "=", 
               RowBox[{"\"\<Exposure \>\"", "<>", 
                RowBox[{"ToString", "[", 
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                "\"\<\[Mu]g/L\>\""}]}]}], "\[IndentingNewLine]", "]"}], ";"}],
            "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"plotMarkers", "=", 
             RowBox[{"{", 
              RowBox[{"Automatic", ",", "18"}], "}"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"plotStyle", "=", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
              "}"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"plotLabel", "=", 
             RowBox[{"\"\<Exposure \>\"", "<>", 
              RowBox[{"ToString", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", 
                 RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
              "\"\<\[Mu]g/L\>\""}]}], ";"}]}], "\[IndentingNewLine]", 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mlist", "=", 
             RowBox[{"Mean", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"dlist", "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Stdlist", "=", 
             RowBox[{"StandardDeviation", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tab", "=", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"dlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                   RowBox[{"mlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", 
                 RowBox[{"ErrorBarPlots`ErrorBar", "[", 
                  RowBox[{"Stdlist", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"i", ",", "1", ",", 
                 RowBox[{"Length", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"ga", ",", 
             RowBox[{"Show", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", "plotStyle"}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{"Black", ",", 
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "fsize"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", "\"\<Survivors(%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"LabelStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"Black", ",", "Bold"}], "]"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "1200"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], " ", "]"}], ",", 
                RowBox[{"ErrorBarPlots`ErrorListPlot", "[", "tab", "]"}]}], 
               "}"}], "]"}]}], "]"}], "\[IndentingNewLine]", ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Length", "[", "plotOptions", "]"}], ">", "0"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"AppendTo", "[", 
                RowBox[{"ga", ",", 
                 RowBox[{"ListPlot", "[", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"funcVals", ",", 
                    RowBox[{"expData", "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
                   RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                   RowBox[{"Joined", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{"True", ",", "False"}], "}"}]}], ",", 
                   RowBox[{"BaseStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{"Black", ",", 
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "fsize"}]}], "}"}]}], ",", 
                   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                   RowBox[{"FrameLabel", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", "\"\<Survivors(%)\>\"", ",", 
                    "plotLabel", ",", "\"\<\>\""}], "}"}]}], ",", 
                   RowBox[{"LabelStyle", "\[Rule]", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"Black", ",", "Bold"}], "]"}]}], ",", 
                   RowBox[{"ImageSize", "\[Rule]", "1200"}], ",", 
                   RowBox[{"PlotRange", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", 
                    RowBox[{"tEnd", "+", "0.1"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                   RowBox[{"PlotMarkers", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"MarkersScaledCol2", "[", 
                    RowBox[{"Black", ",", 
                    RowBox[{"lighter", "=", "0"}], ",", 
                    RowBox[{"thick", "=", "0.045"}], ",", 
                    RowBox[{"filled", "=", "1"}], ",", 
                    RowBox[{"size", "=", "0.0005"}]}], "]"}], "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", "plotMarkers"}], 
                    "}"}]}], ",", "\[IndentingNewLine]", 
                   RowBox[{"PlotStyle", "\[Rule]", "plotStyle"}], ",", 
                   RowBox[{"Filling", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{"2", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", "1", "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"Gray", ",", 
                    RowBox[{"Thickness", "[", "0.0035", "]"}]}], "]"}], 
                    "}"}]}], "}"}]}], "}"}]}]}], "]"}]}], "]"}], ";"}], 
              "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
              RowBox[{"AppendTo", "[", 
               RowBox[{"ga", ",", 
                RowBox[{"Show", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Plot", "[", 
                    RowBox[{
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                    RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                    RowBox[{"BaseStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{"Black", ",", 
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "fsize"}]}], "}"}]}], ",", 
                    RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                    RowBox[{"FrameLabel", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", "\"\<Survivors(%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}]}], ",", 
                    "\"\<\>\""}], "}"}]}], ",", 
                    RowBox[{"LabelStyle", "\[Rule]", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"Black", ",", "Bold"}], "]"}]}], ",", 
                    RowBox[{"ImageSize", "\[Rule]", "1200"}], ",", 
                    RowBox[{"PlotRange", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                    RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
                   RowBox[{"ListPlot", "[", 
                    RowBox[{"expData", ",", 
                    RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                    RowBox[{"PlotMarkers", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], 
                  "}"}], "]"}]}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"izz", ">", "1"}], ","}]}], "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"lz", ">", 
               RowBox[{"Length", "[", "rList", "]"}]}], ",", 
              RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<log-Likelihood for this parameter setting: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "opt$Haz$BG", ",", "tEnd"}], "]"}], "]"}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<\!\(\*SuperscriptBox[\(Chi\), \(2\)]\) for this parameter \
setting: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "opt$Haz$BG", ",", "tEnd", ",", "\"\<SSR\>\""}], 
          "]"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", "ga"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.588330631586253*^9, 3.588330660588153*^9}, {
   3.588330692421336*^9, 3.588330704581552*^9}, {3.5883309042905207`*^9, 
   3.588330968569948*^9}, {3.588344221587117*^9, 3.588344237537712*^9}, {
   3.5883451446534147`*^9, 3.588345146735623*^9}, 3.588574658896365*^9, {
   3.588574818769264*^9, 3.58857493753611*^9}, {3.588574969485729*^9, 
   3.5885751421665564`*^9}, {3.589525300529654*^9, 3.5895253527597885`*^9}, {
   3.5895254312906923`*^9, 3.5895254399487476`*^9}, {3.5895254737385645`*^9, 
   3.589525605325408*^9}, {3.5895256512521024`*^9, 3.5895256802682886`*^9}, {
   3.589525712544895*^9, 3.5895257210313497`*^9}, {3.5895257598443985`*^9, 
   3.589525790701396*^9}, {3.589525848109764*^9, 3.589525869064279*^9}, {
   3.5895258997078867`*^9, 3.5895259023772993`*^9}, {3.58952606284266*^9, 
   3.589526065902337*^9}, {3.5895260964376717`*^9, 3.589526159193284*^9}, {
   3.5895262042125793`*^9, 3.5895262049615664`*^9}, {3.5895262354213796`*^9, 
   3.5895262407111015`*^9}, {3.589526295122901*^9, 3.589526368706893*^9}, {
   3.5895264555904083`*^9, 3.5895264587267923`*^9}, {3.589526501351907*^9, 
   3.5895265153932567`*^9}, {3.5895265590774565`*^9, 
   3.5895265901868477`*^9}, {3.589526637678814*^9, 3.589526676152113*^9}, {
   3.58952671835417*^9, 3.589526749151531*^9}, 3.589527071457108*^9, {
   3.589545794929989*^9, 3.5895457983464108`*^9}, {3.5895459094191227`*^9, 
   3.589545944597348*^9}, {3.5895460176682167`*^9, 3.5895460834226384`*^9}, {
   3.589546133077757*^9, 3.589546166695972*^9}, {3.5895464662344923`*^9, 
   3.58954650805836*^9}, {3.5895465418793774`*^9, 3.589546573921983*^9}, {
   3.589547763240019*^9, 3.589547773083682*^9}, {3.5895478161243577`*^9, 
   3.589547833830471*^9}, {3.5895479494428124`*^9, 3.5895479553708506`*^9}, {
   3.590138535607909*^9, 3.590138548339182*^9}, 3.595665914471072*^9, 
   3.595845406985031*^9, {3.595845482620594*^9, 3.5958454931566477`*^9}, {
   3.617019632446067*^9, 3.6170196527630033`*^9}, {3.617019702434523*^9, 
   3.617019704868139*^9}, {3.617019763103312*^9, 3.617019805818393*^9}, {
   3.617019840624033*^9, 3.6170198469157095`*^9}, {3.6170199907726045`*^9, 
   3.617020023596653*^9}, {3.617020283737164*^9, 3.617020285687714*^9}, {
   3.6170203440491705`*^9, 3.617020345125874*^9}, {3.6170203890532603`*^9, 
   3.6170204072388043`*^9}, {3.6170204874034777`*^9, 3.61702053320087*^9}, {
   3.617020569054577*^9, 3.6170206113975906`*^9}, {3.618719539571867*^9, 
   3.6187195485417633`*^9}, 3.6187236428191366`*^9, {3.6278306102651443`*^9, 
   3.6278306382039375`*^9}, {3.6278310712172346`*^9, 
   3.6278310747215853`*^9}, {3.627894727015436*^9, 3.627894727970532*^9}, {
   3.639223734059897*^9, 3.6392237948809786`*^9}, {3.6422947681946397`*^9, 
   3.64229496291004*^9}, {3.64229518853604*^9, 3.64229520232944*^9}, {
   3.6422952337870398`*^9, 3.6422952924378395`*^9}, {3.64229544176124*^9, 
   3.6422954443196397`*^9}, {3.6422954746570396`*^9, 3.64229547510944*^9}, {
   3.6422960928916397`*^9, 3.64229616524284*^9}, {3.64229621301604*^9, 
   3.64229621412364*^9}, {3.64229624603164*^9, 3.6422962486992397`*^9}, {
   3.6422963166668396`*^9, 3.6422963547076397`*^9}, {3.6422963928284397`*^9, 
   3.6422963938424397`*^9}, {3.64229661657044*^9, 3.6422966170696397`*^9}, {
   3.6422966614344397`*^9, 3.6422966665844398`*^9}, {3.64229745765684*^9, 
   3.64229746341724*^9}, {3.64229760584824*^9, 3.6422977147582397`*^9}, {
   3.6422977547334394`*^9, 3.64229779770284*^9}, 3.64229785031804*^9, {
   3.64229795462704*^9, 3.64229795479004*^9}, {3.64229801679344*^9, 
   3.6422980171990395`*^9}, {3.64229805554904*^9, 3.6422980648662395`*^9}, {
   3.6422981162012396`*^9, 3.6422981885544395`*^9}, {3.64229831074704*^9, 
   3.64229833347824*^9}, {3.64229836489964*^9, 3.64229836497764*^9}, {
   3.64229844296144*^9, 3.6422984927384396`*^9}, {3.64229856195504*^9, 
   3.64229857256304*^9}, {3.64229864061124*^9, 3.64229865579204*^9}, {
   3.64229876808284*^9, 3.6422989327924395`*^9}, {3.6422989716288395`*^9, 
   3.6422989785396395`*^9}, {3.64229903620164*^9, 3.6422990385572395`*^9}, {
   3.642345491162204*^9, 3.6423454982932043`*^9}, {3.6423840925392036`*^9, 
   3.6423841012012043`*^9}, {3.648375027947563*^9, 3.6483750658623543`*^9}, {
   3.648462350296235*^9, 3.6484623658947945`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowResultScenarios4p", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenarios4p", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "opt$Haz$BG_", ",", "optRes_", ",", "tEnd_", 
     ",", "replicates_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "rList", ",", "lz", ",", 
       "izz", ",", "Cw$temp"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p4loc", "=", 
          RowBox[{"p4", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Cw$temp", "[", "t_", "]"}], "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
            RowBox[{"s1", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc", ",", 
                 "p4loc"}], "}"}], ",", "tEnd"}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
            RowBox[{"s1", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc", ",", 
                 "p4loc"}], "}"}], ",", "tEnd"}], "]"}]}]}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"allSols", ",", "s1"}], "]"}], ";"}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"s1", "=", 
            RowBox[{"model", "[", 
             RowBox[{"0", ",", "Cw$temp", ",", 
              RowBox[{"{", 
               RowBox[{
               "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
               "}"}], ",", "tEnd"}], "]"}]}], ";"}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"expData", ",", 
             RowBox[{"Transpose", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"data", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", 
                RowBox[{"data", "[", 
                 RowBox[{"[", 
                  RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
               "}"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"ga", ",", 
           RowBox[{"Show", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Plot", "[", 
               RowBox[{
                RowBox[{"s1", "[", 
                 RowBox[{"[", "3", "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                  "}"}]}], ",", 
                RowBox[{"BaseStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                   RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                RowBox[{"FrameLabel", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                  "\"\<time (days)\>\"", ",", 
                   "\"\<Survival over time (%)\>\"", ",", 
                   RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}]}], ",", 
                   "\"\<\>\""}], "}"}]}], ",", 
                RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                RowBox[{"PlotRange", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"Directive", "[", 
                  RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
              RowBox[{"ListPlot", "[", 
               RowBox[{"expData", ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotMarkers", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], "}"}],
             "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"izz", ">", "1"}], ","}]}], "*)"}], "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"lz", ">", 
            RowBox[{"Length", "[", "rList", "]"}]}], ",", 
           RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<log-Likelihood for this parameter setting: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments2p", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "p4loc", ",", "opt$Haz$BG", ",", "tEnd"}], "]"}], 
         "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<\!\(\*SuperscriptBox[\(Chi\), \(2\)]\) for this parameter \
setting: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments2p", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "p4loc", ",", "opt$Haz$BG", ",", "tEnd", ",", 
           "\"\<SSR\>\""}], "]"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]",
       "ga"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6209985827905254`*^9, 3.6209986359478407`*^9}, {
  3.620998674118657*^9, 3.620998674199665*^9}, {3.627830646496767*^9, 
  3.6278306655576725`*^9}, {3.627831122194332*^9, 3.6278311244495573`*^9}, {
  3.627894736872422*^9, 3.627894737113446*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowResultScenarios5p", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenarios5p", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "opt$Haz$BG_", ",", "optRes_", ",", "tEnd_", 
     ",", "replicates_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p4loc", ",", "p5loc", ",", 
       "rList", ",", "lz", ",", "izz", ",", "Cw$temp"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p4loc", "=", 
          RowBox[{"p4", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p5loc", "=", 
          RowBox[{"p5", "/.", 
           RowBox[{"optRes", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Cw$temp", "[", "t_", "]"}], "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
            RowBox[{"s1", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc", ",", 
                 "p4loc", ",", "p5loc"}], "}"}], ",", "tEnd"}], "]"}]}]}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
            RowBox[{"s1", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc", ",", 
                 "p4loc", ",", "p5loc"}], "}"}], ",", "tEnd"}], "]"}]}]}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"AppendTo", "[", 
            RowBox[{"allSols", ",", "s1"}], "]"}], ";"}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"s1", "=", 
            RowBox[{"model", "[", 
             RowBox[{"0", ",", "Cw$temp", ",", 
              RowBox[{"{", 
               RowBox[{
               "opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
               "}"}], ",", "tEnd"}], "]"}]}], ";"}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"expData", ",", 
             RowBox[{"Transpose", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"data", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", 
                RowBox[{"data", "[", 
                 RowBox[{"[", 
                  RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
               "}"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"ga", ",", 
           RowBox[{"Show", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Plot", "[", 
               RowBox[{
                RowBox[{"s1", "[", 
                 RowBox[{"[", "3", "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                  "}"}]}], ",", 
                RowBox[{"BaseStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                   RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                RowBox[{"FrameLabel", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                  "\"\<time (days)\>\"", ",", 
                   "\"\<Survival over time (%)\>\"", ",", 
                   RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}]}], ",", 
                   "\"\<\>\""}], "}"}]}], ",", 
                RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                RowBox[{"PlotRange", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"Directive", "[", 
                  RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
              RowBox[{"ListPlot", "[", 
               RowBox[{"expData", ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotMarkers", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], "}"}],
             "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"izz", ">", "1"}], ","}]}], "*)"}], "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"lz", ">", 
            RowBox[{"Length", "[", "rList", "]"}]}], ",", 
           RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<log-Likelihood for this parameter setting: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments5p", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "p4loc", ",", "p5loc", ",", "opt$Haz$BG", ",", 
           "tEnd"}], "]"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{
       "\"\<\!\(\*SuperscriptBox[\(Chi\), \(2\)]\) for this parameter \
setting: \>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments5p", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "p4loc", ",", "p5loc", ",", "opt$Haz$BG", ",", 
           "tEnd", ",", "\"\<SSR\>\""}], "]"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "ga"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6209985827905254`*^9, 3.6209986359478407`*^9}, {
  3.620998674118657*^9, 3.620998674199665*^9}, {3.626158634320657*^9, 
  3.6261586870189266`*^9}, {3.626158854804703*^9, 3.626158856210844*^9}, {
  3.6278306702671432`*^9, 3.6278307079679136`*^9}, {3.627831133563469*^9, 
  3.627831135627675*^9}, {3.6278947428740225`*^9, 3.627894743445079*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowResultScenariosx2Res", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenariosx2Res", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "title_", ",", "opt$Haz$BG$1_", ",", 
     "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", ",", "tEnd_", ",", 
     "replicates_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "izz", ",", "Cw$temp"}], "}"}],
      ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"s1", "=", 
            RowBox[{"model", "[", 
             RowBox[{"0", ",", "Cw$temp", ",", 
              RowBox[{"{", 
               RowBox[{
               "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
               "}"}], ",", "tEnd"}], "]"}]}], ";"}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc2", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc2", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc2", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"s2", "=", 
            RowBox[{"model", "[", 
             RowBox[{"0", ",", "Cw$temp", ",", 
              RowBox[{"{", 
               RowBox[{
               "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}], 
               "}"}], ",", "tEnd"}], "]"}]}], ";"}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", 
            RowBox[{"s2", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}],
                 "}"}], ",", "tEnd"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", 
            RowBox[{"s2", "=", 
             RowBox[{"model", "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}],
                 "}"}], ",", "tEnd"}], "]"}]}]}]}], "\[IndentingNewLine]", 
          "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"expData", ",", 
             RowBox[{"Transpose", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"data", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], ",", 
                RowBox[{"data", "[", 
                 RowBox[{"[", 
                  RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
               "}"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"ga", ",", 
           RowBox[{"Show", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Plot", "[", 
               RowBox[{
                RowBox[{"s1", "[", 
                 RowBox[{"[", "3", "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                  "}"}]}], ",", 
                RowBox[{"BaseStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                   RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                RowBox[{"FrameLabel", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                  "\"\<time (days)\>\"", ",", 
                   "\"\<Survival over time (%)\>\"", ",", 
                   RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                RowBox[{"PlotRange", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"Directive", "[", 
                  RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"Plot", "[", 
               RowBox[{
                RowBox[{"s2", "[", 
                 RowBox[{"[", "3", "]"}], "]"}], ",", 
                RowBox[{"{", 
                 RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                  "}"}]}], ",", 
                RowBox[{"BaseStyle", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                   RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                RowBox[{"FrameLabel", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                  "\"\<time (days)\>\"", ",", 
                   "\"\<Survival over time (%)\>\"", ",", 
                   RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                RowBox[{"PlotRange", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                RowBox[{"PlotStyle", "\[Rule]", 
                 RowBox[{"Directive", "[", 
                  RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
              RowBox[{"ListPlot", "[", 
               RowBox[{"expData", ",", 
                RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                RowBox[{"PlotMarkers", "\[Rule]", 
                 RowBox[{"{", 
                  RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], "}"}],
             "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"izz", ">", "1"}], ","}]}], "*)"}], "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"lz", ">", 
            RowBox[{"Length", "[", "rList", "]"}]}], ",", 
           RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"Style", "[", 
        RowBox[{"title", ",", "26"}], "]"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", "\"\<Likelihood for this parameter setting: \>\"", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 1 (black):\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc", ",", "p2loc", ",", 
           "p3loc", ",", "opt$Haz$BG$1", ",", "tEnd"}], "]"}], "]"}]}], "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 2 (red)  :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{
          "data", ",", "model", ",", "0", ",", "p1loc2", ",", "p2loc2", ",", 
           "p3loc2", ",", "opt$Haz$BG$2", ",", "tEnd"}], "]"}], "]"}]}], 
       "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", "ga"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.590138358272177*^9, 3.590138458829232*^9}, {
  3.5901384981121597`*^9, 3.5901386180871563`*^9}, {3.5901395471646605`*^9, 
  3.590139549773921*^9}, {3.591080158377359*^9, 3.591080165729094*^9}, {
  3.5945365823736963`*^9, 3.594536621146696*^9}, {3.61702494060225*^9, 
  3.6170249900727453`*^9}, {3.61702514057425*^9, 3.617025146644311*^9}, {
  3.6170252047548923`*^9, 3.617025213154976*^9}, {3.617027128764024*^9, 
  3.617027129824831*^9}, {3.617027830798746*^9, 3.6170278857734985`*^9}, {
  3.6170279383790436`*^9, 3.6170279458826914`*^9}, {3.627830721224239*^9, 
  3.6278307597560916`*^9}, {3.6278311468928013`*^9, 3.627831149221034*^9}, {
  3.6278947500107355`*^9, 3.627894750244759*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "extractTS", "]"}], ";", 
  RowBox[{
   RowBox[{"extractTS", "[", 
    RowBox[{"predDat_", ",", "predix_"}], "]"}], ":=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Transpose", "[", 
        RowBox[{"Drop", "[", 
         RowBox[{
          RowBox[{"predDat", "[", 
           RowBox[{"[", "predix", "]"}], "]"}], ",", "2"}], "]"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Transpose", "[", 
        RowBox[{"Drop", "[", 
         RowBox[{
          RowBox[{"predDat", "[", 
           RowBox[{"[", "predix", "]"}], "]"}], ",", "2"}], "]"}], "]"}], "[", 
       RowBox[{"[", "2", "]"}], "]"}]}], "}"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6405241878856196`*^9, 3.640524215130178*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "createModelTimeSeries1MOSF", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"createModelTimeSeries1MOSF", "[", 
   RowBox[{
   "model_", ",", "data_", ",", "BGmort_", ",", "param_", ",", "tEnd_", ",", 
    "replicates_", ",", 
    RowBox[{"deltaT_:", "0.1"}], ",", 
    RowBox[{"MOSF_:", "1"}]}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "s1", ",", "s2", ",", "simData", ",", "lineVal", ",", "tabTime", ",", 
      "dayNr", ",", "rList", ",", "expData", ",", "lz", ",", "iz", ",", "yuy",
       ",", "tmp", ",", "iix"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"iix", "=", "3"}], ";", "\[IndentingNewLine]", 
     RowBox[{"ga", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"rList", "=", 
        RowBox[{"Table", "[", 
         RowBox[{"replicates", ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", 
            RowBox[{
             RowBox[{"Length", "[", 
              RowBox[{"data", "[", 
               RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
            ",", "replicates"}], "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"rList", "=", "replicates"}], ",", 
       RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"IFform", "=", 
      RowBox[{"TrueQ", "[", " ", 
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", 
            RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
        "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"iz", "=", "1"}], ",", 
       RowBox[{"iz", "<=", 
        RowBox[{"Length", "[", 
         RowBox[{"data", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"iz", "+=", 
        RowBox[{"rList", "[", 
         RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"iz", ">", "1"}], ",", 
          RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "iz"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
          RowBox[{"s1", "=", 
           RowBox[{"model", "[", 
            RowBox[{"0", ",", 
             RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"BGmort", ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}], ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "2", ",", "2"}], "]"}], "]"}], ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "3", ",", "2"}], "]"}], "]"}]}], "}"}], 
             ",", "tEnd", ",", "MOSF"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"s1", "=", 
           RowBox[{"model", "[", 
            RowBox[{"0", ",", "Cw$temp", ",", 
             RowBox[{"{", 
              RowBox[{"BGmort", ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}], ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "2", ",", "2"}], "]"}], "]"}], ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "3", ",", "2"}], "]"}], "]"}]}], "}"}], 
             ",", "tEnd", ",", "MOSF"}], "]"}]}]}], "\[IndentingNewLine]", 
         "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"simData", "=", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"\"\<Treatment:\>\"", "<>", 
             RowBox[{"ToString", "[", 
              RowBox[{"data", "[", 
               RowBox[{"[", 
                RowBox[{"3", ",", "iz"}], "]"}], "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
            "\"\<time\>\"", ",", "\"\<model\>\"", ",", 
             "\"\<Data (replicates)\>\""}], "}"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"dayNr", "=", "1"}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"tabTime", "=", "0"}], ",", 
          RowBox[{"tabTime", "\[LessEqual]", "tEnd"}], ",", 
          RowBox[{"tabTime", "+=", "deltaT"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "@", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"s1", "[", 
                  RowBox[{"[", "iix", "]"}], "]"}], "/.", 
                 RowBox[{"{", 
                  RowBox[{"t", "\[Rule]", "tabTime"}], "}"}]}], ")"}]}], ">", 
              "0"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"tmp", "=", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "iix", "]"}], "]"}], "/.", 
                  RowBox[{"{", 
                   RowBox[{"t", "\[Rule]", "tabTime"}], "}"}]}], ")"}], "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ";", 
              RowBox[{"lineVal", "=", 
               RowBox[{"{", 
                RowBox[{"tabTime", ",", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"tmp", ">", "0"}], ",", "tmp", ",", "0"}], "]"}]}],
                 "}"}]}]}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"tmp", "=", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"s1", "[", 
                  RowBox[{"[", "iix", "]"}], "]"}], "/.", 
                 RowBox[{"{", 
                  RowBox[{"t", "\[Rule]", "tabTime"}], "}"}]}], ")"}]}], ";", 
              RowBox[{"lineVal", "=", 
               RowBox[{"{", 
                RowBox[{"tabTime", ",", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"tmp", ">", "0"}], ",", "tmp", ",", "0"}], "]"}]}],
                 "}"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
           "\[IndentingNewLine]", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"dayNr", "\[LessEqual]", 
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"1", ",", "dayNr"}], "]"}], "]"}], "\[Equal]", 
                 "tabTime"}], ",", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"For", "[", 
                  RowBox[{
                   RowBox[{"yuy", "=", "0"}], ",", 
                   RowBox[{"yuy", "<", 
                    RowBox[{"rList", "[", 
                    RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
                   RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"lineVal", ",", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"3", "+", "yuy", "+", "iz"}], ",", "dayNr"}], 
                    "]"}], "]"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", 
                  "]"}], ";", "\[IndentingNewLine]", 
                 RowBox[{"dayNr", "++"}]}]}], "\[IndentingNewLine]", "]"}], 
              ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"simData", ",", "lineVal"}], "]"}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"ga", ",", "simData"}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"lz", ">", 
           RowBox[{"Length", "[", "rList", "]"}]}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "ga"}]}], 
   "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.640523791996976*^9, 3.640523827691018*^9}, {
  3.6405240306156244`*^9, 3.640524035223888*^9}, {3.640524145558199*^9, 
  3.640524149415419*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "createModelTimeSeries1M", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"createModelTimeSeries1M", "[", 
   RowBox[{
   "model_", ",", "data_", ",", "BGmort_", ",", "param_", ",", "tEnd_", ",", 
    "replicates_", ",", 
    RowBox[{"deltaT_:", "0.1"}]}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "s1", ",", "s2", ",", "simData", ",", "lineVal", ",", "tabTime", ",", 
      "dayNr", ",", "rList", ",", "expData", ",", "lz", ",", "iz", ",", "yuy",
       ",", "tmp", ",", "iix"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"iix", "=", "3"}], ";", "\[IndentingNewLine]", 
     RowBox[{"ga", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"rList", "=", 
        RowBox[{"Table", "[", 
         RowBox[{"replicates", ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", 
            RowBox[{
             RowBox[{"Length", "[", 
              RowBox[{"data", "[", 
               RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
            ",", "replicates"}], "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"rList", "=", "replicates"}], ",", 
       RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"IFform", "=", 
      RowBox[{"TrueQ", "[", " ", 
       RowBox[{
        RowBox[{"Head", "[", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", 
            RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
        "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"iz", "=", "1"}], ",", 
       RowBox[{"iz", "<=", 
        RowBox[{"Length", "[", 
         RowBox[{"data", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
       RowBox[{"iz", "+=", 
        RowBox[{"rList", "[", 
         RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"iz", ">", "1"}], ",", 
          RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
         RowBox[{"data", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "iz"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
          RowBox[{"s1", "=", 
           RowBox[{"model", "[", 
            RowBox[{"0", ",", 
             RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"BGmort", ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}], ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "2", ",", "2"}], "]"}], "]"}], ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "3", ",", "2"}], "]"}], "]"}]}], "}"}], 
             ",", "tEnd"}], "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"s1", "=", 
           RowBox[{"model", "[", 
            RowBox[{"0", ",", "Cw$temp", ",", 
             RowBox[{"{", 
              RowBox[{"BGmort", ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}], ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "2", ",", "2"}], "]"}], "]"}], ",", 
               RowBox[{"param", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "3", ",", "2"}], "]"}], "]"}]}], "}"}], 
             ",", "tEnd"}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"simData", "=", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"\"\<Treatment:\>\"", "<>", 
             RowBox[{"ToString", "[", 
              RowBox[{"data", "[", 
               RowBox[{"[", 
                RowBox[{"3", ",", "iz"}], "]"}], "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{
            "\"\<time\>\"", ",", "\"\<model\>\"", ",", 
             "\"\<Data (replicates)\>\""}], "}"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"dayNr", "=", "1"}], ";", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"tabTime", "=", "0"}], ",", 
          RowBox[{"tabTime", "\[LessEqual]", "tEnd"}], ",", 
          RowBox[{"tabTime", "+=", "deltaT"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Length", "@", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"s1", "[", 
                  RowBox[{"[", "iix", "]"}], "]"}], "/.", 
                 RowBox[{"{", 
                  RowBox[{"t", "\[Rule]", "tabTime"}], "}"}]}], ")"}]}], ">", 
              "0"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"tmp", "=", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "iix", "]"}], "]"}], "/.", 
                  RowBox[{"{", 
                   RowBox[{"t", "\[Rule]", "tabTime"}], "}"}]}], ")"}], "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ";", 
              RowBox[{"lineVal", "=", 
               RowBox[{"{", 
                RowBox[{"tabTime", ",", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"tmp", ">", "0"}], ",", "tmp", ",", "0"}], "]"}]}],
                 "}"}]}]}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"tmp", "=", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"s1", "[", 
                  RowBox[{"[", "iix", "]"}], "]"}], "/.", 
                 RowBox[{"{", 
                  RowBox[{"t", "\[Rule]", "tabTime"}], "}"}]}], ")"}]}], ";", 
              RowBox[{"lineVal", "=", 
               RowBox[{"{", 
                RowBox[{"tabTime", ",", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"tmp", ">", "0"}], ",", "tmp", ",", "0"}], "]"}]}],
                 "}"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
           "\[IndentingNewLine]", "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"dayNr", "\[LessEqual]", 
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"1", ",", "dayNr"}], "]"}], "]"}], "\[Equal]", 
                 "tabTime"}], ",", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"For", "[", 
                  RowBox[{
                   RowBox[{"yuy", "=", "0"}], ",", 
                   RowBox[{"yuy", "<", 
                    RowBox[{"rList", "[", 
                    RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
                   RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"lineVal", ",", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"3", "+", "yuy", "+", "iz"}], ",", "dayNr"}], 
                    "]"}], "]"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", 
                  "]"}], ";", "\[IndentingNewLine]", 
                 RowBox[{"dayNr", "++"}]}]}], "\[IndentingNewLine]", "]"}], 
              ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"simData", ",", "lineVal"}], "]"}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"ga", ",", "simData"}], "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"lz", ">", 
           RowBox[{"Length", "[", "rList", "]"}]}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "ga"}]}], 
   "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.635172112452921*^9, 3.635172183808921*^9}, {
  3.635172306821921*^9, 3.635172326736921*^9}, {3.6351723793909206`*^9, 
  3.635172410611921*^9}, {3.6351725362610693`*^9, 3.635172673190761*^9}, {
  3.638442212545288*^9, 3.6384423298280153`*^9}, {3.638587416450425*^9, 
  3.638587437547535*^9}, {3.640521110137583*^9, 3.6405211585513515`*^9}, {
  3.6405212523367157`*^9, 3.6405212552348814`*^9}, {3.640521896422556*^9, 
  3.640521896886582*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"createModelTimeSeries", "[", 
    RowBox[{
    "data_", ",", "BGmort_", ",", "confSD_", ",", "confIT_", ",", "tEnd_", 
     ",", "replicates_", ",", 
     RowBox[{"deltaT_:", "0.1"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "s1", ",", "s2", ",", "simData", ",", "lineVal", ",", "tabTime", ",", 
       "dayNr", ",", "rList", ",", "expData", ",", "lz", ",", "iz", ",", 
       "yuy", ","}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"iz", "=", "1"}], ",", 
        RowBox[{"iz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"iz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"iz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "iz"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"s1", "=", 
          RowBox[{"SDmodel", "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"BGmort", ",", 
              RowBox[{"confSD", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}], ",", 
              RowBox[{"confSD", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "2", ",", "2"}], "]"}], "]"}], ",", 
              RowBox[{"confSD", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "3", ",", "2"}], "]"}], "]"}]}], "}"}], ",",
             "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s2", "=", 
          RowBox[{"ITmodel", "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"BGmort", ",", 
              RowBox[{"confIT", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}], ",", 
              RowBox[{"confIT", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "2", ",", "2"}], "]"}], "]"}], ",", 
              RowBox[{"confIT", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "3", ",", "2"}], "]"}], "]"}]}], "}"}], ",",
             "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"simData", "=", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"\"\<Treatment:\>\"", "<>", 
              RowBox[{"ToString", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", 
                 RowBox[{"3", ",", "iz"}], "]"}], "]"}], "]"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
             "\"\<time\>\"", ",", "\"\<SDModel\>\"", ",", "\"\<ITmodel\>\"", 
              ",", "\"\<Data (replicates)\>\""}], "}"}]}], "}"}]}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"dayNr", "=", "1"}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"tabTime", "=", "0"}], ",", 
           RowBox[{"tabTime", "\[LessEqual]", "tEnd"}], ",", 
           RowBox[{"tabTime", "+=", "deltaT"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"lineVal", "=", 
             RowBox[{"{", 
              RowBox[{"tabTime", ",", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], "/.", 
                  RowBox[{"{", 
                   RowBox[{"t", "\[Rule]", "tabTime"}], "}"}]}], ")"}], "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"s2", "[", 
                  RowBox[{"[", "3", "]"}], "]"}], "/.", 
                 RowBox[{"{", 
                  RowBox[{"t", "\[Rule]", "tabTime"}], "}"}]}], ")"}]}], 
              "}"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"dayNr", "\[LessEqual]", 
               RowBox[{"Length", "[", 
                RowBox[{"data", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"data", "[", 
                   RowBox[{"[", 
                    RowBox[{"1", ",", "dayNr"}], "]"}], "]"}], "\[Equal]", 
                  "tabTime"}], ",", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"For", "[", 
                   RowBox[{
                    RowBox[{"yuy", "=", "0"}], ",", 
                    RowBox[{"yuy", "<", 
                    RowBox[{"rList", "[", 
                    RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
                    RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"lineVal", ",", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"3", "+", "yuy", "+", "iz"}], ",", "dayNr"}], 
                    "]"}], "]"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", 
                   "]"}], ";", "\[IndentingNewLine]", 
                  RowBox[{"dayNr", "++"}]}]}], "\[IndentingNewLine]", "]"}], 
               ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"simData", ",", "lineVal"}], "]"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"ga", ",", "simData"}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"lz", ">", 
            RowBox[{"Length", "[", "rList", "]"}]}], ",", 
           RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "ga"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5895270911418915`*^9, 3.5895272137456493`*^9}, {
   3.58952727982852*^9, 3.5895273153348026`*^9}, {3.589527510055876*^9, 
   3.58952753568684*^9}, {3.589527570256662*^9, 3.589527796443512*^9}, {
   3.5895278469722357`*^9, 3.589527867080765*^9}, {3.5895279236311274`*^9, 
   3.5895279261115427`*^9}, {3.589528090271395*^9, 3.5895284682940187`*^9}, {
   3.5895338211870914`*^9, 3.5895338634875145`*^9}, {3.589533907458659*^9, 
   3.5895339153055096`*^9}, {3.5895339917615995`*^9, 3.589533992900407*^9}, {
   3.589534031089452*^9, 3.5895340701365023`*^9}, {3.58953469125429*^9, 
   3.589534763857155*^9}, {3.589534804058613*^9, 3.5895348058994246`*^9}, {
   3.5895358294523463`*^9, 3.589535833898375*^9}, {3.5895361908452635`*^9, 
   3.5895365122229233`*^9}, 3.5895377048240647`*^9, {3.5895473433779273`*^9, 
   3.5895473641260605`*^9}, {3.589547746376311*^9, 3.589547747546318*^9}, {
   3.589547929911487*^9, 3.5895479437331758`*^9}, 3.5956018061597824`*^9, {
   3.6278307703821545`*^9, 3.627830771771293*^9}, {3.6278947556873035`*^9, 
   3.6278947562523603`*^9}, {3.6349997696469297`*^9, 3.634999770115977*^9}, {
   3.635170129413282*^9, 3.635170142270568*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "calculateScatterPoints", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"calculateScatterPoints", "[", 
    RowBox[{
    "data_", ",", "model1_", ",", "opt$Haz$BG$1_", ",", "optRes1_", ",", 
     "tEnd_", ",", "replicates_", ",", 
     RowBox[{"mean_:", "\"\<N\>\""}], ",", 
     RowBox[{"ErrBars_:", "\"\<N\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "izz", ",", "timeIX", ",", 
       "time", ",", "scatterData", ",", "nPars", ",", "pars", ",", "p4loc", 
       ",", "p5loc"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"scatterData", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"treatment", " ", "loop"}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"nPars", "=", 
          RowBox[{"Length", "[", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"nPars", "<", "4"}], ",", "\[IndentingNewLine]", 
           RowBox[{"pars", "=", 
            RowBox[{"{", 
             RowBox[{
             "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
             "}"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"nPars", "\[Equal]", "4"}], ",", 
           RowBox[{
            RowBox[{"p4loc", "=", 
             RowBox[{"p4", "/.", 
              RowBox[{"optRes1", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
            RowBox[{"pars", "=", 
             RowBox[{"{", 
              RowBox[{
              "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc", ",", 
               "p4loc"}], "}"}]}]}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"nPars", "\[Equal]", "5"}], ",", 
           RowBox[{
            RowBox[{"p4loc", "=", 
             RowBox[{"p4", "/.", 
              RowBox[{"optRes1", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
            RowBox[{"p5loc", "=", 
             RowBox[{"p5", "/.", 
              RowBox[{"optRes1", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
            RowBox[{"pars", "=", 
             RowBox[{"{", 
              RowBox[{
              "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc", ",", 
               "p4loc", ",", "p5loc"}], "}"}]}]}]}], "]"}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{"s1", "=", 
            RowBox[{
             RowBox[{"model1", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "[", 
             RowBox[{"0", ",", 
              RowBox[{"Cw$temp", "[", "t", "]"}], ",", "pars", ",", "tEnd"}], 
             "]"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"s1", "=", 
            RowBox[{
             RowBox[{"model1", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "[", 
             RowBox[{"0", ",", "Cw$temp", ",", "pars", ",", "tEnd"}], 
             "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"replData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"Transpose", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", 
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
                "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"replData", ",", 
              RowBox[{"data", "[", 
               RowBox[{"[", 
                RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mlist", "=", 
             RowBox[{"Mean", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"dlist", "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Stdlist", "=", 
             RowBox[{"StandardDeviation", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"ErrBars", "\[Equal]", "\"\<Y\>\""}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"tab", "=", 
               RowBox[{"Table", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"mlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"TrueQ", "[", 
                    RowBox[{
                    RowBox[{"Head", "[", 
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "]"}], "\[Equal]", 
                    "List"}], "]"}], "\[Equal]", "False"}], ",", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "/.", 
                    RowBox[{"{", 
                    RowBox[{"t", "\[Rule]", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"1", ",", "i"}], "]"}], "]"}]}], "}"}]}], ")"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "/.", 
                    RowBox[{"{", 
                    RowBox[{"t", "\[Rule]", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"1", ",", "i"}], "]"}], "]"}]}], "}"}]}], ")"}], 
                    "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], "}"}], ",", 
                   RowBox[{"ErrorBarPlots`ErrorBar", "[", 
                    RowBox[{
                    RowBox[{"Stdlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", "0"}], "]"}]}], 
                  "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"i", ",", "1", ",", 
                   RowBox[{"Length", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}],
               ",", "\[IndentingNewLine]", 
              RowBox[{"tab", "=", 
               RowBox[{"Table", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"mlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                   RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"TrueQ", "[", 
                    RowBox[{
                    RowBox[{"Head", "[", 
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "]"}], "\[Equal]", 
                    "List"}], "]"}], "\[Equal]", "False"}], ",", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "/.", 
                    RowBox[{"{", 
                    RowBox[{"t", "\[Rule]", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"1", ",", "i"}], "]"}], "]"}]}], "}"}]}], ")"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "/.", 
                    RowBox[{"{", 
                    RowBox[{"t", "\[Rule]", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"1", ",", "i"}], "]"}], "]"}]}], "}"}]}], ")"}], 
                    "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"i", ",", "1", ",", 
                   RowBox[{"Length", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}], 
                "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"scatterData", ",", "tab"}], "]"}]}], ",", 
           "\[IndentingNewLine]", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"For", "[", 
             RowBox[{
              RowBox[{"timeIX", "=", "1"}], ",", 
              RowBox[{"timeIX", "\[LessEqual]", 
               RowBox[{"Length", "[", 
                RowBox[{"data", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], "]"}]}], ",", 
              RowBox[{"timeIX", "++"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"time", "=", 
                RowBox[{"data", "[", 
                 RowBox[{"[", 
                  RowBox[{"1", ",", "timeIX"}], "]"}], "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"For", "[", 
                RowBox[{
                 RowBox[{"yuy", "=", "0"}], ",", 
                 RowBox[{"yuy", "<", 
                  RowBox[{"rList", "[", 
                   RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
                 RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
                 RowBox[{"AppendTo", "[", 
                  RowBox[{"scatterData", ",", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"3", "+", "yuy", "+", "izz"}], ",", "timeIX"}], 
                    "]"}], "]"}], ",", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"TrueQ", "[", 
                    RowBox[{
                    RowBox[{"Head", "[", 
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "]"}], "\[Equal]", 
                    "List"}], "]"}], "\[Equal]", "False"}], ",", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "/.", 
                    RowBox[{"{", 
                    RowBox[{"t", "\[Rule]", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"1", ",", "i"}], "]"}], "]"}]}], "}"}]}], ")"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], "/.", 
                    RowBox[{"{", 
                    RowBox[{"t", "\[Rule]", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"1", ",", "i"}], "]"}], "]"}]}], "}"}]}], ")"}], 
                    "[", 
                    RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                  "]"}]}], "\[IndentingNewLine]", "]"}]}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"lz", ">", 
               RowBox[{"Length", "[", "rList", "]"}]}], ",", 
              RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", "]"}],
       ";", "\[IndentingNewLine]", "scatterData"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5944496816110377`*^9, 3.594449732532056*^9}, {
   3.5944505933302717`*^9, 3.594450597420354*^9}, 3.594450638591177*^9, {
   3.5944507104736147`*^9, 3.5944507354541144`*^9}, {3.594450927397953*^9, 
   3.594451050380413*^9}, {3.5944510943012915`*^9, 3.594451129591997*^9}, {
   3.5944512378051615`*^9, 3.5944513062865305`*^9}, {3.62774031349706*^9, 
   3.6277403138950996`*^9}, {3.627741990412735*^9, 3.627741993895083*^9}, {
   3.6278307914062567`*^9, 3.627830814013517*^9}, {3.627831234157527*^9, 
   3.627831236938805*^9}, {3.6278938574154854`*^9, 3.6278939215358963`*^9}, {
   3.6278941353142724`*^9, 3.6278942336171017`*^9}, {3.627894398862624*^9, 
   3.6278944001967583`*^9}, {3.6278944307978177`*^9, 3.627894432309969*^9}, {
   3.6278946136831045`*^9, 3.6278946139841347`*^9}, {3.6278948243461685`*^9, 
   3.627894886788412*^9}, {3.627895623893115*^9, 3.6278956254752736`*^9}, {
   3.6278956673794637`*^9, 3.627895670515777*^9}, {3.627895741610886*^9, 
   3.627895772831008*^9}, {3.6278958239941235`*^9, 3.627895873436067*^9}, {
   3.6278961139151125`*^9, 3.6278961312928505`*^9}, {3.62789617309803*^9, 
   3.627896233261046*^9}, 3.6278963054992695`*^9, {3.627896350150734*^9, 
   3.627896362546973*^9}, {3.6278964603287506`*^9, 3.627896492793997*^9}, {
   3.627898239453645*^9, 3.6278982825419536`*^9}, {3.6279025493645935`*^9, 
   3.627902735730228*^9}, 3.6279028895306063`*^9, {3.627902957611414*^9, 
   3.627902960214674*^9}, {3.627903072662918*^9, 3.6279030763312845`*^9}, {
   3.627903113466998*^9, 3.627903120973748*^9}, 3.6279068011527295`*^9, 
   3.627906945731186*^9, {3.627907178385449*^9, 3.62790718159377*^9}, {
   3.627907294741083*^9, 3.6279073286854773`*^9}, {3.6279073891795263`*^9, 
   3.6279073971893272`*^9}, {3.627907431288737*^9, 3.6279074496085687`*^9}, {
   3.6279074834399514`*^9, 3.6279075822318296`*^9}, {3.627907614957102*^9, 
   3.6279077080924144`*^9}, {3.6279078660842123`*^9, 3.627907871354739*^9}, {
   3.6279079129058933`*^9, 3.6279079364762506`*^9}, 3.630158432179177*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowModelResultx2M", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowModelResultx2M", "[", 
    RowBox[{
    "model1_", ",", "model2_", ",", "Cext1_", ",", "Cext2_", ",", 
     "opt$Haz$BG$1_", ",", "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", 
     ",", "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "Cw$temp1", ",", "Cw$temp2"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", "Cext1", "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp1", "[", "t_", "]"}], ":=", "Cext1"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp2", "[", "t_", "]"}], ":=", "Cext2"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"p1loc", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p2loc", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p3loc", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"p1loc2", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p2loc2", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p3loc2", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"s1", "=", 
          RowBox[{
           RowBox[{"model1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp1", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{
             "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
             "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s2", "=", 
          RowBox[{
           RowBox[{"model2", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp2", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{
             "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}], 
             "}"}], ",", "tEnd"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"s1", "=", 
          RowBox[{
           RowBox[{"model1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "[", 
           RowBox[{"0", ",", "Cw$temp1", ",", 
            RowBox[{"{", 
             RowBox[{
             "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
             "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s2", "=", 
          RowBox[{
           RowBox[{"model2", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "[", 
           RowBox[{"0", ",", "Cw$temp2", ",", 
            RowBox[{"{", 
             RowBox[{
             "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}], 
             "}"}], ",", "tEnd"}], "]"}]}]}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"ga", ",", 
        RowBox[{"Show", "[", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s1", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
               "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s2", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
               "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}]}], "}"}], "]"}]}], 
       "]"}], ";", "\[IndentingNewLine]", "ga"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.590138358272177*^9, 3.590138458829232*^9}, {
  3.5901384981121597`*^9, 3.5901386180871563`*^9}, {3.5901395471646605`*^9, 
  3.590139549773921*^9}, {3.5901442169906235`*^9, 3.590144237261029*^9}, {
  3.590144361583515*^9, 3.5901443618035192`*^9}, {3.590205462989811*^9, 
  3.590205534128924*^9}, {3.591080177064227*^9, 3.591080179472468*^9}, {
  3.6247054608004503`*^9, 3.62470554790816*^9}, {3.627830824467562*^9, 
  3.6278308754266577`*^9}, {3.6278947636100955`*^9, 3.627894764481183*^9}, {
  3.633156262899947*^9, 3.6331564537436085`*^9}, {3.6331565728083644`*^9, 
  3.6331565733075705`*^9}, {3.633156612039282*^9, 3.633156660723921*^9}, {
  3.6331578271815042`*^9, 3.6331578280239153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ShowModelResultx5Exp", "[", 
    RowBox[{
    "model_", ",", "CextList_", ",", "opt$Haz$BG_", ",", "optRes_", ",", 
     "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "s1", ",", "s2", ",", "s3", ",", "s4", ",", "s5", ",", "p1loc", ",", 
       "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2", 
       ",", "Cw$temp1", ",", "Cw$temp2", ",", "Cw$temp3", ",", "Cw$temp4", 
       ",", "Cw$temp5"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", "Cext1", "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp1", "[", "t_", "]"}], ":=", 
       RowBox[{"CextList", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp2", "[", "t_", "]"}], ":=", 
       RowBox[{"CextList", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp3", "[", "t_", "]"}], ":=", 
       RowBox[{"CextList", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp4", "[", "t_", "]"}], ":=", 
       RowBox[{"CextList", "[", 
        RowBox[{"[", "4", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp5", "[", "t_", "]"}], ":=", 
       RowBox[{"CextList", "[", 
        RowBox[{"[", "5", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"p1loc", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p2loc", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p3loc", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"s1", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp1", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s2", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp2", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s3", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp3", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s4", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp4", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s5", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", 
            RowBox[{"Cw$temp5", "[", "t", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
        ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"s1", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", "Cw$temp1", ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s2", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", "Cw$temp2", ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s3", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", "Cw$temp3", ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s4", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", "Cw$temp4", ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"s5", "=", 
          RowBox[{"model", "[", 
           RowBox[{"0", ",", "Cw$temp5", ",", 
            RowBox[{"{", 
             RowBox[{"opt$Haz$BG", ",", "p1loc", ",", "p2loc", ",", "p3loc"}],
              "}"}], ",", "tEnd"}], "]"}]}], ";"}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"ga", ",", 
        RowBox[{"Show", "[", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s1", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Green"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
               "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s2", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Blue"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
               "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s3", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
               "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s4", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Orange"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
               "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s5", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
               "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                ",", "\"\<\>\"", ",", "\"\<\>\""}], "}"}]}], ",", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}]}], "}"}], 
         "]"}]}], "]"}], ";", "\[IndentingNewLine]", "ga"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6331580037806125`*^9, 3.6331582497005944`*^9}, {
  3.6331583339636893`*^9, 3.633158344883829*^9}, {3.6331583815027304`*^9, 
  3.6331583912060547`*^9}, {3.6331584250896893`*^9, 3.6331584261037025`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowResultScenariosx2MNinit", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenariosx2MNinit", "[", 
    RowBox[{
    "data_", ",", "model1_", ",", "model2_", ",", "opt$Haz$BG$1_", ",", 
     "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", ",", "tEnd_", ",", 
     "replicates_", ",", "Ninit_", ",", 
     RowBox[{"mean_:", "\"\<N\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "izz"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"allDatVals", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"p1loc2", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc2", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc2", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{
              RowBox[{"model1", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", 
            RowBox[{"s2", "=", 
             RowBox[{
              RowBox[{"model2", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}],
                 "}"}], ",", "tEnd"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{
              RowBox[{"model1", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", 
            RowBox[{"s2", "=", 
             RowBox[{
              RowBox[{"model2", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}],
                 "}"}], ",", "tEnd"}], "]"}]}]}]}], "\[IndentingNewLine]", 
          "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"replData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"Transpose", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", 
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
                "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"replData", ",", 
              RowBox[{"Ninit", "*", 
               RowBox[{"data", "[", 
                RowBox[{"[", 
                 RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}]}], 
             "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mlist", "=", 
             RowBox[{"Mean", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"dlist", "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Stdlist", "=", 
             RowBox[{"StandardDeviation", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tab", "=", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"dlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                   RowBox[{"mlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", 
                 RowBox[{"ErrorBarPlots`ErrorBar", "[", 
                  RowBox[{"Stdlist", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"i", ",", "1", ",", 
                 RowBox[{"Length", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"ga", ",", 
             RowBox[{"Show", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"Ninit", "*", 
                   RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}]}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (absolute)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "Ninit"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], " ", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"Ninit", "*", 
                   RowBox[{"s2", "[", 
                    RowBox[{"[", "3", "]"}], "]"}]}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (absolute)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "Ninit"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
                RowBox[{"ErrorBarPlots`ErrorListPlot", "[", 
                 RowBox[{"tab", ",", 
                  RowBox[{"PlotMarkers", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], 
               "}"}], "]"}]}], "]"}], "\[IndentingNewLine]", ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"ga", ",", 
              RowBox[{"Show", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Plot", "[", 
                  RowBox[{
                   RowBox[{"Ninit", "*", 
                    RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}]}], ",", 
                   RowBox[{"{", 
                    RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                   RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                   RowBox[{"BaseStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                   RowBox[{"FrameLabel", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (absolute)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                   RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                   RowBox[{"PlotRange", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "Ninit"}], "}"}]}], "}"}]}], ",", 
                   RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{"Plot", "[", 
                  RowBox[{
                   RowBox[{"Ninit", "*", 
                    RowBox[{"s2", "[", 
                    RowBox[{"[", "3", "]"}], "]"}]}], ",", 
                   RowBox[{"{", 
                    RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                   RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                    "}"}]}], ",", 
                   RowBox[{"BaseStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                   RowBox[{"FrameLabel", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (absolute)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                   RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                   RowBox[{"PlotRange", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "Ninit"}], "}"}]}], "}"}]}], ",", 
                   RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
                 RowBox[{"ListPlot", "[", 
                  RowBox[{"expData", ",", 
                   RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                   RowBox[{"PlotMarkers", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], 
                "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"izz", ">", "1"}], ","}]}], "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"lz", ">", 
               RowBox[{"Length", "[", "rList", "]"}]}], ",", 
              RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"allDatVals", ",", "tab"}], "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", "\"\<Likelihood for this parameter setting: \>\"", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 1 \>\"", "<>", 
        RowBox[{"model1", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (black) :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{"data", ",", 
           RowBox[{"model1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc", ",", 
           "p2loc", ",", "p3loc", ",", "opt$Haz$BG$1", ",", "tEnd"}], "]"}], 
         "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 2 \>\"", "<>", 
        RowBox[{"model2", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (red)   :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{"data", ",", 
           RowBox[{"model2", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc2", ",", 
           "p2loc2", ",", "p3loc2", ",", "opt$Haz$BG$2", ",", "tEnd"}], "]"}],
          "]"}]}], "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      "ga"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQAWIQvXbrCbul+185CnVGeYPoC9OcA0D0n5NO8SD6mUvq
ZRD97fAOMG2wWV56BZCOjFohC6I11hjpgui8bZZgumPNfaZjQFpV4Qg3iFbj
U5IrPPjKsVQuH0x7zZ25bTLja8dl/LUHQbTrN76jneyvHfWW554G0c9bi/+u
ANK3UvvBNMME4eeusq8da77XvgDRNp4S7G5A+tDyI2A67xDTXncgrVR6/CCI
9tebeRpEh8mtBNMxu0X3+gNpVrd/B0G0SkS6dwiQfqJ/OBBEN1eJJYPoZME3
YHqe9aT9ILrJP/soiFbyUvrTA6Tnf7zwH0SLT/hysRdIm+28dRlEAwDhkp4v

  "]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowResultScenariosx2M", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenariosx2M", "[", 
    RowBox[{
    "data_", ",", "model1_", ",", "model2_", ",", "opt$Haz$BG$1_", ",", 
     "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", ",", "tEnd_", ",", 
     "replicates_", ",", 
     RowBox[{"mean_:", "\"\<N\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "izz"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"p1loc2", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc2", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc2", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{
              RowBox[{"model1", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", 
            RowBox[{"s2", "=", 
             RowBox[{
              RowBox[{"model2", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}],
                 "}"}], ",", "tEnd"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{
              RowBox[{"model1", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", 
            RowBox[{"s2", "=", 
             RowBox[{
              RowBox[{"model2", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}],
                 "}"}], ",", "tEnd"}], "]"}]}]}]}], "\[IndentingNewLine]", 
          "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"replData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"Transpose", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", 
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
                "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"replData", ",", 
              RowBox[{"data", "[", 
               RowBox[{"[", 
                RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mlist", "=", 
             RowBox[{"Mean", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"dlist", "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Stdlist", "=", 
             RowBox[{"StandardDeviation", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tab", "=", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"dlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                   RowBox[{"mlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", 
                 RowBox[{"ErrorBarPlots`ErrorBar", "[", 
                  RowBox[{"Stdlist", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"i", ",", "1", ",", 
                 RowBox[{"Length", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"ga", ",", 
             RowBox[{"Show", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], " ", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s2", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
                RowBox[{"ErrorBarPlots`ErrorListPlot", "[", 
                 RowBox[{"tab", ",", 
                  RowBox[{"PlotMarkers", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], 
               "}"}], "]"}]}], "]"}], "\[IndentingNewLine]", ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"ga", ",", 
              RowBox[{"Show", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Plot", "[", 
                  RowBox[{
                   RowBox[{"s1", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                   RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                   RowBox[{"BaseStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                   RowBox[{"FrameLabel", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                   RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                   RowBox[{"PlotRange", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                   RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{"Plot", "[", 
                  RowBox[{
                   RowBox[{"s2", "[", 
                    RowBox[{"[", "3", "]"}], "]"}], ",", 
                   RowBox[{"{", 
                    RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                   RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                   RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                    "}"}]}], ",", 
                   RowBox[{"BaseStyle", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                   RowBox[{"FrameLabel", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                   RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                   RowBox[{"PlotRange", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                   RowBox[{"PlotStyle", "\[Rule]", 
                    RowBox[{"Directive", "[", 
                    RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
                 RowBox[{"ListPlot", "[", 
                  RowBox[{"expData", ",", 
                   RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                   RowBox[{"PlotMarkers", "\[Rule]", 
                    RowBox[{"{", 
                    RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], 
                "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"izz", ">", "1"}], ","}]}], "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"lz", ">", 
               RowBox[{"Length", "[", "rList", "]"}]}], ",", 
              RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", "\"\<Likelihood for this parameter setting: \>\"", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 1 \>\"", "<>", 
        RowBox[{"model1", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (black) :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{"data", ",", 
           RowBox[{"model1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc", ",", 
           "p2loc", ",", "p3loc", ",", "opt$Haz$BG$1", ",", "tEnd"}], "]"}], 
         "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 2 \>\"", "<>", 
        RowBox[{"model2", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (red)   :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{"data", ",", 
           RowBox[{"model2", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc2", ",", 
           "p2loc2", ",", "p3loc2", ",", "opt$Haz$BG$2", ",", "tEnd"}], "]"}],
          "]"}]}], "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      "ga"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.590138358272177*^9, 3.590138458829232*^9}, {
   3.5901384981121597`*^9, 3.5901386180871563`*^9}, {3.5901395471646605`*^9, 
   3.590139549773921*^9}, {3.5901442169906235`*^9, 3.590144237261029*^9}, {
   3.590144361583515*^9, 3.5901443618035192`*^9}, {3.590205462989811*^9, 
   3.590205534128924*^9}, {3.591080177064227*^9, 3.591080179472468*^9}, {
   3.6247054608004503`*^9, 3.62470554790816*^9}, {3.627830824467562*^9, 
   3.6278308754266577`*^9}, {3.6278947636100955`*^9, 3.627894764481183*^9}, {
   3.639226168611328*^9, 3.6392261799364605`*^9}, {3.6392264247589397`*^9, 
   3.6392264301454782`*^9}, {3.639229928086234*^9, 3.6392299662330484`*^9}, {
   3.639230044786903*^9, 3.639230045284953*^9}, {3.639246312679121*^9, 
   3.6392463519460473`*^9}, {3.6392556432295094`*^9, 3.639255694099596*^9}, {
   3.639255832702455*^9, 3.6392558393771224`*^9}, {3.6392802623778715`*^9, 
   3.639280273321966*^9}, 3.639281994721388*^9}],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.6392298632227488`*^9, 3.639229879455372*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowResultScenariosx2M4p", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenariosx2M4p", "[", 
    RowBox[{
    "data_", ",", "model1_", ",", "model2_", ",", "opt$Haz$BG$1_", ",", 
     "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", ",", "tEnd_", ",", 
     "replicates_", ",", 
     RowBox[{"mean_:", "\"\<N\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "izz"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"p1loc2", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p2loc2", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p3loc2", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p4loc2", "=", 
          RowBox[{"p4", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "\[Equal]", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{
              RowBox[{"model1", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", 
            RowBox[{"s2", "=", 
             RowBox[{
              RowBox[{"model2", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2", 
                 ",", "p4loc2"}], "}"}], ",", "tEnd"}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{
              RowBox[{"model1", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", 
            RowBox[{"s2", "=", 
             RowBox[{
              RowBox[{"model2", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2", 
                 ",", "p4loc2"}], "}"}], ",", "tEnd"}], "]"}]}]}]}], "]"}], 
         "\[IndentingNewLine]", ";", "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"replData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"Transpose", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", 
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
                "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"replData", ",", 
              RowBox[{"data", "[", 
               RowBox[{"[", 
                RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mlist", "=", 
             RowBox[{"Mean", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"dlist", "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Stdlist", "=", 
             RowBox[{"StandardDeviation", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tab", "=", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"dlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                   RowBox[{"mlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", 
                 RowBox[{"ErrorBarPlots`ErrorBar", "[", 
                  RowBox[{"Stdlist", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"i", ",", "1", ",", 
                 RowBox[{"Length", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"ga", ",", 
             RowBox[{"Show", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], " ", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s2", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
                RowBox[{"ErrorBarPlots`ErrorListPlot", "[", "tab", "]"}]}], 
               "}"}], "]"}]}], "]"}], "\[IndentingNewLine]", ",", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"ga", ",", 
             RowBox[{"Show", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], " ", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s2", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
                RowBox[{"ListPlot", "[", 
                 RowBox[{"expData", ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotMarkers", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{"Automatic", ",", "8"}], "}"}]}]}], "]"}]}], 
               "}"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"izz", ">", "1"}], ","}]}], "*)"}], "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"lz", ">", 
            RowBox[{"Length", "[", "rList", "]"}]}], ",", 
           RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", "\"\<Likelihood for this parameter setting: \>\"", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 1 \>\"", "<>", 
        RowBox[{"model1", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (black) :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", " ", 
          RowBox[{"data", ",", 
           RowBox[{"model1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc", ",", 
           "p2loc", ",", "p3loc", ",", "opt$Haz$BG$1", ",", "tEnd"}], "]"}], 
         "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 2 \>\"", "<>", 
        RowBox[{"model2", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (red)   :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments2p", "[", " ", 
          RowBox[{"data", ",", 
           RowBox[{"model2", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc2", ",", 
           "p2loc2", ",", "p3loc2", ",", "p4loc2", ",", "opt$Haz$BG$2", ",", 
           "tEnd"}], "]"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", "ga"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.6247055533447037`*^9, 3.6247055717935486`*^9}, {
   3.6247056058229513`*^9, 3.6247056093193007`*^9}, {3.6247057259879665`*^9, 
   3.6247057298983574`*^9}, {3.6247058300073676`*^9, 3.624705831247492*^9}, {
   3.6247069295443106`*^9, 3.6247069945148067`*^9}, 3.624948719387402*^9, {
   3.624948749678008*^9, 3.6249488940308948`*^9}, {3.6249489888127904`*^9, 
   3.624949011434243*^9}, {3.6249492909098325`*^9, 3.624949306390142*^9}, {
   3.624949359551205*^9, 3.624949448502984*^9}, {3.6249494867437487`*^9, 
   3.6249494889037924`*^9}, {3.62494957179545*^9, 3.6249495751365166`*^9}, {
   3.6249496227374687`*^9, 3.624949627947573*^9}, {3.6249496735384846`*^9, 
   3.624949681208638*^9}, {3.62494985881219*^9, 3.6249498589621935`*^9}, {
   3.6273787909563246`*^9, 3.62737882081931*^9}, {3.627379068010027*^9, 
   3.6273790736365895`*^9}, {3.6273933434020405`*^9, 3.627393420832815*^9}, {
   3.627396080656309*^9, 3.627396081586318*^9}, {3.6278308958416986`*^9, 
   3.6278309411732316`*^9}, {3.6278947695836926`*^9, 3.627894771383873*^9}, 
   3.6301449219613595`*^9, 3.630158422852244*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "ShowResultScenariosx3M", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ShowResultScenariosx3M", "[", 
    RowBox[{
    "data_", ",", "model1_", ",", "model2_", ",", "model3_", ",", 
     "opt$Haz$BG$1_", ",", "opt$Haz$BG$2_", ",", "opt$Haz$BG$3_", ",", 
     "optRes1_", ",", "optRes2_", ",", "optRes3_", ",", "tEnd_", ",", 
     "replicates_", ",", 
     RowBox[{"mean_:", "\"\<N\>\""}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "izz"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"ga", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "replicates", "]"}], "\[Equal]", "Integer"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"rList", "=", 
         RowBox[{"Table", "[", 
          RowBox[{"replicates", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{
              RowBox[{"Length", "[", 
               RowBox[{"data", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}], "+", "replicates"}], 
             ",", "replicates"}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"rList", "=", "replicates"}], ",", 
        RowBox[{"rList", "=", "replicates"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IFform", "=", 
       RowBox[{"TrueQ", "[", " ", 
        RowBox[{
         RowBox[{"Head", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}], "\[Equal]", 
         "InterpolatingFunction"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lz", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"izz", "=", "1"}], ",", 
        RowBox[{"izz", "<=", 
         RowBox[{"Length", "[", 
          RowBox[{"data", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ",", 
        RowBox[{"izz", "+=", 
         RowBox[{"rList", "[", 
          RowBox[{"[", "lz", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"izz", ">", "1"}], ",", 
           RowBox[{"lz", "++"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
          RowBox[{"data", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"p1loc", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p2loc", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p3loc", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes1", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p1loc2", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p2loc2", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p3loc2", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p4loc2", "=", 
          RowBox[{"p4", "/.", 
           RowBox[{"optRes2", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"p1loc3", "=", 
          RowBox[{"p1", "/.", 
           RowBox[{"optRes3", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p2loc3", "=", 
          RowBox[{"p2", "/.", 
           RowBox[{"optRes3", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p3loc3", "=", 
          RowBox[{"p3", "/.", 
           RowBox[{"optRes3", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p4loc3", "=", 
          RowBox[{"p4", "/.", 
           RowBox[{"optRes3", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", " ", 
         RowBox[{"p5loc3", "=", 
          RowBox[{"p5", "/.", 
           RowBox[{"optRes3", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"IFform", "==", "True"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{
              RowBox[{"model1", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"s2", "=", 
             RowBox[{
              RowBox[{"model2", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2", 
                 ",", "p4loc2"}], "}"}], ",", "tEnd"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"s3", "=", 
             RowBox[{
              RowBox[{"model3", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", 
               RowBox[{"Cw$temp", "[", "t", "]"}], ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$3", ",", "p1loc3", ",", "p2loc3", ",", "p3loc3", 
                 ",", "p4loc3", ",", "p5loc3"}], "}"}], ",", "tEnd"}], 
              "]"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s1", "=", 
             RowBox[{
              RowBox[{"model1", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
                "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"s2", "=", 
             RowBox[{
              RowBox[{"model2", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2", 
                 ",", "p4loc2"}], "}"}], ",", "tEnd"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"s3", "=", 
             RowBox[{
              RowBox[{"model3", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "[", 
              RowBox[{"0", ",", "Cw$temp", ",", 
               RowBox[{"{", 
                RowBox[{
                "opt$Haz$BG$3", ",", "p1loc3", ",", "p2loc3", ",", "p3loc3", 
                 ",", "p4loc3", ",", "p5loc3"}], "}"}], ",", "tEnd"}], 
              "]"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"expData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"replData", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"yuy", "=", "0"}], ",", 
           RowBox[{"yuy", "<", 
            RowBox[{"rList", "[", 
             RowBox[{"[", "lz", "]"}], "]"}]}], ",", 
           RowBox[{"yuy", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"Transpose", "[", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"data", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", 
                 RowBox[{"data", "[", 
                  RowBox[{"[", 
                   RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], 
                "}"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"replData", ",", 
              RowBox[{"data", "[", 
               RowBox[{"[", 
                RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"mlist", "=", 
             RowBox[{"Mean", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"dlist", "=", 
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Stdlist", "=", 
             RowBox[{"StandardDeviation", "[", "replData", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tab", "=", 
             RowBox[{"Table", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"dlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                   RowBox[{"mlist", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", 
                 RowBox[{"ErrorBarPlots`ErrorBar", "[", 
                  RowBox[{"Stdlist", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"i", ",", "1", ",", 
                 RowBox[{"Length", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"mean", "==", "\"\<Y\>\""}], ",", "\[IndentingNewLine]", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"ga", ",", 
             RowBox[{"Show", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], " ", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s2", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s3", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Blue"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Blue", "}"}], "]"}]}]}], "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"ErrorBarPlots`ErrorListPlot", "[", "tab", "]"}]}], 
               "}"}], "]"}]}], "]"}], "\[IndentingNewLine]", 
           "\[IndentingNewLine]", ",", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"ga", ",", 
             RowBox[{"Show", "[", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s1", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Black", "}"}], "]"}]}]}], " ", "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s2", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Plot", "[", 
                 RowBox[{
                  RowBox[{"s3", "[", 
                   RowBox[{"[", "3", "]"}], "]"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Blue"}], 
                    "}"}]}], ",", 
                  RowBox[{"BaseStyle", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                    RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
                  RowBox[{"Frame", "\[Rule]", "True"}], ",", 
                  RowBox[{"FrameLabel", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    "\"\<time (days)\>\"", ",", 
                    "\"\<Survival over time (%)\>\"", ",", 
                    RowBox[{"\"\<Exposure \>\"", "<>", 
                    RowBox[{"ToString", "[", 
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                    "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
                  RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
                  RowBox[{"PlotRange", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
                  RowBox[{"PlotStyle", "\[Rule]", 
                   RowBox[{"Directive", "[", 
                    RowBox[{"{", "Blue", "}"}], "]"}]}]}], "]"}], ",", 
                RowBox[{"ListPlot", "[", 
                 RowBox[{"expData", ",", 
                  RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
                  RowBox[{"PlotMarkers", "\[Rule]", 
                   RowBox[{"{", 
                    RowBox[{"Automatic", ",", "8"}], "}"}]}]}], "]"}]}], 
               "}"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"izz", ">", "1"}], ","}]}], "*)"}], "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"lz", ">", 
            RowBox[{"Length", "[", "rList", "]"}]}], ",", 
           RowBox[{"Break", "[", "]"}]}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", "\"\<Likelihood for this parameter setting: \>\"", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 1 \>\"", "<>", 
        RowBox[{"model1", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (black) :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments", "[", 
          RowBox[{"data", ",", 
           RowBox[{"model1", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc", ",", 
           "p2loc", ",", "p3loc", ",", "opt$Haz$BG$1", ",", "tEnd"}], "]"}], 
         "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 2 \>\"", "<>", 
        RowBox[{"model2", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (red)   :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments2p", "[", 
          RowBox[{"data", ",", 
           RowBox[{"model2", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc2", ",", 
           "p2loc2", ",", "p3loc2", ",", "p4loc2", ",", "opt$Haz$BG$2", ",", 
           "tEnd"}], "]"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Parameter set 3 \>\"", "<>", 
        RowBox[{"model3", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "<>", "\"\< (blue)   :\>\"", "<>", 
        RowBox[{"ToString", "[", 
         RowBox[{"goalFunc$treatments5p", "[", 
          RowBox[{"data", ",", 
           RowBox[{"model3", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", "0", ",", "p1loc3", ",", 
           "p2loc3", ",", "p3loc3", ",", "p4loc3", ",", "p5loc3", ",", 
           "opt$Haz$BG$3", ",", "tEnd"}], "]"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", "ga"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6247055533447037`*^9, 3.6247055717935486`*^9}, {
   3.6247056058229513`*^9, 3.6247056093193007`*^9}, {3.6247057259879665`*^9, 
   3.6247057298983574`*^9}, {3.6247058300073676`*^9, 3.624705831247492*^9}, {
   3.6247069295443106`*^9, 3.6247069945148067`*^9}, 3.624948719387402*^9, {
   3.624948749678008*^9, 3.6249488940308948`*^9}, {3.6249489888127904`*^9, 
   3.624949011434243*^9}, {3.6249492909098325`*^9, 3.624949306390142*^9}, {
   3.624949359551205*^9, 3.624949448502984*^9}, {3.6249494867437487`*^9, 
   3.6249494889037924`*^9}, {3.62494957179545*^9, 3.6249495751365166`*^9}, {
   3.6249496227374687`*^9, 3.624949627947573*^9}, {3.6249496735384846`*^9, 
   3.624949681208638*^9}, {3.62494985881219*^9, 3.6249498589621935`*^9}, {
   3.6273787909563246`*^9, 3.62737882081931*^9}, {3.627379068010027*^9, 
   3.6273790736365895`*^9}, {3.6273933434020405`*^9, 3.627393399392601*^9}, {
   3.6273934302529097`*^9, 3.627393596694574*^9}, {3.6273961735152454`*^9, 
   3.6273961747852583`*^9}, 3.6273963081366043`*^9, {3.627396441069935*^9, 
   3.6273964464599895`*^9}, {3.6273965215047436`*^9, 
   3.6273965217147455`*^9}, {3.627396566425193*^9, 3.627396619555724*^9}, {
   3.627396684416373*^9, 3.6273966901564302`*^9}, {3.6277087078626857`*^9, 
   3.627708709534853*^9}, {3.6277087438252816`*^9, 3.6277087520241013`*^9}, {
   3.6277097036022496`*^9, 3.6277097037082605`*^9}, {3.6277985907473907`*^9, 
   3.6277985954348593`*^9}, 3.6278047106553855`*^9, {3.627830959815096*^9, 
   3.627831022793393*^9}, {3.627894776616396*^9, 3.627894777234458*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"exportFit", "[", 
    RowBox[{
    "data_", ",", "specName_", ",", "compName_", ",", "expType_", ",", 
     "optResSD_", ",", "parConfSD_", ",", "optResIT_", ",", "parConfIT_", ",",
      "BGmort_", ",", "replicates_", ",", "tEnd_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"timeSeries", ",", "pic", ",", "graph", ",", "mid"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"output", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", "specName", "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", "compName", "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", "expType", "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", "\"\<SD-model: (red lines in Image)\>\"", "}"}]}], "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{"\"\<Likelihood\>\"", ",", 
          RowBox[{"optResSD", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Parameter values\>\"", ",", "\"\<lower CL\>\"", ",", 
          "\"\<best\>\"", ",", "\"\<upper CL\>\""}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{
         "\"\<BG-mortality\>\"", ",", "\"\<\>\"", ",", "BGmort", ",", 
          "\"\<constant\>\""}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{"\"\<dominant kD\>\"", ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "1", ",", "1"}], "]"}], "]"}], ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}], ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "1", ",", "3"}], "]"}], "]"}]}], "}"}]}], "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{"\"\<killing rate (kk)\>\"", ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "2", ",", "1"}], "]"}], "]"}], ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "2", ",", "2"}], "]"}], "]"}], ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "2", ",", "3"}], "]"}], "]"}]}], "}"}]}], "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{"\"\<threshold (z)\>\"", ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "3", ",", "1"}], "]"}], "]"}], ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "3", ",", "2"}], "]"}], "]"}], ",", 
          RowBox[{"parConfSD", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "3", ",", "3"}], "]"}], "]"}]}], "}"}]}], "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", "\"\<IT-model:(black lines in Image)\>\"", "}"}]}], 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{"\"\<Likelihood\>\"", ",", 
          RowBox[{"optResIT", "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Parameter values\>\"", ",", "\"\<lower CL\>\"", ",", 
          "\"\<best\>\"", ",", "\"\<upper CL\>\""}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{
         "\"\<BG-mortality\>\"", ",", "\"\<\>\"", ",", "BGmort", ",", 
          "\"\<constant\>\""}], "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{"\"\<dominant kD\>\"", ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "1", ",", "1"}], "]"}], "]"}], ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}], ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "1", ",", "3"}], "]"}], "]"}]}], "}"}]}], "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{"\"\<alpha\>\"", ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "2", ",", "1"}], "]"}], "]"}], ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "2", ",", "2"}], "]"}], "]"}], ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "2", ",", "3"}], "]"}], "]"}]}], "}"}]}], "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"output", ",", 
        RowBox[{"{", 
         RowBox[{"\"\<beta\>\"", ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "3", ",", "1"}], "]"}], "]"}], ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "3", ",", "2"}], "]"}], "]"}], ",", 
          RowBox[{"parConfIT", "[", 
           RowBox[{"[", 
            RowBox[{"1", ",", "3", ",", "3"}], "]"}], "]"}]}], "}"}]}], "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"timeSeries", "=", 
       RowBox[{"createModelTimeSeries", "[", 
        RowBox[{
        "data", ",", "BGmort", ",", "parConfSD", ",", "parConfIT", ",", 
         "tEnd", ",", "replicates"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"pic", "=", 
       RowBox[{"ShowResultScenariosx2M", "[", 
        RowBox[{"data", ",", 
         RowBox[{"{", 
          RowBox[{"ITmodel", ",", "\"\<IT\>\""}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"SDmodel", ",", "\"\<SD\>\""}], "}"}], ",", "BGmort", ",", 
         "BGmort", ",", "optResIT", ",", "optResSD", ",", "replicates", ",", 
         "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"mid", "=", 
       RowBox[{"Quotient", "[", 
        RowBox[{
         RowBox[{"Length", "[", "pic", "]"}], ",", "2"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"graph", "=", 
       RowBox[{"Show", "[", 
        RowBox[{"GraphicsGrid", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"pic", "[", 
            RowBox[{"[", 
             RowBox[{"1", ";;", "mid"}], "]"}], "]"}], ",", 
           RowBox[{"pic", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"mid", "+", "1"}], ";;", 
              RowBox[{"Length", "[", "pic", "]"}]}], "]"}], "]"}]}], "}"}], 
         "]"}], "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"Export", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"NotebookDirectory", "[", "]"}], "<>", "\"\<\\\\\>\"", "<>", 
         "specName", "<>", "\"\<_\>\"", "<>", "compName", "<>", "\"\<_\>\"", "<>",
          "expType", "<>", "\"\<.xlsx\>\""}], ",", 
        RowBox[{"{", 
         RowBox[{"graph", ",", 
          RowBox[{"Prepend", "[", 
           RowBox[{"timeSeries", ",", "output"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"\"\<Images\>\"", ",", "\"\<Sheets\>\""}], "}"}], "}"}]}], 
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBmIQfSZun2flvleOLhHp/iC67PrvUBDtNWVOPoguvpda
DKIDdkxZBqIzf7tZ1AFpf6alXiA65mdOCIg+lt8CplmvS6eD6DzjNRkgOktv
y0cQ/cHWlqEeSF/T3cMOopXmPQLTm96y8IDomoDdciDaJGySMoj2Z5miCqKf
3ftkAKL38G03BNGRN3YngOiqz6+TQPS2j233QPQ6L6/7INpDy0Zixf5Xjlti
08F05MHM6cKnXjnOZb2yEETHNj1aBaL9XCtWg2i54zzbQPSliPk7QfQSvvyj
INrAtesUiJ7gdbZIHUj7BPRWg2gA1bSiSQ==
  "]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualisation: Multi-image ", "Title",
 CellChangeTimes->{{3.62788852853265*^9, 3.6278885356763644`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"transparentLabels", "[", "range_", "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"Style", "[", 
       RowBox[{
        RowBox[{"ToString", "[", "i", "]"}], ",", "Black", ",", 
        RowBox[{"Opacity", "[", "0", "]"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.03", ",", "0"}], "}"}], ",", "Black"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"range", "[", 
       RowBox[{"[", "1", "]"}], "]"}], ",", 
      RowBox[{"range", "[", 
       RowBox[{"[", "2", "]"}], "]"}], ",", 
      RowBox[{"range", "[", 
       RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"visibleLabels", "[", "range_", "]"}], ":=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"Style", "[", 
       RowBox[{
        RowBox[{"ToString", "[", "i", "]"}], ",", "Black"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.03", ",", "0"}], "}"}], ",", "Black"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"range", "[", 
       RowBox[{"[", "1", "]"}], "]"}], ",", 
      RowBox[{"range", "[", 
       RowBox[{"[", "2", "]"}], "]"}], ",", 
      RowBox[{"range", "[", 
       RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}], "]"}]}]}], "Input",
 CellChangeTimes->{3.6085555575238256`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ShowSingleResultScenariosx2Res$PL", "[", 
    RowBox[{
    "data_", ",", "izz_", ",", "model_", ",", "title_", ",", "opt$Haz$BG$1_", 
     ",", "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", ",", "tEnd_", 
     ",", "replicates_", ",", 
     RowBox[{"ec50_:", "0"}], ",", 
     RowBox[{"slope_:", "0"}], ",", 
     RowBox[{"scenario_:", "\"\<\>\""}], ",", 
     RowBox[{"duration_:", "0"}], ",", 
     RowBox[{"specIx_:", "0"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "pic"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"xrange", "=", 
       RowBox[{"{", 
        RowBox[{"0", ",", "tEnd", ",", 
         RowBox[{"tEnd", "/", "5"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"yrange", "=", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1", ",", "0.2"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
       RowBox[{"data", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"p1loc", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p2loc", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p3loc", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"s1", "=", 
       RowBox[{"model", "[", 
        RowBox[{"0", ",", "Cw$temp", ",", 
         RowBox[{"{", 
          RowBox[{"opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
          "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p1loc2", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p2loc2", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p3loc2", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"s2", "=", 
       RowBox[{"model", "[", 
        RowBox[{"0", ",", "Cw$temp", ",", 
         RowBox[{"{", 
          RowBox[{
          "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}], 
          "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"expData", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"yuy", "=", "0"}], ",", 
        RowBox[{"yuy", "<", "replicates"}], ",", 
        RowBox[{"yuy", "++"}], ",", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"expData", ",", 
          RowBox[{"Transpose", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "}"}], 
           "]"}]}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ec50", ">", "0"}], ",", "\[IndentingNewLine]", 
        "\[IndentingNewLine]", 
        RowBox[{"pic", "=", 
         RowBox[{"Show", "[", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s1", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"transparentLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"visibleLabels", "[", "yrange", "]"}], ",", "None", 
                 ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                "\"\<\>\"", ",", "\"\<Survival over time (%)\>\"", ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s2", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"transparentLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"visibleLabels", "[", "yrange", "]"}], ",", "None", 
                 ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                "\"\<\>\"", ",", "\"\<Survival over time (%)\>\"", ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"ListPlot", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"MarkersScaled", "[", 
                    RowBox[{"0.05", ",", "MyMarkers"}], "]"}], "[", 
                   RowBox[{"[", 
                    RowBox[{"-", "#"}], "]"}], "]"}], "&"}], "/@", 
                 RowBox[{"Range", "[", "replicates", "]"}]}], ")"}]}]}], 
             "]"}], ",", "\[IndentingNewLine]", 
            RowBox[{"ListPlot", "[", 
             RowBox[{
              RowBox[{"Flatten", "[", 
               RowBox[{
                RowBox[{"Join", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"t", ",", 
                    RowBox[{"logLogistic$SOT", "[", 
                    RowBox[{
                    "t", ",", "ec50", ",", "slope", ",", "scenario", ",", 
                    "specIx", ",", "duration"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "7", ",", "7.9", ",", "0.1"}], "}"}]}], 
                    "]"}], ",", 
                   RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"t", ",", 
                    RowBox[{"logLogistic$SOT", "[", 
                    RowBox[{
                    "t", ",", "ec50", ",", "slope", ",", "scenario", ",", 
                    "specIx", ",", "duration"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "21", ",", "21.9", ",", "0.1"}], 
                    "}"}]}], "]"}]}], "}"}], "]"}], ",", "1"}], "]"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"Automatic", ",", "Medium"}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Green", "}"}], "]"}]}]}], "]"}]}], "}"}], 
          "]"}]}], "\[IndentingNewLine]", ",", 
        RowBox[{"(*", " ", "else", " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"pic", "=", 
         RowBox[{"Show", "[", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s1", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"transparentLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"visibleLabels", "[", "yrange", "]"}], ",", "None", 
                 ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                "\"\<\>\"", ",", "\"\<Survival over time (%)\>\"", ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s2", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"transparentLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"visibleLabels", "[", "yrange", "]"}], ",", "None", 
                 ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                "\"\<\>\"", ",", "\"\<Survival over time (%)\>\"", ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
            RowBox[{"ListPlot", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], 
           "\[IndentingNewLine]", "}"}], "]"}]}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", "pic"}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.6040688737429414`*^9, 3.6040689804116073`*^9}, {
   3.604069016701236*^9, 3.604069029658531*^9}, {3.604069231931757*^9, 
   3.6040692326198254`*^9}, 3.6040694173903008`*^9, 3.608555557533826*^9, {
   3.6085557315355663`*^9, 3.6085557653869047`*^9}, {3.608555888218133*^9, 
   3.608555960468856*^9}, 3.608555997109222*^9, 3.6085560823900747`*^9, {
   3.6085561628908796`*^9, 3.6085561719609704`*^9}, {3.608556206411315*^9, 
   3.608556276192013*^9}, {3.6085565660849113`*^9, 3.608556668475936*^9}, {
   3.608557270322954*^9, 3.608557280443055*^9}, 3.608558182993081*^9, {
   3.608558314094392*^9, 3.608558318654437*^9}, {3.6085592088943396`*^9, 
   3.608559275345004*^9}, {3.6085593176754274`*^9, 3.608559339035641*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ShowSingleResultScenariosx2Res$PBL", "[", 
    RowBox[{
    "data_", ",", "izz_", ",", "model_", ",", "title_", ",", "opt$Haz$BG$1_", 
     ",", "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", ",", "tEnd_", 
     ",", "replicates_", ",", 
     RowBox[{"ec50_:", "0"}], ",", 
     RowBox[{"slope_:", "0"}], ",", 
     RowBox[{"scenario_:", "\"\<\>\""}], ",", 
     RowBox[{"duration_:", "0"}], ",", 
     RowBox[{"specIx_:", "0"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "pic"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"xrange", "=", 
       RowBox[{"{", 
        RowBox[{"0", ",", "tEnd", ",", 
         RowBox[{"tEnd", "/", "5"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"yrange", "=", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1", ",", "0.2"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
       RowBox[{"data", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"p1loc", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p2loc", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p3loc", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"s1", "=", 
       RowBox[{"model", "[", 
        RowBox[{"0", ",", "Cw$temp", ",", 
         RowBox[{"{", 
          RowBox[{"opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
          "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p1loc2", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p2loc2", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p3loc2", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"s2", "=", 
       RowBox[{"model", "[", 
        RowBox[{"0", ",", "Cw$temp", ",", 
         RowBox[{"{", 
          RowBox[{
          "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}], 
          "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"expData", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"yuy", "=", "0"}], ",", 
        RowBox[{"yuy", "<", "replicates"}], ",", 
        RowBox[{"yuy", "++"}], ",", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"expData", ",", 
          RowBox[{"Transpose", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "}"}], 
           "]"}]}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ec50", ">", "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{"pic", "=", 
         RowBox[{"Show", "[", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s1", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"visibleLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"visibleLabels", "[", "yrange", "]"}], ",", "None", 
                 ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                 ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s2", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"visibleLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"visibleLabels", "[", "yrange", "]"}], ",", "None", 
                 ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                 ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"ListPlot", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"MarkersScaled", "[", 
                    RowBox[{"0.05", ",", "MyMarkers"}], "]"}], "[", 
                   RowBox[{"[", 
                    RowBox[{"-", "#"}], "]"}], "]"}], "&"}], "/@", 
                 RowBox[{"Range", "[", "replicates", "]"}]}], ")"}]}]}], 
             "]"}], ",", "\[IndentingNewLine]", 
            RowBox[{"ListPlot", "[", 
             RowBox[{
              RowBox[{"Flatten", "[", 
               RowBox[{
                RowBox[{"Join", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"t", ",", 
                    RowBox[{"logLogistic$SOT", "[", 
                    RowBox[{
                    "t", ",", "ec50", ",", "slope", ",", "scenario", ",", 
                    "specIx", ",", "duration"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "7", ",", "7.9", ",", "0.1"}], "}"}]}], 
                    "]"}], ",", 
                   RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"t", ",", 
                    RowBox[{"logLogistic$SOT", "[", 
                    RowBox[{
                    "t", ",", "ec50", ",", "slope", ",", "scenario", ",", 
                    "specIx", ",", "duration"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "21", ",", "21.9", ",", "0.1"}], 
                    "}"}]}], "]"}]}], "}"}], "]"}], ",", "1"}], "]"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"Automatic", ",", "Medium"}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Green", "}"}], "]"}]}]}], "]"}]}], 
           "\[IndentingNewLine]", "}"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"pic", "=", 
         RowBox[{"Show", "[", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s1", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"visibleLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"visibleLabels", "[", "yrange", "]"}], ",", "None", 
                 ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                 ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s2", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"visibleLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"visibleLabels", "[", "yrange", "]"}], ",", "None", 
                 ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                "\"\<time (days)\>\"", ",", "\"\<Survival over time (%)\>\"", 
                 ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"ListPlot", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], 
           "\[IndentingNewLine]", "}"}], "]"}]}]}], "\[IndentingNewLine]", 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "pic"}]}], 
    "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.604069423893951*^9, 3.6040694641529765`*^9}, 
   3.608555557533826*^9, 3.608556691706168*^9, {3.6085567244574957`*^9, 
   3.6085567937581882`*^9}, {3.608557260222853*^9, 3.608557266312914*^9}, 
   3.608558179283044*^9, {3.608558303094282*^9, 3.6085583087343383`*^9}, 
   3.60855935691582*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ShowSingleResultScenariosx2Res$PB", "[", 
    RowBox[{
    "data_", ",", "izz_", ",", "model_", ",", "title_", ",", "opt$Haz$BG$1_", 
     ",", "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", ",", "tEnd_", 
     ",", "replicates_", ",", 
     RowBox[{"ec50_:", "0"}], ",", 
     RowBox[{"slope_:", "0"}], ",", 
     RowBox[{"scenario_:", "\"\<\>\""}], ",", 
     RowBox[{"duration_:", "0"}], ",", 
     RowBox[{"specIx_:", "0"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
       "p3loc2", ",", "rList", ",", "lz", ",", "pic"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"xrange", "=", 
       RowBox[{"{", 
        RowBox[{"0", ",", "tEnd", ",", 
         RowBox[{"tEnd", "/", "5"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"yrange", "=", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1", ",", "0.2"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
       RowBox[{"data", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"p1loc", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p2loc", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p3loc", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes1", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"s1", "=", 
       RowBox[{"model", "[", 
        RowBox[{"0", ",", "Cw$temp", ",", 
         RowBox[{"{", 
          RowBox[{"opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
          "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"p1loc2", "=", 
       RowBox[{"p1", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p2loc2", "=", 
       RowBox[{"p2", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"p3loc2", "=", 
       RowBox[{"p3", "/.", 
        RowBox[{"optRes2", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
      RowBox[{"s2", "=", 
       RowBox[{"model", "[", 
        RowBox[{"0", ",", "Cw$temp", ",", 
         RowBox[{"{", 
          RowBox[{
          "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}], 
          "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"expData", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"yuy", "=", "0"}], ",", 
        RowBox[{"yuy", "<", "replicates"}], ",", 
        RowBox[{"yuy", "++"}], ",", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"expData", ",", 
          RowBox[{"Transpose", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"data", "[", 
              RowBox[{"[", "1", "]"}], "]"}], ",", 
             RowBox[{"data", "[", 
              RowBox[{"[", 
               RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "}"}], 
           "]"}]}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ec50", ">", "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{"pic", "=", 
         RowBox[{"Show", "[", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s1", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"visibleLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"transparentLabels", "[", "yrange", "]"}], ",", 
                 "None", ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"\"\<time (days)\>\"", ",", "\"\<\>\"", ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s2", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"visibleLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"transparentLabels", "[", "yrange", "]"}], ",", 
                 "None", ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"\"\<time (days)\>\"", ",", "\"\<\>\"", ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"ListPlot", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{
                   RowBox[{"MarkersScaled", "[", 
                    RowBox[{"0.05", ",", "MyMarkers"}], "]"}], "[", 
                   RowBox[{"[", 
                    RowBox[{"-", "#"}], "]"}], "]"}], "&"}], "/@", 
                 RowBox[{"Range", "[", "replicates", "]"}]}], ")"}]}]}], 
             "]"}], ",", "\[IndentingNewLine]", 
            RowBox[{"ListPlot", "[", 
             RowBox[{
              RowBox[{"Flatten", "[", 
               RowBox[{
                RowBox[{"Join", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"t", ",", 
                    RowBox[{"logLogistic$SOT", "[", 
                    RowBox[{
                    "t", ",", "ec50", ",", "slope", ",", "scenario", ",", 
                    "specIx", ",", "duration"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "7", ",", "7.9", ",", "0.1"}], "}"}]}], 
                    "]"}], ",", 
                   RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"t", ",", 
                    RowBox[{"logLogistic$SOT", "[", 
                    RowBox[{
                    "t", ",", "ec50", ",", "slope", ",", "scenario", ",", 
                    "specIx", ",", "duration"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "21", ",", "21.9", ",", "0.1"}], 
                    "}"}]}], "]"}]}], "}"}], "]"}], ",", "1"}], "]"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"Automatic", ",", "Medium"}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Green", "}"}], "]"}]}]}], "]"}]}], "}"}], 
          "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"pic", "=", 
         RowBox[{"Show", "[", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s1", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"visibleLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"transparentLabels", "[", "yrange", "]"}], ",", 
                 "None", ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"\"\<time (days)\>\"", ",", "\"\<\>\"", ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"Directive", "[", 
                RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Plot", "[", 
             RowBox[{
              RowBox[{"s2", "[", 
               RowBox[{"[", "3", "]"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
                "}"}]}], ",", 
              RowBox[{"BaseStyle", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                 RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
              RowBox[{"Frame", "\[Rule]", "True"}], ",", 
              RowBox[{"FrameTicks", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"visibleLabels", "[", "xrange", "]"}], ",", 
                 RowBox[{"transparentLabels", "[", "yrange", "]"}], ",", 
                 "None", ",", "None"}], "}"}]}], ",", 
              RowBox[{"FrameLabel", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"\"\<time (days)\>\"", ",", "\"\<\>\"", ",", 
                 RowBox[{"\"\<Exposure \>\"", "<>", 
                  RowBox[{"ToString", "[", 
                   RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                  "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
              RowBox[{"PlotRange", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                 RowBox[{"{", 
                  RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"ListPlot", "[", 
             RowBox[{"expData", ",", 
              RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
              RowBox[{"PlotMarkers", "\[Rule]", 
               RowBox[{"{", 
                RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], "}"}], 
          "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
       "pic"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.604069474007962*^9, 3.604069502339795*^9}, 
   3.608555557533826*^9, 3.6085566931661825`*^9, {3.6085568009582605`*^9, 
   3.6085568384586353`*^9}, {3.6085572474627256`*^9, 3.6085572553628044`*^9}, 
   3.6085581759930105`*^9, {3.608558256143812*^9, 3.608558294334194*^9}, 
   3.608559361095862*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"ShowSingleResultScenariosx2Res$P", "[", 
   RowBox[{
   "data_", ",", "izz_", ",", "model_", ",", "title_", ",", "opt$Haz$BG$1_", 
    ",", "opt$Haz$BG$2_", ",", "optRes1_", ",", "optRes2_", ",", "tEnd_", ",",
     "replicates_", ",", 
    RowBox[{"ec50_:", "0"}], ",", 
    RowBox[{"slope_:", "0"}], ",", 
    RowBox[{"scenario_:", "\"\<\>\""}], ",", 
    RowBox[{"duration_:", "0"}], ",", 
    RowBox[{"specIx_:", "0"}]}], "]"}], ":=", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "p1loc", ",", "p2loc", ",", "p3loc", ",", "p1loc2", ",", "p2loc2", ",", 
      "p3loc2", ",", "rList", ",", "lz", ",", "pic"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"xrange", "=", 
      RowBox[{"{", 
       RowBox[{"0", ",", "tEnd", ",", 
        RowBox[{"tEnd", "/", "5"}]}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"yrange", "=", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "0.2"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Cw$temp", "[", "t_", "]"}], ":=", 
      RowBox[{"data", "[", 
       RowBox[{"[", 
        RowBox[{
         RowBox[{"-", "1"}], ",", "izz"}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"p1loc", "=", 
      RowBox[{"p1", "/.", 
       RowBox[{"optRes1", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
     RowBox[{"p2loc", "=", 
      RowBox[{"p2", "/.", 
       RowBox[{"optRes1", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
     RowBox[{"p3loc", "=", 
      RowBox[{"p3", "/.", 
       RowBox[{"optRes1", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
     RowBox[{"s1", "=", 
      RowBox[{"model", "[", 
       RowBox[{"0", ",", "Cw$temp", ",", 
        RowBox[{"{", 
         RowBox[{"opt$Haz$BG$1", ",", "p1loc", ",", "p2loc", ",", "p3loc"}], 
         "}"}], ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"p1loc2", "=", 
      RowBox[{"p1", "/.", 
       RowBox[{"optRes2", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
     RowBox[{"p2loc2", "=", 
      RowBox[{"p2", "/.", 
       RowBox[{"optRes2", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
     RowBox[{"p3loc2", "=", 
      RowBox[{"p3", "/.", 
       RowBox[{"optRes2", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}]}], ";", 
     RowBox[{"s2", "=", 
      RowBox[{"model", "[", 
       RowBox[{"0", ",", "Cw$temp", ",", 
        RowBox[{"{", 
         RowBox[{
         "opt$Haz$BG$2", ",", "p1loc2", ",", "p2loc2", ",", "p3loc2"}], "}"}],
         ",", "tEnd"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"expData", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"yuy", "=", "0"}], ",", 
       RowBox[{"yuy", "<", "replicates"}], ",", 
       RowBox[{"yuy", "++"}], ",", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"expData", ",", 
         RowBox[{"Transpose", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"data", "[", 
             RowBox[{"[", "1", "]"}], "]"}], ",", 
            RowBox[{"data", "[", 
             RowBox[{"[", 
              RowBox[{"3", "+", "yuy", "+", "izz"}], "]"}], "]"}]}], "}"}], 
          "]"}]}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"ec50", ">", "0"}], ",", "\[IndentingNewLine]", 
       RowBox[{"pic", "=", 
        RowBox[{"Show", "[", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s1", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
             RowBox[{"FrameTicks", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"transparentLabels", "[", "xrange", "]"}], ",", 
                RowBox[{"transparentLabels", "[", "yrange", "]"}], ",", 
                "None", ",", "None"}], "}"}]}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", 
                RowBox[{"\"\<Exposure \>\"", "<>", 
                 RowBox[{"ToString", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                 "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Blue", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s2", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameTicks", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"transparentLabels", "[", "xrange", "]"}], ",", 
                RowBox[{"transparentLabels", "[", "yrange", "]"}], ",", 
                "None", ",", "None"}], "}"}]}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", 
                RowBox[{"\"\<Exposure \>\"", "<>", 
                 RowBox[{"ToString", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                 "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"ListPlot", "[", 
            RowBox[{"expData", ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotMarkers", "\[Rule]", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"MarkersScaled", "[", 
                   RowBox[{"0.05", ",", "MyMarkers"}], "]"}], "[", 
                  RowBox[{"[", 
                   RowBox[{"-", "#"}], "]"}], "]"}], "&"}], "/@", 
                RowBox[{"Range", "[", "replicates", "]"}]}], ")"}]}]}], "]"}],
            ",", "\[IndentingNewLine]", 
           RowBox[{"ListPlot", "[", 
            RowBox[{
             RowBox[{"Flatten", "[", 
              RowBox[{
               RowBox[{"Join", "[", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Table", "[", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{"t", ",", 
                    RowBox[{"logLogistic$SOT", "[", 
                    RowBox[{
                    "t", ",", "ec50", ",", "slope", ",", "scenario", ",", 
                    "specIx", ",", "duration"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "7", ",", "7.9", ",", "0.1"}], "}"}]}], 
                   "]"}], ",", 
                  RowBox[{"Table", "[", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{"t", ",", 
                    RowBox[{"logLogistic$SOT", "[", 
                    RowBox[{
                    "t", ",", "ec50", ",", "slope", ",", "scenario", ",", 
                    "specIx", ",", "duration"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"t", ",", "21", ",", "21.9", ",", "0.1"}], 
                    "}"}]}], "]"}]}], "}"}], "]"}], ",", "1"}], "]"}], ",", 
             RowBox[{"PlotMarkers", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"Automatic", ",", "Medium"}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Green", "}"}], "]"}]}]}], "]"}]}], "}"}], 
         "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"pic", "=", 
        RowBox[{"Show", "[", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s1", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Black"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", "\[IndentingNewLine]", 
             RowBox[{"FrameTicks", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"transparentLabels", "[", "xrange", "]"}], ",", 
                RowBox[{"transparentLabels", "[", "yrange", "]"}], ",", 
                "None", ",", "None"}], "}"}]}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", 
                RowBox[{"\"\<Exposure \>\"", "<>", 
                 RowBox[{"ToString", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                 "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Black", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Plot", "[", 
            RowBox[{
             RowBox[{"s2", "[", 
              RowBox[{"[", "3", "]"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"t", ",", "0", ",", "tEnd"}], "}"}], ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Thickness", "[", "0.01", "]"}], ",", "Red"}], 
               "}"}]}], ",", 
             RowBox[{"BaseStyle", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"FontFamily", "\[Rule]", "\"\<Arial\>\""}], ",", 
                RowBox[{"FontSize", "\[Rule]", "20"}]}], "}"}]}], ",", 
             RowBox[{"Frame", "\[Rule]", "True"}], ",", 
             RowBox[{"FrameTicks", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"transparentLabels", "[", "xrange", "]"}], ",", 
                RowBox[{"transparentLabels", "[", "yrange", "]"}], ",", 
                "None", ",", "None"}], "}"}]}], ",", 
             RowBox[{"FrameLabel", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"\"\<\>\"", ",", "\"\<\>\"", ",", 
                RowBox[{"\"\<Exposure \>\"", "<>", 
                 RowBox[{"ToString", "[", 
                  RowBox[{"data", "[", 
                   RowBox[{"[", 
                    RowBox[{"3", ",", "izz"}], "]"}], "]"}], "]"}], "<>", 
                 "\"\<\[Mu]g/L\>\""}], ",", "\"\<\>\""}], "}"}]}], ",", 
             "\[IndentingNewLine]", 
             RowBox[{"ImageSize", "\[Rule]", "400"}], ",", 
             RowBox[{"PlotRange", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0", ",", "tEnd"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
             RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Directive", "[", 
               RowBox[{"{", "Red", "}"}], "]"}]}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"ListPlot", "[", 
            RowBox[{"expData", ",", 
             RowBox[{"PlotRangeClipping", "\[Rule]", "False"}], ",", 
             RowBox[{"PlotMarkers", "\[Rule]", 
              RowBox[{"{", 
               RowBox[{"Automatic", ",", "18"}], "}"}]}]}], "]"}]}], "}"}], 
         "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "pic"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.60406903814538*^9, 3.6040690614947147`*^9}, {
   3.604069237015265*^9, 3.6040692376213255`*^9}, 3.608555557533826*^9, 
   3.6085566964862156`*^9, {3.6085568466487174`*^9, 3.608556879819049*^9}, {
   3.6085572206224575`*^9, 3.608557243692688*^9}, {3.6085581607628584`*^9, 
   3.6085581675629263`*^9}, {3.6085586110483613`*^9, 3.6085586127283783`*^9}, 
   3.6085593676859274`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SOTMultiImage", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "label_", ",", "BGA_", ",", "BGB_", ",", 
     "resA_", ",", "resB_", ",", "tEnd_", ",", "replicates_", ",", "nrSpec_", 
     ",", "scenarioIX_"}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"plots", ",", "plotLine", ",", "specIx", ",", "scenIx"}], "}"}],
      ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"plots", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"specIx", "=", "1"}], ",", 
        RowBox[{"specIx", "\[LessEqual]", "nrSpec"}], ",", 
        RowBox[{"specIx", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"plotLine", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"specIx", "<", "nrSpec"}], ",", "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"scenIx", "=", "1"}], ",", 
             RowBox[{"scenIx", "\[LessEqual]", 
              RowBox[{"Length", "[", "scenarioIX", "]"}]}], ",", 
             RowBox[{"scenIx", "++"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"scenIx", "\[Equal]", "1"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"plotLine", ",", 
                  RowBox[{"ShowSingleResultScenariosx2Res$PL", "[", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"scenarioIX", "[", 
                    RowBox[{"[", "scenIx", "]"}], "]"}], ",", "model", ",", 
                    "label", ",", 
                    RowBox[{"BGA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"BGB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", "tEnd", ",", 
                    "replicates"}], "]"}]}], "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"plotLine", ",", 
                  RowBox[{"ShowSingleResultScenariosx2Res$P", "[", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"scenarioIX", "[", 
                    RowBox[{"[", "scenIx", "]"}], "]"}], ",", "model", ",", 
                    "label", ",", 
                    RowBox[{"BGA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"BGB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", "tEnd", ",", 
                    "replicates"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
               "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"scenIx", "=", "1"}], ",", 
             RowBox[{"scenIx", "\[LessEqual]", 
              RowBox[{"Length", "[", "scenarioIX", "]"}]}], ",", 
             RowBox[{"scenIx", "++"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"scenIx", "\[Equal]", "1"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"plotLine", ",", 
                  RowBox[{"ShowSingleResultScenariosx2Res$PBL", "[", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"scenarioIX", "[", 
                    RowBox[{"[", "scenIx", "]"}], "]"}], ",", "model", ",", 
                    "label", ",", 
                    RowBox[{"BGA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"BGB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", "tEnd", ",", 
                    "replicates"}], "]"}]}], "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"plotLine", ",", 
                  RowBox[{"ShowSingleResultScenariosx2Res$PB", "[", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"scenarioIX", "[", 
                    RowBox[{"[", "scenIx", "]"}], "]"}], ",", "model", ",", 
                    "label", ",", 
                    RowBox[{"BGA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"BGB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", "tEnd", ",", 
                    "replicates"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
               "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"plots", ",", "plotLine"}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"gg", "=", 
       RowBox[{"GraphicsGrid", "[", 
        RowBox[{"plots", ",", 
         RowBox[{"ImageSize", "\[Rule]", "1800"}], ",", 
         RowBox[{"BaseStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"FontSize", "\[Rule]", "22"}], "}"}]}], ",", 
         RowBox[{"Spacings", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Scaled", "[", 
             RowBox[{"-", "0.3"}], "]"}], ",", 
            RowBox[{"Scaled", "[", 
             RowBox[{"-", "0.2"}], "]"}]}], "}"}]}]}], "]"}]}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.604116687550741*^9, 3.6041170763236146`*^9}, {
  3.6041173412411036`*^9, 3.6041173579127707`*^9}, {3.6041174299969783`*^9, 
  3.604117438997878*^9}, {3.6041175252485027`*^9, 3.6041175417691545`*^9}, {
  3.6041176003640137`*^9, 3.6041176004120183`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "SOTMultiImagePlusTWA", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SOTMultiImagePlusTWA", "[", 
    RowBox[{
    "data_", ",", "model_", ",", "label_", ",", "BGA_", ",", "BGB_", ",", 
     "resA_", ",", "resB_", ",", "tEnd_", ",", "replicates_", ",", "nrSpec_", 
     ",", "scenarioIX_", ",", "ec50s_", ",", "slopes_", ",", 
     RowBox[{"durationG_:", "0"}]}], "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "durations", ",", "duration", ",", "plots", ",", "plotLine", ",", 
       "specIx", ",", "scenIx", ",", "scenarios", ",", "scenario", ",", 
       "slope", ",", "ec50"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"plots", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"scenarios", "=", 
       RowBox[{"{", 
        RowBox[{
        "\"\<A\>\"", ",", "\"\<B\>\"", ",", "\"\<C\>\"", ",", "\"\<D\>\""}], 
        "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"durations", "=", 
       RowBox[{"{", 
        RowBox[{"2", ",", "7", ",", "14", ",", "21"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"specIx", "=", "1"}], ",", 
        RowBox[{"specIx", "\[LessEqual]", "nrSpec"}], ",", 
        RowBox[{"specIx", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"plotLine", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"scenario", "=", 
          RowBox[{"scenarios", "[", 
           RowBox[{"[", "specIx", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"ec50", "=", 
          RowBox[{"ec50s", "[", 
           RowBox[{"[", "specIx", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"slope", "=", 
          RowBox[{"slopes", "[", 
           RowBox[{"[", "specIx", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"durationG", ">", "0"}], ",", "\[IndentingNewLine]", 
           RowBox[{"duration", "=", " ", "durationG"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"duration", "=", 
            RowBox[{"durations", "[", 
             RowBox[{"[", "specIx", "]"}], "]"}]}]}], "\[IndentingNewLine]", 
          "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"specIx", "<", "nrSpec"}], ",", "\[IndentingNewLine]", 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"scenIx", "=", "1"}], ",", 
             RowBox[{"scenIx", "\[LessEqual]", 
              RowBox[{"Length", "[", "scenarioIX", "]"}]}], ",", 
             RowBox[{"scenIx", "++"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"scenIx", "\[Equal]", "1"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"plotLine", ",", 
                  RowBox[{"ShowSingleResultScenariosx2Res$PL", "[", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"scenarioIX", "[", 
                    RowBox[{"[", "scenIx", "]"}], "]"}], ",", "model", ",", 
                    "label", ",", 
                    RowBox[{"BGA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"BGB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", "tEnd", ",", 
                    "replicates", ",", "ec50", ",", "slope", ",", "scenario", 
                    ",", "duration", ",", "scenIx"}], "]"}]}], "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"plotLine", ",", 
                  RowBox[{"ShowSingleResultScenariosx2Res$P", "[", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"scenarioIX", "[", 
                    RowBox[{"[", "scenIx", "]"}], "]"}], ",", "model", ",", 
                    "label", ",", 
                    RowBox[{"BGA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"BGB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", "tEnd", ",", 
                    "replicates", ",", "ec50", ",", "slope", ",", "scenario", 
                    ",", "duration", ",", "scenIx"}], "]"}]}], "]"}]}], 
               "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", 
            "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"scenIx", "=", "1"}], ",", 
             RowBox[{"scenIx", "\[LessEqual]", 
              RowBox[{"Length", "[", "scenarioIX", "]"}]}], ",", 
             RowBox[{"scenIx", "++"}], ",", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"scenIx", "\[Equal]", "1"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"plotLine", ",", 
                  RowBox[{"ShowSingleResultScenariosx2Res$PBL", "[", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"scenarioIX", "[", 
                    RowBox[{"[", "scenIx", "]"}], "]"}], ",", "model", ",", 
                    "label", ",", 
                    RowBox[{"BGA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"BGB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", "tEnd", ",", 
                    "replicates", ",", "ec50", ",", "slope", ",", "scenario", 
                    ",", "duration", ",", "scenIx"}], "]"}]}], "]"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"AppendTo", "[", 
                 RowBox[{"plotLine", ",", 
                  RowBox[{"ShowSingleResultScenariosx2Res$PB", "[", 
                   RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"scenarioIX", "[", 
                    RowBox[{"[", "scenIx", "]"}], "]"}], ",", "model", ",", 
                    "label", ",", 
                    RowBox[{"BGA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"BGB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resA", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", 
                    RowBox[{"resB", "[", 
                    RowBox[{"[", "specIx", "]"}], "]"}], ",", "tEnd", ",", 
                    "replicates", ",", "ec50", ",", "slope", ",", "scenario", 
                    ",", "duration", ",", "scenIx"}], "]"}]}], "]"}]}], 
               "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", 
            "]"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"plots", ",", "plotLine"}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"gg", "=", 
       RowBox[{"GraphicsGrid", "[", 
        RowBox[{"plots", ",", 
         RowBox[{"ImageSize", "\[Rule]", "1800"}], ",", 
         RowBox[{"BaseStyle", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"FontSize", "\[Rule]", "22"}], "}"}]}], ",", 
         RowBox[{"Spacings", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Scaled", "[", 
             RowBox[{"-", "0.3"}], "]"}], ",", 
            RowBox[{"Scaled", "[", 
             RowBox[{"-", "0.2"}], "]"}]}], "}"}]}]}], "]"}]}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.608555650104752*^9, 3.6085556600748515`*^9}, {
  3.6085563168024187`*^9, 3.6085563554528055`*^9}, {3.6085569015292664`*^9, 
  3.6085570030702815`*^9}, {3.60855719590221*^9, 3.608557212012371*^9}, {
  3.6085574687449384`*^9, 3.6085576072763233`*^9}, {3.6085576434076853`*^9, 
  3.6085576825680766`*^9}, {3.608557723038481*^9, 3.608557742318674*^9}, {
  3.60855777893904*^9, 3.608557796109212*^9}, {3.608557850699758*^9, 
  3.608557891580167*^9}, {3.608557960100852*^9, 3.608558002711278*^9}, {
  3.6085580416716676`*^9, 3.608558043421685*^9}, {3.6085581287625384`*^9, 
  3.6085581318825693`*^9}, {3.6085584816660676`*^9, 3.608558484076092*^9}, {
  3.6086200866462665`*^9, 3.6086201295666957`*^9}}]
}, Closed]]
},
WindowSize->{1664, 948},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
TaggingRules->{"NotebookTemplateVersion" -> 1.},
FrontEndVersion->"10.0 for Microsoft Windows (64-bit) (December 4, 2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 134, 1, 81, "Subsection"],
Cell[CellGroupData[{
Cell[716, 25, 286, 6, 57, "Subsubtitle"],
Cell[1005, 33, 193, 4, 31, "Input"],
Cell[1201, 39, 5815, 122, 364, "Input"],
Cell[7019, 163, 3928, 97, 343, "Input"],
Cell[10950, 262, 6107, 126, 384, "Input"],
Cell[17060, 390, 6842, 143, 424, "Input"],
Cell[23905, 535, 9508, 208, 639, "Input"],
Cell[33416, 745, 7212, 178, 639, "Input"],
Cell[40631, 925, 9498, 208, 659, "Input"],
Cell[50132, 1135, 10388, 226, 779, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[60557, 1366, 123, 1, 101, "Title"],
Cell[60683, 1369, 320, 6, 57, "Subsubtitle"],
Cell[61006, 1377, 4970, 114, 332, "Input"],
Cell[CellGroupData[{
Cell[66001, 1495, 240, 3, 57, "Subsubtitle"],
Cell[66244, 1500, 18679, 365, 992, "Input"],
Cell[84926, 1867, 18822, 367, 1032, "Input"],
Cell[103751, 2236, 19318, 379, 1032, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[123118, 2621, 131, 1, 101, "Title"],
Cell[123252, 2624, 868, 24, 112, "Input"],
Cell[124123, 2650, 988, 29, 112, "Input"],
Cell[125114, 2681, 5658, 140, 392, "Input"],
Cell[130775, 2823, 77061, 1652, 6052, "Input"],
Cell[207839, 4477, 77328, 1652, 6052, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[285204, 6134, 112, 1, 101, "Title"],
Cell[285319, 6137, 4387, 101, 352, "Input"],
Cell[289709, 6240, 3685, 89, 332, "Input"],
Cell[293397, 6331, 14551, 306, 852, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.}],
Cell[307951, 6639, 14160, 302, 812, "Input"],
Cell[322114, 6943, 18259, 374, 1012, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[340410, 7322, 109, 1, 101, "Title"],
Cell[340522, 7325, 96, 1, 31, "Input"],
Cell[340621, 7328, 3676, 90, 392, "Input"],
Cell[344300, 7420, 5070, 123, 432, "Input"],
Cell[349373, 7545, 5279, 127, 552, "Input"],
Cell[354655, 7674, 2549, 62, 272, "Input"],
Cell[357207, 7738, 2553, 62, 272, "Input"],
Cell[359763, 7802, 2776, 65, 292, "Input"],
Cell[362542, 7869, 4857, 120, 567, "Input"],
Cell[367402, 7991, 1757, 45, 262, "Input"],
Cell[CellGroupData[{
Cell[369184, 8040, 1016, 22, 112, "Input"],
Cell[370203, 8064, 478, 6, 31, "Output"],
Cell[370684, 8072, 482, 6, 31, "Output"],
Cell[371169, 8080, 482, 6, 31, "Output"]
}, Open  ]],
Cell[371666, 8089, 9848, 217, 772, "Input"],
Cell[381517, 8308, 70, 1, 31, "Input"],
Cell[CellGroupData[{
Cell[381612, 8313, 131, 2, 31, "Input"],
Cell[381746, 8317, 316, 4, 31, "Output"]
}, Open  ]],
Cell[382077, 8324, 7801, 180, 552, "Input"],
Cell[389881, 8506, 7532, 179, 552, "Input"],
Cell[397416, 8687, 12385, 290, 1092, "Input"],
Cell[409804, 8979, 12481, 309, 832, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[422322, 9293, 112, 1, 101, "Title"],
Cell[CellGroupData[{
Cell[422459, 9298, 4868, 134, 252, "Input"],
Cell[427330, 9434, 1647, 39, 197, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[429014, 9478, 10943, 279, 312, "Input"],
Cell[439960, 9759, 2717, 60, 409, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[442714, 9824, 22325, 542, 592, "Input"],
Cell[465042, 10368, 6358, 123, 445, "Output"]
}, Open  ]],
Cell[471415, 10494, 988, 25, 31, "Input"],
Cell[CellGroupData[{
Cell[472428, 10523, 663, 14, 52, "Input"],
Cell[473094, 10539, 2424, 38, 197, "Output"],
Cell[475521, 10579, 2426, 38, 197, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[477984, 10622, 135, 2, 31, "Input"],
Cell[478122, 10626, 5822, 116, 445, "Output"]
}, Open  ]],
Cell[483959, 10745, 553, 16, 31, "Input"],
Cell[484515, 10763, 89, 1, 52, "Input"],
Cell[CellGroupData[{
Cell[484629, 10768, 23833, 577, 672, "Input"],
Cell[508465, 11347, 5385, 103, 445, "Output"],
Cell[513853, 11452, 5780, 116, 478, "Output"]
}, Open  ]],
Cell[519648, 11571, 810, 22, 31, "Input"],
Cell[520461, 11595, 89, 1, 52, "Input"],
Cell[520553, 11598, 6990, 158, 419, "Input"],
Cell[527546, 11758, 26048, 543, 1600, "Input"],
Cell[553597, 12303, 10032, 235, 740, "Input"],
Cell[563632, 12540, 10401, 242, 760, "Input"],
Cell[574036, 12784, 13716, 311, 932, "Input"],
Cell[587755, 13097, 843, 23, 31, "Input"],
Cell[588601, 13122, 9353, 222, 852, "Input"],
Cell[597957, 13346, 9583, 225, 852, "Input"],
Cell[607543, 13573, 9070, 203, 692, "Input"],
Cell[616616, 13778, 17741, 399, 1212, "Input"],
Cell[634360, 14179, 8216, 189, 652, "Input"],
Cell[642579, 14370, 13994, 313, 832, "Input"],
Cell[656576, 14685, 20745, 455, 1432, "Input"],
Cell[677324, 15142, 20813, 451, 1412, "Input"],
Cell[698140, 15595, 94, 1, 31, "Input"],
Cell[698237, 15598, 20690, 448, 1292, "Input"],
Cell[718930, 16048, 27674, 592, 1452, "Input"],
Cell[746607, 16642, 8743, 211, 552, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[755387, 16858, 111, 1, 101, "Title"],
Cell[755501, 16861, 1417, 41, 91, "Input"],
Cell[756921, 16904, 16255, 362, 91, "Input"],
Cell[773179, 17268, 15885, 360, 91, "Input"],
Cell[789067, 17630, 15423, 347, 91, "Input"],
Cell[804493, 17979, 15274, 348, 91, "Input"],
Cell[819770, 18329, 7068, 148, 91, "Input"],
Cell[826841, 18479, 9461, 192, 91, "Input"]
}, Closed]]
}
]
*)

(* End of internal cache information *)
